@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,20 +24,27 @@
 #include <assert.h>
 #include <float.h>
 
+#include "algos/cie_standard_observer.h" // Do not include this from anywhere else
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/icc_profile.h"
 #include "core/processing.h"
-#include "core/undo.h"
 #include "core/siril_log.h"
 #include "core/OS_utils.h"
 #include "gui/progress_and_log.h"
 #include "gui/histogram.h"
-#include "io/single_image.h"
 #include "io/image_format_fits.h"
 #include "algos/colors.h"
 #include "algos/statistics.h"
 #include "algos/extraction.h"
 
+/******************************************************************************
+ * Note for maintainers: do not use the translation macro on the following    *
+ * string. Color management relies on being able to detect "Extraction"       *
+ * in FITS HISTORY header.                                                    *
+ ******************************************************************************/
+const gchar *extractionstring = "Extraction";
+
 static gchar *add_filter_str[] = { "R", "G", "B"};
 /*
  * A Fast HSL-to-RGB Transform
@@ -48,6 +55,7 @@ static gchar *add_filter_str[] = { "R", "G", "B"};
  *  * given h,s,l on [0..1],
  *   * return r,g,b on [0..1]
  *    */
+
 void hsl_to_rgb_float_sat(float h, float sl, float l, float * r, float * g,
 		float * b) {
 	float v;
@@ -262,99 +270,79 @@ void rgb_to_hsl(double r, double g, double b, double *h, double *s, double *l) {
 	*h /= 6;
 }
 
-// Single precision versions of the above
-void hsl_to_rgbf(float h, float s, float l, float * r, float * g,
-		float * b) {
-	float v;
+void rgb_to_hslf(float r, float g, float b, float *h, float *s, float *l) {
+    float v = fmaxf(fmaxf(r, g), b);
+    float m = fminf(fminf(r, g), b);
+    float vm = v - m;
 
-	assert(h >= 0.f && h <= 1.f);
-	if (h >= 1.f) h -= 1.f;		// this code doesn't work for h = 1
-	v = (l <= 0.5f) ? (l * (1.f + s)) : (l + s - l * s);
-	if (v <= 0.f) {
-		*r = *g = *b = 0.f;
-	} else {
-		float m;
-		float sv;
-		int sextant;
-		float fract, vsf, mid1, mid2;
+    *l = (m + v) / 2.0f;
 
-		m = l + l - v;
-		sv = (v - m) / v;
-		h *= 6.f;
-		sextant = h;
-		fract = h - sextant;
-		vsf = v * sv * fract;
-		mid1 = m + vsf;
-		mid2 = v - vsf;
-		switch (sextant) {
-			case 0:
-				*r = v;
-				*g = mid1;
-				*b = m;
-				break;
-			case 1:
-				*r = mid2;
-				*g = v;
-				*b = m;
-				break;
-			case 2:
-				*r = m;
-				*g = v;
-				*b = mid1;
-				break;
-			case 3:
-				*r = m;
-				*g = mid2;
-				*b = v;
-				break;
-			case 4:
-				*r = mid1;
-				*g = m;
-				*b = v;
-				break;
-			case 5:
-				*r = v;
-				*g = m;
-				*b = mid2;
-				break;
-		}
-	}
+    if (vm == 0.0f) {
+        *h = 0.0f;
+        *s = 0.0f;
+        return;
+    }
+
+    *s = (*l <= 0.5f) ? (vm / (v + m)) : (vm / (2.0f - v - m));
+
+    float r2 = (v - r) / vm;
+    float g2 = (v - g) / vm;
+    float b2 = (v - b) / vm;
+
+    float hr = (r == v) * (g == m ? 5.0f + b2 : 1.0f - g2);
+    float hg = (g == v) * (b == m ? 1.0f + r2 : 3.0f - b2);
+    float hb = (b == v) * (r == m ? 3.0f + g2 : 5.0f - r2);
+
+    *h = hr + hg + hb;
+    *h /= 6.0f;
 }
 
-void rgb_to_hslf(float r, float g, float b, float *h, float *s, float *l) {
-	float v;
-	float m;
-	float vm;
-	float r2, g2, b2;
+void hsl_to_rgbf(float h, float s, float l, float *r, float *g, float *b) {
 
-	v = max(r, g);
-	v = max(v, b);
-	m = min(r, g);
-	m = min(m, b);
-	*h = 0.f;
-	*s = 0.f;	// init values
+	h = fmodf(h, 1.0f);  // ensure h is in the range [0, 1)
+	float v = (l <= 0.5f) ? (l * (1.f + s)) : (l + s - l * s);
 
-	if ((*l = (m + v) / 2.f) <= 0.f) {
-		*l = 0.f;
+	if (v <= 0.f) {
+		*r = *g = *b = 0.f;
 		return;
 	}
-	if ((*s = vm = v - m) > 0.f) {
-		*s /= (*l <= 0.5f) ? (v + m) : (2.f - v - m);
-	} else
-		return;
 
-	r2 = (v - r) / vm;
-	g2 = (v - g) / vm;
-	b2 = (v - b) / vm;
+	float m = l + l - v;
+	float sv = (v - m) / v;
+	float h6 = h * 6.0f;
+	int sextant = (int)h6;
+	float fract = h6 - sextant;
+	float vsf = v * sv * fract;
+	float mid1 = m + vsf;
+	float mid2 = v - vsf;
 
-	if (r == v)
-		*h = (g == m ? 5.f + b2 : 1.f - g2);
-	else if (g == v)
-		*h = (b == m ? 1.f + r2 : 3.f - b2);
-	else
-		*h = (r == m ? 3.f + g2 : 5.f - r2);
+	*r = (sextant == 0 || sextant == 5) ? v : (sextant == 2 || sextant == 3) ? m : (sextant == 4) ? mid1 : mid2;
+	*g = (sextant == 1 || sextant == 2) ? v : (sextant == 4 || sextant == 5) ? m : (sextant == 0) ? mid1 : mid2;
+	*b = (sextant == 3 || sextant == 4) ? v : (sextant == 0 || sextant == 1) ? m : (sextant == 2) ? mid1 : mid2;
+}
 
-	*h /= 6.f;
+void rgbw_to_hslw(uint16_t r, uint16_t g, uint16_t b, uint16_t *h, uint16_t *s, uint16_t *l) {
+	// Normalize RGB values to [0, 1]
+	float rf = r / USHRT_MAX_SINGLE;
+	float gf = g / USHRT_MAX_SINGLE;
+	float bf = b / USHRT_MAX_SINGLE;
+	float hf, sf, lf;
+	rgb_to_hslf(rf, gf, bf, &hf, &sf, &lf);
+	*h = roundf_to_WORD(hf * USHRT_MAX_SINGLE);
+	*s = roundf_to_WORD(sf * USHRT_MAX_SINGLE);
+	*l = roundf_to_WORD(lf * USHRT_MAX_SINGLE);
+}
+
+void hslw_to_rgbw(uint16_t h, uint16_t s, uint16_t l, uint16_t *r, uint16_t *g, uint16_t *b) {
+	// Normalize HSL values to [0, 1]
+	float hf = h / USHRT_MAX_SINGLE;
+	float sf = s / USHRT_MAX_SINGLE;
+	float lf = l / USHRT_MAX_SINGLE;
+	float rf, gf, bf;
+	hsl_to_rgbf(hf, sf, lf, &rf, &gf, &bf);
+	*r = roundf_to_WORD(rf * USHRT_MAX_SINGLE);
+	*g = roundf_to_WORD(gf * USHRT_MAX_SINGLE);
+	*b = roundf_to_WORD(bf * USHRT_MAX_SINGLE);
 }
 
 /* all variables are between 0 and 1. h takes 0 for grey */
@@ -385,6 +373,33 @@ void rgb_to_hsv(double r, double g, double b, double *h, double *s, double *v) {
 		*h += 1.0;
 }
 
+void rgb_to_hsvf(float r, float g, float b, float *h, float *s, float *v) {
+	float cmax, cmin, delta;
+
+	cmax = max(r, g);
+	cmax = max(cmax, b);
+	cmin = min(r, g);
+	cmin = min(cmin, b);
+	delta = cmax - cmin;
+	*v = cmax;
+	if (delta == 0.f) {
+		*s = 0.f;
+		*h = 0.f;
+		return;
+	}
+	*s = delta / cmax;
+
+	if (cmax == r)
+		*h = (((g - b) / delta)) / 6.f;
+	else if (cmax == g)
+		*h = (((b - r) / delta) + 2.f) / 6.f;
+	else
+		*h = (((r - g) / delta) + 4.f) / 6.f;
+
+	if (*h < 0.f)
+		*h += 1.f;
+}
+
 void hsv_to_rgb(double h, double s, double v, double *r, double *g, double *b) {
 	double p, q, t, f;
 	int i;
@@ -531,7 +546,6 @@ void yuv_to_rgbf(float y, float u, float v, float *red, float *green, float *blu
 	*blue = (a * y) - (b * u) + (c * 0.5f * v);
 }
 
-
 void xyz_to_LAB(double x, double y, double z, double *L, double *a, double *b) {
 	x /= 95.047;
 	y /= 100.000;
@@ -627,7 +641,8 @@ void xyz_to_rgbf(float x, float y, float z, float *r, float *g, float *b) {
 	*b = (*b > 0.0031308f) ? 1.055f * (powf(*b, (1.f / 2.4f))) - 0.055f : 12.92f * (*b);
 }
 
-// color index to temperature in kelvin
+// Reference: https://en.wikipedia.org/wiki/Color_index and https://arxiv.org/abs/1201.1809 (Ballesteros, F. J., 2012)
+// Uses Ballesteros' formula based on considering stars as black bodies
 double BV_to_T(double BV) {
 	double T;
 
@@ -644,6 +659,83 @@ double BV_to_T(double BV) {
 	return T;
 }
 
+// CIE XYZ Color Matching Functions
+float x1931(float w) {
+	int index = w - 360;
+	float w2_w = w - (float) w;
+	float w1_w = 1.f - w2_w;
+	index = max(min(index, 470), 0);
+	float retval = xyz1931_1nm[index][0] * w1_w + xyz1931_1nm[index+1][0] * w2_w;
+	return retval;
+}
+
+float y1931(float w) {
+	int index = w - 360;
+	float w2_w = w - (float) w;
+	float w1_w = 1.f - w2_w;
+	index = max(min(index, 470), 0);
+	float retval = xyz1931_1nm[index][1] * w1_w + xyz1931_1nm[index+1][1] * w2_w;
+	return retval;
+}
+
+float z1931(float w) {
+	int index = w - 360;
+	float w2_w = w - (float) w;
+	float w1_w = 1.f - w2_w;
+	index = max(min(index, 470), 0);
+	float retval = xyz1931_1nm[index][2] * w1_w + xyz1931_1nm[index+1][2] * w2_w;
+	return retval;
+}
+
+float x1964(float w) {
+	int index = w - 360;
+	float w2_w = w - (float) w;
+	float w1_w = 1.f - w2_w;
+	index = max(min(index, 470), 0);
+	float retval = xyz1964_1nm[index][0] * w1_w + xyz1964_1nm[index+1][0] * w2_w;
+	return retval;
+}
+
+float y1964(float w) {
+	int index = w - 360;
+	float w2_w = w - (float) w;
+	float w1_w = 1.f - w2_w;
+	index = max(min(index, 470), 0);
+	float retval = xyz1964_1nm[index][1] * w1_w + xyz1964_1nm[index+1][1] * w2_w;
+	return retval;
+}
+
+float z1964(float w) {
+	int index = w - 360;
+	float w2_w = w - (float) w;
+	float w1_w = 1.f - w2_w;
+	index = max(min(index, 470), 0);
+	float retval = xyz1964_1nm[index][2] * w1_w + xyz1964_1nm[index+1][2] * w2_w;
+	return retval;
+}
+
+cmsCIExyY wl_to_xyY(double wl) {
+	cmsCIEXYZ XYZ;
+	cmsCIExyY xyY;
+	if (com.pref.icc.cmf == CMF_1931_2DEG) {
+		XYZ = (cmsCIEXYZ) { x1931(wl), y1931(wl), z1931(wl) };
+	} else {
+		XYZ = (cmsCIEXYZ) { x1964(wl), y1964(wl), z1964(wl) };
+	}
+	cmsXYZ2xyY(&xyY, &XYZ);
+	return xyY;
+}
+
+// Returns the emittance of a Planckian black body spectrum at wavelength
+// wl and temperature bbTemp
+// from "Colour Rendering of Spectra", John Walker, Fourmilab. Public domain code, last updated March 9 2003
+/*
+float bb_spectrum(float wl, float bbTemp) {
+	float wlm = wl * 1e-9;   // Wavelength in meters
+	return (3.74183e-16f * pow(wlm, -5.f)) / (expf(1.4388e-2f / (wlm * bbTemp)) - 1.f);
+}
+*/
+
 int equalize_cfa_fit_with_coeffs(fits *fit, float coeff1, float coeff2, const char *cfa_string) {
 	unsigned int row, col, pat_cell;
 	/* compute width of the (square) CFA pattern */
@@ -701,11 +793,33 @@ static gpointer extract_channels_ushort(gpointer p) {
 	siril_log_color_message(_("%s channel extraction: processing...\n"), "green",
 			args->str_type);
 	gettimeofday(&t_start, NULL);
+	gchar *histstring = NULL;
+	cmsHPROFILE cielab_profile = NULL, image_profile = NULL;
+	cmsColorSpaceSignature sig;
+	cmsUInt32Number trans_type, lab_type;
+	gboolean threaded;
+	cmsHTRANSFORM transform = NULL;
+	cmsUInt32Number datasize;
+	cmsUInt32Number bytesperline;
+	cmsUInt32Number bytesperplane;
+	gchar *desc = NULL;
+	if(args->fit->icc_profile) {
+		desc = siril_color_profile_get_description(args->fit->icc_profile);
+		cmsCloseProfile(args->fit->icc_profile);
+	}
+	/* The extracted channels are considered raw data, and are not color
+		* managed. It is up to the user to ensure that future use of them is
+		* with similar data and an appropriate color profile is assigned.
+		* See also the HSV and CIELAB cases below.*/
+	args->fit->icc_profile = NULL;
+	color_manage(args->fit, FALSE);
 
 	switch (args->type) {
 	case EXTRACT_RGB:
+		histstring = g_strdup_printf(_("%s: extract RGB channel"), extractionstring);
 		break;
 	case EXTRACT_HSL:
+		histstring = g_strdup_printf(_("%s: extract HSL channel"), extractionstring);
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
@@ -714,13 +828,15 @@ static gpointer extract_channels_ushort(gpointer p) {
 			double r = (double) buf[RLAYER][i] / USHRT_MAX_DOUBLE;
 			double g = (double) buf[GLAYER][i] / USHRT_MAX_DOUBLE;
 			double b = (double) buf[BLAYER][i] / USHRT_MAX_DOUBLE;
+			// RGB to HSL is a coordinate transform and does not require lcms
 			rgb_to_hsl(r, g, b, &h, &s, &l);
-			buf[RLAYER][i] = round_to_WORD(h * 360.0);	// TODO: what's that?
+			buf[RLAYER][i] = round_to_WORD(h * 360.0);
 			buf[GLAYER][i] = round_to_WORD(s * USHRT_MAX_DOUBLE);
 			buf[BLAYER][i] = round_to_WORD(l * USHRT_MAX_DOUBLE);
 		}
 		break;
 	case EXTRACT_HSV:
+		histstring = g_strdup_printf(_("%s: extract HSV channel"), extractionstring);
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
@@ -729,6 +845,7 @@ static gpointer extract_channels_ushort(gpointer p) {
 			double r = (double) buf[RLAYER][i] / USHRT_MAX_DOUBLE;
 			double g = (double) buf[GLAYER][i] / USHRT_MAX_DOUBLE;
 			double b = (double) buf[BLAYER][i] / USHRT_MAX_DOUBLE;
+			// RGB to HSV is a coordinate transform and does not require lcms
 			rgb_to_hsv(r, g, b, &h, &s, &v);
 			buf[RLAYER][i] = round_to_WORD(h * 360.0);
 			buf[GLAYER][i] = round_to_WORD(s * USHRT_MAX_DOUBLE);
@@ -736,33 +853,62 @@ static gpointer extract_channels_ushort(gpointer p) {
 		}
 		break;
 	case EXTRACT_CIELAB:
-#ifdef _OPENMP
-#pragma omp parallel for num_threads(com.max_thread) schedule(static)
-#endif
-		for (size_t i = 0; i < n; i++) {
-			double x, y, z, L, a, b;
-			double red = (double) buf[RLAYER][i] / USHRT_MAX_DOUBLE;
-			double green = (double) buf[GLAYER][i] / USHRT_MAX_DOUBLE;
-			double blue = (double) buf[BLAYER][i] / USHRT_MAX_DOUBLE;
-			rgb_to_xyz(red, green, blue, &x, &y, &z);
-			xyz_to_LAB(x, y, z, &L, &a, &b);
-			buf[RLAYER][i] = round_to_WORD(L / 100. * USHRT_MAX_DOUBLE);// 0 < L < 100
-			buf[GLAYER][i] = round_to_WORD(
-					((a + 128) / 255.) * USHRT_MAX_DOUBLE);	// -128 < a < 127
-			buf[BLAYER][i] = round_to_WORD(
-					((b + 128) / 255.) * USHRT_MAX_DOUBLE);	// -128 < b < 127
+		histstring = g_strdup_printf(_("%s: extract LAB channel"), extractionstring);
+		cielab_profile = cmsCreateLab4Profile(NULL);
+		if (args->fit->icc_profile) {
+			image_profile = copyICCProfile(args->fit->icc_profile);
+		} else {
+			siril_log_message(_("Image is not color managed. Assuming sRGB.\n"));
+			image_profile = srgb_trc();
 		}
-
+		sig = cmsGetColorSpace(image_profile);
+		trans_type = get_planar_formatter_type(sig, args->fit->type, FALSE);
+		lab_type = TYPE_Lab_16_PLANAR;
+		threaded = !get_thread_run();
+		// We use sRGB as the fallback for non-color managed images
+		transform = cmsCreateTransformTHR((threaded ? com.icc.context_threaded : com.icc.context_single), image_profile, trans_type, cielab_profile, lab_type, INTENT_PERCEPTUAL, com.icc.rendering_flags);
+		cmsCloseProfile(cielab_profile);
+		cmsCloseProfile(image_profile);
+		datasize = sizeof(WORD);
+		bytesperline = args->fit->rx * datasize;
+		bytesperplane = args->fit->rx * args->fit->ry * datasize;
+		cmsDoTransformLineStride(transform, args->fit->data, args->fit->data, args->fit->rx, args->fit->ry, bytesperline, bytesperline, bytesperplane, bytesperplane);
+		cmsDeleteTransform(transform);
+	}
+	gchar *fitfilter = g_strdup(args->fit->keywords.filter);
+	if (desc) {
+		args->fit->history = g_slist_append(args->fit->history, g_strdup_printf(_("Channel extraction from 3-channel image with ICC profile:")));
+		args->fit->history = g_slist_append(args->fit->history, g_strdup_printf("%s", desc));
 	}
-	gchar *fitfilter = g_strdup(args->fit->filter);
 	for (int i = 0; i < 3; i++) {
 		if (args->channel[i]) {
 			update_filter_information(args->fit, add_filter_str[i], TRUE);
+			if (i > 0) {
+				GSList *current = args->fit->history;
+				while (current->next != NULL && current->next->next != NULL) {
+					current = current->next;
+				}
+				// Check if there is only one element in the list.
+				if (current->next == NULL) {
+					g_slist_free_full(args->fit->history, g_free);
+					args->fit->history = NULL;
+				} else {
+					// Remove the last element.
+					GSList *last = current->next;
+					current->next = NULL;
+					g_free(last->data);
+					g_slist_free_1(last);
+				}
+			}
+			args->fit->history = g_slist_append(args->fit->history, g_strdup_printf("%s %d", histstring, i));
+			args->fit->keywords.bayer_pattern[0] = '\0'; // Mark this as no longer having a Bayer pattern
 			save1fits16(args->channel[i], args->fit, i);
 			update_filter_information(args->fit, fitfilter, FALSE); //reinstate original filter name
 		}
 	}
 	g_free(fitfilter);
+	g_free(histstring);
+	g_free(desc);
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
 
@@ -785,65 +931,130 @@ static gpointer extract_channels_float(gpointer p) {
 	siril_log_color_message(_("%s channel extraction: processing...\n"), "green",
 			args->str_type);
 	gettimeofday(&t_start, NULL);
+	gchar *histstring = NULL;
+	cmsHPROFILE cielab_profile = NULL, image_profile = NULL;
+	cmsColorSpaceSignature sig;
+	cmsUInt32Number trans_type, lab_type;
+	gboolean threaded;
+	cmsHTRANSFORM transform = NULL;
+	cmsUInt32Number datasize;
+	cmsUInt32Number bytesperline;
+	cmsUInt32Number bytesperplane;
+	gchar *desc = NULL;
+	if(args->fit->icc_profile) {
+		desc = siril_color_profile_get_description(args->fit->icc_profile);
+		cmsCloseProfile(args->fit->icc_profile);
+	}
+	/* The extracted channels are considered raw data, and are not color
+		* managed. It is up to the user to ensure that future use of them is
+		* with similar data and an appropriate color profile is assigned.
+		* See also the HSV and CIELAB cases below.*/
+	args->fit->icc_profile = NULL;
+	color_manage(args->fit, FALSE);
 
 	switch (args->type) {
-	case EXTRACT_RGB:
-		break;
-	case EXTRACT_HSL:
+		case EXTRACT_RGB:
+			histstring = g_strdup_printf(_("%s: extract RGB channel"), extractionstring);
+			break;
+		case EXTRACT_HSL:
+			histstring = g_strdup_printf(_("%s: extract HSL channel"), extractionstring);
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
-		for (size_t i = 0; i < n; i++) {
-			double h, s, l;
-			double r = (double) buf[RLAYER][i];
-			double g = (double) buf[GLAYER][i];
-			double b = (double) buf[BLAYER][i];
-			rgb_to_hsl(r, g, b, &h, &s, &l);
-			buf[RLAYER][i] = (float) h;
-			buf[GLAYER][i] = (float) s;
-			buf[BLAYER][i] = (float) l;
-		}
-		break;
-	case EXTRACT_HSV:
+			for (size_t i = 0; i < n; i++) {
+				double h, s, l;
+				double r = (double) buf[RLAYER][i];
+				double g = (double) buf[GLAYER][i];
+				double b = (double) buf[BLAYER][i];
+				rgb_to_hsl(r, g, b, &h, &s, &l);
+				buf[RLAYER][i] = (float) h;
+				buf[GLAYER][i] = (float) s;
+				buf[BLAYER][i] = (float) l;
+			}
+			break;
+		case EXTRACT_HSV:
+			histstring = g_strdup_printf(_("%s: extract HSV channel"), extractionstring);
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
-		for (size_t i = 0; i < n; i++) {
-			double h, s, v;
-			double r = (double) buf[RLAYER][i];
-			double g = (double) buf[GLAYER][i];
-			double b = (double) buf[BLAYER][i];
-			rgb_to_hsv(r, g, b, &h, &s, &v);
-			buf[RLAYER][i] = (float) h;
-			buf[GLAYER][i] = (float) s;
-			buf[BLAYER][i] = (float) v;
-		}
-		break;
-	case EXTRACT_CIELAB:
+			for (size_t i = 0; i < n; i++) {
+				double h, s, v;
+				double r = (double) buf[RLAYER][i];
+				double g = (double) buf[GLAYER][i];
+				double b = (double) buf[BLAYER][i];
+				rgb_to_hsv(r, g, b, &h, &s, &v);
+				buf[RLAYER][i] = (float) h;
+				buf[GLAYER][i] = (float) s;
+				buf[BLAYER][i] = (float) v;
+			}
+			break;
+		case EXTRACT_CIELAB:
+			histstring = g_strdup_printf(_("%s: extract LAB channel"), extractionstring);
+			cielab_profile = cmsCreateLab4Profile(NULL);
+			if (args->fit->icc_profile) {
+				image_profile = copyICCProfile(args->fit->icc_profile);
+			} else {
+				siril_log_message(_("Image is not color managed. Assuming sRGB.\n"));
+				image_profile = srgb_trc();
+			}
+			sig = cmsGetColorSpace(image_profile);
+			trans_type = get_planar_formatter_type(sig, args->fit->type, FALSE);
+			lab_type = TYPE_Lab_FLT_PLANAR;
+			threaded = !get_thread_run();
+			transform = cmsCreateTransformTHR((threaded ? com.icc.context_threaded : com.icc.context_single), image_profile, trans_type, cielab_profile, lab_type, com.pref.icc.processing_intent, com.icc.rendering_flags);
+			cmsCloseProfile(cielab_profile);
+			cmsCloseProfile(image_profile);
+			datasize = sizeof(float);
+			bytesperline = args->fit->rx * datasize;
+			bytesperplane = args->fit->rx * args->fit->ry * datasize;
+			cmsDoTransformLineStride(transform, args->fit->fdata, args->fit->fdata, args->fit->rx, args->fit->ry, bytesperline, bytesperline, bytesperplane, bytesperplane);
+			cmsDeleteTransform(transform);
+			/*  Convert L* to Siril 0.0 - 1.0 range
+			* TODO: long term, make Siril able to cope with images with different ranges
+			* by accounting for them in the display transform
+			*/
+			const size_t npixels = args->fit->rx * args->fit->ry;
 #ifdef _OPENMP
-#pragma omp parallel for num_threads(com.max_thread) schedule(static)
+#pragma omp parallel for schedule(static) num_threads(com.max_thread) if (com.max_thread > 1)
 #endif
-		for (size_t i = 0; i < n; i++) {
-			double x, y, z, L, a, b;
-			double red = (double) buf[RLAYER][i];
-			double green = (double) buf[GLAYER][i];
-			double blue = (double) buf[BLAYER][i];
-			rgb_to_xyz(red, green, blue, &x, &y, &z);
-			xyz_to_LAB(x, y, z, &L, &a, &b);
-			buf[RLAYER][i] = (float) (L / 100.);		// 0 < L < 100
-			buf[GLAYER][i] = (float) ((a + 128.) / 255.);	// -128 < a < 127
-			buf[BLAYER][i] = (float) ((b + 128.) / 255.);	// -128 < b < 127
-		}
-
+			for (size_t i = 0 ; i < npixels ; i++) {
+				args->fit->fdata[i] *= 0.01f;
+			}
+			siril_log_message(_("Note: L* channel is scaled to have values in the rang 0.0 - 1.0 to aid editing in Siril. The CIE L*a*b* colorspace defines L* to have a range of 0.0 - 100.0, so if you need to use this as a true L* channel in other software you will need to multiply the values by 100, for example using the Siril command \"fmul 100\"\n"));
+	}
+	gchar *fitfilter = g_strdup(args->fit->keywords.filter);
+	if (desc) {
+		args->fit->history = g_slist_append(args->fit->history, g_strdup_printf(_("Channel extraction from 3-channel image with ICC profile:")));
+		args->fit->history = g_slist_append(args->fit->history, g_strdup_printf("%s", desc));
 	}
-	gchar *fitfilter = g_strdup(args->fit->filter);
 	for (int i = 0; i < 3; i++) {
 		if (args->channel[i]) {
 			update_filter_information(args->fit, add_filter_str[i], TRUE);
+			if (i > 0) {
+				GSList *current = args->fit->history;
+				while (current->next != NULL && current->next->next != NULL) {
+					current = current->next;
+				}
+				// Check if there is only one element in the list.
+				if (current->next == NULL) {
+					g_slist_free_full(args->fit->history, g_free);
+					args->fit->history = NULL;
+				} else {
+					// Remove the last element.
+					GSList *last = current->next;
+					current->next = NULL;
+					g_free(last->data);
+					g_slist_free_1(last);
+				}
+			}
+			args->fit->history = g_slist_append(args->fit->history, g_strdup_printf("%s %d", histstring, i));
+			args->fit->keywords.bayer_pattern[0] = '\0'; // Mark this as no longer having a Bayer pattern
 			save1fits32(args->channel[i], args->fit, i);
 			update_filter_information(args->fit, fitfilter, FALSE); //reinstate original filter name
 		}
 	}
+	g_free(desc);
+	g_free(histstring);
 	g_free(fitfilter);
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
@@ -1049,3 +1260,84 @@ int pos_to_neg(fits *fit) {
 	return 0;
 }
 
+void ccm_float(fits *fit, ccm matrix, float power) {
+	size_t npixels = fit->rx * fit->ry;
+#ifdef _OPENMP
+#pragma omp parallel for num_threads(com.max_thread) schedule(static)
+#endif
+	for (size_t i = 0 ; i < npixels ; i++) {
+		float r = fit->fpdata[RLAYER][i] * matrix[0][0] + fit->fpdata[GLAYER][i] * matrix[0][1] + fit->fpdata[BLAYER][i] * matrix[0][2];
+		float g = fit->fpdata[RLAYER][i] * matrix[1][0] + fit->fpdata[GLAYER][i] * matrix[1][1] + fit->fpdata[BLAYER][i] * matrix[1][2];
+		float b = fit->fpdata[RLAYER][i] * matrix[2][0] + fit->fpdata[GLAYER][i] * matrix[2][1] + fit->fpdata[BLAYER][i] * matrix[2][2];
+		fit->fpdata[RLAYER][i] = powf(r, 1.f / power);
+		fit->fpdata[GLAYER][i] = powf(g, 1.f / power);
+		fit->fpdata[BLAYER][i] = powf(b, 1.f / power);
+	}
+}
+
+void ccm_ushort(fits *fit, ccm matrix, float power) {
+	size_t npixels = fit->rx * fit->ry;
+	float norm = USHRT_MAX_SINGLE;
+	float invnorm = 1.f / norm;
+#ifdef _OPENMP
+#pragma omp parallel for num_threads(com.max_thread) schedule(static)
+#endif
+	for (size_t i = 0 ; i < npixels ; i++) {
+		float r = (float) fit->pdata[RLAYER][i] * matrix[0][0] + (float) fit->pdata[GLAYER][i] * matrix[0][1] + (float) fit->pdata[BLAYER][i] * matrix[0][2];
+		float g = (float) fit->pdata[RLAYER][i] * matrix[1][0] + (float) fit->pdata[GLAYER][i] * matrix[1][1] + (float) fit->pdata[BLAYER][i] * matrix[1][2];
+		float b = (float) fit->pdata[RLAYER][i] * matrix[2][0] + (float) fit->pdata[GLAYER][i] * matrix[2][1] + (float) fit->pdata[BLAYER][i] * matrix[2][2];
+		fit->pdata[RLAYER][i] = roundf_to_WORD(norm * powf(r * invnorm, 1.f / power));
+		fit->pdata[GLAYER][i] = roundf_to_WORD(norm * powf(g * invnorm, 1.f/ power));
+		fit->pdata[BLAYER][i] = roundf_to_WORD(norm * powf(b * invnorm, 1.f / power));
+	}
+}
+
+int ccm_calc(fits *fit, ccm matrix, float power) {
+	// We require a 3-channel FITS
+	if (!isrgb(fit))
+		return 1;
+	fit->type == DATA_FLOAT ? ccm_float(fit, matrix, power) : ccm_ushort(fit, matrix, power);
+	return 0;
+}
+
+static int ccm_image_hook(struct generic_seq_args *args, int o, int i, fits *fit,
+		rectangle *_, int threads) {
+	struct ccm_data *c_args = (struct ccm_data*) args->user;
+	int ret = ccm_calc(fit, c_args->matrix, c_args->power);
+	if (ret) {
+		siril_log_color_message(_("Color Conversion Matrices can only be applied to 3-channel images.\n"), "red");
+	}
+	return ret;
+}
+
+static int ccm_finalize_hook(struct generic_seq_args *args) {
+	struct ccm_data *c_args = (struct ccm_data*) args->user;
+	int retval = seq_finalize_hook(args);
+	free(c_args);
+	return retval;
+}
+
+void apply_ccm_to_sequence(struct ccm_data *ccm_args) {
+	struct generic_seq_args *args = create_default_seqargs(ccm_args->seq);
+	args->filtering_criterion = seq_filter_included;
+	args->nb_filtered_images = ccm_args->seq->selnum;
+	args->compute_mem_limits_hook = NULL;
+	args->prepare_hook = seq_prepare_hook;
+	args->finalize_hook = ccm_finalize_hook;
+	args->image_hook = ccm_image_hook;
+	args->stop_on_error = FALSE;
+	args->description = _("Color Conversion Matrices");
+	args->has_output = TRUE;
+	args->output_type = get_data_type(args->seq->bitpix);
+	args->new_seq_prefix = strdup(ccm_args->seqEntry);
+	args->load_new_sequence = TRUE;
+	args->user = ccm_args;
+
+	ccm_args->fit = NULL;	// not used here
+
+	if (!start_in_new_thread(generic_sequence_worker, args)) {
+		free(ccm_args->seqEntry);
+		free(ccm_args);
+		free_generic_seq_args(args, TRUE);
+	}
+}
