@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include "core/siril.h"
 #include "core/siril_log.h"
 #include "algos/statistics.h"
+#include "algos/siril_random.h"
 #include "io/conversion.h"
 #include "io/image_format_fits.h"
 
@@ -141,7 +142,7 @@ static int subtract_fudge(fits *fit, rectangle af, float fudge, af_pixel_matrix
 					// This is an auto focus pixel.  Subtract the fudge.
 
 					// Randomly add a 1 to some pixels to bring the average correction close to the computed value.
-					WORD fudgew_rand = rand() / (float) RAND_MAX >= fudge - (float) fudgew ? fudgew : fudgew + 1;
+					WORD fudgew_rand = siril_random_float() >= fudge - (float) fudgew ? fudgew : fudgew + 1;
 
 					// Save for debugging.
 					total_fudgew += fudgew_rand;
@@ -183,9 +184,9 @@ int fix_xtrans_ac(fits *fit) {
 	rectangle af, sam;
 	gboolean read_bottom_up = FALSE;
 
-	int model = get_model(fit->instrume);
+	int model = get_model(fit->keywords.instrume);
 	if (model < 0) {
-		siril_log_color_message(_("Fix X-Trans: Unknown camera %s, trying to read information from preferences.\n"), "red", fit->instrume);
+		siril_log_color_message(_("Fix X-Trans: Unknown camera %s, trying to read information from preferences.\n"), "red", fit->keywords.instrume);
 		if (com.pref.prepro.xtrans_af.w != 0 && com.pref.prepro.xtrans_af.h != 0) {
 			if (com.pref.prepro.xtrans_sample.w > fit->rx || com.pref.prepro.xtrans_sample.h > fit->ry) {
 				siril_log_color_message(_("Sample box cannot be bigger than the image.\n"), "red");
@@ -217,8 +218,9 @@ int fix_xtrans_ac(fits *fit) {
 	// Flip the image so the xtrans pattern makes sense.
 	// This matches logic in demosaicing.c.
 	read_bottom_up = (com.pref.debayer.use_bayer_header
-			&& !g_strcmp0(fit->row_order, "BOTTOM-UP"))
-			|| (!com.pref.debayer.top_down);
+			&& !g_strcmp0(fit->keywords.row_order, "BOTTOM-UP"))
+			// || (!com.pref.debayer.top_down);
+			;
 	if (read_bottom_up) { fits_flip_top_to_bottom(fit); }
 
 
@@ -250,7 +252,7 @@ int fix_xtrans_ac(fits *fit) {
 
 	// af_matrix is an RGB pattern where lowercase letters represent AF pixels and their color.
 	af_pixel_matrix af_matrix = { 0 };
-	set_af_matrix(fit->bayer_pattern, af_matrix);
+	set_af_matrix(fit->keywords.bayer_pattern, af_matrix);
 
 	WORD *buf = fit->pdata[RLAYER];
 	float *fbuf = fit->fpdata[RLAYER];
