@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
-#include "gui/utils.h"
 #include "gui/progress_and_log.h"
 #include "algos/quality.h"
 
@@ -214,7 +213,11 @@ static double Gradient(const float *buf, int width, int height) {
 				pixels++;
 			}
 	}
-
+	if (pixels == 0) {
+		siril_debug_print("Warning: pixels = 0 in Gradient(). Returning -1.0 to avoid div/0\n");
+		val = -1.0;
+		goto end;
+	}
 	val = val / (double)pixels; // normalise value to per-pixel
 	val = val / 10.0;
 end:
