@@ -2,8 +2,8 @@
  * This file is part of Siril, an astronomy image processor.
  *
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,28 +24,25 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-#include <locale.h>
 #include <gsl/gsl_matrix.h>
 #include <gsl/gsl_statistics.h>
 
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/siril_log.h"
+#include "algos/demosaicing.h"
 #include "algos/PSF.h"
 #include "algos/star_finder.h"
 #include "algos/statistics.h"
 #include "algos/sorting.h"
 #include "algos/siril_wcs.h"
-#include "io/single_image.h"
 #include "io/image_format_fits.h"
 #include "io/sequence.h"
 #include "gui/PSF_list.h"
+#include "gui/utils.h"
 #include "registration/registration.h"
 #include "opencv/opencv.h"
-#ifdef HAVE_WCSLIB
-#include <wcslib.h>
 #include <wcsfix.h>
-#endif
 
 #define _SQRT_EXP1 1.6487212707
 #define KERNEL_SIZE 2.  // sigma of the gaussian smoothing kernel
@@ -72,17 +69,25 @@ static float compute_threshold(image *image, double ksigma, int layer, rectangle
 		*bg = 0.0;
 		return 0;
 	}
-	threshold = (float)(stat->median + ksigma * stat->bgnoise);
-	*norm = stat->normValue;
-	*bg = stat->median;
-	*bgnoise = stat->bgnoise;
-	*max = stat->max;
+	double normvalue = 1.;
+	if (image->from_seq && image->from_seq->bitpix != image->fit->bitpix) { // stats are not correctly normalized, we need to correct that
+		double normavalueseq =  (image->from_seq->bitpix == BYTE_IMG) ? UCHAR_MAX_DOUBLE : 
+								(image->from_seq->bitpix == USHORT_IMG) ? USHRT_MAX_DOUBLE : 1.;
+		double normavaluefit =  (image->fit->bitpix == BYTE_IMG) ? UCHAR_MAX_DOUBLE : 
+								(image->fit->bitpix == USHORT_IMG) ? USHRT_MAX_DOUBLE : 1.;
+		normvalue	= (normavaluefit / normavalueseq);
+	}
+	threshold = (float)(stat->median * normvalue + ksigma * stat->bgnoise * normvalue);
+	*norm = stat->normValue * normvalue;
+	*bg = stat->median * normvalue;;
+	*bgnoise = stat->bgnoise * normvalue;;
+	*max = stat->max * normvalue;
 	free_stats(stat);
 
 	return threshold;
 }
 
-static sf_errors reject_star(psf_star *result, star_finder_params *sf, starc *se, double dynrange, double minA, double maxA, gchar *errmsg) {
+static sf_errors reject_star(psf_star *result, const star_finder_params *sf, starc *se, double dynrange, double minA, double maxA, gchar *errmsg) {
 	if (isnan(result->fwhmx) || isnan(result->fwhmy))
 		return SF_NO_FWHM; //crit 11
 	if (isnan(result->x0) || isnan(result->y0))
@@ -102,18 +107,18 @@ static sf_errors reject_star(psf_star *result, star_finder_params *sf, starc *se
 		if (errmsg) g_snprintf(errmsg, SF_ERRMSG_LEN, "fwhmx: %3.1f, fwhmy: %3.1f\n", result->fwhmx, result->fwhmy);
 		return SF_ROUNDNESS_BELOW_CRIT; //crit 16
 	}
-	if ((fabs(result->x0 - (double)se->R) >= se->sx) || (fabs(result->y0 - (double)se->R) >= se->sy)) {
-		// if star center off from original candidate detection by more than sigma radius
-		if (errmsg) g_snprintf(errmsg, SF_ERRMSG_LEN, "x0: %3.1f, y0: %3.1f, R:%d\n", result->x0, result->y0, se->R);
-		return SF_CENTER_OFF; //crit 10
-	}
+	// if ((fabs(result->x0 - (double)se->R) >= se->sx) || (fabs(result->y0 - (double)se->R) >= se->sy)) {
+	// 	// if star center off from original candidate detection by more than sigma radius
+	// 	if (errmsg) g_snprintf(errmsg, SF_ERRMSG_LEN, "x0: %3.1f, y0: %3.1f, R:%d\n", result->x0, result->y0, se->R);
+	// 	return SF_CENTER_OFF; //crit 10
+	// }
 	if (result->fwhmx > max(se->sx, se->sy) * _2_SQRT_2_LOG2 * (1 + 0.5 * log(max(se->sx, se->sy) / KERNEL_SIZE))) {
 		// criteria gets looser as guessed fwhm gets larger than kernel
 		if (errmsg) g_snprintf(errmsg, SF_ERRMSG_LEN, "fwhm: %3.1f, s: %3.1f, m: %3.1f, R: %3d\n", result->fwhmx, max(se->sx, se->sy), _2_SQRT_2_LOG2 * (1 + 0.5 * log(max(se->sx, se->sy) / KERNEL_SIZE)), se->R);
 		return SF_FWHM_TOO_LARGE; //crit 2
 	}
 	if (((result->rmse * sf->sigma / result->A) > 0.2) && (!(result->A > dynrange))) {
-		//  do not apply for saturated stars to keep them for alignement purposes
+		//  do not apply for saturated stars to keep them for alignment purposes
 		if (errmsg) g_snprintf(errmsg, SF_ERRMSG_LEN, "RMSE: %4.3e, A: %4.3e, B: %4.3e\n", result->rmse, result->A, result->B);
 		return SF_RMSE_TOO_LARGE; //crit 3
 	}
@@ -164,7 +169,7 @@ static gboolean candidate_is_duplicate(int xx, int yy, int boxradius, starc *can
  * profiles (PSF).
  */
 
-static int minimize_candidates(fits *image, star_finder_params *sf, starc *candidates, int nb_candidates, int layer, double dynrange, psf_star ***retval, gboolean limit_nbstars, int maxstars, starprofile profile, int threads);
+static int minimize_candidates(fits *image, star_finder_params *sf, starc *candidates, int nb_candidates, int layer, double bg, double dynrange, psf_star ***retval, gboolean limit_nbstars, int maxstars, starprofile profile, int threads);
 
 psf_star **peaker(image *image, int layer, star_finder_params *sf, int *nb_stars, rectangle *area, gboolean showtime, gboolean limit_nbstars, int maxstars, starprofile profile, int threads) {
 	int nx = image->fit->rx;
@@ -542,7 +547,7 @@ psf_star **peaker(image *image, int layer, star_finder_params *sf, int *nb_stars
 	siril_debug_print("Candidates for stars: %d\n", nbstars);
 	/* Check if candidates are stars by minimizing a PSF on each */
 	psf_star **results;
-	nbstars = minimize_candidates(image->fit, sf, candidates, nbstars, layer, dynrange, &results, limit_nbstars, maxstars, profile, threads);
+	nbstars = minimize_candidates(image->fit, sf, candidates, nbstars, layer, bg, dynrange, &results, limit_nbstars, maxstars, profile, threads);
 	if (nbstars == 0)
 		results = NULL;
 	sort_stars_by_mag(results, nbstars);
@@ -562,14 +567,13 @@ psf_star **peaker(image *image, int layer, star_finder_params *sf, int *nb_stars
 }
 
 /* returns number of stars found, result is in parameters */
-static int minimize_candidates(fits *image, star_finder_params *sf, starc *candidates, int nb_candidates, int layer, double dynrange, psf_star ***retval, gboolean limit_nbstars, int maxstars, starprofile profile, int threads) {
+static int minimize_candidates(fits *image, star_finder_params *sf, starc *candidates, int nb_candidates, int layer, double bg, double dynrange, psf_star ***retval, gboolean limit_nbstars, int maxstars, starprofile profile, int threads) {
 	int nx = image->rx;
 	int ny = image->ry;
 	WORD **image_ushort = NULL;
 	float **image_float = NULL;
 	gint nbstars = 0;
 	sf_errors accepted_level = (com.pref.starfinder_conf.relax_checks) ? SF_RMSE_TOO_LARGE : SF_OK;
-	double bg = background(image, layer, NULL, SINGLE_THREADED);
 	int psf_failure = 0;
 	double minA = 0.0, maxA = 0.0;
 	if (sf->min_A > 0.0 || sf->max_A > 0.0) {
@@ -679,10 +683,7 @@ static int minimize_candidates(fits *image, star_finder_params *sf, starc *candi
 #if DEBUG_STAR_DETECTION
 				siril_debug_print("Candidate #%5d: X: %5d, Y: %5d - PSF fit failed with error %d\n", candidate, x, y, error);
 #endif
-#ifdef _OPENMP
-#pragma omp atomic
-#endif
-				psf_failure++;
+				g_atomic_int_inc(&psf_failure);
 			}
 		}
 		if (threads > 1) {
@@ -711,9 +712,9 @@ static int minimize_candidates(fits *image, star_finder_params *sf, starc *candi
 	return nbstars;
 }
 
-int compare_stars_by_mag(const void* star1, const void* star2) {
-	psf_star *s1 = *(psf_star**) star1;
-	psf_star *s2 = *(psf_star**) star2;
+static int compare_stars_by_mag(const void* star1, const void* star2) {
+	const psf_star *s1 = *(psf_star**) star1;
+	const psf_star *s2 = *(psf_star**) star2;
 	if (s1->mag < s2->mag)
 		return -1;
 	if (s1->mag > s2->mag)
@@ -819,20 +820,20 @@ float filtered_FWHM_average(psf_star **stars, int nb) {
 	return retval;
 }
 
-/* looks for the list file, tries to load it, returns 1 on success */
-static int check_star_list(gchar *filename, struct starfinder_data *sfargs) {
+/* looks for the list file, tries to load it, returns TRUE on success */
+static gboolean check_star_list(gchar *filename, struct starfinder_data *sfargs) {
 	FILE *fd = g_fopen(filename, "r");
 	if (!fd)
-		return 0;
+		return FALSE;
 	siril_debug_print("star list file %s found, checking...\n", filename);
 	char buffer[300];
-	gboolean params_ok = FALSE, read_failure = FALSE;
-	star_finder_params *sf = &com.pref.starfinder_conf;
+	gboolean params_ok = FALSE, read_failure = FALSE, discard_file = FALSE;;
+	const star_finder_params *sf = &com.pref.starfinder_conf;
 	int star = 0, nb_stars = -1;
 	while (fgets(buffer, 300, fd)) {
 		if (buffer[0] != '#' && !params_ok) {
-			fclose(fd);
-			return 0;
+			read_failure = TRUE;
+			break;
 		}
 		if (!strncmp(buffer, "# ", 2)) {
 			if (sscanf(buffer, "# %d stars found ", &nb_stars) == 1) { // nbstars tainted as based on data from file, needs extra checking
@@ -888,14 +889,15 @@ static int check_star_list(gchar *filename, struct starfinder_data *sfargs) {
 		psf_star *s = new_psf_star();
 		memset(s, 0, sizeof(psf_star));
 		int fi, tokens;
-		char dump[9];
+		char dump[256];
 		tokens = sscanf(buffer,
-				"%d\t%d\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%le\t%lf\t%s",
+				"%d\t%d\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%lf\t%le\t%lf\t%d\t%s",
 				&fi, &s->layer, &s->B, &s->A, &s->beta, &s->xpos, &s->ypos,
-				&s->fwhmx, &s->fwhmy, &s->fwhmx_arcsec, &s->fwhmy_arcsec, &s->angle, &s->rmse, &s->mag, dump);
-		if (tokens != 15) {
+				&s->fwhmx, &s->fwhmy, &s->fwhmx_arcsec, &s->fwhmy_arcsec, &s->angle, &s->rmse, &s->mag, &s->has_saturated, dump);
+		if (tokens != 16) {
 			siril_debug_print("malformed line: %s", buffer);
 			read_failure = TRUE;
+			discard_file = TRUE;
 			free(s);
 			s = NULL;
 			break;
@@ -916,6 +918,9 @@ static int check_star_list(gchar *filename, struct starfinder_data *sfargs) {
 		}
 	}
 	fclose(fd);
+	if (discard_file && g_unlink(filename)) {
+		siril_debug_print("g_unlink failed\n");
+	}
 	return !read_failure;
 }
 
@@ -926,9 +931,18 @@ static int check_star_list(gchar *filename, struct starfinder_data *sfargs) {
 	g_object_unref(file); \
 	return 1
 
-int save_list(gchar *filename, int max_stars_fitted, psf_star **stars, int nbstars, star_finder_params *sf, int layer, gboolean verbose) {
+int save_list(gchar *filename, int max_stars_fitted, psf_star **stars, int nbstars, const star_finder_params *sf, int layer, gboolean verbose) {
 	int i = 0;
 	GError *error = NULL;
+	gchar *dirname = g_path_get_dirname(filename);
+	GDir *dir = g_dir_open(dirname, 0, NULL);
+	if (!dir && siril_mkdir_with_parents(dirname, 0755) < 0) {
+		g_free(dirname);
+		return 1;
+	}
+	if (dir)
+		g_dir_close(dir);
+	g_free(dirname);
 
 	GFile *file = g_file_new_for_path(filename);
 	GOutputStream *output_stream = (GOutputStream*) g_file_replace(file, NULL, FALSE,
@@ -964,7 +978,7 @@ int save_list(gchar *filename, int max_stars_fitted, psf_star **stars, int nbsta
 		HANDLE_WRITE_ERR;
 	}
 	len = snprintf(buffer, 320,
-			"# star#\tlayer\tB\tA\tbeta\tX\tY\tFWHMx [px]\tFWHMy [px]\tFWHMx [\"]\tFWHMy [\"]\tangle\tRMSE\tmag\tProfile\tRA\tDec%s",
+			"# star#\tlayer\tB\tA\tbeta\tX\tY\tFWHMx [px]\tFWHMy [px]\tFWHMx [\"]\tFWHMy [\"]\tangle\tRMSE\tmag\tSat\tProfile\tRA\tDec%s",
 			SIRIL_EOL);
 	if (!g_output_stream_write_all(output_stream, buffer, len, NULL, NULL, &error)) {
 		HANDLE_WRITE_ERR;
@@ -973,18 +987,18 @@ int save_list(gchar *filename, int max_stars_fitted, psf_star **stars, int nbsta
 		while (stars[i]) {
 			if (stars[i]->profile == PSF_GAUSSIAN) {
 				beta = -1;
-				starprof = gausstr;
+				starprof = N_(gausstr);
 			} else {
 				beta = stars[i]->beta;
-				starprof = moffstr;
+				starprof = N_(moffstr);
 			}
 			len = snprintf(buffer, 320,
-					"%d\t%d\t%10.6f\t%10.6f\t%10.2f\t%10.2f\t%10.2f\t%10.2f\t%10.2f\t%10.2f\t%10.2f\t%3.2f\t%10.3e\t%10.2f\t%s\t%f\t%f%s",
+					"%d\t%d\t%10.6f\t%10.6f\t%10.2f\t%10.2f\t%10.2f\t%10.2f\t%10.2f\t%10.2f\t%10.2f\t%3.2f\t%10.3e\t%10.2f\t%d\t%s\t%f\t%f%s",
 					i + 1, stars[i]->layer, stars[i]->B, stars[i]->A, beta,
 					stars[i]->xpos, stars[i]->ypos, stars[i]->fwhmx,
 					stars[i]->fwhmy, stars[i]->fwhmx_arcsec ,stars[i]->fwhmy_arcsec,
 					stars[i]->angle, stars[i]->rmse, stars[i]->mag + com.magOffset,
-					starprof,stars[i]->ra, stars[i]->dec, SIRIL_EOL);
+					stars[i]->has_saturated, starprof, stars[i]->ra, stars[i]->dec, SIRIL_EOL);
 			if (!g_output_stream_write_all(output_stream, buffer, len, NULL, NULL, &error)) {
 				HANDLE_WRITE_ERR;
 			}
@@ -1017,9 +1031,9 @@ int save_list_as_FITS_table(const char *filename, psf_star **stars, int nbstars,
 		return 1;
 	}
 
-	char* rownames[] = { "X", "Y" };
-	char* rowtypes[] = { "1E", "1E" };	// rE is float, rD is double
-	if (fits_create_tbl(fptr, BINARY_TBL, nbstars, 2, rownames, rowtypes, NULL, NULL, &status)) {
+	char* rownames[] = { "X", "Y", "FLUX", "BACKGROUND"};
+	char* rowtypes[] = { "1E", "1E", "1E", "1E"};	// rE is float, rD is double
+	if (fits_create_tbl(fptr, BINARY_TBL, nbstars, 4, rownames, rowtypes, NULL, NULL, &status)) {
 		report_fits_error(status);
 		status = 0;
 		fits_close_file(fptr, &status);
@@ -1052,8 +1066,33 @@ int save_list_as_FITS_table(const char *filename, psf_star **stars, int nbstars,
 
 	for (int i = 0; i < nbstars; i++)
 		data[i] = ry - stars[i]->ypos + 0.5; // asnet convention
-	if (fits_write_col(fptr, TFLOAT, 2, 1, 1, nbstars, data, &status))
+	if (fits_write_col(fptr, TFLOAT, 2, 1, 1, nbstars, data, &status)) {
 		report_fits_error(status);
+		status = 0;
+		fits_close_file(fptr, &status);
+		free(data);
+		return 1;
+	}
+
+	for (int i = 0; i < nbstars; i++)
+		data[i] = stars[i]->A;
+	if (fits_write_col(fptr, TFLOAT, 3, 1, 1, nbstars, data, &status)) {
+		report_fits_error(status);
+		status = 0;
+		fits_close_file(fptr, &status);
+		free(data);
+		return 1;
+	}
+
+	for (int i = 0; i < nbstars; i++)
+		data[i] = stars[i]->B;
+	if (fits_write_col(fptr, TFLOAT, 4, 1, 1, nbstars, data, &status)) {
+		report_fits_error(status);
+		status = 0;
+		fits_close_file(fptr, &status);
+		free(data);
+		return 1;
+	}
 
 	int retval = status;
 	status = 0;
@@ -1064,7 +1103,8 @@ int save_list_as_FITS_table(const char *filename, psf_star **stars, int nbstars,
 
 static int findstar_compute_mem_limits(struct generic_seq_args *args, gboolean for_writer) {
 	unsigned int MB_per_image, MB_avail, required = 0;
-	int limit = compute_nb_images_fit_memory(args->seq, 1.0, FALSE, &MB_per_image, NULL, &MB_avail);
+	// using output scale of 0. as this does not produce output image
+	int limit = compute_nb_images_fit_memory(args->seq, 0.0, FALSE, &MB_per_image, NULL, &MB_avail);
 
 	if (limit > 0) {
 		int is_float = get_data_type(args->seq->bitpix) == DATA_FLOAT;
@@ -1079,10 +1119,22 @@ static int findstar_compute_mem_limits(struct generic_seq_args *args, gboolean f
 		// * As the 1-channel fits populated by extract_fits() is DATA_FLOAT no
 		//   memory is required for the Gaussian blur as the in-place RT algorithm is
 		//   used;
+		// * If the sequence is CFA, we also need to make a copy of the input image
+		// to interpolate non green pixels
 		// * Also allow MAX_STARS * sizeof(psf_star) + 1MB margin for gslsolver data;
 		//   and the indexing arrays e.g. smooth_array (ry * sizeof(float*)).
 		int stars_and_overhead = 1 + (int) ceilf(((float) MAX_STARS * (float) sizeof(psf_star)) / (float) BYTES_IN_A_MB);
 		required = MB_per_image + MB_per_float_chan + stars_and_overhead;
+
+		// we need to read the bayer pattern of ref image...
+		fits fit = { 0 };
+		// load ref metadata in fit
+		if (seq_read_frame_metadata(args->seq, sequence_find_refimage(args->seq), &fit))
+			return 0;
+		if (fit.keywords.bayer_pattern[0] != '\0')
+			required += MB_per_image;
+		clearfits(&fit);
+
 		int thread_limit = MB_avail / required;
 		if (thread_limit > com.max_thread)
 				thread_limit = com.max_thread;
@@ -1108,41 +1160,125 @@ static int findstar_compute_mem_limits(struct generic_seq_args *args, gboolean f
 	return limit;
 }
 
-int findstar_image_hook(struct generic_seq_args *args, int o, int i, fits *fit, rectangle *_, int threads) {
+/* return FALSE to avoid reading image */
+static gboolean findstar_image_read_hook(struct generic_seq_args *args, int index) {
 	struct starfinder_data *findstar_args = (struct starfinder_data *)args->user;
 
-	struct starfinder_data *curr_findstar_args = malloc(sizeof(struct starfinder_data));
+	struct starfinder_data *curr_findstar_args = calloc(1, sizeof(struct starfinder_data));
 	memcpy(curr_findstar_args, findstar_args, sizeof(struct starfinder_data));
-	curr_findstar_args->im.index_in_seq = i;
-	curr_findstar_args->im.fit = fit;
+	curr_findstar_args->im.index_in_seq = index;
+	curr_findstar_args->im.fit = NULL;
 	if (findstar_args->stars && findstar_args->nb_stars) {
-		curr_findstar_args->stars = findstar_args->stars + i;
-		curr_findstar_args->nb_stars = findstar_args->nb_stars + i;
+		curr_findstar_args->stars = findstar_args->stars + index;
+		curr_findstar_args->nb_stars = findstar_args->nb_stars + index;
 	}
-	curr_findstar_args->threading = threads;
+	curr_findstar_args->threading = SINGLE_THREADED;
 
-	// build the star list file name in all cases to try reading it
-	char root[256];
-	if (!fit_sequence_get_image_filename(args->seq, i, root, FALSE)) {
+	gchar *star_filename = get_sequence_cache_filename(args->seq, index, "cache", "lst", NULL);
+	if (!star_filename) {
 		free(curr_findstar_args);
-		return 1;
+		return TRUE;
 	}
-	gchar *star_filename = g_strdup_printf("%s.lst", root);
 
 	if (findstar_args->save_to_file)
 		curr_findstar_args->starfile = star_filename;
 
-	int retval = 0;
-	if (!check_starfile_date(args->seq, i, star_filename) ||
-			!check_star_list(star_filename, curr_findstar_args))
-		retval = GPOINTER_TO_INT(findstar_worker(curr_findstar_args));
-
+	cache_status status = check_cachefile_date(args->seq, index, star_filename);
+	if (status <= CACHE_NOT_FOUND) { // cached file is older and was deleted or does not exist, we need to read the image
+		free(curr_findstar_args);
+		g_free(star_filename);
+		return TRUE; 
+	}
+	status = (int)check_star_list(star_filename, curr_findstar_args);
+	free(curr_findstar_args);
 	g_free(star_filename);
+	return !status; // check_star_list returns TRUE on success
+}
+
+// contrarily to findstar_worker, this function first checks if a lst file exists:
+// - if yes and star finder params are consistent, the list is used to populate stars
+// - else, it calls peaker
+// It is standalone to be:
+// - wrapped by findstar_image_hook
+// - called by registration
+struct starfinder_data *findstar_image_worker(const struct starfinder_data *findstar_args, int o, int i, fits *fit, rectangle *_, int threads) {
+	struct starfinder_data *curr_findstar_args = calloc(1, sizeof(struct starfinder_data));
+	memcpy(curr_findstar_args, findstar_args, sizeof(struct starfinder_data));
+	curr_findstar_args->im.index_in_seq = i;
+	curr_findstar_args->im.fit = fit;
+	sequence *seq = findstar_args->im.from_seq;
+	if (findstar_args->stars && findstar_args->nb_stars) { // used by 2pass reg which needs to store all star lists
+		curr_findstar_args->stars = findstar_args->stars + i;
+		curr_findstar_args->nb_stars = findstar_args->nb_stars + i;
+		curr_findstar_args->onepass = FALSE;
+	} else if (findstar_args->keep_stars) { // used by global reg which needs to store the current star list
+		curr_findstar_args->stars = calloc(1, sizeof(psf_star **));
+		curr_findstar_args->nb_stars = calloc(1, sizeof(int));
+		curr_findstar_args->onepass = TRUE;
+	}
+	curr_findstar_args->threading = threads;
+	rectangle selection = findstar_args->selection;
+	gboolean can_use_cache = !(selection.w != 0 && selection.h != 0);
+
+	int retval = 0;
+	gchar *star_filename = NULL;
+
+	if (can_use_cache) {// otherwise, we don't try to read the lst nor save it
+		// build the star list file name in all cases to try reading it
+		star_filename = get_sequence_cache_filename(seq, i, "cache", "lst", NULL);
+		if (!star_filename) {
+			if (curr_findstar_args->onepass == TRUE) {
+				free(curr_findstar_args->nb_stars);
+				free(curr_findstar_args->stars);
+			}
+			free(curr_findstar_args);
+			curr_findstar_args = NULL;
+			return curr_findstar_args;
+		}
+
+		if (seq->type == SEQ_INTERNAL || !(check_cachefile_date(seq, i, star_filename) == CACHE_NEWER) ||
+				!check_star_list(star_filename, curr_findstar_args))
+				can_use_cache = FALSE;
+		if (findstar_args->save_to_file)
+			curr_findstar_args->starfile = star_filename;
+		else
+			g_free(star_filename);
+	}
+
+	if (!can_use_cache) {
+		fits *green_fit = NULL;
+		if (fit->keywords.bayer_pattern[0] != '\0') {
+			green_fit = calloc(1, sizeof(fits));
+			copyfits(fit, green_fit, CP_ALLOC | CP_COPYA | CP_FORMAT, -1);
+			interpolate_nongreen(green_fit);
+			curr_findstar_args->im.fit = green_fit;
+			// uncomment these lines to save the green fit for each image
+			// const gchar *green_filename = get_sequence_cache_filename(seq, i, "cache", "fit", "green_");
+			// savefits(green_filename, green_fit);
+		}
+		retval = GPOINTER_TO_INT(findstar_worker(curr_findstar_args));
+		clearfits(green_fit);
+		free(green_fit);
+		if (retval) {
+			if (curr_findstar_args->onepass == TRUE) {
+				free(curr_findstar_args->nb_stars);
+				free(curr_findstar_args->stars);
+			}
+			free(curr_findstar_args);
+			curr_findstar_args = NULL;
+		}
+	}
+	return curr_findstar_args;
+}
+
+static int findstar_image_hook(struct generic_seq_args *args, int o, int i, fits *fit, rectangle *_, int threads) {
+	const struct starfinder_data *findstar_args = (struct starfinder_data *)args->user;
+	struct starfinder_data *curr_findstar_args = findstar_image_worker(findstar_args, o, i, fit, _, threads);
+	gboolean retval = !curr_findstar_args;
 	free(curr_findstar_args);
 	return retval;
 }
 
-
 gboolean end_findstar_sequence(gpointer p) {
 	struct generic_seq_args *args = (struct generic_seq_args *) p;
 	struct starfinder_data *findstar_args = (struct starfinder_data*) args->user;
@@ -1157,19 +1293,18 @@ gboolean end_findstar_sequence(gpointer p) {
 	}
 	if (!check_seq_is_comseq(args->seq))
 		free_sequence(args->seq, TRUE);
+
 	free(findstar_args);
 	free(p);
 	return end_generic(NULL);
 }
 
-int findstar_finalize_hook (struct generic_seq_args *args) {
+int findstar_finalize_hook(struct generic_seq_args *args) {
 	struct starfinder_data *data = (struct starfinder_data *) args->user;
-#ifdef HAVE_WCSLIB
 	if (data->ref_wcs) {
 		if (!wcsfree(data->ref_wcs))
 			free(data->ref_wcs);
 	}
-#endif
 	if (data->startable)
 		g_free(data->startable);
 	if (data->starfile)
@@ -1184,6 +1319,7 @@ int apply_findstar_to_sequence(struct starfinder_data *findstar_args) {
 		args->filtering_criterion = seq_filter_included;
 		args->nb_filtered_images = args->seq->selnum;
 	}
+	args->image_read_hook = findstar_image_read_hook;
 	args->image_hook = findstar_image_hook;
 	args->finalize_hook = findstar_finalize_hook;
 	args->compute_mem_limits_hook = findstar_compute_mem_limits;
@@ -1210,17 +1346,13 @@ int apply_findstar_to_sequence(struct starfinder_data *findstar_args) {
 			findstar_args->save_eqcoords = FALSE;
 		else {
 			findstar_args->reference_image = refidx;
-#ifdef HAVE_WCSLIB
-			findstar_args->ref_wcs = ref.wcslib;
-#endif
+			findstar_args->ref_wcs = ref.keywords.wcslib;
 			if (args->seq->regparam[findstar_args->layer] &&
 					guess_transform_from_H(args->seq->regparam[findstar_args->layer][refidx].H) != NULL_TRANSFORMATION) {
 				findstar_args->reference_H = args->seq->regparam[findstar_args->layer][refidx].H;
 			}
 		}
-#ifdef HAVE_WCSLIB
-		ref.wcslib = NULL;	// don't free it
-#endif
+		ref.keywords.wcslib = NULL;	// don't free it
 		clearfits(&ref);
 	}
 	if (findstar_args->already_in_thread) {
@@ -1228,7 +1360,10 @@ int apply_findstar_to_sequence(struct starfinder_data *findstar_args) {
 		free(args);
 		return retval;
 	}
-	start_in_new_thread(generic_sequence_worker, args);
+	if (!start_in_new_thread(generic_sequence_worker, args)) {
+		free(args->user);
+		free_generic_seq_args(args, TRUE);
+	}
 	return 0;
 }
 
@@ -1239,13 +1374,26 @@ gpointer findstar_worker(gpointer p) {
 	struct starfinder_data *args = (struct starfinder_data *)p;
 	int retval = 0;
 	int nbstars = 0;
-	rectangle *selection = NULL;
-	if (com.selection.w != 0 && com.selection.h != 0)
-		selection = &com.selection;
 	gboolean limit_stars = (args->max_stars_fitted > 0);
 	int threads = check_threading(&args->threading);
+	fits *green_fit = NULL;
+	fits *orig = NULL;
+	gboolean has_selection = args->selection.w != 0 && args->selection.h != 0;
+	if (fit_is_cfa(args->im.fit)) {
+		green_fit = calloc(1, sizeof(fits));
+		orig = args->im.fit;
+		copyfits(args->im.fit, green_fit, CP_ALLOC | CP_COPYA | CP_FORMAT, -1);
+		interpolate_nongreen(green_fit);
+		args->im.fit = green_fit;
+		siril_log_color_message(_("Undebayered CFA image. Detection is done on green pixels only, using interpolation\n"), "salmon");
+	}
 	psf_star **stars = peaker(&args->im, args->layer, &com.pref.starfinder_conf, &nbstars,
-			selection, args->update_GUI, limit_stars, args->max_stars_fitted, com.pref.starfinder_conf.profile, threads);
+			&args->selection, args->update_GUI, limit_stars, args->max_stars_fitted, com.pref.starfinder_conf.profile, threads);
+	if (green_fit) {
+		args->im.fit = orig;
+		clearfits(green_fit);
+		free(green_fit);
+	}
 
 	double fwhm = 0.0;
 	if (stars) {
@@ -1255,37 +1403,32 @@ gpointer findstar_worker(gpointer p) {
 			sum += stars[i]->fwhmx;
 			fwhm_to_arcsec_if_needed(args->im.fit, stars[i]);
 
-			if (args->starfile && args->save_eqcoords) {
+			if (args->starfile || args->save_eqcoords) {
 				sequence *seq = args->im.from_seq;
 				double dx = stars[i]->xpos, dy = stars[i]->ypos;
 				if (has_wcs(args->im.fit)) {
 					double ra = 0.0, dec = 0.0;
-#ifdef HAVE_WCSLIB
 					// coordinates of the star in FITS/WCS coordinates
 					double fx, fy;
-					display_to_fits(dx, dy, &fx, &fy, args->im.fit->ry);
-					pix2wcs2(args->ref_wcs, fx, fy, &ra, &dec);
-#endif
+					display_to_siril(dx, dy, &fx, &fy, args->im.fit->ry);
+					pix2wcs2(args->im.fit->keywords.wcslib, fx, fy, &ra, &dec);
 					// ra and dec = -1 is the error code
 					stars[i]->ra = ra;
 					stars[i]->dec = dec;
-				}
-				else if (!seq->regparam[args->layer] ||
-						guess_transform_from_H(seq->regparam[args->layer][args->im.index_in_seq].H) == NULL_TRANSFORMATION) {
-					// image was not registered, ignore
-				}
-				else {
-					cvTransfPoint(&dx, &dy, seq->regparam[args->layer][args->im.index_in_seq].H, args->reference_H);
+				} else if (seq && seq->regparam[args->layer] &&
+					guess_transform_from_H(seq->regparam[args->layer][args->im.index_in_seq].H) > NULL_TRANSFORMATION) {
+					cvTransfPoint(&dx, &dy, seq->regparam[args->layer][args->im.index_in_seq].H, args->reference_H, 1.);
 					double ra = 0.0, dec = 0.0;
-#ifdef HAVE_WCSLIB
 					// coordinates of the star in FITS/WCS coordinates
 					double fx, fy;
-					display_to_fits(dx, dy, &fx, &fy, args->im.fit->ry);
+					display_to_siril(dx, dy, &fx, &fy, args->im.fit->ry);
 					pix2wcs2(args->ref_wcs, fx, fy, &ra, &dec);
-#endif
 					// ra and dec = -1 is the error code
 					stars[i]->ra = ra;
 					stars[i]->dec = dec;
+				} else { // fallback to a default invalid value
+					stars[i]->ra = 9.99E9;
+					stars[i]->dec = 9.99E9;
 				}
 			}
 			i++;
@@ -1298,11 +1441,11 @@ gpointer findstar_worker(gpointer p) {
 	}
 
 	if (args->update_GUI)
-		update_star_list(stars, TRUE);
+		update_star_list(stars, TRUE, com.python_script);
 
 	siril_log_message(_("Found %d %s profile stars in %s, channel #%d (FWHM %f)\n"), nbstars,
 			com.pref.starfinder_conf.profile == PSF_GAUSSIAN ? _("Gaussian") : _("Moffat"),
-			selection ? _("selection") : _("image"), args->layer, fwhm);
+			has_selection ? _("selection") : _("image"), args->layer, fwhm);
 	if (args->starfile &&
 			save_list(args->starfile, args->max_stars_fitted, stars, nbstars,
 				&com.pref.starfinder_conf, args->layer, args->update_GUI)) {
@@ -1324,7 +1467,7 @@ gpointer findstar_worker(gpointer p) {
 		free_fitted_stars(stars);
 END:
 	if (args->update_GUI)
-		siril_add_idle(end_findstar, args);
+		execute_idle_and_wait_for_it(end_findstar, args);
 	/*gettimeofday(&t_end, NULL);
 	gchar *msg = g_strdup_printf("findstar for image %d", args->im.index_in_seq);
 	show_time_msg(t_start, t_end, msg);
