@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 #include <float.h>
 #include <assert.h>
 #include <gsl/gsl_statistics.h>
+#include "algos/sorting.h"
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/processing.h"
@@ -46,7 +47,6 @@
 #include "gui/progress_and_log.h"
 #include "io/image_format_fits.h"
 #include "io/sequence.h"
-#include "sorting.h"
 #include "statistics.h"
 #include "statistics_float.h"
 #include "demosaicing.h"
@@ -56,15 +56,6 @@
 #undef siril_debug_print
 #define siril_debug_print(fmt, ...) { }
 
-/* Activating nullcheck will treat pixels with 0 value as null and remove them
- * from stats computation. This can be useful when a large area is black, but
- * this shouldn't happen often. Maybe we could detect it instead of hardcoding
- * it...
- * Deactivating this will take less memory and make a faster statistics
- * computation. ngoodpix will be equal to total if deactivated.
- * Set to 0 to deactivate or 1 to activate. */
-#define ACTIVATE_NULLCHECK 1
-
 static void stats_set_default_values(imstats *stat);
 
 // copies the area of an image into the memory buffer data
@@ -162,8 +153,7 @@ static double siril_stats_ushort_bwmv(const WORD* data, const size_t n,
 			up += ai * SQR((double ) data[i] - median) * SQR(SQR (1 - yi2));
 			down += (ai * (1 - yi2) * (1 - 5 * yi2));
 		}
-
-		bwmv = n * (up / (down * down));
+		bwmv = down ? n * (up / (down * down)) : 0.0;
 	}
 
 	return bwmv;
@@ -221,7 +211,7 @@ static void siril_stats_ushort_minmax(WORD *min_out, WORD *max_out,
  * computes them and stores them in it if they have not already been */
 static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *selection,
 		int option, imstats *stats, int bitpix, threading_type threads) {
-	int nx, ny;
+	int nx = 0, ny = 0;
 	WORD *data = NULL;
 	int stat_is_local = 0, free_data = 0;
 	imstats* stat = stats;
@@ -246,8 +236,9 @@ static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *sele
 			if (layer < 0) {
 				size_t newsz;
 				data = extract_CFA_buffer_area_ushort(fit, -layer - 1, selection, &newsz);
-				if (!data) {
+				if (!data || newsz == 0) {
 					siril_log_message(_("Failed to compute CFA statistics for channel %d\n"), -layer-1);
+					free(data);
 					return NULL;
 				}
 				nx = newsz;
@@ -318,7 +309,7 @@ static imstats* statistics_internal_ushort(fits *fit, int layer, rectangle *sele
 			return NULL;	// not in cache, don't compute
 		}
 		siril_debug_print("- stats %p fit %p (%d): computing basic\n", stat, fit, layer);
-		siril_fits_img_stats_ushort(data, nx, ny, ACTIVATE_NULLCHECK, 0, &stat->ngoodpix,
+		siril_fits_img_stats_ushort(data, nx, ny, &stat->ngoodpix,
 				NULL, NULL, &stat->mean, &stat->sigma, &stat->bgnoise,
 				NULL, NULL, NULL, threads, &status);
 		if (status) {
@@ -477,14 +468,14 @@ imstats* statistics(sequence *seq, int image_index, fits *fit, int super_layer,
 		g_assert(layer < fit->naxes[2]);
 		if (fit->stats && fit->stats[layer]) {
 			oldstat = fit->stats[layer];
-			atomic_int_incref(oldstat->_nb_refs);
+			g_atomic_int_inc(&oldstat->_nb_refs);
 		}
 		stat = statistics_internal(fit, layer, NULL, option, oldstat, fit->bitpix, threads);
 		if (!stat) {
 			fprintf(stderr, "- stats failed for fit %p (%d)\n", fit, layer);
 			if (oldstat) {
 				stats_set_default_values(oldstat);
-				atomic_int_decref(oldstat->_nb_refs);
+				g_atomic_int_dec_and_test(&oldstat->_nb_refs);
 			}
 			return NULL;
 		}
@@ -496,7 +487,7 @@ imstats* statistics(sequence *seq, int image_index, fits *fit, int super_layer,
 		if (seq->stats && seq->stats[layer]) {
 			oldstat = seq->stats[layer][image_index];
 			if (oldstat)	// can be NULL here
-				atomic_int_incref(oldstat->_nb_refs);
+				g_atomic_int_inc(&oldstat->_nb_refs);
 		}
 		stat = statistics_internal(fit, layer, NULL, option, oldstat, seq->bitpix, threads);
 		if (!stat) {
@@ -505,7 +496,7 @@ imstats* statistics(sequence *seq, int image_index, fits *fit, int super_layer,
 						image_index, layer);
 			if (oldstat) {
 				stats_set_default_values(oldstat);
-				atomic_int_decref(oldstat->_nb_refs);
+				g_atomic_int_dec_and_test(&oldstat->_nb_refs);
 			}
 			return NULL;
 		}
@@ -579,7 +570,7 @@ void add_stats_to_fit(fits *fit, int layer, imstats *stat) {
 		} else return;
 	}
 	fit->stats[layer] = stat;
-	atomic_int_incref(stat->_nb_refs);
+	g_atomic_int_inc(&stat->_nb_refs);
 	siril_debug_print("- stats %p saved to fit %p (%d)\n", stat, fit, layer);
 }
 
@@ -608,7 +599,7 @@ static void add_stats_to_stats(sequence *seq, int nb_layers, imstats ****stats,
 	siril_debug_print("- stats %p, %d in seq (%d): saving data\n", stat, image_index, layer);
 	(*stats)[layer][image_index] = stat;
 	seq->needs_saving = TRUE;
-	atomic_int_incref(stat->_nb_refs);
+	g_atomic_int_inc(&stat->_nb_refs);
 }
 
 void add_stats_to_seq(sequence *seq, int image_index, int layer, imstats *stat) {
@@ -682,7 +673,7 @@ void allocate_stats(imstats **stat) {
 			*stat = malloc(sizeof(imstats));
 		if (!*stat) { PRINT_ALLOC_ERR; return; } // OOM
 		stats_set_default_values(*stat);
-		(*stat)->_nb_refs = atomic_int_alloc();
+		(*stat)->_nb_refs = 1;
 		siril_debug_print("- stats %p allocated\n", *stat);
 	}
 }
@@ -690,8 +681,7 @@ void allocate_stats(imstats **stat) {
 /* frees an imstats struct if there are no more references to it.
  * returns NULL if it was freed, the argument otherwise. */
 imstats* free_stats(imstats *stat) {
-	int n;
-	if (stat && (n = atomic_int_decref(stat->_nb_refs)) == 0) {
+	if (stat && (g_atomic_int_dec_and_test(&stat->_nb_refs)) == TRUE) {
 		siril_debug_print("- stats %p has no more refs, freed\n", stat);
 		free(stat);
 		return NULL;
@@ -755,7 +745,7 @@ static int stat_image_hook(struct generic_seq_args *args, int o, int i, fits *fi
 		rectangle *_, int threads) {
 	struct stat_data *s_args = (struct stat_data*) args->user;
 
-	gboolean is_cfa = fit->bayer_pattern[0] != '\0' && s_args->cfa;
+	gboolean is_cfa = fit->keywords.bayer_pattern[0] != '\0' && s_args->cfa;
 	int nb_image_layers = (int)fit->naxes[2];
 	if (is_cfa)
 		nb_image_layers = 3;
@@ -826,6 +816,10 @@ static int stat_image_hook(struct generic_seq_args *args, int o, int i, fits *fi
 static int stat_finalize_hook(struct generic_seq_args *args) {
 	GError *error = NULL;
 	struct stat_data *s_args = (struct stat_data*) args->user;
+	GFile *file = NULL;
+	GOutputStream* output_stream = NULL;
+	int result = 1; // Default to error
+
 	if (!s_args->list) {
 		free(s_args);
 		return 1;
@@ -833,20 +827,21 @@ static int stat_finalize_hook(struct generic_seq_args *args) {
 
 	int nb_data_layers = s_args->cfa ? 3 : s_args->seq->nb_layers;
 	int size = nb_data_layers * args->nb_filtered_images;
-	GFile *file = g_file_new_for_path(s_args->csv_name);
-	GOutputStream* output_stream = (GOutputStream*) g_file_replace(file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, &error);
+
+	file = g_file_new_for_path(s_args->csv_name);
+	output_stream = (GOutputStream*) g_file_replace(file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, &error);
 	g_free(s_args->csv_name);
-	if (output_stream == NULL) {
+
+	if (output_stream == NULL || error != NULL) {
 		if (error != NULL) {
-			g_warning("%s\n", error->message);
+			g_warning("Cannot create output file: %s", error->message);
 			g_clear_error(&error);
-			fprintf(stderr, "Cannot save histo\n");
 		}
-		g_object_unref(file);
-		free_stat_list(s_args->list, size);
-		free(s_args);
-		return 1;
+		fprintf(stderr, "Cannot save histo\n");
+		goto cleanup;
 	}
+
+	// Write header
 	const gchar *header;
 	if (s_args->option == STATS_BASIC) {
 		header = "image\tchan\tmean\tmedian\tsigma\tmin\tmax\tnoise\n";
@@ -855,42 +850,53 @@ static int stat_finalize_hook(struct generic_seq_args *args) {
 	} else {
 		header = "image\tchan\tmean\tmedian\tsigma\tmin\tmax\tnoise\tavgDev\tmad\tsqrtbwmv\tlocation\tscale\n";
 	}
+
 	if (!g_output_stream_write_all(output_stream, header, strlen(header), NULL, NULL, &error)) {
-		g_warning("%s\n", error->message);
+		g_warning("Failed to write header: %s", error->message);
 		g_clear_error(&error);
-		g_object_unref(output_stream);
-		g_object_unref(file);
-		free_stat_list(s_args->list, size);
-		free(s_args);
-		return 1;
+		goto cleanup;
 	}
 
+	// Write data
 	for (int i = 0; i < args->nb_filtered_images * nb_data_layers; i++) {
 		if (!s_args->list[i]) continue; //stats can fail
 		if (!g_output_stream_write_all(output_stream, s_args->list[i], strlen(s_args->list[i]), NULL, NULL, &error)) {
-			g_warning("%s\n", error->message);
+			g_warning(_("Failed to write seqstat data: %s"), error->message);
 			g_clear_error(&error);
-			g_object_unref(output_stream);
-			g_object_unref(file);
-			free_stat_list(s_args->list, size);
-			free(s_args);
-			return 1;
+			goto cleanup;
 		}
 	}
 
+	// Success path
 	siril_log_message(_("Statistic file %s was successfully created.\n"), g_file_peek_path(file));
 	writeseqfile(args->seq);
-	g_object_unref(output_stream);
-	g_object_unref(file);
-	free_stat_list(s_args->list, size);
+	result = 0; // Success
+
+cleanup:
+	// Close and cleanup in proper order
+	if (output_stream) {
+		if (!g_output_stream_close(output_stream, NULL, &error)) {
+			g_warning(_("Failed to close seqstat output stream: %s"), error->message);
+			g_clear_error(&error);
+		}
+		g_object_unref(output_stream);
+	}
+
+	if (file) {
+		g_object_unref(file);
+	}
+
+	if (s_args->list) {
+		free_stat_list(s_args->list, size);
+	}
 	free(s_args);
 
-	return 0;
+	return result;
 }
 
 static int stat_compute_mem_limit(struct generic_seq_args *args, gboolean for_writer) {
 	unsigned int MB_per_image, MB_avail, required;
-	int limit = compute_nb_images_fit_memory(args->seq, 1.0, FALSE, &MB_per_image, NULL, &MB_avail);
+	int limit = compute_nb_images_fit_memory(args->seq, 0.0, FALSE, &MB_per_image, NULL, &MB_avail); // no output - output_scale is set to 0.
 
 	int is_color = args->seq->nb_layers == 3;
 	required = is_color ? MB_per_image * 4 / 3 : MB_per_image * 2;
@@ -942,7 +948,14 @@ void apply_stats_to_sequence(struct stat_data *stat_args) {
 
 	stat_args->fit = NULL;	// not used here
 
-	start_in_new_thread(generic_sequence_worker, args);
+	if (!start_in_new_thread(generic_sequence_worker, args)) {
+		int nb_data_layers = stat_args->cfa ? 3 : stat_args->seq->nb_layers;
+		int size = nb_data_layers * args->nb_filtered_images;
+		free_stat_list(stat_args->list, size);
+		g_free(stat_args->csv_name);
+		free (stat_args);
+		free_generic_seq_args(args, TRUE);
+	}
 }
 
 /**** callbacks ****/
@@ -1005,8 +1018,8 @@ int compute_all_channels_statistics_seqimage(sequence *seq, int image_index, fit
 			if (threads >= required_computations) {
 				channels_per_thread = 1;
 				threads_per_thread = compute_thread_distribution(required_computations, threading);
-				omp_set_nested(1);	// to be done at each level
-				if ((!omp_get_nested() || omp_get_max_active_levels() < 2) && threads_per_thread[0] > 1)
+				omp_set_max_active_levels(INT_MAX);	// to be done at each level
+				if ((omp_get_max_active_levels() < 2) && threads_per_thread[0] > 1)
 					siril_log_message(_("Threading statistics computation per channel, but enabling threading in channels too is not supported.\n"));
 				else siril_debug_print("threading statistics computation per channel (at most %d threads each)\n", threads_per_thread[0]);
 			}
@@ -1050,7 +1063,7 @@ int compute_all_channels_statistics_seqimage(sequence *seq, int image_index, fit
 int compute_all_channels_statistics_single_image(fits *fit, int option,
 		threading_type threading, imstats **stats) {
 	g_assert(fit);
-	gboolean cfa = (option & STATS_FOR_CFA) && fit->bayer_pattern[0] != '\0';
+	gboolean cfa = (option & STATS_FOR_CFA) && fit->keywords.bayer_pattern[0] != '\0';
 	int required_computations = cfa ? 3 : (int)fit->naxes[2];
 	if (required_computations == 0) {
 		stats[0] = NULL;
@@ -1075,8 +1088,8 @@ int compute_all_channels_statistics_single_image(fits *fit, int option,
 		if (threads >= required_computations) {
 			channels_per_thread = 1;
 			threads_per_thread = compute_thread_distribution(required_computations, threading);
-			omp_set_nested(1);	// to be done at each level
-			if ((!omp_get_nested() || omp_get_max_active_levels() < 2) && threads_per_thread[0] > 1)
+			omp_set_max_active_levels(INT_MAX);	// to be done at each level
+			if ((omp_get_max_active_levels() < 2) && threads_per_thread[0] > 1)
 				siril_log_message(_("Threading statistics computation per channel, but enabling threading in channels too is not supported.\n"));
 			else siril_debug_print("threading statistics computation per channel (at most %d threads each)\n", threads_per_thread[0]);
 		}
@@ -1122,7 +1135,7 @@ int copy_cached_stats_for_image(sequence *seq, int image, imstats **channels) {
 	for (int i = 0; i < seq->nb_layers; i++) {
 		if (all_copied && seq->stats[i] && seq->stats[i][image]) {
 			imstats *stats = seq->stats[i][image];
-			atomic_int_incref(stats->_nb_refs);
+			g_atomic_int_inc(&stats->_nb_refs);
 			channels[i] = stats;
 		} else {
 			all_copied = 0;
@@ -1143,7 +1156,7 @@ int sos_update_noise_float(float *array, long nx, long ny, long nchans, double *
 	float *colarray[3];
 	double fSigma = 0.0;
 	if (nchans == 1) {
-		retval = siril_fits_img_stats_float(array, nx, ny, ACTIVATE_NULLCHECK, 0.0f, NULL, NULL, NULL,
+		retval = siril_fits_img_stats_float(array, nx, ny, NULL, NULL, NULL,
 		NULL, NULL, noise, NULL, NULL, NULL, MULTI_THREADED, &status);
 		return retval;
 	} else {
@@ -1151,7 +1164,7 @@ int sos_update_noise_float(float *array, long nx, long ny, long nchans, double *
 		colarray[1] = array + (nx * ny);
 		colarray[2] = array + 2 * (nx * ny);
 		for (unsigned i = 0 ; i < nchans ; i++) {
-			retval += siril_fits_img_stats_float(colarray[i], nx, ny, ACTIVATE_NULLCHECK, 0.0f, NULL, NULL, NULL,
+			retval += siril_fits_img_stats_float(colarray[i], nx, ny, NULL, NULL, NULL,
 						NULL, NULL, noise, NULL, NULL, NULL, MULTI_THREADED, &status);
 			fSigma += *noise;
 		}
@@ -1159,3 +1172,68 @@ int sos_update_noise_float(float *array, long nx, long ny, long nchans, double *
 	}
 	return retval;
 }
+
+double robust_median_w(fits *fit, rectangle *area, int chan, float lower, float upper) {
+	uint32_t x0, y0, x1,y1;
+	if (area) {
+		x0 = area->x;
+		y0 = area->y;
+		x1 = area->x + area->w;
+		y1 = area->y + area->h;
+	} else {
+		x0 = y0 = 0;
+		x1 = fit->rx;
+		y1 = fit->ry;
+	}
+	size_t npixels = (x1 - x0) * (y1 - y0);
+	WORD *data = fit->pdata[chan];
+	WORD *filtered_data = malloc(npixels * sizeof(WORD));
+	size_t count = 0;
+	WORD lowerw = roundf_to_WORD(lower);
+	WORD upperw = roundf_to_WORD(upper);
+	for (uint32_t y = y0 ; y < y1 ; y++) {
+		size_t j = y * fit->rx;
+		for (uint32_t x = x0 ; x < x1 ; x++) {
+			size_t i = x + j;
+			if (data[i] >= lowerw && data[i] <= upperw) {
+				filtered_data[count++] = data[i];
+			}
+		}
+	}
+
+	// Check if there are any elements in the specified range
+	if (count == 0) {
+		free(filtered_data);
+		return 0.0; // No elements in the range, return 0 as median
+	}
+
+	// use histogram_median here instead of quickmedian for speed (see #1458)
+	double retval = histogram_median(filtered_data, count, MULTI_THREADED);
+
+	// Free the allocated memory for filtered_data
+	free(filtered_data);
+
+	return retval;
+}
+
+// Function to quickly compute min and max values
+int quick_minmax(fits *fit, double *minval, double *maxval) {
+    imstats *stats[3] = { NULL };
+    int retval = compute_all_channels_statistics_single_image(fit, STATS_MINMAX, MULTI_THREADED, stats);
+
+    if (retval) {
+        siril_log_color_message(_("Error: statistics computation failed. Unable to check for out-of-range values.\n"), "red");
+    } else {
+        if (fit->naxes[2] == 1) {
+            *maxval = stats[0]->max;
+            *minval = stats[0]->min;
+        } else {
+            *maxval = max(max(stats[RLAYER]->max, stats[GLAYER]->max), stats[BLAYER]->max);
+            *minval = min(min(stats[RLAYER]->min, stats[GLAYER]->min), stats[BLAYER]->min);
+        }
+        for (int i = 0; i < fit->naxes[2]; i++) {
+            free_stats(stats[i]);
+        }
+    }
+    return retval;
+}
