@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,28 +37,28 @@
 #include <string.h>
 #include <float.h>
 #include <assert.h>
+#include <gsl/gsl_sort.h>
 #include <gsl/gsl_statistics.h>
+#include <gsl/gsl_multifit.h>
+#include <gsl/gsl_fit.h>
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/processing.h"
 #include "core/siril_log.h"
-#include "gui/dialogs.h"
 #include "sorting.h"
 #include "statistics.h"
 #include "demosaicing.h"
 #include "gui/progress_and_log.h"
 
-#define ACTIVATE_NULLCHECK_FLOAT 1
-
 // uncomment to debug statistics
 #undef siril_debug_print
 #define siril_debug_print(fmt, ...) { }
 
 // copies the area of an image into the memory buffer data
-static void select_area_float(fits *fit, float *data, int layer, rectangle *bounds) {
+static void select_area_float(fits *fit, float *data, int layer, const rectangle *bounds) {
 	int i, j, k = 0;
 
-	float *from = fit->fpdata[layer] +
+	const float *from = fit->fpdata[layer] +
 		(fit->ry - bounds->y - bounds->h) * fit->rx + bounds->x;
 	int stridefrom = fit->rx - bounds->w;
 
@@ -134,8 +134,7 @@ static double siril_stats_float_bwmv(const float* data, const size_t n,
 			up += SQR(i_med * SQR (1 - yi2));
 			down += (1 - yi2) * (1 - 5 * yi2);
 		}
-
-		bwmv = n * (up / (down * down));
+		bwmv = down ? n * (up / (down * down)) : 0.0;
 	}
 
 	return bwmv;
@@ -252,7 +251,7 @@ static float* reassign_to_non_null_data_float(float *data, size_t inputlen, size
 	}
 
 	for (i = 0; i < inputlen; i++) {
-		if (data[i] != 0.f) {
+		if (data[i] != 0.f && !isnan(data[i])) {
 			if (j >= outputlen) {
 				fprintf(stderr, "\n- stats MISMATCH in sizes (in: %zu, out: %zu), THIS IS A BUG: seqfile is wrong *********\n\n", inputlen, outputlen);
 				break;
@@ -279,8 +278,10 @@ static void siril_stats_float_minmax(float *min_out, float *max_out,
 #endif
 		for (size_t i = 0; i < n; i++) {
 			const float xi = data[i];
-			min = min(min, xi);
-			max = max(max, xi);
+			if (!isnan(xi)) {
+				min = min(min, xi);
+				max = max(max, xi);
+			}
 		}
 
 		*min_out = min;
@@ -292,7 +293,7 @@ static void siril_stats_float_minmax(float *min_out, float *max_out,
 /* this function tries to get the requested stats from the passed stats,
  * computes them and stores them in it if they have not already been */
 imstats* statistics_internal_float(fits *fit, int layer, rectangle *selection, int option, imstats *stats, int bitpix, threading_type threads) {
-	int nx, ny;
+	int nx = 0, ny = 0;
 	float *data = NULL;
 	int stat_is_local = 0, free_data = 0;
 	imstats* stat = stats;
@@ -317,8 +318,9 @@ imstats* statistics_internal_float(fits *fit, int layer, rectangle *selection, i
 			if (layer < 0) {
 				size_t newsz;
 				data = extract_CFA_buffer_area_float(fit, -layer - 1, selection, &newsz);
-				if (!data) {
+				if (!data || newsz == 0) {
 					siril_log_message(_("Failed to compute CFA statistics for channel %d\n"), -layer-1);
+					free(data);
 					return NULL;
 				}
 				nx = newsz;
@@ -398,7 +400,7 @@ imstats* statistics_internal_float(fits *fit, int layer, rectangle *selection, i
 			return NULL;	// not in cache, don't compute
 		}
 		siril_debug_print("- stats %p fit %p (%d): computing basic\n", stat, fit, layer);
-		siril_fits_img_stats_float(data, nx, ny, ACTIVATE_NULLCHECK_FLOAT, 0.0f, &stat->ngoodpix,
+		siril_fits_img_stats_float(data, nx, ny, &stat->ngoodpix,
 				NULL, NULL, &stat->mean, &stat->sigma, &stat->bgnoise,
 				NULL, NULL, NULL, threads, &status);
 		if (status) {
@@ -418,9 +420,9 @@ imstats* statistics_internal_float(fits *fit, int layer, rectangle *selection, i
 	}
 
 
-	/* we exclude 0 if some computations remain to be done or copy data if
+	/* we exclude 0 and nans if some computations remain to be done or copy data if
 	 * median has to be computed */
-	if (ACTIVATE_NULLCHECK_FLOAT && fit && compute_median && stat->total != stat->ngoodpix) {
+	if (fit && compute_median && stat->total != stat->ngoodpix) {
 		data = reassign_to_non_null_data_float(data, stat->total, stat->ngoodpix, free_data);
 		if (!data) {
 			if (stat_is_local) free(stat);
@@ -492,7 +494,7 @@ imstats* statistics_internal_float(fits *fit, int layer, rectangle *selection, i
 	return stat;
 }
 
-int compute_means_from_flat_cfa_float(fits *fit, double mean[36]) {
+int compute_means_from_flat_cfa_float(const fits *fit, double mean[36]) {
 	int row, col, c, i[36] = {0};
 	float *data;
 	unsigned int width, height;
@@ -683,7 +685,7 @@ float siril_stats_robust_mean(const float sorted_data[],
 
 /************* another robust mean ***************/
 
-int robustmean(int n, double *x, double *mean, double *stdev)
+int robustmean(int n, const double *x, double *mean, double *stdev)
 	/* Newton's iterations */
 {
 	int i, it;
@@ -761,3 +763,42 @@ int robustmean(int n, double *x, double *mean, double *stdev)
 	return 0;
 }
 
+double robust_median_f(fits *fit, rectangle *area, int chan, float lower, float upper) {
+	uint32_t x0, y0, x1,y1;
+	if (area) {
+		x0 = area->x;
+		y0 = area->y;
+		x1 = area->x + area->w;
+		y1 = area->y + area->h;
+	} else {
+		x0 = y0 = 0;
+		x1 = gfit.rx;
+		y1 = gfit.ry;
+	}
+	size_t npixels = (x1 - x0) * (y1 - y0);
+	float *data = fit->fpdata[chan];
+	float *filtered_data = malloc(npixels * sizeof(float));
+	size_t count = 0;
+	for (uint32_t y = y0 ; y < y1 ; y++) {
+		size_t j = y * fit->rx;
+		for (uint32_t x = x0 ; x < x1 ; x++) {
+			size_t i = x + j;
+			if (data[i] >= lower && data[i] <= upper) {
+				filtered_data[count++] = data[i];
+			}
+		}
+	}
+	// Check if there are any elements in the specified range
+	if (count == 0) {
+		free(filtered_data);
+		return 0.0; // No elements in the range, return 0 as median
+	}
+	// Sort the filtered data
+	// use histogram_median_float here instead of quickmedian for speed (see #1458)
+	double retval = histogram_median_float(filtered_data, count, MULTI_THREADED);
+
+	// Free the allocated memory for filtered_data
+	free(filtered_data);
+
+	return retval;
+}
