@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -84,58 +84,124 @@
  * @param name the path of the directory to be tested
  * @return the disk space remaining in bytes, or a negative value if error
  */
-#if OS_OSX
+#ifdef OS_OSX
 static gint64 find_space(const gchar *name) {
 	gint64 result = -1;
-	@autoreleasepool
-	{
+	@autoreleasepool {
+		// Check input parameter
+		if (!name) {
+			NSLog(@"Error: null path provided");
+			return result;
+		}
+
 		NSString *path = [NSString stringWithUTF8String:name];
 		if (!path) {
+			NSLog(@"Error: invalid path string encoding");
+			return result;
+		}
+
+		NSFileManager *fileManager = [NSFileManager defaultManager];
+
+		// Check if path exists
+		BOOL isDirectory;
+		if (![fileManager fileExistsAtPath:path isDirectory:&isDirectory]) {
+			NSLog(@"Error: path does not exist: %@", path);
+			return result;
+		}
+
+		// Resolve symlink if needed
+		NSString *resolvedPath = path;
+		NSError *attrError = nil;
+		NSDictionary *attributes = [fileManager attributesOfItemAtPath:path error:&attrError];
+
+		if (!attributes) {
+			NSLog(@"Error: could not get attributes: %@", attrError);
+			return result;
+		}
+
+		// Check if it's a symbolic link
+		if ([[attributes fileType] isEqualToString:NSFileTypeSymbolicLink]) {
+			NSError *linkError = nil;
+			NSString *destination = [fileManager destinationOfSymbolicLinkAtPath:path error:&linkError];
+
+			if (!destination) {
+				NSLog(@"Error: could not resolve symbolic link: %@", linkError);
+				return result;
+			}
+
+			// If the symlink path is relative, combine it with the parent directory
+			if (![destination hasPrefix:@"/"]) {
+				NSString *parentDir = [path stringByDeletingLastPathComponent];
+				resolvedPath = [parentDir stringByAppendingPathComponent:destination];
+			} else {
+				resolvedPath = destination;
+			}
+
+			// Verify the resolved path exists
+			if (![fileManager fileExistsAtPath:resolvedPath isDirectory:&isDirectory]) {
+				NSLog(@"Error: resolved path does not exist: %@", resolvedPath);
+				return result;
+			}
+		}
+
+		NSURL *fileURL = [[NSURL alloc] initFileURLWithPath:resolvedPath];
+		if (!fileURL) {
+			NSLog(@"Error: could not create NSURL from path");
 			return result;
 		}
 
-		NSURL *fileURL = [[NSURL alloc] initFileURLWithPath:path];
 		NSError *error = nil;
 
-		// First get filesystem type
-		NSDictionary *fsInfo = [fileURL resourceValuesForKeys:@[NSURLVolumeLocalizedFormatDescriptionKey]
-		error:&error];
+		// Check if volume is accessible
+		NSDictionary *fsInfo = [fileURL resourceValuesForKeys:@[NSURLVolumeLocalizedFormatDescriptionKey] error:&error];
 		if (!fsInfo) {
 			NSLog(@"Error getting filesystem info: %@", error);
+			#if !__has_feature(objc_arc)
+			[fileURL release];
+			#endif
 			return result;
 		}
 
 		NSString *fsType = fsInfo[NSURLVolumeLocalizedFormatDescriptionKey];
+		if (!fsType) {
+			NSLog(@"Error: could not determine filesystem type");
+			#if !__has_feature(objc_arc)
+			[fileURL release];
+			#endif
+			return result;
+		}
 
-		// For APFS or HFS+, we can use the important usage key
+		// For APFS or HFS+, use important usage key
 		if ([fsType containsString:@"APFS"] || [fsType containsString:@"HFS"]) {
-			NSDictionary *results = [fileURL resourceValuesForKeys:@[NSURLVolumeAvailableCapacityForImportantUsageKey]
-			error:&error];
+			NSDictionary *results = [fileURL resourceValuesForKeys:@[NSURLVolumeAvailableCapacityForImportantUsageKey] error:&error];
 			if (!results) {
 				NSLog(@"Error getting space info: %@", error);
-			} else if (results) {
+			} else {
 				NSNumber *freeSpace = results[NSURLVolumeAvailableCapacityForImportantUsageKey];
 				if (freeSpace) {
 					result = (gint64)[freeSpace longLongValue];
+				} else {
+					NSLog(@"Error: no free space information available");
 				}
 			}
 		} else {
-			// For other filesystems (FAT32, etc.), fall back to basic capacity
-			NSDictionary *results = [fileURL resourceValuesForKeys:@[NSURLVolumeAvailableCapacityKey]
-			error:&error];
+			// For other filesystems (FAT32, etc.), use basic capacity
+			NSDictionary *results = [fileURL resourceValuesForKeys:@[NSURLVolumeAvailableCapacityKey] error:&error];
 			if (!results) {
 				NSLog(@"Error getting space info: %@", error);
-			} else if (results) {
+			} else {
 				NSNumber *freeSpace = results[NSURLVolumeAvailableCapacityKey];
 				if (freeSpace) {
-				result = (gint64)[freeSpace longLongValue];
+					result = (gint64)[freeSpace longLongValue];
+				} else {
+					NSLog(@"Error: no free space information available");
+				}
 			}
 		}
-	}
 
-	#if !__has_feature(objc_arc)
-	[fileURL release];
-	#endif
+		#if !__has_feature(objc_arc)
+		[fileURL release];
+		#endif
 	}
 
 	return result;
@@ -260,7 +326,7 @@ static guint64 get_used_RAM_memory() {
  * @param the path of the folder to be checked
  * @return TRUE if free space disk is available, FALSE otherwise
  */
-gboolean is_space_disk_available(gchar *disk) {
+gboolean is_space_disk_available(const gchar *disk) {
 	return (!(find_space(disk) < 1L));
 }
 
@@ -269,7 +335,9 @@ gboolean is_space_disk_available(gchar *disk) {
  * and displays information on the control window.
  * @return always return TRUE
  */
-gboolean update_displayed_memory() {
+gboolean update_displayed_memory(gpointer data) {
+	// Remove unused argument warnings
+	(void) data;
 	set_GUI_MEM(get_used_RAM_memory(), "labelmem");
 	set_GUI_DiskSpace(find_space(com.wd), "labelFreeSpace");
 	set_GUI_DiskSpace(find_space(com.pref.swap_dir), "free_mem_swap");
@@ -560,8 +628,9 @@ void init_num_procs() {
 		siril_log_message(_("Using cgroups limit on the number of processors: %d\n"), cgroups_num_proc);
 		com.max_thread = cgroups_num_proc;
 	}
-	omp_set_nested(1);
-	int supports_nesting = omp_get_nested() && omp_get_max_active_levels() > 1;
+	omp_set_max_active_levels(INT_MAX);
+	int max_levels_supported = omp_get_max_active_levels();
+	int supports_nesting = max_levels_supported > 1;
 	siril_log_message(
 			_("Parallel processing enabled: using %d logical %s%s.\n"),
 			com.max_thread,
@@ -986,3 +1055,88 @@ long get_pathmax(void) {
 	return MAX_PATH;
 #endif
 }
+
+
+#ifdef _WIN32
+
+gchar *get_siril_bundle_path() {
+	gchar *sirilexepath = NULL;
+	// g_find_program_in_path doc:
+	/* This means first in the directory where the executing program was loaded from,
+		then in the current directory, then in the Windows 32-bit system directory,
+		then in the Windows directory, and finally in the directories in the PATH environment variable
+	*/
+	// so g_find_program_in_path should return the path to currently loaded siril. Note we don't
+	// care whether we are looking for siril.exe or siril-cli.exe as both will be installed in the
+	// same place: it is the directory path we care about.
+	sirilexepath = g_find_program_in_path("siril.exe");
+	if (!sirilexepath)  // should not happen
+		return NULL;
+	const gchar *bin_folder = g_path_get_dirname(sirilexepath);
+	g_free(sirilexepath);
+	return g_path_get_dirname(bin_folder);
+}
+
+// will find an executable in PATH if path is set to NULL
+// will find an executable in passed path otherwise
+gchar *find_executable_in_path(const char *exe_name, const char *path) {
+	char full_path[MAX_PATH];
+	DWORD result;
+	const gchar *path_value;
+	if (!path) { // we need to remove mingw64 tokens from PATH
+		const gchar *tmp_path_value = g_getenv("PATH");
+		// siril_debug_print("Unfiltered: %s\n", tmp_path_value);
+		gchar **tokens = g_strsplit(tmp_path_value, ";", -1);
+		GPtrArray *filtered_tokens = g_ptr_array_new_with_free_func(g_free);
+		for (guint i = 0; tokens[i] != NULL; i++) {
+			if (!g_strstr_len(tokens[i], -1, "mingw64") && !g_strstr_len(tokens[i], -1, "msys64")) {
+				g_ptr_array_add(filtered_tokens, g_strdup(tokens[i]));
+			}
+		}
+		g_ptr_array_add(filtered_tokens, NULL);
+		path_value = g_strjoinv(";", (gchar **)filtered_tokens->pdata);
+		// siril_debug_print("Filtered: %s\n", path_value);
+		// Free memory
+		g_strfreev(tokens);
+		g_ptr_array_free(filtered_tokens, TRUE);
+	} else
+		path_value = path;
+
+	// Use SearchPath to find the executable in PATH
+	result = SearchPath(
+		path_value,		// Search in PATH directories
+		exe_name,	// The executable name
+		".exe",		// Default extension (NULL if no default extension)
+		MAX_PATH,	// Buffer size
+		full_path,	// Buffer to store the full path
+		NULL		// Address of a pointer to the file part of the path (can be NULL)
+	);
+
+	if (result > 0 && result < MAX_PATH) {
+		// Found the executable, so return a copy of the path
+		return g_strdup(full_path);
+	} else {
+		// Executable not found
+		return NULL;
+	}
+}
+
+#endif
+
+gchar *get_siril_version_string() {
+#ifdef _WIN32
+#ifdef SIRIL_UNSTABLE
+	return g_strdup_printf(_("unreleased %s %s-%s for %s (%s %s)"), PACKAGE, VERSION, SIRIL_GIT_VERSION_ABBREV,
+			SIRIL_BUILD_PLATFORM_FAMILY, CPU_ARCH, SIRIL_BUILD_PLATFORM_ENV);
+#else
+	return g_strdup_printf(_("%s %s for %s (%s %s)"), PACKAGE, VERSION, SIRIL_BUILD_PLATFORM_FAMILY, CPU_ARCH, SIRIL_BUILD_PLATFORM_ENV);
+#endif
+#else
+#ifdef SIRIL_UNSTABLE
+	return g_strdup_printf(_("unreleased %s %s-%s for %s (%s)"), PACKAGE, VERSION, SIRIL_GIT_VERSION_ABBREV,
+			SIRIL_BUILD_PLATFORM_FAMILY, CPU_ARCH);
+#else
+	return g_strdup_printf(_("%s %s for %s (%s)"), PACKAGE, VERSION, SIRIL_BUILD_PLATFORM_FAMILY, CPU_ARCH);
+#endif
+#endif
+}
\ No newline at end of file
