@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include <ctype.h>
 
 #include "core/siril.h"
+#include "algos/statistics.h"
 #include "core/proto.h"
 #include "core/initfile.h"
 #include "core/OS_utils.h"
@@ -33,7 +34,8 @@
 #include "gui/image_interactions.h"
 #include "gui/image_display.h"
 #include "gui/callbacks.h"
-#include "gui/preferences.h"
+#include "gui/histogram.h"
+#include "gui/script_menu.h"
 #include "core/processing.h"
 #include "core/command_list.h"
 #include "io/sequence.h"
@@ -44,7 +46,7 @@
 #include "command.h"
 #include "command_line_processor.h"
 
-static const char *cmd_err_to_str(cmd_errors err) {
+const char *cmd_err_to_str(cmd_errors err) {
 	switch (err) {
 		case CMD_NOT_FOUND:
 			return _("unknown command name");
@@ -93,6 +95,8 @@ static const char *cmd_err_to_str(cmd_errors err) {
 			return _("memory allocation error");
 		case CMD_THREAD_RUNNING:
 			return _("a processing is already running");
+        case CMD_DIR_NOT_FOUND:
+			return _("directory not found");
 	}
 }
 
@@ -197,7 +201,20 @@ int execute_command(int wordnb) {
 	// process the command
 	siril_log_color_message(_("Running command: %s\n"), "salmon", word[0]);
 	fprintf(stdout, "%lu: running command %s\n", time(NULL), word[0]);
-	return commands[i].process(wordnb);
+	int retval = commands[i].process(wordnb);
+	if (gui.roi.active)
+		populate_roi();
+	if (retval & CMD_NOTIFY_GFIT_MODIFIED) {
+		if (!com.python_script) {
+			notify_gfit_modified();
+		} else {
+			invalidate_stats_from_fit(&gfit);
+			invalidate_gfit_histogram();
+			execute_idle_and_wait_for_it(end_gfit_operation, NULL);
+		}
+		retval = retval & ~CMD_NOTIFY_GFIT_MODIFIED;
+	}
+	return (int) retval;
 }
 
 static void update_log_icon(gboolean is_running) {
@@ -254,16 +271,16 @@ static void clear_status_bar() {
 static gboolean end_script(gpointer p) {
 	/* GTK+ code is ignored during scripts, this is a good place to redraw everything */
 	clear_status_bar();
-	set_GUI_CWD();
-	update_MenuItem();
-	adjust_cutoff_from_updated_gfit();
+	gui_function(set_GUI_CWD, NULL);
+	gui_function(update_MenuItem, NULL);
+	notify_gfit_modified();
 	redraw(REMAP_ALL);
-	redraw_previews();
+	gui_function(redraw_previews, NULL);
 	update_zoom_label();
 	update_display_fwhm();
 	display_filename();
-	new_selection_zone();
-	update_spinCPU(0);
+	gui_function(new_selection_zone, NULL);
+	update_spinCPU(GINT_TO_POINTER(0));
 	set_cursor_waiting(FALSE);
 	return FALSE;
 }
@@ -354,7 +371,9 @@ gpointer execute_script(gpointer p) {
 		};
 
 		retval = execute_command(wordnb);
-
+		remove_child_from_children((GPid) -2); // remove the processing thread child
+		// reference (speculative - not always necessary, but simplest to
+		// call it every time just in case the command ran in the thread.)
 		if (retval && retval != CMD_NO_WAIT) {
 			siril_log_message(_("Error in line %d ('%s'): %s.\n"), line, buffer, cmd_err_to_str(retval));
 			siril_log_message(_("Exiting batch processing.\n"));
@@ -377,7 +396,7 @@ gpointer execute_script(gpointer p) {
 
 	if (!com.headless) {
 		com.script = FALSE;
-		siril_add_idle(end_script, NULL);
+		gui_function(end_script, NULL);
 	}
 
 	/* Now we want to restore the saved cwd */
@@ -398,10 +417,13 @@ gpointer execute_script(gpointer p) {
 		siril_debug_print("Script thread exiting\n");
 		com.script_thread_exited = TRUE;
 	}
+	/* If called from the GUI, re-enable widgets blocked during the script */
+	siril_add_idle(script_widgets_idle, NULL);
 	return GINT_TO_POINTER(retval);
 }
 
-static void show_command_help_popup(GtkEntry *entry) {
+static gboolean show_command_help_popup(gpointer user_data) {
+	GtkEntry *entry = (GtkEntry*) user_data;
 	gchar *helper = NULL;
 
 	const gchar *text = gtk_entry_get_text(entry);
@@ -464,6 +486,7 @@ static void show_command_help_popup(GtkEntry *entry) {
 	gtk_widget_show(popover);
 #endif
 	g_free(helper);
+	return FALSE;
 }
 
 /* handler for the single-line console */
@@ -540,16 +563,17 @@ static gboolean on_command_key_press_event(GtkWidget *widget, GdkEventKey *event
 	return (handled == 1);
 }
 
-int processcommand(const char *line) {
+int processcommand(const char *line, gboolean wait_for_completion) {
 	int wordnb = 0;
 	GError *error = NULL;
+	int ret = 0;
 
 	if (line[0] == '\0' || line[0] == '\n')
-		return 0;
+		return CMD_NOT_FOUND;
 	if (line[0] == '@') { // case of files
 		if (get_thread_run() || (get_script_thread_run() && !com.script_thread_exited)) {
 			PRINT_ANOTHER_THREAD_RUNNING;
-			return 1;
+			return CMD_THREAD_RUNNING;
 		}
 		if (get_script_thread_run())
 			wait_for_script_thread();
@@ -571,7 +595,7 @@ int processcommand(const char *line) {
 			}
 
 			g_object_unref(file);
-			return 1;
+			return CMD_FILE_NOT_FOUND;
 		}
 		/* Run the script */
 		siril_log_message(_("Starting script %s\n"), filename);
@@ -585,19 +609,35 @@ int processcommand(const char *line) {
 		int len = strlen(line);
 		if (len > 0)
 			g_print("input command:%s\n", myline);
+
 		parse_line(myline, len, &wordnb);
-		int ret = execute_command(wordnb);
+		ret = execute_command(wordnb);
+
 		if (ret) {
 			siril_log_color_message(_("Command execution failed: %s.\n"), "red", cmd_err_to_str(ret));
-			if (!com.script && !com.headless && (ret == CMD_WRONG_N_ARG || ret == CMD_ARG_ERROR)) {
-				show_command_help_popup(GTK_ENTRY(lookup_widget("command")));
+			if (!(com.script || com.python_script) && !com.headless &&
+				(ret == CMD_WRONG_N_ARG || ret == CMD_ARG_ERROR)) {
+				gui_function(show_command_help_popup, GTK_ENTRY(lookup_widget("command")));
 			}
 			free(myline);
-			return 1;
+			return ret;
 		}
+
+		if (wait_for_completion && ret != CMD_NO_WAIT) {
+			while (get_thread_run()) {
+				if (waiting_for_thread()) {
+					ret = CMD_GENERIC_ERROR;  // Command failed during execution
+					break;
+				}
+				g_usleep(100000);  // Sleep for 100ms to avoid busy waiting
+				remove_child_from_children((GPid) -2); // remove processing thread from list
+			}
+		}
+
 		free(myline);
 	}
-	return 0;
+
+	return ret;
 }
 
 // loads the sequence from com.wd
@@ -782,9 +822,9 @@ static void history_add_line(char *line) {
 void on_command_activate(GtkEntry *entry, gpointer user_data) {
 	const gchar *text = gtk_entry_get_text(entry);
 	history_add_line(strdup(text));
-	if (!(processcommand(text))) {
+	if (!(processcommand(text, FALSE))) {
 		gtk_entry_set_text(entry, "");
-		set_precision_switch();
+		gui_function(set_precision_switch, NULL);
 	}
 }
 
