@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +31,6 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <string.h>
-#include <sys/stat.h>
 #include <fcntl.h>
 #include <signal.h>
 #ifdef _WIN32
@@ -383,7 +382,7 @@ void *read_pipe(void *p) {
 		char buf[PIPE_MSG_SZ];
 		do {
 			int select_return;
-			pipe_read_status read_return;
+			pipe_read_status read_return = PIPE_READ_UNUSED;
 			fd_set rfds;
 			FD_ZERO(&rfds);
 			FD_SET(pipe_fd_r, &rfds);
