@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -177,8 +177,27 @@ static int darkOptimization(fits *raw, struct preprocessing_data *args, int in_i
 	if (copyfits(dark, &dark_tmp, CP_ALLOC | CP_COPYA | CP_FORMAT, 0))
 		return 1;
 
-	/* Minimization of background noise to find better k */
-	k0 = goldenSectionSearch(raw, &dark_tmp, lo, up, 0.001f, args->allow_32bit_output);
+	if (args->use_exposure) {
+		if (dark->keywords.exposure <= 0.0) {
+			siril_log_color_message(_("The dark frame contains no exposure data or incorrect exposure data.\n"), "red");
+			clearfits(&dark_tmp);
+			return 1;
+		}
+		if (raw->keywords.exposure <= 0.0) {
+			siril_log_color_message(_("The light frame (%d) contains no exposure data or incorrect exposure data.\n"), "red", in_index + 1);
+			clearfits(&dark_tmp);
+			return 1;
+		}
+		/* linear scale with time */
+		k0 = raw->keywords.exposure / dark->keywords.exposure;
+		if (k0 > 1.f) {
+			siril_log_color_message(_("Warning: master dark is shorter than lights. It is "
+						"recommended that the master dark be at least as long as the lights.\n"), "salmon");
+		}
+	} else {
+		/* Minimization of background noise to find better k */
+		k0 = goldenSectionSearch(raw, &dark_tmp, lo, up, 0.001f, args->allow_32bit_output);
+	}
 	if (k0 < 0.f) {
 		siril_log_message(_("Dark optimization of image %d failed\n"), in_index);
 		ret = -1;
@@ -369,7 +388,7 @@ int prepro_prepare_hook(struct generic_seq_args *args) {
 
 	// proceed to cosmetic correction
 	if (prepro->use_cosmetic_correction && prepro->use_dark && prepro->cc_from_dark) {
-		if (strlen(prepro->dark->bayer_pattern) > 4) {
+		if (strlen(prepro->dark->keywords.bayer_pattern) > 4) {
 			siril_log_color_message(_("Cosmetic correction cannot be applied on X-Trans files.\n"), "red");
 			prepro->use_cosmetic_correction = FALSE;
 		} else {
@@ -443,7 +462,7 @@ int prepro_image_hook(struct generic_seq_args *args, int out_index, int in_index
 	 */
 	full_stats_invalidation_from_fit(fit);
 	fit->history = g_slist_concat(fit->history, history);
-	fit->lo = 0;
+	fit->keywords.lo = 0;
 	return 0;
 }
 
@@ -492,11 +511,14 @@ void start_sequence_preprocessing(struct preprocessing_data *prepro) {
 			(args->seq->type == SEQ_SER && !args->force_fitseq_output)) ?
 		DATA_USHORT : DATA_FLOAT;
 
-	start_in_new_thread(generic_sequence_worker, args);
+	if (!start_in_new_thread(generic_sequence_worker, args)) {
+		free(prepro->ppprefix);
+		free_generic_seq_args(args, TRUE);
+	}
 }
 
 /********** SINGLE IMAGE (from com.uniq) ************/
-int preprocess_single_image(struct preprocessing_data *args) {
+int calibrate_single_image(struct preprocessing_data *args) {
 	gchar *msg;
 	fits fit = { 0 };
 	int ret = 0;
@@ -539,6 +561,8 @@ int preprocess_single_image(struct preprocessing_data *args) {
 		free(filename_noext);
 		g_free(dest_filename);
 		g_free(msg);
+	} else {
+		clearfits(&fit);
 	}
 
 	return ret;
@@ -607,7 +631,7 @@ int evaluateoffsetlevel(const char* expression, fits *fit) {
 	}
 	if (!multiplier) goto free_on_error; // multiplier not parsed
 	if (end[0] != '\0') goto free_on_error; // some characters were found after the multiplier
-	offsetlevel = (int)(multiplier * fit->key_offset);
+	offsetlevel = (int)(multiplier * fit->keywords.key_offset);
 	if (expressioncpy) g_free(expressioncpy);
 	return offsetlevel;
 free_on_error:
@@ -775,13 +799,14 @@ static gboolean test_for_master_files(struct preprocessing_data *args) {
 				has_error = TRUE;
 			}
 			g_free(expression);
-			
+
 		}
 
 		if (args->use_dark) {
 			// dark optimization
-			tbutton = GTK_TOGGLE_BUTTON(lookup_widget("checkDarkOptimize"));
-			args->use_dark_optim = gtk_toggle_button_get_active(tbutton);
+			int optim = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget("comboDarkOptimize")));
+			args->use_dark_optim = optim != 0;
+			args->use_exposure = optim == 2;
 			const char *error = NULL;
 			if ((gfit.orig_bitpix == BYTE_IMG) && args->use_dark_optim) {
 				error = _("Dark optimization: This process cannot be applied to 8b images");
@@ -870,8 +895,8 @@ static gboolean test_for_master_files(struct preprocessing_data *args) {
 					}
 
 				} else error = _("NOT USING FLAT: cannot open the file");
-				g_free(expression);
 			}
+			g_free(expression); // expression not used again after here, free before it falls out of scope
 			if (error) {
 				siril_log_color_message("%s\n", "red", error);
 				set_progress_bar_data(error, PROGRESS_DONE);
@@ -918,10 +943,7 @@ void on_prepro_button_clicked(GtkButton *button, gpointer user_data) {
 		free(args);
 		return;
 	}
-	if (!args->use_bias && !args->use_dark && !args->use_flat) {
-		free(args);
-		return;
-	}
+
 	siril_log_color_message(_("Preprocessing...\n"), "green");
 
 	// set output filename (preprocessed file name prefix)
@@ -953,7 +975,7 @@ void on_prepro_button_clicked(GtkButton *button, gpointer user_data) {
 		set_cursor_waiting(TRUE);
 		control_window_switch_to_tab(OUTPUT_LOGS);
 
-		retval = preprocess_single_image(args);
+		retval = calibrate_single_image(args);
 
 		free(args);
 
@@ -962,7 +984,7 @@ void on_prepro_button_clicked(GtkButton *button, gpointer user_data) {
 		else {
 			set_progress_bar_data(PROGRESS_TEXT_RESET, PROGRESS_RESET);
 			invalidate_gfit_histogram();
-			open_single_image_from_gfit();
+			gui_function(open_single_image_from_gfit, NULL);
 		}
 		set_cursor_waiting(FALSE);
 	}
@@ -994,8 +1016,8 @@ void on_GtkButtonEvaluateCC_clicked(GtkButton *button, gpointer user_data) {
 	}
 
 	set_cursor_waiting(TRUE);
-	sig[0] = gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spinSigCosmeColdBox")));
-	sig[1] = gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spinSigCosmeHotBox")));
+	sig[0] = gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spinSigCosmeCold")));
+	sig[1] = gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spinSigCosmeHot")));
 	widget[0] = lookup_widget("GtkLabelColdCC");
 	widget[1] = lookup_widget("GtkLabelHotCC");
 	label[0] = GTK_LABEL(lookup_widget("GtkLabelColdCC"));
@@ -1013,6 +1035,8 @@ void on_GtkButtonEvaluateCC_clicked(GtkButton *button, gpointer user_data) {
 		return;
 	}
 	find_deviant_pixels(&fit, sig, &icold, &ihot, TRUE);
+	printf("sig[0]=%lf\n", sig[0]);
+	printf("sig[1]=%lf\n", sig[1]);
 	total = fit.rx * fit.ry;
 	clearfits(&fit);
 	rate = (double)icold / total;
