@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,8 @@
 #include "core/settings.h"
 #include "core/siril.h"
 #include "core/siril_log.h"
-#include "io/catalogues.h"
+#include "core/icc_profile.h"
+#include "io/local_catalogues.h"
 #include "stacking/stacking.h"
 
 /* the settings as initialized in static.
@@ -42,15 +43,11 @@ preferences pref_init = {
 	.hd_bitdepth = 20,
 	.script_check_requires = TRUE,
 	.pipe_check_requires = FALSE,
-#ifdef HAVE_JSON_GLIB
  #ifdef SIRIL_UNSTABLE
 	.check_update = FALSE,
  #else
 	.check_update = TRUE,
  #endif
-#else
-	.check_update = FALSE,
-#endif
 	.lang = 0,
 	.swap_dir = NULL,
 	.binning_update = TRUE,
@@ -59,12 +56,19 @@ preferences pref_init = {
 	.catalogue_paths[1] = NULL,
 	.catalogue_paths[2] = NULL,
 	.catalogue_paths[3] = NULL,
+	.catalogue_paths[4] = NULL,
+	.catalogue_paths[5] = NULL,
 	.rgb_aladin = FALSE,
+	.use_checksum = FALSE,
 	.copyright = NULL,
 	.starnet_exe = NULL,
 	.starnet_weights = NULL,
-	.gnuplot_dir = NULL,
+	.graxpert_path = NULL,
 	.asnet_dir = NULL,
+	.selected_scripts = NULL,
+	.use_scripts_repository = TRUE,
+	.auto_script_update = TRUE,
+	.drizz_weight_match_bitpix = FALSE,
 	.starfinder_conf = { // starfinder_conf
 		.radius = DEF_BOX_RADIUS,
 		.sigma = 1.0,
@@ -101,6 +105,8 @@ preferences pref_init = {
 		.use_dark_lib = FALSE,
 		.flat_lib = NULL,
 		.use_flat_lib = FALSE,
+		.disto_lib = NULL,
+		.use_disto_lib = FALSE,
 		.stack_default = NULL,
 		.use_stack_default = TRUE,
 	},
@@ -122,7 +128,7 @@ preferences pref_init = {
 		.font_scale = 100,
 		.icon_symbolic = FALSE,
 		.script_path = NULL,
-		.warn_script_run = TRUE,
+		.warn_scripts_run = TRUE,
 		.show_thumbnails = TRUE,
 		.thumbnail_size = 256,
 		.default_rendering_mode = LINEAR_DISPLAY,
@@ -136,6 +142,8 @@ preferences pref_init = {
 		.catalog[6] = TRUE,
 		.catalog[7] = TRUE,
 		.catalog[8] = TRUE,
+		.catalog[9] = TRUE,
+		.catalog[10] = TRUE,
 		.position_compass = 1,
 		.selection_guides = 0,
 		.show_deciasec = FALSE,
@@ -143,16 +151,42 @@ preferences pref_init = {
 		.reg_interpolation = OPENCV_LANCZOS4,
 		.reg_clamping = TRUE,
 		.pm_presets = NULL,
+		.roi_mode = ROI_MANUAL,
+		.enable_roi_warning = TRUE,
 		.config_colors.color_bkg_samples = NULL,
 		.config_colors.color_std_annotations = NULL,
-		.mmb_action = MMB_ZOOM_FIT
+		.config_colors.color_dso_annotations = NULL,
+		.config_colors.color_sso_annotations = NULL,
+		.config_colors.color_tmp_annotations = NULL,
+		.mmb_action = MMB_ZOOM_FIT,
+		.mouse_speed_limit = 0.0,
+		.mouse_cfg = {
+			.mouse_actions_array = NULL,
+			.scroll_actions_array = NULL
+		},
+		.editor_cfg = {
+			.highlight_syntax = TRUE,
+			.highlight_bracketmatch = TRUE,
+			.rmargin = TRUE,
+			.rmargin_pos = 80,
+			.show_linenums = TRUE,
+			.show_linemarks = FALSE,
+			.highlight_currentline = TRUE,
+			.autoindent = TRUE,
+			.indentontab = TRUE,
+			.smartbs = TRUE,
+			.smarthomeend = TRUE,
+			.showspaces = FALSE,
+			.shownewlines = FALSE,
+			.minimap = FALSE
+		}
 	},
 	.debayer = {
 		.open_debayer = FALSE,
 		.use_bayer_header = TRUE,
 		.bayer_pattern = BAYER_FILTER_RGGB,
 		.bayer_inter = BAYER_RCD,
-		.top_down = TRUE,
+		.orientation = ROW_ORDER_HEADER_TOPDOWN,
 		.xbayeroff = 0,
 		.ybayeroff = 0,
 		.xtrans_passes = 1
@@ -164,19 +198,22 @@ preferences pref_init = {
 		.auto_inner_factor = 4.2,
 		.auto_outer_factor = 6.3,
 		.aperture = 10.0,
+		.auto_aperture_factor = 4.0,
 		.force_radius = TRUE,
 		.minval = -1500.0,
 		.maxval = 60000.0,
+		.discard_var_catalogues = 4,
 	},
 	.astrometry = {
 		.update_default_scale = TRUE,
 		.percent_scale_range = 20,
-		.sip_correction_order = 0,
+		.sip_correction_order = 3,
 		.radius_degrees = 10.0,
 		.keep_xyls_files = FALSE,
 		.keep_wcs_files = FALSE,
-		.max_seconds_run = 10,
+		.max_seconds_run = 30,
 		.show_asnet_output = FALSE,
+		.default_obscode = NULL,
 	},
 	.analysis = {
 		.mosaic_panel = 256,
@@ -203,6 +240,49 @@ preferences pref_init = {
 		.multithreaded = TRUE,
 		.wisdom_file = NULL,
 		.fft_cutoff = 15,
+	},
+	.max_slice_size = 32769,
+	.fits_save_icc = TRUE,
+	.icc = {
+		.rendering_intent = INTENT_RELATIVE_COLORIMETRIC,
+		.proofing_intent = INTENT_RELATIVE_COLORIMETRIC,
+		.export_intent = INTENT_RELATIVE_COLORIMETRIC,
+		.processing_intent = INTENT_PERCEPTUAL,
+		.working_gamut = TYPE_SRGB,
+		.icc_path_monitor = NULL,
+		.icc_path_soft_proof = NULL,
+		.custom_monitor_profile_active = FALSE,
+		.soft_proofing_profile_active = FALSE,
+		.custom_icc_trc = NULL,
+		.custom_icc_gray = NULL,
+		.export_8bit_method = EXPORT_SRGB,
+		.export_16bit_method = EXPORT_WORKING,
+		.default_to_srgb = TRUE,
+		.rendering_bpc = TRUE,
+		.autoconversion = ICC_NEVER_AUTOCONVERT,
+		.autoassignment = ICC_ASSIGN_ON_STRETCH,
+		.pedantic_linear = FALSE,
+		.cmf = CMF_1931_2DEG
+	},
+	.spcc = {
+		.use_spcc_repository = TRUE,
+		.auto_spcc_update = TRUE,
+		.redpref = NULL,
+		.greenpref = NULL,
+		.bluepref = NULL,
+		.lpfpref = NULL,
+		.oscfilterpref = NULL,
+		.monosensorpref = NULL,
+		.oscsensorpref = NULL,
+		.is_mono = TRUE,
+		.is_dslr = FALSE,
+		.nb_mode = FALSE,
+		.red_wl = 656.28,
+		.green_wl = 500.70,
+		.blue_wl = 500.70,
+		.red_bw = 6.0,
+		.green_bw = 6.0,
+		.blue_bw = 6.0
 	}
 };
 
@@ -215,18 +295,20 @@ void free_preferences(preferences *pref) {
 	pref->copyright = NULL;
 	g_free(pref->starnet_exe);
 	pref->starnet_exe = NULL;
+	g_free(pref->graxpert_path);
+	pref->graxpert_path = NULL;
 	g_free(pref->starnet_weights);
 	pref->starnet_weights = NULL;
-	g_free(pref->gnuplot_dir);
-	pref->gnuplot_dir = NULL;
 	g_free(pref->asnet_dir);
 	pref->asnet_dir = NULL;
 	g_free(pref->lang);
 	pref->lang = NULL;
-	g_slist_free_full(pref->gui.script_path, g_free);
+	g_slist_free_full(g_steal_pointer(&pref->gui.script_path), g_free);
 	pref->gui.script_path = NULL;
 	g_free(pref->fftw_conf.wisdom_file);
 	pref->fftw_conf.wisdom_file = NULL;
+	g_free(pref->astrometry.default_obscode);
+	pref->astrometry.default_obscode = NULL;
 }
 
 void set_wisdom_file() {
@@ -241,28 +323,32 @@ void set_wisdom_file() {
 static void initialize_configurable_colors() {
 	com.pref.gui.config_colors.color_bkg_samples = g_strdup("rgba(255, 51, 26, 1.0)");
 	com.pref.gui.config_colors.color_std_annotations = g_strdup("rgba(128, 255, 77, 0.9)");
+	com.pref.gui.config_colors.color_dso_annotations = g_strdup("rgba(255, 128, 0, 0.9)");
+	com.pref.gui.config_colors.color_sso_annotations = g_strdup("rgba(255, 255, 0, 0.9)");
+	com.pref.gui.config_colors.color_tmp_annotations = g_strdup("rgba(255, 0, 0, 0.9)");
 }
 
 /* static + dynamic settings initialization */
 void initialize_default_settings() {
 	com.pref = pref_init;
+	com.pref.spcc.oscfilterpref = g_strdup("No filter");
 	com.pref.ext = g_strdup(".fit");
 	com.pref.prepro.stack_default = g_strdup("$seqname$stacked");
 	com.pref.swap_dir = g_strdup(g_get_tmp_dir());
 	initialize_local_catalogues_paths();
 	initialize_configurable_colors();
-	// set_wisdom_file();
 }
 
 void update_gain_from_gfit() {
-	if (gfit.cvf > 0.0)
-		com.pref.phot_set.gain = gfit.cvf;
+	if (gfit.keywords.cvf > 0.0)
+		com.pref.phot_set.gain = gfit.keywords.cvf;
 }
 
 struct settings_access all_settings[] = {
 	{ "core", "wd", STYPE_STRDIR, N_("current working directory"), &com.pref.wd },
 	{ "core", "extension", STYPE_STR, N_("FITS file extension"), &com.pref.ext },
 	{ "core", "force_16bit", STYPE_BOOL, N_("don't use 32 bits for pixel depth"), &com.pref.force_16bit },
+	{ "core", "fits_save_icc", STYPE_BOOL, N_("embed ICC profiles in FITS when saving"), &com.pref.fits_save_icc },
 	{ "core", "allow_heterogeneous_fitseq", STYPE_BOOL, N_("allow FITS cubes to have different sizes"), &com.pref.allow_heterogeneous_fitseq },
 	{ "core", "mem_mode", STYPE_INT, N_("memory mode (0 ratio, 1 amount)"), &com.pref.mem_mode, { .range_int = { 0, 1 } } },
 	{ "core", "mem_ratio", STYPE_DOUBLE, N_("memory ratio of available"), &com.pref.memory_ratio, { .range_double = { 0.05, 4.0 } } },
@@ -279,11 +365,14 @@ struct settings_access all_settings[] = {
 	{ "core", "catalogue_unnamedstars", STYPE_STR, N_("Path of the unnamedstars.dat catalogue"), &com.pref.catalogue_paths[1] },
 	{ "core", "catalogue_tycho2", STYPE_STR, N_("Path of the deepstars.dat catalogue"), &com.pref.catalogue_paths[2] },
 	{ "core", "catalogue_nomad", STYPE_STR, N_("Path of the USNO-NOMAD-1e8.dat catalogue"), &com.pref.catalogue_paths[3] },
+	{ "core", "catalogue_gaia_astro", STYPE_STR, N_("Path of the local Gaia astrometric catalogue"), &com.pref.catalogue_paths[4] },
+	{ "core", "catalogue_gaia_photo", STYPE_STR, N_("Path of the local Gaia photometric catalogue"), &com.pref.catalogue_paths[5] },
 	{ "core", "rgb_aladin", STYPE_BOOL, N_("add CTYPE3='RGB' in the FITS header"), &com.pref.rgb_aladin },
+	{ "core", "use_checksum", STYPE_BOOL, N_("Verify file checksums if they exist"), &com.pref.use_checksum },
 	{ "core", "copyright", STYPE_STR, N_("user copyright to put in file header"), &com.pref.copyright },
 	{ "core", "starnet_exe", STYPE_STR, N_("location of the StarNet executable"), &com.pref.starnet_exe },
 	{ "core", "starnet_weights", STYPE_STR, N_("location of the StarNet-torch weights file"), &com.pref.starnet_weights },
-	{ "core", "gnuplot_dir", STYPE_STR, N_("directory of the gnuplot installation"), &com.pref.gnuplot_dir },
+	{ "core", "graxpert_path", STYPE_STR, N_("location of the GraXpert executable"), &com.pref.graxpert_path },
 #ifdef _WIN32
 	{ "core", "asnet_dir", STYPE_STR, N_("directory of the asnet_ansvr installation"), &com.pref.asnet_dir },
 #else
@@ -293,6 +382,7 @@ struct settings_access all_settings[] = {
 	{ "core", "fftw_conv_fft_cutoff", STYPE_INT, N_("Convolution minimum kernel size to use FFTW"), &com.pref.fftw_conf.fft_cutoff },
 	{ "core", "fftwf_strategy", STYPE_INT, N_("FFTW planning strategy"), &com.pref.fftw_conf.strategy },
 	{ "core", "fftw_multithreaded", STYPE_BOOL, N_("multithreaded FFTW"), &com.pref.fftw_conf.multithreaded },
+	{ "core", "max_slice_size", STYPE_INT, N_("Maximum slice size for automated slice processing"), &com.pref.max_slice_size, { .range_int = { 512, 32769 } } },
 
 	{ "starfinder", "focal_length", STYPE_DOUBLE, N_("focal length in mm for radius adjustment"), &com.pref.starfinder_conf.focal_length, { .range_double = { 0., 999999. } } },
 	{ "starfinder", "pixel_size", STYPE_DOUBLE, N_("pixel size in µm for radius adjustment"), &com.pref.starfinder_conf.pixel_size_x, { .range_double = { 0., 99. } } },
@@ -300,7 +390,7 @@ struct settings_access all_settings[] = {
 	{ "debayer", "use_bayer_header", STYPE_BOOL, N_("use pattern from the file header"), &com.pref.debayer.use_bayer_header },
 	{ "debayer", "pattern", STYPE_INT, N_("index of the Bayer pattern"), &com.pref.debayer.bayer_pattern, { .range_int = { 0, XTRANS_FILTER_4 } } },
 	{ "debayer", "interpolation", STYPE_INT, N_("type of interpolation"), &com.pref.debayer.bayer_inter, { .range_int = { 0, XTRANS } } },
-	{ "debayer", "top_down", STYPE_BOOL, N_("force debayer top-down"), &com.pref.debayer.top_down },
+	{ "debayer", "orientation", STYPE_INT, N_("row-order preference"), &com.pref.debayer.orientation, { .range_int = { 0, ROW_ORDER_FORCE_BOTTOMUP } } },
 	{ "debayer", "offset_x", STYPE_INT, N_("Bayer matrix offset X"), &com.pref.debayer.xbayeroff, { .range_int = { 0, 1 } } },
 	{ "debayer", "offset_y", STYPE_INT, N_("Bayer matrix offset Y"), &com.pref.debayer.ybayeroff, { .range_int = { 0, 1 } } },
 	{ "debayer", "xtrans_passes", STYPE_INT, N_("Number of passes for the X-Trans Markesteijn algorithm"), &com.pref.debayer.xtrans_passes, { .range_int = { 1, 4 } } },
@@ -311,19 +401,38 @@ struct settings_access all_settings[] = {
 	{ "photometry", "inner_factor", STYPE_DOUBLE, N_("factor for inner radius automatic computation"), &com.pref.phot_set.auto_inner_factor, { .range_double = { 2.0, 50.0 } } },
 	{ "photometry", "outer_factor", STYPE_DOUBLE, N_("factor for outer radius automatic computation"), &com.pref.phot_set.auto_outer_factor, { .range_double = { 2.0, 50.0 } } },
 	{ "photometry", "force_radius", STYPE_BOOL, N_("force flux aperture value"), &com.pref.phot_set.force_radius },
+	{ "photometry", "auto_aperture_factor", STYPE_DOUBLE, N_("Radius/halfFWHM ratio"), &com.pref.phot_set.auto_aperture_factor, { .range_double = { 1., 5. } }  },
 	{ "photometry", "aperture", STYPE_DOUBLE, N_("forced aperture for flux computation"), &com.pref.phot_set.aperture, { .range_double = { 1., 100. } } },
 	{ "photometry", "minval", STYPE_DOUBLE, N_("minimum valid pixel value for photometry"), &com.pref.phot_set.minval, { .range_double = { -65536.0, 65534.0 } } },
 	{ "photometry", "maxval", STYPE_DOUBLE, N_("maximum valid pixel value for photometry"), &com.pref.phot_set.maxval, { .range_double = { 1.0, 65535.0 } } },
+	{ "photometry", "discard_var_catalogues", STYPE_INT, N_("catalogues to be used to discard the variable stars from the comparison stars list"), &com.pref.phot_set.discard_var_catalogues, { .range_int = { 0, 7 } } },
+	{ "photometry", "redpref", STYPE_STR, N_("preferred SPCC red filter"), &com.pref.spcc.redpref },
+	{ "photometry", "greenpref", STYPE_STR, N_("preferred SPCC green filter"), &com.pref.spcc.greenpref },
+	{ "photometry", "bluepref", STYPE_STR, N_("preferred SPCC blue filter"), &com.pref.spcc.bluepref },
+	{ "photometry", "lpfpref", STYPE_STR, N_("preferred SPCC DSLR LPF filter"), &com.pref.spcc.lpfpref },
+	{ "photometry", "oscfilterpref", STYPE_STR, N_("preferred SPCC OSC filter"), &com.pref.spcc.oscfilterpref },
+	{ "photometry", "monosensorpref", STYPE_STR, N_("preferred SPCC mono sensor"), &com.pref.spcc.monosensorpref },
+	{ "photometry", "oscsensorpref", STYPE_STR, N_("preferred SPCC OSC sensor"), &com.pref.spcc.oscsensorpref },
+	{ "photometry", "is_mono", STYPE_BOOL, N_("is the SPCC sensor mono?"), &com.pref.spcc.is_mono },
+	{ "photometry", "is_dslr", STYPE_BOOL, N_("is the SPCC OSC sensor a DSLR?"), &com.pref.spcc.is_dslr },
+	{ "photometry", "nb_mode", STYPE_BOOL, N_("are we in narrowband mode?"), &com.pref.spcc.nb_mode },
+	{ "photometry", "r_wl", STYPE_DOUBLE, N_("red NB filter wavelength"), &com.pref.spcc.red_wl },
+	{ "photometry", "r_bw", STYPE_DOUBLE, N_("red NB filter bandwidth"), &com.pref.spcc.red_bw },
+	{ "photometry", "g_wl", STYPE_DOUBLE, N_("green NB filter wavelength"), &com.pref.spcc.green_wl },
+	{ "photometry", "g_bw", STYPE_DOUBLE, N_("green NB filter bandwidth"), &com.pref.spcc.green_bw },
+	{ "photometry", "b_wl", STYPE_DOUBLE, N_("blue NB filter wavelength"), &com.pref.spcc.blue_wl },
+	{ "photometry", "b_bw", STYPE_DOUBLE, N_("blue NB filter bandwidth"), &com.pref.spcc.blue_bw },
 
-	{ "astrometry", "asnet_sip_order", STYPE_INT, N_("degrees of the polynomial correction"), &com.pref.astrometry.sip_correction_order, { .range_int = { 0, 6 } } },
-	{ "astrometry", "asnet_radius", STYPE_DOUBLE, N_("radius around the target coordinates (degrees)"), &com.pref.astrometry.radius_degrees, { .range_double = { 0.01, 180.0 } } },
 	{ "astrometry", "asnet_keep_xyls", STYPE_BOOL, N_("do not delete .xyls FITS tables"), &com.pref.astrometry.keep_xyls_files },
 	{ "astrometry", "asnet_keep_wcs", STYPE_BOOL, N_("do not delete .wcs result files"), &com.pref.astrometry.keep_wcs_files },
-	{ "astrometry", "asnet_max_seconds_run", STYPE_INT, N_("maximum seconds to try solving"), &com.pref.astrometry.max_seconds_run, { .range_int = { 0, 100000 } } },
 	{ "astrometry", "asnet_show_output", STYPE_BOOL, N_("show solve-field output in main log"), &com.pref.astrometry.show_asnet_output },
 
+	{ "astrometry", "sip_order", STYPE_INT, N_("degrees of the polynomial correction"), &com.pref.astrometry.sip_correction_order, { .range_int = { 1, 5 } } },
+	{ "astrometry", "radius", STYPE_DOUBLE, N_("radius around the target coordinates (degrees)"), &com.pref.astrometry.radius_degrees, { .range_double = { 0.01, 30.0 } } },
+	{ "astrometry", "max_seconds_run", STYPE_INT, N_("maximum seconds to try solving"), &com.pref.astrometry.max_seconds_run, { .range_int = { 0, 100000 } } },
 	{ "astrometry", "update_default_scale", STYPE_BOOL, N_("update default focal length and pixel size from the result"), &com.pref.astrometry.update_default_scale },
-	{ "astrometry", "percent_scale_range", STYPE_INT, N_("percent below and above the expected sampling to allow"), &com.pref.astrometry.percent_scale_range, { .range_int = { 0, 50 } } },
+	{ "astrometry", "percent_scale_range", STYPE_INT, N_("percent below and above the expected sampling to allow"), &com.pref.astrometry.percent_scale_range, { .range_int = { 1, 50 } } },
+	{ "astrometry", "default_obscode", STYPE_STR, N_("default IAU observatory code"), &com.pref.astrometry.default_obscode },
 
 	{ "analysis", "panel", STYPE_INT, N_("panel size of aberration inspector"), &com.pref.analysis.mosaic_panel, { .range_int = { 127, 1024 } } },
 	{ "analysis", "window", STYPE_INT, N_("window size of aberration inspector"), &com.pref.analysis.mosaic_window, { .range_int = { 300, 1600 } } },
@@ -351,12 +460,15 @@ struct settings_access all_settings[] = {
 	{ "gui_prepro", "use_dark_lib", STYPE_BOOL, N_("use default master dark"), &com.pref.prepro.use_dark_lib },
 	{ "gui_prepro", "flat_lib", STYPE_STR, N_("default master flat"), &com.pref.prepro.flat_lib },
 	{ "gui_prepro", "use_flat_lib", STYPE_BOOL, N_("use default master flat"), &com.pref.prepro.use_flat_lib },
+	{ "gui_prepro", "disto_lib", STYPE_STR, N_("default distortion master"), &com.pref.prepro.disto_lib },
+	{ "gui_prepro", "use_disto_lib", STYPE_BOOL, N_("use default master distortion"), &com.pref.prepro.use_disto_lib },
 	{ "gui_prepro", "stack_default", STYPE_STR, N_("default stack name"), &com.pref.prepro.stack_default },
 	{ "gui_prepro", "use_stack_default", STYPE_BOOL, N_("use preferred stack name"), &com.pref.prepro.use_stack_default },
 
 	{ "gui_registration", "method", STYPE_INT, N_("index of the selected registration method"), &com.pref.gui.reg_settings, { .range_int = { 0, 7 } } },
 	{ "gui_registration", "interpolation", STYPE_INT, N_("index of the selected interpolation method"), &com.pref.gui.reg_interpolation, { .range_int = { 0, 5 } } },
 	{ "gui_registration", "clamping", STYPE_BOOL, N_("use clamping method with Lanczos and Cubic interpolation"), &com.pref.gui.reg_clamping },
+	{ "gui_registration", "drizz_weight_match_bitpix", STYPE_BOOL, N_("Match Drizzle weights bitpix to sequence"), &com.pref.drizz_weight_match_bitpix },
 
 	{ "gui_stack", "method", STYPE_INT, N_("index of the selected method"), &com.pref.stack.method, { .range_int = { 0, STACK_MIN } } },
 	{ "gui_stack", "normalization", STYPE_INT, N_("index of the normalization method"), &com.pref.stack.normalisation_method, { .range_int = { 0, MULTIPLICATIVE_SCALING } } },
@@ -384,17 +496,46 @@ struct settings_access all_settings[] = {
 	{ "gui", "font_scale", STYPE_DOUBLE, N_("font scale in percent"), &com.pref.gui.font_scale },
 	{ "gui", "icon_symbolic", STYPE_BOOL, N_("icon style"), &com.pref.gui.icon_symbolic },
 	{ "gui", "script_path", STYPE_STRLIST, N_("list of script directories"), &com.pref.gui.script_path },
-	{ "gui", "warn_script_run", STYPE_BOOL, N_("warn when launching a script"), &com.pref.gui.warn_script_run },
+	{ "gui", "use_scripts_repository", STYPE_BOOL, N_("use and sync online scripts repository"), &com.pref.use_scripts_repository },
+	{ "gui", "use_spcc_repository", STYPE_BOOL, N_("use and sync spcc-database repository"), &com.pref.spcc.use_spcc_repository },
+	{ "gui", "auto_update_scripts", STYPE_BOOL, N_("auto sync online scripts repository"), &com.pref.auto_script_update },
+	{ "gui", "auto_update_spcc", STYPE_BOOL, N_("auto sync spcc-database repository"), &com.pref.spcc.auto_spcc_update },
+	{ "gui", "selected_scripts", STYPE_STRLIST, N_("list of scripts selected from the repository"), &com.pref.selected_scripts },
+	{ "gui", "warn_scripts_run", STYPE_BOOL, N_("warn when launching a script"), &com.pref.gui.warn_scripts_run },
 	{ "gui", "show_thumbnails", STYPE_BOOL, N_("show thumbnails in open dialog"), &com.pref.gui.show_thumbnails },
 	{ "gui", "thumbnail_size", STYPE_INT, N_("size of the thumbnails"), &com.pref.gui.thumbnail_size },
 	{ "gui", "selection_guides", STYPE_INT, N_("number of elements of the grid guides"), &com.pref.gui.selection_guides },
 	{ "gui", "show_deciasec", STYPE_BOOL, N_("show tenths of arcseconds on hover"), &com.pref.gui.show_deciasec },
 	{ "gui", "default_rendering_mode", STYPE_INT, N_("default display mode"), &com.pref.gui.default_rendering_mode, { .range_int = { 0, 6 } } },
 	{ "gui", "display_histogram_mode", STYPE_INT, N_("default histogram display mode"), &com.pref.gui.display_histogram_mode, { .range_int = { 0, 1 } } },
+	{ "gui", "roi_mode", STYPE_INT, N_("ROI selection mode"), &com.pref.gui.roi_mode },
+	{ "gui", "roi_warning", STYPE_BOOL, N_("enable ROI dialog warning"), &com.pref.gui.enable_roi_warning },
 	{ "gui", "mmb_zoom_action", STYPE_INT, N_("Middle mouse button double click zoom action"), &com.pref.gui.mmb_action },
+	{ "gui", "mouse_speed_limit", STYPE_DOUBLE, N_("Mouse smooth scroll speed limit"), &com.pref.gui.mouse_speed_limit },
 	{ "gui", "color_bkg_samples", STYPE_STR, N_("configure background samples color"), &com.pref.gui.config_colors.color_bkg_samples },
 	{ "gui", "color_std_annotations", STYPE_STR, N_("configure standard annotation color"), &com.pref.gui.config_colors.color_std_annotations },
-
+	{ "gui", "color_dso_annotations", STYPE_STR, N_("configure dso annotation color"), &com.pref.gui.config_colors.color_dso_annotations },
+	{ "gui", "color_sso_annotations", STYPE_STR, N_("configure sso annotation color"), &com.pref.gui.config_colors.color_sso_annotations },
+	{ "gui", "color_tmp_annotations", STYPE_STR, N_("configure tmp annotation color"), &com.pref.gui.config_colors.color_tmp_annotations },
+	{ "gui", "custom_monitor_profile", STYPE_STR, N_("path to custom monitor ICC profile"), &com.pref.icc.icc_path_monitor },
+	{ "gui", "soft_proofing_profile", STYPE_STR, N_("path to soft proofing ICC profile"), &com.pref.icc.icc_path_soft_proof },
+	{ "gui", "icc_custom_monitor_active", STYPE_BOOL, N_("custom monitor profile active"), &com.pref.icc.custom_monitor_profile_active },
+	{ "gui", "icc_soft_proofing_active", STYPE_BOOL, N_("output proofing profile active"), &com.pref.icc.soft_proofing_profile_active },
+	{ "gui", "custom_RGB_ICC_profile", STYPE_STR, N_("path to custom RGB ICC profile"), &com.pref.icc.custom_icc_trc },
+	{ "gui", "custom_gray_ICC_profile", STYPE_STR, N_("path to custom gray ICC profile"), &com.pref.icc.custom_icc_gray },
+	{ "gui", "rendering_intent", STYPE_INT, N_("color management rendering intent"), &com.pref.icc.rendering_intent },
+	{ "gui", "proofing_intent", STYPE_INT, N_("color management soft proofing intent"), &com.pref.icc.proofing_intent },
+	{ "gui", "export_intent", STYPE_INT, N_("color management export intent"), &com.pref.icc.export_intent },
+	{ "gui", "default_to_srgb", STYPE_BOOL, N_("default to sRGB when exporting non color managed images"), &com.pref.icc.default_to_srgb },
+	{ "gui", "working_gamut", STYPE_INT, N_("color management working gamut"), &com.pref.icc.working_gamut },
+	{ "gui", "export_8bit_method", STYPE_INT, N_("color management export profile for 8bit files"), &com.pref.icc.export_8bit_method },
+	{ "gui", "export_16bit_method", STYPE_INT, N_("color management export profile for 16bit files"), &com.pref.icc.export_16bit_method },
+	{ "gui", "icc_autoconversion", STYPE_INT, N_("autoconvert images with an ICC profile to the working color space"), &com.pref.icc.autoconversion },
+	{ "gui", "icc_autoassignment", STYPE_INT, N_("encodes ICC profile auto-assignment options"), &com.pref.icc.autoassignment },
+	{ "gui", "icc_rendering_bpc", STYPE_BOOL, N_("enable rendering BPC"), &com.pref.icc.rendering_bpc },
+	{ "gui", "icc_pedantic_linear", STYPE_BOOL, N_("pedantically assign linear ICC profiles"), &com.pref.icc.pedantic_linear },
+	{ "gui", "mouse_actions", STYPE_STRLIST, N_("mouse actions config. Opaque data structure, edit this using the GUI"), &com.pref.gui.mouse_cfg.mouse_actions_array },
+	{ "gui", "scroll_actions", STYPE_STRLIST, N_("scroll actions config. Opaque data structure, edit this using the GUI"), &com.pref.gui.mouse_cfg.scroll_actions_array },
 	{ "gui_astrometry", "compass_position", STYPE_INT, N_("index of the compass position over grid"), &com.pref.gui.position_compass, { .range_int = { 0, 5 } } },
 	{ "gui_astrometry", "cat_messier", STYPE_BOOL, N_("show Messier objects in annotations"), &com.pref.gui.catalog[0] },
 	{ "gui_astrometry", "cat_ngc", STYPE_BOOL, N_("show NGC objects in annotations"), &com.pref.gui.catalog[1] },
@@ -402,10 +543,28 @@ struct settings_access all_settings[] = {
 	{ "gui_astrometry", "cat_ldn", STYPE_BOOL, N_("show LDN objects in annotations"), &com.pref.gui.catalog[3] },
 	{ "gui_astrometry", "cat_sh2", STYPE_BOOL, N_("show SH2 objects in annotations"), &com.pref.gui.catalog[4] },
 	{ "gui_astrometry", "cat_stars", STYPE_BOOL, N_("show stars in annotations"), &com.pref.gui.catalog[5] },
-	{ "gui_astrometry", "cat_user", STYPE_BOOL, N_("show user objects in annotations"), &com.pref.gui.catalog[6] },
+	{ "gui_astrometry", "cat_const", STYPE_BOOL, N_("show constellations in annotations"), &com.pref.gui.catalog[6] },
+	{ "gui_astrometry", "cat_const_names", STYPE_BOOL, N_("show constellations names in annotations"), &com.pref.gui.catalog[7] },
+	{ "gui_astrometry", "cat_user_dso", STYPE_BOOL, N_("show user DSO objects in annotations"), &com.pref.gui.catalog[8] },
+	{ "gui_astrometry", "cat_user_sso", STYPE_BOOL, N_("show user SSO objects in annotations"), &com.pref.gui.catalog[9] },
 
 	{ "gui_pixelmath", "pm_presets", STYPE_STRLIST, N_("list of pixel math presets"), &com.pref.gui.pm_presets },
 
+	{ "script_editor", "highlight_syntax", STYPE_BOOL, N_("highlight syntax in the script editor"), &com.pref.gui.editor_cfg.highlight_syntax },
+	{ "script_editor", "highlight_bracketmatch", STYPE_BOOL, N_("highlight matching brackets in the script editor"), &com.pref.gui.editor_cfg.highlight_bracketmatch },
+	{ "script_editor", "rmargin", STYPE_BOOL, N_("show the right margin in the script editor"), &com.pref.gui.editor_cfg.rmargin },
+	{ "script_editor", "rmargin_pos", STYPE_INT, N_("position of the right margin in the script editor"), &com.pref.gui.editor_cfg.rmargin_pos },
+	{ "script_editor", "show_linenums", STYPE_BOOL, N_("show line numbers in the script editor"), &com.pref.gui.editor_cfg.show_linenums },
+	{ "script_editor", "show_linemarks", STYPE_BOOL, N_("show line marks in the script editor"), &com.pref.gui.editor_cfg.show_linemarks },
+	{ "script_editor", "highlight_currentline", STYPE_BOOL, N_("highlight the current line in the script editor"), &com.pref.gui.editor_cfg.highlight_currentline },
+	{ "script_editor", "autoindent", STYPE_BOOL, N_("automatically indent new lines"), &com.pref.gui.editor_cfg.autoindent },
+	{ "script_editor", "indentontab", STYPE_BOOL, N_("indent selected blocks of lines in the script editor using the tab key"), &com.pref.gui.editor_cfg.indentontab },
+	{ "script_editor", "smartbs", STYPE_BOOL, N_("Smart Backspace behaviour in the script editor"), &com.pref.gui.editor_cfg.smartbs },
+	{ "script_editor", "smarthomeend", STYPE_BOOL, N_("Smart Home / End behaviour in the script editor"), &com.pref.gui.editor_cfg.smarthomeend },
+	{ "script_editor", "showspaces", STYPE_BOOL, N_("Show visible space and tab characters in the script editor"), &com.pref.gui.editor_cfg.showspaces },
+	{ "script_editor", "shownewlines", STYPE_BOOL, N_("Show visible newline characters in the script editor"), &com.pref.gui.editor_cfg.shownewlines },
+	{ "script_editor", "minimap", STYPE_BOOL, N_("Show a minimap in the script editor"), &com.pref.gui.editor_cfg.minimap },
+
 	{ NULL, NULL, STYPE_BOOL, NULL, NULL }
 };
 
@@ -441,11 +600,11 @@ static const char *settings_type_to_string(enum settings_type type) {
 	}
 }
 
-int print_settings_key(const char *group, const char *key, gboolean with_details) {
+gchar* get_settings_key(const char *group, const char *key, gboolean with_details) {
 	struct settings_access *desc = get_key_settings(group, key);
 	if (!desc) {
 		siril_log_message(_("Unknown settings variable %s.%s\n"), group, key);
-		return 1;
+		return NULL;
 	}
 	GString *str = g_string_sized_new(120);
 	g_string_printf(str, "%s.%s = ", desc->group, desc->key);
@@ -482,10 +641,18 @@ int print_settings_key(const char *group, const char *key, gboolean with_details
 			g_string_append_printf(str, " [%g, %g]",
 					desc->range_double.min, desc->range_double.max);
 		g_string_append_printf(str, " (%s)", settings_type_to_string(desc->type));
-		g_string_append_printf(str, ", %s", desc->desc);
+		g_string_append_printf(str, ", %s", _(desc->desc));
 	}
 	gchar *s = g_string_free(str, FALSE);
-	siril_log_message("%s\n", s);
+	return s;
+}
+
+int print_settings_key(const char *group, const char *key, gboolean with_details) {
+	gchar *s = get_settings_key(group, key, with_details);
+	if (s) {
+		siril_log_message("%s\n", s);
+		g_free(s);
+	}
 	return 0;
 }
 
