@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,7 +34,6 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
-#include "core/undo.h"
 
 #include "signals.h"
 
@@ -48,6 +47,11 @@ static void signal_handled(int s) {
 		g_printf("Error, signal %d:\n", s);
 		const gchar *visit = _("Please report this bug to: " PACKAGE_BUGREPORT);
 		switch (s) {
+#ifndef _WIN32
+		case SIGINT: // useful for devs who often use CTRL+C
+			gtk_main_quit();
+			break;
+#endif
 		case SIGSEGV:
 		case SIGFPE:
 		case SIGABRT:
