@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,9 +28,7 @@
 #include "core/processing.h"
 #include "core/arithm.h"
 #include "core/siril_log.h"
-#include "gui/utils.h"
 #include "gui/callbacks.h"
-#include "gui/message_dialog.h"
 #include "gui/image_display.h"
 #include "gui/histogram.h"
 #include "gui/progress_and_log.h"
@@ -195,7 +193,7 @@ int unsharp(fits *fit, double sigma, double amount, gboolean verbose) {
  * sigma value, and when it is given, the entropy will only be computed for
  * pixels with values above background + 1 * sigma. It must be NULL otherwise.
  */
-float entropy(fits *fit, int layer, rectangle *area, imstats *opt_stats) {
+float entropy(fits *fit, int layer, rectangle *area, const imstats *opt_stats) {
 	float e = 0.f;
 	gsl_histogram *histo;
 
@@ -309,12 +307,12 @@ int visu(fits *fit, int low, int high) {
 	gui.hi = high;
 	set_cutoff_sliders_values();
 	redraw(REMAP_ALL);
-	redraw_previews();
+	gui_function(redraw_previews, NULL);
 	return 0;
 }
 
 /* fill an image or selection with the value 'level' */
-int fill(fits *fit, int level, rectangle *arearg) {
+int fill(fits *fit, int level, const rectangle *arearg) {
 	rectangle area;
 
 	if (arearg) {
@@ -456,11 +454,6 @@ double background(fits* fit, int reqlayer, rectangle *selection, threading_type
 	return bg;
 }
 
-void show_FITS_header(fits *fit) {
-	if (fit->header)
-		show_data_dialog(fit->header, "FITS Header", NULL, NULL);
-}
-
 void compute_grey_flat(fits *fit) {
 	float coeff1, coeff2;
 	double mean[36] = {0};
