@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,15 +30,16 @@ static const gchar *siril_share_dir = NULL;
 static const gchar *siril_config_dir = NULL;
 static const gchar *siril_startup_dir = NULL;
 static const gchar *siril_locale_dir = NULL;
+static const gchar *siril_scripts_repo_dir = NULL;
+static const gchar *siril_spcc_repo_dir = NULL;
+static const gchar *siril_user_data_dir = NULL;
 
 /* To set the data dir we are looking for the glade file */
 static void search_for_data_dir() {
-	gchar *path;
-
 	/* First we are looking for in the package_data_dir */
 #ifdef _WIN32 // in the case where the app is started with double click on seq file
 	gchar *execname = g_win32_get_package_installation_directory_of_module(NULL);
-	path = g_build_filename(execname, "share", PACKAGE, NULL);
+	gchar *path = g_build_filename(execname, "share", PACKAGE, NULL);
 	g_free(execname);
 	if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
 		siril_share_dir = g_strdup(path);
@@ -47,7 +48,7 @@ static void search_for_data_dir() {
 #elif defined(ENABLE_RELOCATABLE_RESOURCES) && defined(OS_OSX)
 	const gchar *relocated_path = g_getenv("SIRIL_RELOCATED_RES_DIR");
 	if (relocated_path != NULL) {
-		path = g_build_filename(relocated_path, "share", PACKAGE, NULL);
+		gchar *path = g_build_filename(relocated_path, "share", PACKAGE, NULL);
 		if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
 			siril_share_dir = g_strdup(path);
 		}
@@ -60,37 +61,12 @@ static void search_for_data_dir() {
 		siril_share_dir = g_strdup(g_build_filename(relocated_path, "usr", "share", PACKAGE, NULL));
 	}
 #else
-	path = g_build_filename(PACKAGE_DATA_DIR, NULL);
+	gchar *path = g_build_filename(PACKAGE_DATA_DIR, NULL);
 	if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
 		siril_share_dir = g_strdup(path);
 	}
 	g_free(path);
 #endif
-	/* if not found we are looking for in the common dirs */
-	if (siril_share_dir == NULL) {
-		int i = 0;
-		const gchar *const*system_data_dirs;
-
-		system_data_dirs = g_get_system_data_dirs();
-
-		do {
-			path = g_build_filename(system_data_dirs[i], PACKAGE, NULL);
-			gchar *gladefile = g_build_filename(path, GLADE_FILE, NULL);
-
-			/* data dir is the dir when a glade file is found */
-			if (g_file_test(gladefile, G_FILE_TEST_EXISTS)) {
-				siril_share_dir = g_strdup(path);
-
-				g_free(path);
-				g_free(gladefile);
-				break;
-			}
-			g_free(path);
-			g_free(gladefile);
-
-			i++;
-		} while (system_data_dirs[i] != NULL);
-	}
 }
 
 static void search_for_config_dir() {
@@ -144,6 +120,9 @@ static void search_for_locale_dir() {
 		gchar *path = g_build_filename(LOCALEDIR, NULL);
 		if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
 			siril_locale_dir = g_strdup(path);
+		} else {
+			g_warning("Locale directory %s not found in OSX, using fallback", path);
+			siril_locale_dir = g_strdup("/usr/share/locale");
 		}
 		g_free(path);
 	}
@@ -151,16 +130,42 @@ static void search_for_locale_dir() {
 	const gchar *relocated_path = g_getenv("APPDIR");
 	if (relocated_path != NULL) {
 		siril_locale_dir = g_build_filename(relocated_path, "usr", "share", "locale", NULL);
+	} else {
+		g_warning("APPDIR environment variable not set in AppImage");
+		gchar *path = g_build_filename(LOCALEDIR, NULL);
+		if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
+			siril_locale_dir = g_strdup(path);
+		} else {
+			g_warning("Locale directory %s not found in AppImage, using fallback", path);
+			siril_locale_dir = g_strdup("/usr/share/locale");
+		}
+		g_free(path);
 	}
 #else
 	gchar *path = g_build_filename(LOCALEDIR, NULL);
+
 	if (g_file_test(path, G_FILE_TEST_IS_DIR)) {
 		siril_locale_dir = g_strdup(path);
+	} else {
+		g_warning("Locale directory %s not found, using fallback", path);
+		siril_locale_dir = g_strdup("/usr/share/locale");
 	}
 	g_free(path);
 #endif
 }
 
+static void search_for_scripts_repo_dir() {
+	siril_scripts_repo_dir = g_build_filename(g_get_user_data_dir(), "siril-scripts", NULL);
+}
+
+static void search_for_spcc_repo_dir() {
+	siril_spcc_repo_dir = g_build_filename(g_get_user_data_dir(), "siril-spcc-database", NULL);
+}
+
+static void search_for_user_data_dir() {
+	siril_user_data_dir = g_build_filename(g_get_user_data_dir(), "siril", NULL);
+}
+
 /** Public functions **/
 
 const gchar* siril_get_locale_dir() {
@@ -183,5 +188,20 @@ void initialize_siril_directories() {
 	search_for_locale_dir();
 	search_for_startup_dir();
 	search_for_config_dir();
+	search_for_scripts_repo_dir();
+	search_for_spcc_repo_dir();
+	search_for_user_data_dir();
 }
 
+const gchar* siril_get_scripts_repo_path() {
+	return siril_scripts_repo_dir;
+}
+
+const gchar* siril_get_spcc_repo_path() {
+	return siril_spcc_repo_dir;
+}
+
+const gchar *siril_get_user_data_dir() {
+	return siril_user_data_dir;
+
+}
