@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,5 +25,8 @@ const gchar* siril_get_locale_dir();
 const gchar *siril_get_startup_dir();
 const gchar *siril_get_system_data_dir();
 const gchar* siril_get_config_dir();
+const gchar* siril_get_scripts_repo_path();
+const gchar* siril_get_spcc_repo_path();
+const gchar *siril_get_user_data_dir();
 
 #endif /* SRC_CORE_SIRIL_APP_DIRS_H_ */
