@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -99,6 +99,49 @@ gdouble date_time_to_Julian(GDateTime *dt) {
 	}
 }
 
+/**
+ * Converts a Julian date to GDateTime format
+ * (inverse of date_time_to_Julian function)
+ * @param jd the Julian date (Modified Julian Date referenced to J2000.0)
+ * @return a GDateTime pointer (needs to be freed with g_date_time_unref when no longer needed)
+ */
+GDateTime* Julian_to_date_time(gdouble jd) {
+	if (jd <= 0)
+		return NULL;
+
+	// Using MJD (Modified Julian Date) reference
+	// MJD 0 corresponds to November 17, 1858
+
+	// Create base date: November 17, 1858 (MJD 0)
+	GDateTime *epoch = g_date_time_new_utc(1858, 11, 17, 0, 0, 0);
+	if (!epoch)
+		return NULL;
+
+	// Convert Julian date to seconds since epoch
+	gdouble seconds_since_epoch = jd * 86400.0; // Convert days to seconds
+
+	// Split into seconds and microseconds
+	gint64 full_seconds = (gint64) seconds_since_epoch;
+	gdouble fractional_seconds = seconds_since_epoch - full_seconds;
+	guint microseconds = (guint) (fractional_seconds * 1000000.0);
+
+	// Add full seconds
+	GDateTime *date_with_seconds = g_date_time_add_seconds(epoch, full_seconds);
+	g_date_time_unref(epoch);
+	if (!date_with_seconds)
+		return NULL;
+
+	// Add microseconds if needed
+	if (microseconds > 0) {
+		GDateTime *final_date = g_date_time_add_seconds(date_with_seconds,
+				microseconds / 1000000.0);
+		g_date_time_unref(date_with_seconds);
+		return final_date;
+	}
+
+	return date_with_seconds;
+}
+
 /**
  * Build filename in the iso8601 format
  * @return a newly allocated string formatted in ISO 8601 format
@@ -159,11 +202,12 @@ GDateTime *FITS_date_to_date_time(char *date) {
 	gint year = 0, month = 0, day = 0, hour = 0, min = 0;
 	gdouble sec = 0.0;
 
-	if (date[0] == '\0')
+	if (!date || date[0] == '\0')
 		return NULL;
 
 	if (sscanf(date, "%04d-%02d-%02dT%02d:%02d:%lf", &year, &month, &day, &hour, &min, &sec) != 6) {
-		return NULL;
+		if (sscanf(date, "%04d-%02d-%02dT%02d-%02d-%lf", &year, &month, &day, &hour, &min, &sec) != 6)
+			return NULL;
 	}
 	GTimeZone *tz = g_time_zone_new_utc();
 	GDateTime *new_date = g_date_time_new(tz, year, month, day, hour, min, sec);
@@ -226,7 +270,21 @@ gchar *date_time_to_date(GDateTime *datetime) {
  * @return a newly allocated string formatted as year-month-day_hour-minutes-seconds in numbers.
  */
 gchar *date_time_to_date_time(GDateTime *datetime) {
-	gchar *format = "%Y-%m-%d_%H-%M-%S";
+	gchar *format = "%Y-%m-%dT%H-%M-%S";
 	return g_date_time_format(datetime, format);
 }
 
+// jsecs is the number of seconds since julian day 2450000.5: 00:00:00 UT on October 10, 1995
+GDateTime *julian_sec_to_date(uint32_t jsecs, uint32_t us) {
+	//double day = jsecs / 86400.0 - 2440587.5;
+	GDateTime *JD245 = g_date_time_new_utc(1995, 10, 10, 0, 0, 0.0);
+	GDateTime *date1 = g_date_time_add_seconds(JD245, jsecs);
+	GDateTime *date = g_date_time_add_seconds(date1, us / 1000000.0);
+	g_date_time_unref(JD245);
+	g_date_time_unref(date1);
+	return date;
+}
+
+double timediff_in_s(GDateTime *dt1, GDateTime *dt2) {
+	return (double)g_date_time_difference(dt2, dt1) * 1.e-6;
+}
