@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2025 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
  * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -35,6 +35,10 @@
 #include "core/processing.h"
 #include "gui/progress_and_log.h"
 
+#define STR_INDIR(x) #x 
+#define STR(x) STR_INDIR(x)
+#define SIRIL_USER_AGENT "siril/" STR(SIRIL_MAJOR_VERSION) "." STR(SIRIL_MINOR_VERSION) " (https://gitlab.com/free-astro/siril/)"
+
 // Uncomment the next line for some additional debug printing
 // #define NETWORKING_DEBUG
 
@@ -68,7 +72,7 @@ static CURL* initialize_curl(const gchar *url, struct ucontent *content, HttpReq
 	retval |= curl_easy_setopt(curl, CURLOPT_VERBOSE, 0L);
 	retval |= curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, cbk_curl);
 	retval |= curl_easy_setopt(curl, CURLOPT_WRITEDATA, content);
-	retval |= curl_easy_setopt(curl, CURLOPT_USERAGENT, "siril/0.0");
+	retval |= curl_easy_setopt(curl, CURLOPT_USERAGENT, SIRIL_USER_AGENT);
 	retval |= curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
 	if (request_type == HTTP_POST) {
 		retval |= curl_easy_setopt(curl, CURLOPT_POSTFIELDS, post_data);
@@ -199,7 +203,7 @@ char* fetch_url_range_with_curl(void* curlp, const gchar *url, size_t start, siz
 	retval |= curl_easy_setopt(curl, CURLOPT_VERBOSE, 0L);
 	retval |= curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, cbk_curl);
 	retval |= curl_easy_setopt(curl, CURLOPT_WRITEDATA, &content);
-	retval |= curl_easy_setopt(curl, CURLOPT_USERAGENT, "siril/0.0");
+	retval |= curl_easy_setopt(curl, CURLOPT_USERAGENT, SIRIL_USER_AGENT);
 	retval |= curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
 	retval |= curl_easy_setopt(curl, CURLOPT_RANGE, range_header);
 
@@ -235,7 +239,7 @@ char* fetch_url_range_with_curl(void* curlp, const gchar *url, size_t start, siz
 		long code;
 		curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &code);
 
-		// Accept both 206 (Partial Content) and 200 (OK)
+		// A range response will always be 206. Don't accept 200 because that will be the entire file
 		if (code == 206 || code == 200) {
 			result = content.data;
 			*response_length = content.len;
@@ -264,91 +268,21 @@ char* fetch_url_range_with_curl(void* curlp, const gchar *url, size_t start, siz
 
 char* fetch_url_range(const gchar *url, size_t start, size_t length,
                       gsize *response_length, int *error, gboolean quiet) {
-	*error = 0;
-	*response_length = 0;
-	struct ucontent content = {NULL, 0};
+    CURL *curl = curl_easy_init();
+    if (!curl) {
+        if (!quiet) {
+            siril_log_color_message(_("Error initialising CURL handle.\n"), "red");
+        }
+        if (error) *error = 1;
+        return NULL;
+    }
 
-	CURL *curl = curl_easy_init();
-	if (!curl) {
-		if (!quiet) {
-			siril_log_color_message(_("Error initialising CURL handle for range request.\n"), "red");
-		}
-		*error = 1;
-		return NULL;
-	}
+    char *result = fetch_url_range_with_curl(curl, url, start, length, response_length, error, quiet);
 
-	// Construct the range header
-	gchar *range_header = g_strdup_printf("%zu-%zu", start, start + length - 1);
+    // Clean up the local handle
+    curl_easy_cleanup(curl);
 
-	CURLcode retval;
-	retval = curl_easy_setopt(curl, CURLOPT_URL, url);
-	retval |= curl_easy_setopt(curl, CURLOPT_VERBOSE, 0L);
-	retval |= curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, cbk_curl);
-	retval |= curl_easy_setopt(curl, CURLOPT_WRITEDATA, &content);
-	retval |= curl_easy_setopt(curl, CURLOPT_USERAGENT, "siril/0.0");
-	retval |= curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
-	retval |= curl_easy_setopt(curl, CURLOPT_RANGE, range_header);
-
-	g_free(range_header);
-
-	if (retval) {
-		if (!quiet) {
-			siril_log_color_message(_("Error in curl_easy_setopt() for range request\n"), "red");
-		}
-		curl_easy_cleanup(curl);
-		*error = 1;
-		return NULL;
-	}
-
-	if (g_getenv("CURL_CA_BUNDLE")) {
-		if (curl_easy_setopt(curl, CURLOPT_CAINFO, g_getenv("CURL_CA_BUNDLE"))) {
-			if (!quiet) {
-				siril_log_color_message(_("Error configuring CURL with CA bundle.\n"), "red");
-			}
-		}
-	}
-
-	content.data = calloc(1, 1);
-	if (content.data == NULL) {
-		PRINT_ALLOC_ERR;
-		curl_easy_cleanup(curl);
-		*error = 1;
-		return NULL;
-	}
-
-	CURLcode res = curl_easy_perform(curl);
-	char *result = NULL;
-
-	if (res == CURLE_OK) {
-		long code;
-		curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &code);
-
-		// Accept both 206 (Partial Content) and 200 (OK)
-		if (code == 206 || code == 200) {
-			result = content.data;
-			*response_length = content.len;
-#ifdef NETWORKING_DEBUG
-			siril_debug_print("Retrieved result from %s, length %lu\n", url, content.len);
-#endif
-		} else {
-			if (!quiet) {
-				siril_log_color_message(_("HTTP range request failed with code %ld for URL %s\n"),
-				                       "red", code, url);
-			}
-			free(content.data);
-			*error = 1;
-		}
-	} else {
-		if (!quiet) {
-			siril_log_color_message(_("URL range request failed. libcurl error: %s\n"),
-			                       "red", curl_easy_strerror(res));
-		}
-		free(content.data);
-		*error = 1;
-	}
-
-	curl_easy_cleanup(curl);
-	return result;
+    return result;
 }
 
 int submit_post_request(const char *url, const char *post_data, char **post_response) {
@@ -369,42 +303,74 @@ int submit_post_request(const char *url, const char *post_data, char **post_resp
 	return (res != CURLE_OK ? 1 : 0);
 }
 
+// abort callback
+static size_t cbk_abort(void *ptr, size_t size, size_t nmemb, void *data) {
+	size_t total = size * nmemb;
+	
+	// If it's exactly 1 byte, we assume it's our requested range.
+	// We "consume" it to let the connection close gracefully (CURLE_OK).
+	if (total == 1) {
+		return total;
+	}
+
+	// If it's anything else, it's likely the entire request body so we
+	// return zero to kill the connection immediately (CURLE_WRITE_ERROR).
+	return 0;
+}
+
+/**
+ * Checks if a URL is up and supports Range requests.
+ * Returns:
+ * - >0 (ms) if UP and supports Range (206)
+ * - -2      if UP but NO Range support (200)
+ * - -1      if DOWN or Error
+ */
 int http_check(const gchar *url) {
 	CURL *curl;
 	CURLcode res;
-	int time_ms = -1;
+	int result_val = -1;
 
-	if (!url) {
-		return -1;
-	}
+	if (!url) return -1;
 
 	curl = curl_easy_init();
-	if (!curl) {
+	if (!curl) return -1;
+
+	CURLcode retval;
+	retval = curl_easy_setopt(curl, CURLOPT_URL, url);
+	retval |= curl_easy_setopt(curl, CURLOPT_VERBOSE, 0L);
+	retval |= curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, cbk_abort);
+	retval |= curl_easy_setopt(curl, CURLOPT_USERAGENT, SIRIL_USER_AGENT);
+	retval |= curl_easy_setopt(curl, CURLOPT_RANGE, "0-0");
+	retval |= curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
+	retval |= curl_easy_setopt(curl, CURLOPT_TIMEOUT, 5L);
+	retval |= curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1L);
+
+	if (retval) {
+		siril_debug_print("Error in curl_easy_setopt()\n");
+		curl_easy_cleanup(curl);
 		return -1;
 	}
 
-	curl_easy_setopt(curl, CURLOPT_URL, url);
-	curl_easy_setopt(curl, CURLOPT_NOBODY, 1L);        // HEAD request
-	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 5L);       // 5 second timeout
-	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L); // Follow redirects
-	curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1L);      // Thread-safe
-
 	res = curl_easy_perform(curl);
 
-	if (res == CURLE_OK) {
+	if (res == CURLE_OK || res == CURLE_WRITE_ERROR) {
 		long response_code;
 		curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response_code);
 
-		// Consider 2xx and 3xx status codes as "up"
-		if (response_code >= 200 && response_code < 400) {
-			double total_time;
+		if (response_code == 206) {
+			// Success: Server is up and handled the range request
+			double total_time = 0;
 			curl_easy_getinfo(curl, CURLINFO_TOTAL_TIME, &total_time);
-			time_ms = (int)(total_time * 1000.0);
+			result_val = (int)(total_time * 1000.0);
+		}
+		else if (response_code == 200) {
+			// Server is up, but ignored the Range header
+			result_val = -2;
 		}
 	}
 
 	curl_easy_cleanup(curl);
-	return time_ms;
+	return result_val;
 }
 
 gboolean siril_compiled_with_networking() {
@@ -476,7 +442,7 @@ gboolean siril_compiled_with_networking() {
 }
 
 int http_check(const gchar *url) {
-	retutn -1;
+	return -1;
 }
 
 #endif
