@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,6 +60,14 @@ void siril_world_cs_unref(SirilWorldCS *world_cs) {
 	}
 }
 
+SirilWorldCS *siril_world_cs_copy(SirilWorldCS *world_cs) {
+	g_return_val_if_fail(world_cs != NULL, NULL);
+	SirilWorldCS *world_cs_new = siril_world_cs_alloc();
+	world_cs_new->alpha = world_cs->alpha;
+	world_cs_new->delta = world_cs->delta;
+	return world_cs_new;
+}
+
 SirilWorldCS* siril_world_cs_new_from_a_d(gdouble alpha, gdouble delta) {
 	SirilWorldCS *world_cs;
 	/*
@@ -105,6 +113,8 @@ SirilWorldCS* siril_world_cs_new_from_ra_dec(gdouble ra_h, gdouble ra_m, gdouble
 
 /* parse RA 'hours minutes seconds' or 'hours:minutes:seconds' to degrees */
 double parse_hms(const char *objctra) {
+	if (!objctra)
+		return NAN;
 	double ra = NAN;
 	int ra_h, ra_m;
 	gdouble ra_s;
@@ -116,6 +126,8 @@ double parse_hms(const char *objctra) {
 
 /* parse Dec 'degrees minutes seconds' or 'degrees:minutes:seconds' to degrees */
 double parse_dms(const char *objctdec) {
+	if (!objctdec)
+		return NAN;
 	double dec = NAN;
 	int dec_deg, dec_m;
 	gdouble dec_s;
@@ -199,14 +211,8 @@ static void dec2dms(double dec, int *sign, int *d, int *m, double *s) {
 	*s = rem;
 }
 
-/* Warning: for format with decimal seconds, %lf is required, not %f (this is actually a bug) */
-gchar* siril_world_cs_delta_format(SirilWorldCS *world_cs, const gchar *format) {
-	g_return_val_if_fail(world_cs != NULL, NULL);
+gchar *siril_world_cs_delta_format_from_double(gdouble dec, const gchar *format) {
 	g_return_val_if_fail(format != NULL, NULL);
-	g_return_val_if_fail(g_utf8_validate (format, -1, NULL), NULL);
-
-	gdouble dec = world_cs->delta;
-
 	int degree, min, sign;
 	double sec;
 	dec2dms(dec, &sign, &degree, &min, &sec);
@@ -229,12 +235,17 @@ gchar* siril_world_cs_delta_format(SirilWorldCS *world_cs, const gchar *format)
 }
 
 /* Warning: for format with decimal seconds, %lf is required, not %f (this is actually a bug) */
-gchar* siril_world_cs_alpha_format(SirilWorldCS *world_cs, const gchar *format) {
+gchar *siril_world_cs_delta_format(SirilWorldCS *world_cs, const gchar *format) {
 	g_return_val_if_fail(world_cs != NULL, NULL);
 	g_return_val_if_fail(format != NULL, NULL);
 	g_return_val_if_fail(g_utf8_validate (format, -1, NULL), NULL);
 
-	gdouble ra = world_cs->alpha;
+	return siril_world_cs_delta_format_from_double(world_cs->delta, format);
+}
+
+/* Warning: for format with decimal seconds, %lf is required, not %f (this is actually a bug) */
+gchar *siril_world_cs_alpha_format_from_double(gdouble ra, const gchar *format) {
+	g_return_val_if_fail(format != NULL, NULL);
 
 	int hour, min;
 	double sec;
@@ -257,6 +268,16 @@ gchar* siril_world_cs_alpha_format(SirilWorldCS *world_cs, const gchar *format)
 	return g_strdup_printf(format, hour, min,  new_sec);
 }
 
+
+/* Warning: for format with decimal seconds, %lf is required, not %f (this is actually a bug) */
+gchar *siril_world_cs_alpha_format(SirilWorldCS *world_cs, const gchar *format) {
+	g_return_val_if_fail(world_cs != NULL, NULL);
+	g_return_val_if_fail(format != NULL, NULL);
+	g_return_val_if_fail(g_utf8_validate (format, -1, NULL), NULL);
+
+	return siril_world_cs_alpha_format_from_double(world_cs->alpha, format);
+}
+
 void siril_world_cs_get_ra_hour_min_sec(SirilWorldCS *world_cs, int *hour, int *min, double *sec) {
 	int h, m;
 	gdouble s;
