@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,14 +26,16 @@
 
 #include "core/siril.h"
 #include "core/siril_log.h"
+#include "core/icc_profile.h"
 #include "gui/utils.h"
 #include "gui/callbacks.h"
-#include "gui/dialogs.h"
 #include "gui/image_display.h"
 #include "gui/histogram.h"
 #include "gui/progress_and_log.h"
+#include "gui/siril_preview.h"
 #include "io/single_image.h"
 #include "io/image_format_fits.h"
+#include "io/annotation_catalogues.h"
 #include "core/undo.h"
 #include "core/proto.h"
 #include "algos/statistics.h"
@@ -91,6 +93,8 @@ static int undo_remove_item(historic *histo, int index) {
 	if (histo[index].filename) {
 		if (g_unlink(histo[index].filename))
 			siril_debug_print("g_unlink() failed\n");
+		if (histo[index].icc_profile)
+			cmsCloseProfile(histo[index].icc_profile);
 		g_free(histo[index].filename);
 		histo[index].filename = NULL;
 		memset(&histo[index].wcsdata, 0, sizeof(wcs_info));
@@ -99,7 +103,7 @@ static int undo_remove_item(historic *histo, int index) {
 	return 0;
 }
 
-static void undo_add_item(fits *fit, char *filename, char *histo) {
+static void undo_add_item(fits *fit, char *filename, const char *histo) {
 
 	if (!com.history) {
 		com.hist_size = HISTORY_SIZE;
@@ -112,12 +116,18 @@ static void undo_add_item(fits *fit, char *filename, char *histo) {
 		com.hist_current--;
 		undo_remove_item(com.history, com.hist_current);
 	}
+	int status = -1;
 	com.history[com.hist_current].filename = filename;
 	com.history[com.hist_current].rx = fit->rx;
 	com.history[com.hist_current].ry = fit->ry;
+	com.history[com.hist_current].nchans = fit->naxes[2];
 	com.history[com.hist_current].type = fit->type;
-	com.history[com.hist_current].wcsdata = fit->wcsdata;
-	com.history[com.hist_current].focal_length = fit->focal_length;
+	com.history[com.hist_current].wcsdata = fit->keywords.wcsdata;
+	com.history[com.hist_current].wcslib = wcs_deepcopy(fit->keywords.wcslib, &status);
+	if (status)
+		siril_debug_print("could not copy wcslib struct\n");
+	com.history[com.hist_current].focal_length = fit->keywords.focal_length;
+	com.history[com.hist_current].icc_profile = copyICCProfile(fit->icc_profile);
 	snprintf(com.history[com.hist_current].history, FLEN_VALUE, "%s", histo);
 
 	if (com.hist_current == com.hist_size - 1) {
@@ -175,14 +185,17 @@ static int undo_get_data_ushort(fits *fit, historic *hist) {
 	} else {
 		fit->pdata[GLAYER] = fit->pdata[BLAYER] = fit->pdata[RLAYER];
 	}
-	memcpy(&fit->wcsdata, &hist->wcsdata, sizeof(wcs_info));
-	fit->focal_length = hist->focal_length;
-	if (!has_wcsdata(fit)) {
-		free_wcs(fit, TRUE);
+	memcpy(&fit->keywords.wcsdata, &hist->wcsdata, sizeof(wcs_info));
+	if (hist->wcslib) {
+		int status = -1;
+		fit->keywords.wcslib = wcs_deepcopy(hist->wcslib, &status);
+		if (status)
+			siril_debug_print("could not copy wcslib struct\n");
 	} else {
-		load_WCS_from_memory(fit);
+		free_wcs(fit);
+		reset_wcsdata(fit);
 	}
-
+	fit->keywords.focal_length = hist->focal_length;
 
 	full_stats_invalidation_from_fit(fit);
 	free(buf);
@@ -230,13 +243,17 @@ static int undo_get_data_float(fits *fit, historic *hist) {
 	} else {
 		fit->fpdata[GLAYER] = fit->fpdata[BLAYER] = fit->fpdata[RLAYER];
 	}
-	memcpy(&fit->wcsdata, &hist->wcsdata, sizeof(wcs_info));
-	fit->focal_length = hist->focal_length;
-	if (!has_wcsdata(fit)) {
-		free_wcs(fit, TRUE);
+	memcpy(&fit->keywords.wcsdata, &hist->wcsdata, sizeof(wcs_info));
+	if (hist->wcslib) {
+		int status = -1;
+		fit->keywords.wcslib = wcs_deepcopy(hist->wcslib, &status);
+		if (status)
+			siril_debug_print("could not copy wcslib struct\n");
 	} else {
-		load_WCS_from_memory(fit);
+		free_wcs(fit);
+		reset_wcsdata(fit);
 	}
+	fit->keywords.focal_length = hist->focal_length;
 
 	full_stats_invalidation_from_fit(fit);
 	free(buf);
@@ -245,18 +262,24 @@ static int undo_get_data_float(fits *fit, historic *hist) {
 }
 
 static int undo_get_data(fits *fit, historic *hist) {
+	if (fit->icc_profile)
+		cmsCloseProfile(fit->icc_profile);
+	fit->icc_profile = copyICCProfile(hist->icc_profile);
+	color_manage(fit, (fit->icc_profile != NULL));
+	fits_change_depth(fit, hist->nchans);
+
 	if (hist->type == DATA_USHORT) {
 		if (gfit.type != DATA_USHORT) {
 			size_t ndata = fit->naxes[0] * fit->naxes[1] * fit->naxes[2];
 			fit_replace_buffer(fit, float_buffer_to_ushort(fit->fdata, ndata), DATA_USHORT);
-			set_precision_switch();
+			gui_function(set_precision_switch, NULL);
 		}
 		return undo_get_data_ushort(fit, hist);
 	} else if (hist->type == DATA_FLOAT) {
 		if (gfit.type != DATA_FLOAT) {
 			size_t ndata = fit->naxes[0] * fit->naxes[1] * fit->naxes[2];
 			fit_replace_buffer(fit, ushort_buffer_to_float(fit->data, ndata), DATA_FLOAT);
-			set_precision_switch();
+			gui_function(set_precision_switch, NULL);
 		}
 		return undo_get_data_float(fit, hist);
 	}
@@ -291,7 +314,7 @@ int undo_save_state(fits *fit, const char *message, ...) {
 		undo_add_item(fit, filename, histo);
 
 		/* update menus */
-		update_MenuItem();
+		gui_function(update_MenuItem, NULL);
 	}
 	va_end(args);
 	return 0;
@@ -304,8 +327,16 @@ int undo_display_data(int dir) {
 	switch (dir) {
 	case UNDO:
 		if (is_undo_available()) {
-			/* first we close all liveview dialog to avoid issue, especially with crop */
-			siril_close_preview_dialogs();
+			// Avoid any issues with ROI or preview
+			gboolean preview_was_active = is_preview_active();
+			// Can't reactivate the ROI if the size has changed
+			gboolean roi_was_active = (gui.roi.active && gfit.rx == com.history[com.hist_display - 1].rx
+					&& gfit.ry == com.history[com.hist_display - 1].ry
+					&& gfit.naxes[2] == com.history[com.hist_display - 1].nchans);
+			rectangle roi_rect;
+			memcpy(&roi_rect, &gui.roi.selection, sizeof(rectangle));
+			siril_preview_hide();
+			on_clear_roi();
 			if (com.hist_current == com.hist_display) {
 				undo_save_state(&gfit, NULL);
 				com.hist_display--;
@@ -316,22 +347,63 @@ int undo_display_data(int dir) {
 			invalidate_gfit_histogram();
 			invalidate_stats_from_fit(&gfit);
 			update_gfit_histogram_if_needed();
-			update_MenuItem();
+			gui_function(update_MenuItem, NULL);
+			lock_display_transform();
+			if (gui.icc.proofing_transform)
+				cmsDeleteTransform(gui.icc.proofing_transform);
+			gui.icc.proofing_transform = NULL;
+			unlock_display_transform();
+			refresh_annotations(TRUE);
+			gui_function(close_tab, NULL); // These 2 lines account for possible change from mono to RGB
 			redraw(REMAP_ALL);
+			if (preview_was_active) {
+				copy_gfit_to_backup();
+				siril_log_message(_("Following undo / redo with a preview active you may need "
+						"to toggle the preview off and on again to reactivate the preview effect\n"));
+				// TODO: To be perfect, we would need a register of preview functions
+				// look up the correct one for the open dialog and re-apply the preview
+			}
+			if (roi_was_active) {
+				memcpy(&com.selection, &roi_rect, sizeof(rectangle));
+				on_set_roi();
+			}
+			update_fits_header(&gfit);
 		}
 		break;
 	case REDO:
 		if (is_redo_available()) {
-			/* first we close all liveview dialog to avoid issue, especially with crop */
-			siril_close_preview_dialogs();
+			// Avoid any issues with ROI or preview
+			gboolean preview_was_active = is_preview_active();
+			// Can't reactivate the ROI if the size has changed
+			gboolean roi_was_active = (gui.roi.active && gfit.rx == com.history[com.hist_display + 1].rx
+					&& gfit.ry == com.history[com.hist_display + 1].ry
+					&& gfit.naxes[2] == com.history[com.hist_display + 1].nchans);
+			rectangle roi_rect;
+			memcpy(&roi_rect, &gui.roi.selection, sizeof(rectangle));
+			on_clear_roi();
+			siril_preview_hide();
 			siril_log_message(_("Redo: %s\n"), com.history[com.hist_display].history);
 			com.hist_display++;
 			undo_get_data(&gfit, &com.history[com.hist_display]);
 			invalidate_gfit_histogram();
 			invalidate_stats_from_fit(&gfit);
 			update_gfit_histogram_if_needed();
-			update_MenuItem();
+			gui_function(update_MenuItem, NULL);
+			refresh_annotations(TRUE);
+			lock_display_transform();
+			if (gui.icc.proofing_transform)
+				cmsDeleteTransform(gui.icc.proofing_transform);
+			gui.icc.proofing_transform = NULL;
+			unlock_display_transform();
+			gui_function(close_tab, NULL); // These 2 lines account for possible change from mono to RGB
 			redraw(REMAP_ALL);
+			if (preview_was_active)
+				copy_gfit_to_backup();
+			if (roi_was_active) {
+				memcpy(&gui.roi.selection, &roi_rect, sizeof(rectangle));
+				on_set_roi();
+			}
+			update_fits_header(&gfit);
 		}
 		break;
 	default:
@@ -341,6 +413,16 @@ int undo_display_data(int dir) {
 	return 0;
 }
 
+gboolean undo_in_thread(gpointer user_data) {
+	undo_display_data(UNDO);
+	return FALSE;
+}
+
+gboolean redo_in_thread(gpointer user_data) {
+	undo_display_data(REDO);
+	return FALSE;
+}
+
 int undo_flush() {
 	if (!com.history) {
 		return 1;
