@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,5 +30,7 @@ int undo_display_data(int dir);
 int undo_save_state(fits *fit, const char *message, ...);
 int	undo_flush();
 void set_undo_redo_tooltip();
+gboolean undo_in_thread(gpointer user_data);
+gboolean redo_in_thread(gpointer user_data);
 
 #endif /* UNDO_H_ */
