@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,14 +19,12 @@
  */
 
 #include <string.h>
-#include <opencv2/core/version.hpp>
 
 #include "core/siril.h"
 #include "core/proto.h"
 #include "algos/statistics.h"
 #include "core/processing.h"
 #include "core/undo.h"
-#include "algos/colors.h"
 #include "opencv/opencv.h"
 #include "gui/image_display.h"
 #include "gui/dialogs.h"
@@ -34,7 +32,6 @@
 #include "gui/registration_preview.h"
 #include "gui/utils.h"
 #include "io/single_image.h"
-#include "gui/message_dialog.h"
 #include "gui/siril_preview.h"
 
 #include "clahe.h"
@@ -65,7 +62,7 @@ static void clahe_close(gboolean revert) {
 static int clahe_update_preview() {
 	copy_backup_to_gfit();
 
-	struct CLAHE_data *args = malloc(sizeof(struct CLAHE_data));
+	struct CLAHE_data *args = calloc(1, sizeof(struct CLAHE_data));
 
 	set_cursor_waiting(TRUE);
 
@@ -73,7 +70,8 @@ static int clahe_update_preview() {
 	args->clip = clahe_limit_value;
 	args->tileSize = clahe_tile_size;
 
-	start_in_new_thread(clahe, args);
+	if (!start_in_new_thread(clahe, args))
+		free(args);
 	return 0;
 }
 
@@ -84,9 +82,9 @@ static gboolean end_clahe(gpointer p) {
 
 	free(args);
 
-	adjust_cutoff_from_updated_gfit();
+	notify_gfit_modified();
 	redraw(REMAP_ALL);
-	redraw_previews();
+	gui_function(redraw_previews, NULL);
 	return FALSE;
 }
 
@@ -106,9 +104,10 @@ void apply_clahe_cancel() {
 
 /** callbacks **/
 
-void on_clahe_cancel_clicked(GtkMenuItem *menuitem, gpointer user_data) {
+gboolean on_clahe_cancel_clicked(GtkMenuItem *menuitem, gpointer user_data) {
 	clahe_close(TRUE);
 	siril_close_dialog("CLAHE_dialog");
+	return FALSE;
 }
 
 void on_CLAHE_dialog_close(GtkDialog *dialog, gpointer user_data) {
@@ -116,14 +115,10 @@ void on_CLAHE_dialog_close(GtkDialog *dialog, gpointer user_data) {
 }
 
 void on_clahe_undo_clicked(GtkButton *button, gpointer user_data) {
-	GtkSpinButton *liit_value = GTK_SPIN_BUTTON(lookup_widget("spin_clahe"));
-	GtkSpinButton *tiles_size = GTK_SPIN_BUTTON(lookup_widget("clahe_tiles_size_spin"));
-	clahe_limit_value = 2.0;
-	clahe_tile_size = 8;
-
 	set_notify_block(TRUE);
-	gtk_spin_button_set_value(liit_value, clahe_limit_value);
-	gtk_spin_button_set_value(tiles_size, clahe_tile_size);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spin_clahe")), 2);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("clahe_tiles_size_spin")), 8);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget("clahe_preview")), TRUE);
 	set_notify_block(FALSE);
 
 	copy_backup_to_gfit();
@@ -136,6 +131,8 @@ void on_clahe_undo_clicked(GtkButton *button, gpointer user_data) {
 }
 
 void on_clahe_Apply_clicked(GtkButton *button, gpointer user_data) {
+	if (!check_ok_if_cfa())
+		return;
 	if (clahe_show_preview == FALSE) {
 		update_image *param = malloc(sizeof(update_image));
 		param->update_preview_fn = clahe_update_preview;
@@ -182,7 +179,9 @@ void on_clahe_tiles_size_spin_value_changed(GtkSpinButton *button, gpointer user
 }
 
 void on_clahe_preview_toggled(GtkToggleButton *button, gpointer user_data) {
-	if (clahe_show_preview == TRUE) {
+	cancel_pending_update();
+	clahe_show_preview = gtk_toggle_button_get_active(button);
+	if (!clahe_show_preview) {
 		/* if user click very fast */
 		waiting_for_thread();
 		siril_preview_hide();
@@ -194,6 +193,5 @@ void on_clahe_preview_toggled(GtkToggleButton *button, gpointer user_data) {
 		param->show_preview = TRUE;
 		notify_update((gpointer) param);
 	}
-	clahe_show_preview = !clahe_show_preview;
 }
 
