@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include <glib.h>
 
 #ifdef __cplusplus
-#define EXTERNC extern "C" {
+#define EXTERNC extern "C"
 #else
 #define EXTERNC
 #endif
@@ -97,38 +97,12 @@ EXTERNC typedef struct estk_data {
 	char* savepsf_filename; // for the makepsf -savepsf= option
 	gboolean recalc_ks; // for the makepsf stars option
 	gboolean stars_need_clearing; // for the makepsf stars option
+	gboolean previewing;
 } estk_data;
 
-#ifdef __cplusplus
-}
-#endif
-
 EXTERNC float *estimate_kernel(estk_data *args, int max_threads);
-#ifdef __cplusplus
-}
-#endif
-
 EXTERNC float *gf_estimate_kernel(estk_data *args, int max_threads);
-#ifdef __cplusplus
-}
-#endif
-
 EXTERNC int split_bregman(float *fdata, unsigned rx, unsigned ry, unsigned nchans, float *kernel, int kernelsize, unsigned kchans, float lambda, int iters, int max_threads);
-#ifdef __cplusplus
-}
-#endif
-
-EXTERNC int fft_richardson_lucy(float *fdata, unsigned rx, unsigned ry, unsigned nchans, float *kernel, int kernelsize, unsigned kchans, float lambda, int maxiter, float stopcriterion, int max_threads, int regtype, float stepsize, int stopcriterion_active);
-#ifdef __cplusplus
-}
-#endif
-
-EXTERNC int naive_richardson_lucy(float *fdata, unsigned rx, unsigned ry, unsigned nchans, float *kernel, int kernelsize, unsigned kchans, float lambda, int maxiter, float stopcriterion, int max_threads, int regtype, float stepsize, int stopcriterion_active);
-#ifdef __cplusplus
-}
-#endif
-
+EXTERNC int fft_richardson_lucy(float *fdata, unsigned rx, unsigned ry, unsigned nchans, float *kernel, int kernelsize, unsigned kchans, float lambda, int maxiter, float stopcriterion, int max_threads, regtype_t regtype, float stepsize, int stopcriterion_active);
+EXTERNC int naive_richardson_lucy(float *fdata, unsigned rx, unsigned ry, unsigned nchans, float *kernel, int kernelsize, unsigned kchans, float lambda, int maxiter, float stopcriterion, int max_threads, regtype_t regtype, float stepsize, int stopcriterion_active);
 EXTERNC int wienerdec(float *fdata, unsigned rx, unsigned ry, unsigned nchans, float *kernel, int kernelsize, unsigned kchans, float sigma, int max_threads);
-#ifdef __cplusplus
-}
-#endif
