@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -161,10 +161,10 @@ static void normalisation_spectra_float(unsigned int w, unsigned int h, const fl
 static void save_dft_information_in_gfit(fits *fit) {
 	int i;
 
-	strcpy(gfit.dft.ord, fit->dft.type);
-	strcpy(gfit.dft.ord, fit->dft.ord);
+	strcpy(gfit.keywords.dft.ord, fit->keywords.dft.type);
+	strcpy(gfit.keywords.dft.ord, fit->keywords.dft.ord);
 	for (i = 0; i < fit->naxes[2]; i++)
-		gfit.dft.norm[i] = fit->dft.norm[i];
+		gfit.keywords.dft.norm[i] = fit->keywords.dft.norm[i];
 
 }
 
@@ -210,14 +210,14 @@ static void FFTD_ushort(fits *fit, fits *x, fits *y, int type_order, int layer)
 	//We normalize the modulus and the phase
 	normalisation_spectra_ushort(width, height, modul, phase, xbuf, ybuf, maxi);
 	if (type_order == TYPE_CENTERED) {
-		strcpy(x->dft.ord, "CENTERED");
+		strcpy(x->keywords.dft.ord, "CENTERED");
 		centered_ushort(xbuf, width, height, FFTW_FORWARD);
 		centered_ushort(ybuf, width, height, FFTW_FORWARD);
 	} else {
-		strcpy(x->dft.ord, "REGULAR");
+		strcpy(x->keywords.dft.ord, "REGULAR");
 	}
-	strcpy(y->dft.ord, x->dft.ord);
-	x->dft.norm[layer] = maxi / USHRT_MAX_SINGLE;
+	strcpy(y->keywords.dft.ord, x->keywords.dft.ord);
+	x->keywords.dft.norm[layer] = maxi / USHRT_MAX_SINGLE;
 
 	free(modul);
 	free(phase);
@@ -268,14 +268,14 @@ static void FFTD_float(fits *fit, fits *x, fits *y, int type_order, int layer) {
 	//We normalize the modulus and the phase
 	normalisation_spectra_float(width, height, modul, phase, xbuf, ybuf, maxi);
 	if (type_order == TYPE_CENTERED) {
-		strcpy(x->dft.ord, "CENTERED");
+		strcpy(x->keywords.dft.ord, "CENTERED");
 		centered_float(xbuf, width, height, FFTW_FORWARD);
 		centered_float(ybuf, width, height, FFTW_FORWARD);
 	} else {
-		strcpy(x->dft.ord, "REGULAR");
+		strcpy(x->keywords.dft.ord, "REGULAR");
 	}
-	strcpy(y->dft.ord, x->dft.ord);
-	x->dft.norm[layer] = maxi;
+	strcpy(y->keywords.dft.ord, x->keywords.dft.ord);
+	x->keywords.dft.norm[layer] = maxi;
 
 	free(modul);
 	free(phase);
@@ -309,7 +309,7 @@ static void FFTI_ushort(fits *fit, fits *xfit, fits *yfit, int type_order, int l
 	}
 
 	for (i = 0; i < height * width; i++) {
-		modul[i] = (float) xbuf[i] * (xfit->dft.norm[layer]);
+		modul[i] = (float) xbuf[i] * (xfit->keywords.dft.norm[layer]);
 		phase[i] = (float) ybuf[i] * (2.f * (float)M_PI / USHRT_MAX_SINGLE);
 		phase[i] -= (float)M_PI;
 	}
@@ -368,7 +368,7 @@ static void FFTI_float(fits *fit, fits *xfit, fits *yfit, int type_order, int la
 	}
 
 	for (i = 0; i < height * width; i++) {
-		modul[i] = xbuf[i] * (xfit->dft.norm[layer]);
+		modul[i] = xbuf[i] * (xfit->keywords.dft.norm[layer]);
 		phase[i] = ybuf[i] * (2.f * (float)M_PI);
 		phase[i] -= (float)M_PI;
 	}
@@ -422,10 +422,12 @@ static void FFTI(fits *fit, fits *xfit, fits *yfit, int type_order, int layer) {
 static gboolean end_fourier_transform(gpointer p) {
 	struct fft_data *args = (struct fft_data *)p;
 	stop_processing_thread();
-	adjust_cutoff_from_updated_gfit();
+	notify_gfit_modified();
 	redraw(REMAP_ALL);
-	redraw_previews();
-	free(args->type);
+	gui_function(redraw_previews, NULL);
+	g_free(args->type);
+	g_free(args->modulus);
+	g_free(args->phase);
 	free(args);
 	set_cursor_waiting(FALSE);
 
@@ -446,22 +448,7 @@ gpointer fourier_transform(gpointer p) {
 	fprintf(stdout, "fftwf initialized with %d threads\n", n);
 #endif
 
-// The following switch is no longer used: the overhead generated by expensive
-// planning strategies is not worth it for one-off FFTs
 
-/*	switch (com.pref.fftw_conf.strategy) {
-		case 1:
-			strategy = FFTW_MEASURE;
-			break;
-		case 2:
-			strategy = FFTW_PATIENT;
-			break;
-		case 3:
-			strategy = FFTW_EXHAUSTIVE;
-			break;
-		default:
-			strategy = FFTW_ESTIMATE;
-	}*/
 	strategy = FFTW_ESTIMATE;
 	set_wisdom_file();
     if (fftwf_import_wisdom_from_filename(com.pref.fftw_conf.wisdom_file) == 1) {
@@ -490,12 +477,12 @@ gpointer fourier_transform(gpointer p) {
 
 		for (chan = 0; chan < args->fit->naxes[2]; chan++)
 			FFTD(args->fit, tmp1, tmp2, args->type_order, chan);
-		strcpy(tmp1->dft.type, "SPECTRUM");
+		strcpy(tmp1->keywords.dft.type, "SPECTRUM");
 		if (savefits(args->modulus, tmp1)) {
 			args->retval = 1;
 			goto end;
 		}
-		strcpy(tmp2->dft.type, "PHASE");
+		strcpy(tmp2->keywords.dft.type, "PHASE");
 		if (savefits(args->phase, tmp2)) {
 			args->retval = 1;
 			goto end;
@@ -529,9 +516,9 @@ gpointer fourier_transform(gpointer p) {
 			siril_log_color_message(_("Images must have same dimensions.\n"), "red");
 			goto end;
 		}
-		if (tmp->dft.ord[0] == 'C')		// CENTERED
+		if (tmp->keywords.dft.ord[0] == 'C')		// CENTERED
 			args->type_order = TYPE_CENTERED;
-		else if (tmp->dft.ord[0] == 'R')	// REGULAR
+		else if (tmp->keywords.dft.ord[0] == 'R')	// REGULAR
 			args->type_order = TYPE_REGULAR;
 		else {
 			args->retval = 1;
@@ -570,7 +557,9 @@ end:
 /************************* GUI for FFT ********************************/
 
 void on_button_fft_apply_clicked(GtkButton *button, gpointer user_data) {
-	const char *mag, *phase;
+	if (!check_ok_if_cfa())
+		return;
+	gchar *mag, *phase;
 	char *type = NULL, page;
 	int type_order = -1;
 	static GtkToggleButton *order = NULL;
@@ -609,8 +598,8 @@ void on_button_fft_apply_clicked(GtkButton *button, gpointer user_data) {
 
 		type_order = !gtk_toggle_button_get_active(order);
 		type = strdup("fftd");
-		mag = gtk_entry_get_text(entry_mag);
-		phase = gtk_entry_get_text(entry_phase);
+		mag = g_strdup(gtk_entry_get_text(entry_mag));
+		phase = g_strdup(gtk_entry_get_text(entry_phase));
 	} else {
 		type = strdup("ffti");
 		mag = siril_file_chooser_get_filename(GTK_FILE_CHOOSER(lookup_widget("filechooser_mag")));
@@ -621,6 +610,8 @@ void on_button_fft_apply_clicked(GtkButton *button, gpointer user_data) {
 			siril_message_dialog(GTK_MESSAGE_ERROR, _("Error"), msg);
 			set_cursor_waiting(FALSE);
 			free(type);
+			g_free(mag);
+			g_free(phase);
 			return;
 		}
 		open_single_image(mag);
@@ -628,16 +619,23 @@ void on_button_fft_apply_clicked(GtkButton *button, gpointer user_data) {
 
 	if ((mag != NULL) && (phase != NULL)) {
 		set_cursor_waiting(TRUE);
-		struct fft_data *args = malloc(sizeof(struct fft_data));
+		struct fft_data *args = calloc(1, sizeof(struct fft_data));
 		args->fit = &gfit;
 		args->type = type;
 		args->modulus = mag;
 		args->phase = phase;
 		args->type_order = type_order;
 		set_cursor_waiting(TRUE);
-		start_in_new_thread(fourier_transform, args);
+		if (!start_in_new_thread(fourier_transform, args)) {
+			free(args->type);
+			g_free(args->modulus);
+			g_free(args->phase);
+			free(args);
+		}
 	} else {
 		free(type);
+		g_free(mag);
+		g_free(phase);
 	}
 }
 
@@ -645,3 +643,7 @@ void on_button_fft_close_clicked(GtkButton *button, gpointer user_data) {
 	siril_close_dialog("dialog_FFT");
 }
 
+gboolean fft_hide_on_delete(GtkWidget *widget) {
+	siril_close_dialog("dialog_FFT");
+	return TRUE;
+}
