@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@
 #include "core/siril_log.h"
 #include "algos/statistics.h"
 #include "algos/sorting.h"
+#include "gui/callbacks.h"
+#include "gui/siril_preview.h"
 #include "gui/image_display.h"
 #include "gui/progress_and_log.h"
 #include "gui/registration_preview.h"
@@ -39,11 +41,35 @@
 #include "median.h"
 #include "algos/median_fast.h"
 
-void on_Median_cancel_clicked(GtkButton *button, gpointer user_data) {
+void median_roi_callback() {
+	gui.roi.operation_supports_roi = TRUE;
+	gtk_widget_set_visible(lookup_widget("Median_roi_preview"), gui.roi.active);
+	copy_backup_to_gfit();
+	notify_gfit_modified();
+}
+
+void on_Median_dialog_show(GtkWidget *widget, gpointer user_data) {
+	roi_supported(TRUE);
+	median_roi_callback();
+	add_roi_callback(median_roi_callback);
+	copy_gfit_to_backup();
+}
+
+void median_close() {
+	siril_preview_hide();
+	roi_supported(FALSE);
+	remove_roi_callback(median_roi_callback);
 	siril_close_dialog("Median_dialog");
 }
 
+void on_Median_cancel_clicked(GtkButton *button, gpointer user_data) {
+	median_close();
+}
+
 void on_Median_Apply_clicked(GtkButton *button, gpointer user_data) {
+	if (!check_ok_if_cfa())
+		return;
+
 	int combo_size = gtk_combo_box_get_active(
 			GTK_COMBO_BOX(
 				gtk_builder_get_object(gui.builder, "combo_ksize_median")));
@@ -56,7 +82,9 @@ void on_Median_Apply_clicked(GtkButton *button, gpointer user_data) {
 		return;
 	}
 
-	struct median_filter_data *args = malloc(sizeof(struct median_filter_data));
+	struct median_filter_data *args = calloc(1, sizeof(struct median_filter_data));
+
+	args->previewing = ((GtkWidget*) button == lookup_widget("Median_roi_preview"));
 
 	switch (combo_size) {
 		default:
@@ -82,14 +110,13 @@ void on_Median_Apply_clicked(GtkButton *button, gpointer user_data) {
 			args->ksize = 15;
 			break;
 	}
-	undo_save_state(&gfit, _("Median Filter (filter=%dx%d px)"),
-			args->ksize, args->ksize);
 
-	args->fit = &gfit;
+	args->fit = args->previewing && gui.roi.active ? &gui.roi.fit : &gfit;
 	args->amount = amount;
 	args->iterations = iterations;
 	set_cursor_waiting(TRUE);
-	start_in_new_thread(median_filter, args);
+	if (!start_in_new_thread(median_filter, args))
+		free(args);
 
 }
 
@@ -234,12 +261,15 @@ double get_median_gsl(gsl_matrix *mat, const int xx, const int yy, const int w,
 
 static gboolean end_median_filter(gpointer p) {
 	struct median_filter_data *args = (struct median_filter_data *) p;
-	stop_processing_thread();// can it be done here in case there is no thread?
-	adjust_cutoff_from_updated_gfit();
+	if (!args->previewing) {
+		copy_gfit_to_backup();
+		populate_roi();
+	}
+	stop_processing_thread();
+	notify_gfit_modified();
 	redraw(REMAP_ALL);
-	redraw_previews();
+	gui_function(redraw_previews, NULL);
 	set_cursor_waiting(FALSE);
-
 	free(args);
 	return FALSE;
 }
@@ -763,11 +793,19 @@ static gpointer median_filter_float(gpointer p) {
  * ksize x ksize aperture. Each channel of a multi-channel image is
  * processed independently. In-place operation is supported. */
 gpointer median_filter(gpointer p) {
+	lock_roi_mutex();
 	struct median_filter_data *args = (struct median_filter_data *)p;
+	copy_backup_to_gfit();
+	if (!com.script && !args->previewing)
+		undo_save_state(&gfit, _("Median Filter (filter=%dx%d px)"),
+			args->ksize, args->ksize);
+	gpointer retval = GINT_TO_POINTER(1);
 	if (args->fit->type == DATA_USHORT)
-		return median_filter_ushort(p);
+		retval = median_filter_ushort(p);
 	if (args->fit->type == DATA_FLOAT)
-		return median_filter_float(p);
-	siril_add_idle(end_median_filter, args);
-	return GINT_TO_POINTER(1);
+		retval = median_filter_float(p);
+	unlock_roi_mutex();
+	if (args->fit == &gfit)
+		notify_gfit_modified();
+	return GINT_TO_POINTER(retval);
 }
