@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,15 +21,15 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/undo.h"
+#include "core/icc_profile.h"
 #include "core/processing.h"
 #include "core/siril_log.h"
 #include "core/OS_utils.h"
 #include "algos/colors.h"
-#include "algos/statistics.h"
 #include "io/single_image.h"
-#include "gui/image_display.h"
+#include "gui/callbacks.h"
 #include "gui/progress_and_log.h"
-#include "gui/registration_preview.h"
+#include "gui/siril_preview.h" // For copy_gfit_to_backup() etc
 #include "gui/utils.h"
 #include "gui/histogram.h"
 #include "gui/dialogs.h"
@@ -52,18 +52,28 @@ const char *scnr_type_to_string(scnr_type t) {
 
 /* Subtractive Chromatic Noise Reduction */
 gpointer scnr(gpointer p) {
+	lock_roi_mutex(); // this prevents changes to the ROI occurring while the thread
+	// is running as this could rip out data structures from under us
+
+	// Ensure we are starting with a fresh copy of gfit with no previous
+	// ROI preview data
+	copy_backup_to_gfit();
 	struct scnr_data *args = (struct scnr_data *) p;
 	g_assert(args->fit->type == DATA_USHORT || args->fit->type == DATA_FLOAT);
 	size_t i, nbdata = args->fit->naxes[0] * args->fit->naxes[1];
 	gint nb_above_1 = 0;
 	struct timeval t_start, t_end;
+	gettimeofday(&t_start, NULL);
 	double norm = get_normalized_value(args->fit);
 	double invnorm = 1.0 / norm;
+	// If we aren't being run from a command or ROI previewing, set undo state
+	if (!com.script && !args->previewing)
+		undo_save_state(&gfit, _("SCNR (type=%s, amount=%0.2lf, preserve=%s)"),
+				scnr_type_to_string(args->type), args->amount, args->preserve ? "true" : "false");
 
 	siril_log_color_message(_("SCNR: processing with %s algorithm%s...\n"),
 			"green", scnr_type_to_string(args->type),
 			args->preserve ? _(", preserving lightness") : "");
-	gettimeofday(&t_start, NULL);
 
 	int error = 0;
 #ifdef _OPENMP
@@ -142,15 +152,45 @@ gpointer scnr(gpointer p) {
 	if (args->preserve && nb_above_1)
 		siril_log_message("%d pixels were truncated to a maximum value of 1\n", nb_above_1);
 
+	// If we are not previewing, we now update the backup *and the ROI*
+	// This must be done before notify_gfit_modified() is called or the ROI will
+	// be drawn with the old data
+	if (!args->previewing) {
+		copy_gfit_to_backup();
+		populate_roi();
+	}
+
 	free(args);
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
 
+	unlock_roi_mutex(); // We're done, so unlock the ROI mutex to allow ROI changes to happen again
 	notify_gfit_modified();
 	return GINT_TO_POINTER(error);
 }
 
+void scnr_roi_callback() {
+	gui.roi.operation_supports_roi = TRUE;
+	// Sets the visbility of the ROI preview button
+	gtk_widget_set_visible(lookup_widget("SCNR_roi_preview"), gui.roi.active);
+	// The next 2 lines ensure gfit looks fresh again and there isn't the
+	// remains of an old preview in the wrong place
+	copy_backup_to_gfit();
+	notify_gfit_modified();
+}
+
 void on_SCNR_dialog_show(GtkWidget *widget, gpointer user_data) {
+	// Notify the overlay that this dialog supports ROI processing
+	roi_supported(TRUE);
+	gtk_widget_set_visible(lookup_widget("SCNR_roi_preview"), gui.roi.active);
+	if (gui.roi.active) {
+		copy_gfit_to_backup();
+		// Call this directly on startup to set the ROI preview
+		scnr_roi_callback();
+	}
+	add_roi_callback(scnr_roi_callback);
+	// Set up the backup (this is used to go back to a fresh copy of the
+	// image after abandoning a ROI preview)
 	GtkComboBox *comboscnr = GTK_COMBO_BOX(
 			gtk_builder_get_object(gui.builder, "combo_scnr"));
 	int type = gtk_combo_box_get_active(comboscnr);
@@ -159,6 +199,17 @@ void on_SCNR_dialog_show(GtkWidget *widget, gpointer user_data) {
 		gtk_combo_box_set_active(comboscnr, 0);
 }
 
+void on_SCNR_cancel_clicked(GtkButton *button, gpointer user_data) {
+	// Notify the overlay that we are leaving a dialog that supports ROI
+	roi_supported(FALSE);
+	// Get rid of the gfit backup
+	if (is_preview_active())
+		siril_preview_hide();
+	// Remove the callback
+	remove_roi_callback(scnr_roi_callback);
+	siril_close_dialog("SCNR_dialog");
+}
+
 void on_SCNR_Apply_clicked(GtkButton *button, gpointer user_data) {
 	int type = gtk_combo_box_get_active(
 			GTK_COMBO_BOX(gtk_builder_get_object(gui.builder, "combo_scnr")));
@@ -173,20 +224,21 @@ void on_SCNR_Apply_clicked(GtkButton *button, gpointer user_data) {
 		return;
 	}
 
-	struct scnr_data *args = malloc(sizeof(struct scnr_data));
-	undo_save_state(&gfit, _("SCNR (type=%s, amount=%0.2lf, preserve=%s)"),
-			scnr_type_to_string(type), amount, preserve ? "true" : "false");
+	struct scnr_data *args = calloc(1, sizeof(struct scnr_data));
+	// Tell the threaded function if this is a preview or for real
+	args->previewing = ((GtkWidget*) button == lookup_widget("SCNR_roi_preview"));
+	// undo_save_state(...) // We don't do this here, it has to be done
+	// in the thread after copy_gfit_to_backup
 
-	args->fit = &gfit;
+	// The fit to be operated on depends on if we are previewing with a ROI
+	// active or carrying out the final operation on gfit
+	args->fit = (args->previewing && gui.roi.active) ? &gui.roi.fit : &gfit;
 	args->type = type;
 	args->amount = amount;
 	args->preserve = preserve;
 	set_cursor_waiting(TRUE);
-	start_in_new_thread(scnr, args);
-}
-
-void on_SCNR_cancel_clicked(GtkButton *button, gpointer user_data) {
-	siril_close_dialog("SCNR_dialog");
+	if (!start_in_new_thread(scnr, args))
+		free(args);
 }
 
 void on_combo_scnr_changed(GtkComboBoxText *box, gpointer user_data) {
