@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,39 +28,29 @@
 #include <fcntl.h>
 #include <gio/gwin32inputstream.h>
 #else
-#include <sys/types.h> // for waitpid(2)
-#include <sys/wait.h> // for waitpid(2)
 #include <gio/gunixinputstream.h>
 #endif
 
 #include "core/siril.h"
+#include "core/icc_profile.h"
 #include "core/proto.h"
 #include "core/arithm.h"
-#include "core/undo.h"
 #include "core/processing.h"
 #include "core/OS_utils.h"
 #include "core/siril_log.h"
+#include "core/siril_spawn.h"
 #include "algos/colors.h"
 #include "algos/extraction.h"
-#include "algos/geometry.h"
 #include "algos/statistics.h"
 #include "io/single_image.h"
 #include "io/image_format_fits.h"
 #include "io/sequence.h"
 #include "filters/mtf.h"
-#include "gui/image_display.h"
-#include "gui/image_interactions.h"
 #include "gui/progress_and_log.h"
-#include "gui/registration_preview.h"
 #include "gui/remixer.h"
-#include "gui/utils.h"
-#include "gui/histogram.h"
-#include "gui/dialogs.h"
-#include "gui/siril_preview.h"
 #include "opencv/opencv.h"
 
 #include <unistd.h>
-#include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -69,13 +59,8 @@
 #include "starnet.h"
 
 #ifdef HAVE_LIBTIFF
-fits *current_fit = NULL;
-gboolean verbose = TRUE;
-
-static void child_watch_cb(GPid pid, gint status, gpointer user_data) {
-	siril_debug_print("starnet is being closed\n");
-	g_spawn_close_pid(pid);
-}
+static fits *current_fit = NULL;
+static gboolean verbose = TRUE;
 
 static int exec_prog_starnet(char **argv, starnet_version version) {
 	gint child_stdout;
@@ -89,7 +74,10 @@ static int exec_prog_starnet(char **argv, starnet_version version) {
 	}
 	fprintf(stdout, "\n");
 	// g_spawn handles wchar so not need to convert
-	g_spawn_async_with_pipes(NULL, argv, NULL,
+	if (!get_thread_run()) {
+		return 1;
+	}
+	siril_spawn_host_async_with_pipes(NULL, argv, NULL,
 			G_SPAWN_SEARCH_PATH |
 			G_SPAWN_LEAVE_DESCRIPTORS_OPEN | G_SPAWN_STDERR_TO_DEV_NULL | G_SPAWN_DO_NOT_REAP_CHILD,
 			NULL, NULL, &child_pid, NULL, &child_stdout,
@@ -99,13 +87,17 @@ static int exec_prog_starnet(char **argv, starnet_version version) {
 		siril_log_color_message(_("Spawning starnet failed: %s\n"), "red", error->message);
 		return retval;
 	}
-	g_child_watch_add(child_pid, child_watch_cb, NULL);
-	com.child_is_running = EXT_STARNET;
-#ifdef _WIN32
-	com.childhandle = child_pid;		// For Windows, handle of a child process
-#else
-	com.childpid = child_pid;			// For other OSes, PID of a child process
-#endif
+
+	// At this point, remove the processing thread from the list of children and replace it
+	// with the starnet process. This avoids tracking two children for the same task.
+	if (get_thread_run())
+		remove_child_from_children((GPid) -2);
+
+	// Add the Starnet child to the list of child processes
+	if (!add_child(child_pid, EXT_STARNET, "Starnet")) {
+		siril_log_color_message(_("Error adding Starnet to child process list\n"), "red");
+		return 1;
+	}
 
 	GInputStream *stream = NULL;
 #ifdef _WIN32
@@ -166,7 +158,7 @@ static int exec_prog_starnet(char **argv, starnet_version version) {
 }
 
 starnet_version starnet_executablecheck(gchar* executable) {
-	char *test_argv[3] = {0};
+	char *test_argv[3] = { NULL };
 	int retval = NIL;
 	gint child_stdout;
 	g_autoptr(GError) error = NULL;
@@ -202,8 +194,9 @@ starnet_version starnet_executablecheck(gchar* executable) {
 	if (retval2) {
 		retval = NIL;
 		g_free(dir);
+		g_free(v1dir);
 		g_free(currentdir);
-		return NIL;
+		return retval;
 	}
 	g_free(dir);
 
@@ -212,7 +205,7 @@ starnet_version starnet_executablecheck(gchar* executable) {
 	gchar *versionarg = g_strdup("--version");
 	test_argv[nb++] = versionarg;
 	// g_spawn handles wchar so not need to convert
-	g_spawn_async_with_pipes(NULL, test_argv, NULL,
+	siril_spawn_host_async_with_pipes(NULL, test_argv, NULL,
 			G_SPAWN_SEARCH_PATH |
 			G_SPAWN_LEAVE_DESCRIPTORS_OPEN | G_SPAWN_STDERR_TO_DEV_NULL,
 			NULL, NULL, NULL, NULL, &child_stdout,
@@ -221,6 +214,7 @@ starnet_version starnet_executablecheck(gchar* executable) {
 	if (error != NULL) {
 		siril_log_color_message(_("Spawning starnet failed during version check: %s\n"), "red", error->message);
 		g_free(versionarg);
+		g_free(v1dir);
 		return NIL;
 	}
 
@@ -272,12 +266,11 @@ starnet_version starnet_executablecheck(gchar* executable) {
 
 END:
 	g_free(v1dir);
-
 	return retval;
 }
 
 void free_starnet_args(starnet_data *args) {
-	g_free(args->seqEntry);
+	// do not free multi_args, it is only a reference
 	g_free(args->stride);
 	free(args);
 	siril_debug_print("starnet_args freed\n");
@@ -306,8 +299,8 @@ gpointer do_starnet(gpointer p) {
 	int retval2 = 0;
 	fits workingfit = { 0 }, fit = { 0 };
 	starnet_data *args = (starnet_data *) p;
-	verbose = (args->seq == NULL); // To suppress log messages during seq working
-	args->follow_on = args->seq ? FALSE : args->follow_on;
+	verbose = (args->multi_args == NULL); // To suppress log messages during seq working
+	args->follow_on = args->multi_args ? FALSE : args->follow_on;
 	current_fit = args->starnet_fit;
 	int orig_x = current_fit->rx, orig_y = current_fit->ry;
 	struct remixargs *blendargs = NULL;
@@ -324,6 +317,7 @@ gpointer do_starnet(gpointer p) {
 	gchar *starnetcommand = NULL;
 	gchar *temptif = NULL;
 	gchar *starlesstif = NULL;
+	gchar *starmasktif = NULL;
 	gchar *starlessfit = NULL;
 	gchar *starmaskfit = NULL;
 	gchar *imagenoext = NULL;
@@ -364,10 +358,10 @@ gpointer do_starnet(gpointer p) {
 		if (current_fit->naxes[2] == 3) {
 			starnetcommand = g_strdup(com.pref.starnet_exe);
 		} else {
-			gchar *temp = g_path_get_dirname(com.pref.starnet_exe);
+			gchar *temp2 = g_path_get_dirname(com.pref.starnet_exe);
 			gchar *winext = g_str_has_suffix(com.pref.starnet_exe, ".exe") ? g_strdup(".exe") : g_strdup("\0");
-			starnetcommand = g_strdup_printf("%s/mono_starnet++%s", temp, winext);
-			g_free(temp);
+			starnetcommand = g_strdup_printf("%s/mono_starnet++%s", temp2, winext);
+			g_free(temp2);
 			g_free(winext);
 			if (starnet_executablecheck(starnetcommand) != V1MONO) {
 				siril_log_color_message(_("No suitable StarNet executable found for a mono image.\n"), "red");
@@ -379,10 +373,10 @@ gpointer do_starnet(gpointer p) {
 		if (current_fit->naxes[2] == 1) {
 			starnetcommand = g_strdup(com.pref.starnet_exe);
 		} else {
-			gchar *temp = g_path_get_dirname(com.pref.starnet_exe);
+			gchar *temp2 = g_path_get_dirname(com.pref.starnet_exe);
 			gchar *winext = g_str_has_suffix(com.pref.starnet_exe, ".exe") ? g_strdup(".exe") : g_strdup("\0");
-			starnetcommand = g_strdup_printf("%s/rgb_starnet++%s", temp, winext);
-			g_free(temp);
+			starnetcommand = g_strdup_printf("%s/rgb_starnet++%s", temp2, winext);
+			g_free(temp2);
 			g_free(winext);
 			if (starnet_executablecheck(starnetcommand) != V1RGB) {
 				siril_log_color_message(_("No suitable StarNet executable found for a RGB image.\n"), "red");
@@ -390,7 +384,7 @@ gpointer do_starnet(gpointer p) {
 				goto CLEANUP3;
 			}
 		}
-	} else if (version == V2 || version & TORCH) {
+	} else if (version == V2 || (version & TORCH)) {
 		starnetcommand = g_strdup(com.pref.starnet_exe);
 	}
 
@@ -401,15 +395,14 @@ gpointer do_starnet(gpointer p) {
 		temp = g_path_get_basename(com.uniq->filename);
 		imagenoextorig = g_strdup_printf("%s", temp);
 		g_free(temp);
-	} else if (args->seq) {
-		imagenoextorig = g_strdup_printf("%s%.5d", args->seq->seqname, args->imgnumber + 1);
+	} else if (args->multi_args) {
+		imagenoextorig = g_strdup_printf("%s%.5d", args->multi_args->seq->seqname, args->imgnumber + 1);
 	} else {
 		imagenoextorig = g_strdup_printf("image");
 	}
-	imagenoext = g_strdup(imagenoextorig);
-	starlessnoext = g_strdup_printf("%s%s", starlessprefix, imagenoext);
-	starmasknoext = g_strdup_printf("%s%s", starmaskprefix, imagenoext);
-	imagenoext = g_strdup_printf("%s%s", starnetprefix, imagenoext);
+	starlessnoext = g_strdup_printf("%s%s", starlessprefix, imagenoextorig);
+	starmasknoext = g_strdup_printf("%s%s", starmaskprefix, imagenoextorig);
+	imagenoext = g_strdup_printf("%s%s", starnetprefix, imagenoextorig);
 	temp = g_build_filename(com.wd, imagenoext, NULL);
 	g_free(imagenoext);
 	imagenoext = remove_ext_from_filename(temp);
@@ -432,6 +425,16 @@ gpointer do_starnet(gpointer p) {
 	starlesstif = g_strdup(temp);
 	g_free(temp);
 
+	starmasktif = g_build_filename(com.wd, starmasknoext, NULL);
+	temp = remove_ext_from_filename(starmasktif);
+	g_free(starmasktif);
+	starmasktif = g_strdup(temp);
+	g_free(temp);
+	temp = g_strdup_printf("%s.tif", starmasktif);
+	g_free(starmasktif);
+	starmasktif = g_strdup(temp);
+	g_free(temp);
+
 	if (strlen(starlesstif) > pathmax) {
 		retval = 1;
 		siril_log_color_message(_("Error: file path too long!\n"), "red");
@@ -461,7 +464,15 @@ gpointer do_starnet(gpointer p) {
 	starmaskfit = g_strdup(temp);
 	g_free(temp);
 
-//	printf("%s\n%s\n%s\n%s\n", starlesstif, starlessfit, starmasktif, starmaskfit);
+	/* Copy the ICC profile because starnet is not color managed: the output will
+	 * be the same color space as the imput but will have no ICC profile embedded
+	 * so we have to replace the original */
+	cmsHPROFILE original_profile = copyICCProfile(current_fit->icc_profile);
+	gboolean original_colormanaged = current_fit->color_managed;
+
+	// Disable color management so that savetif doesn't mess about with the image
+	current_fit->color_managed = FALSE;
+
 	// ok, let's start
 	if (verbose)
 		set_progress_bar_data(_("Starting StarNet"), PROGRESS_NONE);
@@ -546,11 +557,13 @@ gpointer do_starnet(gpointer p) {
 	// default shadow clipping and target background. This does marginally clip the
 	// shadows but generally by less than 0.001% of pixels. The result of starnet using
 	// this stretch is much better than either asinh or GHT stretches.
-	struct mtf_params params;
-	params.do_red = TRUE;
-	params.do_green = TRUE;
-	params.do_blue = TRUE;
-	retval = find_linked_midtones_balance_default(&workingfit, &params);
+	struct mtf_params params[3];
+	for (int i = 0 ; i < 3 ; i++) {
+		params[i].do_red = TRUE;
+		params[i].do_green = TRUE;
+		params[i].do_blue = TRUE;
+	}
+	retval = find_unlinked_midtones_balance_default(&workingfit, params);
 	if (retval && args->linear) {
 		siril_log_color_message(_("Error: unable to find the MTF stretch factors...\n"), "red");
 		goto CLEANUP;
@@ -558,7 +571,18 @@ gpointer do_starnet(gpointer p) {
 	if (args->linear) {
 		if (verbose)
 			siril_log_message(_("StarNet: linear mode. Applying Midtone Transfer Function (MTF) pre-stretch to image.\n"));
-		apply_linked_mtf_to_fits(&workingfit, &workingfit, params, TRUE);
+		apply_unlinked_mtf_to_fits(&workingfit, &workingfit, params);
+		siril_log_message(_("StarNet: linear mode. Applying MTF autostretch to StarNet input image.\n"));
+		if  (workingfit.naxes[2] == 1)
+			siril_debug_print("Applying MTF with values %f, %f, %f\n", params[0].shadows, params[0].midtones, params[0].highlights);
+		else
+			siril_debug_print("Applying MTF with values:\n"
+					"  Red:   %f, %f, %f\n"
+					"  Green: %f, %f, %f\n"
+					"  Blue:  %f, %f, %f\n",
+					params[0].shadows, params[0].midtones, params[0].highlights,
+					params[1].shadows, params[1].midtones, params[1].highlights,
+					params[2].shadows, params[2].midtones, params[2].highlights);
 	}
 
 	// Upscale if needed
@@ -613,12 +637,12 @@ gpointer do_starnet(gpointer p) {
 			torcharg_weights = g_strdup_printf("-w %s", com.pref.starnet_weights);
 			my_argv[nb++] = torcharg_weights;
 		}
+		torcharg_mask = g_strdup_printf("-m %s", starmasktif);
+		my_argv[nb++] = torcharg_mask;
 	}
 
 	// *** Call starnet++ *** //
 	retval = exec_prog_starnet(my_argv, version);
-	g_free(starnetcommand);
-	starnetcommand = NULL;
 	if (retval) {
 		siril_log_color_message(_("Error: StarNet did not execute correctly...\n"), "red");
 		goto CLEANUP;
@@ -632,17 +656,25 @@ gpointer do_starnet(gpointer p) {
 		goto CLEANUP;
 	}
 
+	if (workingfit.icc_profile) {
+		cmsCloseProfile(workingfit.icc_profile);
+		workingfit.icc_profile = copyICCProfile(original_profile);
+		workingfit.color_managed = original_colormanaged;
+	}
 	// Remove working TIFF files, they are no longer required
 	retval = g_remove(starlesstif);
+	retval |= (g_remove(starmasktif) && (version & TORCH));
 	retval |= g_remove(temptif);
 	if (retval) {
 		siril_log_color_message(_("Error: unable to remove temporary working file...\n"), "red");
 		siril_log_message(_("Attempting to continue. You will need to clean up the working files manually.\n"));
 	}
 
-	/* we need to copy metadata as they have been removed with readtif     */
+	/* we need to copy metadata as they have been removed with readtif */
 	copy_fits_metadata(current_fit, &workingfit);
 	copy_fits_metadata(current_fit, &fit);
+	workingfit.header = malloc(strlen(current_fit->header)+1);
+	memcpy(workingfit.header, current_fit->header, strlen(current_fit->header)+1);
 
 	// Increase bit depth of starless image to 32 bit to improve precision
 	// for subsequent processing. Only if !force_16bit otherwise there is an error on subtraction
@@ -650,17 +682,17 @@ gpointer do_starnet(gpointer p) {
 	//force_16_bit needs to be generated carefully because of the stacking result corner case
 	gboolean force_16bit = com.pref.force_16bit;
 	if (sequence_is_loaded()) {
-		if ((!(com.seq.current == RESULT_IMAGE || com.seq.current == UNRELATED_IMAGE)) && args->seq && (args->seq->type == SEQ_SER || args->force_ser)) {
+		if ((!(com.seq.current == RESULT_IMAGE || com.seq.current == UNRELATED_IMAGE)) && args->multi_args && args->multi_args->seq && (args->multi_args->seq->type == SEQ_SER || args->force_ser)) {
 			force_16bit = TRUE;
 		}
-	} else if (args->seq) {
-		if (args->seq->type == SEQ_SER || args->force_ser) {
+	} else if (args->multi_args && args->multi_args->seq) {
+		if (args->multi_args->seq->type == SEQ_SER || args->force_ser) {
 			force_16bit = TRUE;
 		}
 	}
 
+	const size_t ndata = workingfit.naxes[0] * workingfit.naxes[1] * workingfit.naxes[2];
 	if (!force_16bit) {
-		const size_t ndata = workingfit.naxes[0] * workingfit.naxes[1] * workingfit.naxes[2];
 		fit_replace_buffer(&workingfit, ushort_buffer_to_float(workingfit.data, ndata), DATA_FLOAT);
 	}
 
@@ -679,8 +711,8 @@ gpointer do_starnet(gpointer p) {
 	// stretch to the starless version and re-save the final result
 	if (args->linear) {
 		if (verbose)
-			siril_log_message(_("StarNet: linear mode. Applying inverse MTF stretch to starless image.\n"));
-		apply_linked_pseudoinverse_mtf_to_fits(&workingfit, &workingfit, params, TRUE);
+			siril_log_message(_("StarNet: linear mode. Applying inverse autostretch to starless image.\n"));
+		apply_unlinked_pseudoinverse_mtf_to_fits(&workingfit, &workingfit, params, TRUE);
 	}
 
 	// Chdir back to the Siril working directory, we don't need to be in the starnet
@@ -693,7 +725,7 @@ gpointer do_starnet(gpointer p) {
 
 	// Save workingfit as starless stretched image fits
 	update_filter_information(&workingfit, "Starless", TRUE);
-	if ((!args->seq) && get_thread_run()) { // sequence worker will handle saving this in the sequence
+	if ((!args->multi_args) && get_thread_run()) { // sequence worker will handle saving this in the sequence
 		retval = savefits(starlessfit, &workingfit);
 		if (retval) {
 			siril_log_color_message(_("Error: unable to save starless image as FITS...\n"), "red");
@@ -703,16 +735,27 @@ gpointer do_starnet(gpointer p) {
 
 	if (args->starmask) {
 		// Subtract starless stretched from original stretched
-		retval = imoper(&fit, &workingfit, OPER_SUB, !force_16bit);
+		//retval = imoper(&fit, &workingfit, OPER_SUB, !force_16bit);
+		// De-screen starless from original
+		retval = descreen(&fit, &workingfit, !force_16bit, com.max_thread);
 		if (retval) {
 			siril_log_color_message(_("Error: image subtraction failed...\n"), "red");
 			goto CLEANUP;
 		}
 		update_filter_information(&fit, "StarMask", TRUE);
 
+		// Replace ICC profile here too
+		if (original_profile) {
+			if (fit.icc_profile)
+				cmsCloseProfile(fit.icc_profile);
+			fit.icc_profile = copyICCProfile(original_profile);
+			fit.color_managed = original_colormanaged;
+			cmsCloseProfile(original_profile);
+		}
+
 		// Save fit as starmask fits
 		if (get_thread_run()) {
-			if ((!args->seq)) {
+			if ((!args->multi_args)) {
 				retval = savefits(starmaskfit, &fit);
 				if (retval) {
 					siril_log_color_message(_("Error: unable to save starmask image as FITS...\n"), "red");
@@ -739,11 +782,15 @@ gpointer do_starnet(gpointer p) {
 		goto CLEANUP;
 	}
 	copy_fits_metadata(&workingfit, current_fit);
+	if (workingfit.header) {
+		current_fit->header = malloc(strlen(workingfit.header)+1);
+		memcpy(current_fit->header, workingfit.header, strlen(workingfit.header)+1);
+	}
 	// Before CLEANUP so that this doesn't print on failure.
 	if (verbose)
 		siril_log_color_message(_("StarNet: job completed.\n"), "green");
 
-	if ((!args->seq)) {
+	if ((!args->multi_args)) {
 		free(com.uniq->filename);
 		com.uniq->filename = strdup(starlessfit);
 		if (args->follow_on) {
@@ -756,12 +803,15 @@ gpointer do_starnet(gpointer p) {
 				goto CLEANUP;
 			}
 			copy_fits_metadata(&workingfit, blendargs->fit1);
+			update_fits_header(blendargs->fit1);
+
 			retval = copyfits(&fit, blendargs->fit2, (CP_ALLOC | CP_COPYA |CP_FORMAT), -1);
 			if (retval) {
 				siril_log_color_message(_("Error: image copy failed...\n"), "red");
 				goto CLEANUP;
 			}
 			copy_fits_metadata(&fit, blendargs->fit2);
+			update_fits_header(blendargs->fit2);
 		}
 	}
 
@@ -780,14 +830,14 @@ gpointer do_starnet(gpointer p) {
 	CLEANUP2:
 	clearfits(&workingfit);
 	CLEANUP3:
-	if (com.child_is_running == EXT_STARNET)
-		com.child_is_running = EXT_NONE;
 	if (verbose)
 		set_progress_bar_data("Ready.", PROGRESS_RESET);
 	g_free(currentdir);
+	g_free(starnetcommand); // command
 	g_free(starlesstif); // filename
 	g_free(starlessfit); // filename
 	g_free(starmaskfit); // filename
+	g_free(starmasktif); // filename
 	g_free(starlessnoext); // part filename
 	g_free(starmasknoext); // part filename
 	g_free(imagenoext); // part filename
@@ -802,7 +852,7 @@ gpointer do_starnet(gpointer p) {
 	gettimeofday(&t_end, NULL);
 	if (verbose)
 		show_time(t_start, t_end);
-	if ((!args->seq)) {
+	if ((!args->multi_args)) {
 		if (args->follow_on) {
 			free_starnet_args(args);
 			if (!(retval)) {
@@ -847,138 +897,70 @@ static int starnet_compute_mem_limits(struct generic_seq_args *args, gboolean fo
 	return limit;
 }
 
-static int starnet_finalize_hook(struct generic_seq_args *args) {
-	struct starnet_data *starnet_args = (struct starnet_data *) args->user;
-	if (starnet_args->too_small)
-		siril_log_color_message(_("Warning: some images in the sequence were smaller than 512x512. Some versions of StarNet may fail to process these images.\n"), "salmon");
-	args->new_ser = starnet_args->new_ser_starless;
-	args->new_fitseq = starnet_args->new_fitseq_starless;
-	int retval = seq_finalize_hook(args);
-	starnet_args->new_ser_starless = NULL;
-	starnet_args->new_fitseq_starless = NULL;
-
-	if (starnet_args->starmask) {
-		args->new_ser = starnet_args->new_ser_starmask;
-		args->new_fitseq = starnet_args->new_fitseq_starmask;
-		retval |= seq_finalize_hook(args);
-		starnet_args->new_ser_starmask = NULL;
-		starnet_args->new_fitseq_starmask = NULL;
-		seqwriter_set_number_of_outputs(1);
-	}
-	free_starnet_args(starnet_args);
-	return retval;
-}
-
-static int starnet_save_hook(struct generic_seq_args *args, int out_index, int in_index, fits *fit) {
-	starnet_data *seqdata = (starnet_data *) args->user;
-	if (!get_thread_run()) {
-		return 1;
-	}
-	int retval1, retval2 = 0;
-	if (args->force_ser_output || args->seq->type == SEQ_SER) {
-		retval1 = ser_write_frame_from_fit(seqdata->new_ser_starless, seqdata->starnet_fit, out_index);
-		if (seqdata->starmask) {
-			retval2 = ser_write_frame_from_fit(seqdata->new_ser_starmask, seqdata->starmask_fit, out_index);
-		}
-		// the two fits are freed by the writing thread
-	} else if (args->force_fitseq_output || args->seq->type == SEQ_FITSEQ) {
-		retval1 = fitseq_write_image(seqdata->new_fitseq_starless, seqdata->starnet_fit, out_index);
-		if (seqdata->starmask) {
-			retval2 = fitseq_write_image(seqdata->new_fitseq_starmask, seqdata->starmask_fit, out_index);
-		}
-	} else {
-		char *dest = fit_sequence_get_image_filename_prefixed(args->seq, "starless_", in_index);
-		retval1 = savefits(dest, seqdata->starnet_fit);
-		free(dest);
-		clearfits(seqdata->starnet_fit);
-		seqdata->starnet_fit = NULL;
-		if (seqdata->starmask) {
-			dest = fit_sequence_get_image_filename_prefixed(args->seq, "starmask_", in_index);
-			retval2 = savefits(dest, seqdata->starmask_fit);
-			free(dest);
-			clearfits(seqdata->starmask_fit);
-			free(seqdata->starmask_fit);
-			seqdata->starmask_fit = NULL;
-		}
-	}
-	return retval1 || retval2;
-}
-
 int starnet_image_hook(struct generic_seq_args *args, int o, int i, fits *fit, rectangle *_, int threads) {
-	int ret = 0;
-	starnet_data *seqdata = (starnet_data *) args->user;
-	seqdata->force_ser = args->force_ser_output;
+	struct multi_output_data *multi_args = (struct multi_output_data*) args->user;
+	starnet_data *seqdata = (starnet_data *) multi_args->user_data;
 	seqdata->starnet_fit = fit;
 	if (seqdata->starmask)
 		seqdata->starmask_fit = calloc(1, sizeof(fits));
-	seqdata->imgnumber = o;
+	seqdata->force_ser = args->force_ser_output;
 	siril_log_color_message(_("Starnet: Processing image %d\n"), "green", o + 1);
-	do_starnet(seqdata);
-	return ret;
-}
-
-static int starnet_basic_prepare_hook(struct generic_seq_args *args) {
-	int retval = seq_prepare_hook(args);
-	if (!retval && args->new_ser) {
-		retval = ser_reset_to_monochrome(args->new_ser);
+	// Call the starnet process
+	int retval = GPOINTER_TO_INT(do_starnet(seqdata));
+	if (!retval) {
+		// Store results in a struct _multi_split
+		struct _multi_split *multi_data = calloc(1, sizeof(struct _multi_split));
+		multi_data->index = o;
+		int nb_out = ((int) seqdata->starmask) + 1;
+		multi_data->images = calloc(nb_out, sizeof(fits*));
+		// We allocate images[0] and move the data from fit. This avoids any clash
+		// between the generic sequence worker which frees fit and the multi_save
+		// hook which frees all the fits in images.
+		multi_data->images[0] = calloc(1, sizeof(fits));
+		copy_fits_metadata(fit, multi_data->images[0]);
+		fits_swap_image_data(fit, multi_data->images[0]);
+		seqdata->starnet_fit = NULL;
+		if (seqdata->starmask) {
+			multi_data->images[1] = seqdata->starmask_fit;
+			seqdata->starmask_fit = NULL;
+		}
+#ifdef _OPENMP
+		omp_set_lock(&args->lock);
+#endif
+		multi_args->processed_images = g_list_append(multi_args->processed_images, multi_data);
+#ifdef _OPENMP
+		omp_unset_lock(&args->lock);
+#endif
+		siril_debug_print("%s: processed images added to the save list (%d)\n", args->description, o);
 	}
 	return retval;
 }
 
-static int starnet_prepare_hook(struct generic_seq_args *args) {
-	struct starnet_data *starnet_args = (struct starnet_data *) args->user;
-	// we call the generic prepare twice with different prefixes
-	args->new_seq_prefix = strdup("starless_");
-	if (starnet_basic_prepare_hook(args))
-		return 1;
-	// but we copy the result between each call
-	starnet_args->new_ser_starless = args->new_ser;
-	starnet_args->new_fitseq_starless = args->new_fitseq;
-	free(args->new_seq_prefix);
-
-	if (starnet_args->starmask) {
-		args->new_seq_prefix = strdup("starmask_");
-		if (starnet_basic_prepare_hook(args))
-			return 1;
-		starnet_args->new_ser_starmask = args->new_ser;
-		starnet_args->new_fitseq_starmask = args->new_fitseq;
-		free(args->new_seq_prefix);
-	}
-	// Set the prefix for the sequence we want loaded afterwards
-	args->new_seq_prefix = strdup("starless_");
-	args->new_ser = NULL;
-	args->new_fitseq = NULL;
-
-	if (starnet_args->starmask)
-		seqwriter_set_number_of_outputs(2);
-	else
-		seqwriter_set_number_of_outputs(1);
-
-	return 0;
-}
-void apply_starnet_to_sequence(struct starnet_data *seqdata) {
-	seqdata->starnet_fit = NULL;
-	struct generic_seq_args *seqargs = create_default_seqargs(seqdata->seq);
-	seqargs->seq = seqdata->seq;
-	seqargs->filtering_criterion = seq_filter_included;
-	seqargs->nb_filtered_images = seqdata->seq->selnum;
-	seqargs->compute_mem_limits_hook = starnet_compute_mem_limits;
-	seqargs->finalize_hook = starnet_finalize_hook;
-	seqargs->save_hook = starnet_save_hook;
-	seqargs->image_hook = starnet_image_hook;
-	seqargs->prepare_hook = starnet_prepare_hook;
+void apply_starnet_to_sequence(struct multi_output_data *multi_args) {
+    struct generic_seq_args *seqargs = create_default_seqargs(multi_args->seq);
 	seqargs->description = _("StarNet");
+	seqargs->seq = multi_args->seq;
+	seqargs->filtering_criterion = seq_filter_included;
+	seqargs->nb_filtered_images = multi_args->seq->selnum;
+	seqargs->parallel = FALSE;
+	seqargs->max_parallel_images = 1;
+	seqargs->compute_mem_limits_hook = starnet_compute_mem_limits;
+	seqargs->prepare_hook = multi_prepare;
+	seqargs->image_hook = starnet_image_hook;
 	seqargs->has_output = TRUE;
+	seqargs->stop_on_error = TRUE;
 	seqargs->output_type = get_data_type(seqargs->seq->bitpix);
-	seqargs->new_seq_prefix = seqdata->seqEntry;
-	seqargs->load_new_sequence = TRUE;
-	seqargs->user = seqdata;
-	const char *ptr = strrchr(seqdata->seq->seqname, G_DIR_SEPARATOR);
-	if (ptr)
-		seqdata->seqname = g_strdup_printf("%s%s%s", seqdata->seqEntry, ptr + 1, com.pref.ext);
-	else seqdata->seqname = g_strdup_printf("%s%s%s", seqargs->new_seq_prefix, seqargs->seq->seqname, com.pref.ext);
+	seqargs->save_hook = multi_save;
+	seqargs->new_seq_prefix = strdup(multi_args->seqEntry);
+	seqargs->finalize_hook = multi_finalize;
+	seqargs->load_new_sequence = (multi_args->new_seq_index < 2);
+	seqargs->user = multi_args;
 	set_progress_bar_data(_("StarNet: Processing..."), 0.);
-	start_in_new_thread(generic_sequence_worker, seqargs);
+	if (!start_in_new_thread(generic_sequence_worker, seqargs)) {
+		free_starnet_args((starnet_data*)multi_args->user_data);
+		free_multi_args(multi_args);
+		free_generic_seq_args(seqargs, TRUE);
+	}
 }
 
 #endif
