@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,10 +34,8 @@
 #include "gui/dialogs.h"
 #include "gui/siril_preview.h"
 #include "io/image_format_fits.h"
-#include "io/single_image.h"
 #include "algos/Def_Wavelet.h"
 #include "wavelets.h"
-#include "core/OS_utils.h"
 
 static float wavelet_value[6];
 static gboolean wavelet_show_preview;
@@ -45,19 +43,15 @@ static gboolean wavelet_show_preview;
 static void reset_scale_w() {
 	static GtkRange *range_w[6] = { NULL, NULL, NULL, NULL, NULL, NULL };
 
-	if (range_w[0] == NULL) {
-		range_w[0] = GTK_RANGE(lookup_widget("scale_w0"));
-		range_w[1] = GTK_RANGE(lookup_widget("scale_w1"));
-		range_w[2] = GTK_RANGE(lookup_widget("scale_w2"));
-		range_w[3] = GTK_RANGE(lookup_widget("scale_w3"));
-		range_w[4] = GTK_RANGE(lookup_widget("scale_w4"));
-		range_w[5] = GTK_RANGE(lookup_widget("scale_w5"));
-	}
-
 	set_notify_block(TRUE);
 	for (int i = 0; i < 6; i++) {
+		char widget_name[10];
+		sprintf(widget_name, "scale_w%d", i);
+		range_w[i] = GTK_RANGE(lookup_widget(widget_name));
 		gtk_range_set_value(range_w[i], 1.f);
 	}
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spinbutton_plans_w")), 6);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget("wavelet_preview")), TRUE);
 	set_notify_block(FALSE);
 }
 
@@ -80,6 +74,7 @@ static int update_wavelets() {
 		}
 		g_free(dir[i]);
 	}
+	redraw(REMAP_ALL);
 	return 0;
 }
 
@@ -204,6 +199,7 @@ void on_wavelets_dialog_hide(GtkWidget *widget, gpointer user_data) {
 }
 
 void on_button_reset_w_clicked(GtkButton *button, gpointer user_data) {
+	gtk_combo_box_set_active(GTK_COMBO_BOX(lookup_widget("combobox_type_w")), 1);
 	reset_scale_w();
 	update_wavelets();
 }
@@ -213,6 +209,7 @@ void apply_wavelets_cancel() {
 		reset_scale_w();
 		update_wavelets();
 	}
+	siril_close_dialog("wavelets_dialog");
 }
 
 void on_button_ok_w_clicked(GtkButton *button, gpointer user_data) {
@@ -228,15 +225,14 @@ void on_button_ok_w_clicked(GtkButton *button, gpointer user_data) {
 	siril_close_dialog("wavelets_dialog");
 }
 
-void on_button_cancel_w_clicked(GtkButton *button, gpointer user_data) {
+gboolean on_button_cancel_w_clicked(GtkButton *button, gpointer user_data) {
 	apply_wavelets_cancel();
-	siril_close_dialog("wavelets_dialog");
+	return FALSE;
 }
 
 void on_button_compute_w_clicked(GtkButton *button, gpointer user_data) {
 	if (wavelet_show_preview) {
 		copy_backup_to_gfit();
-		undo_save_state(get_preview_gfit_backup(), _("Wavelets Transformation"));
 	}
 
 	int Type_Transform, Nbr_Plan, maxplan, mins, i;
@@ -334,12 +330,13 @@ void on_button_extract_w_ok_clicked(GtkButton *button, gpointer user_data) {
 		return;
 	}
 
-	struct wavelets_filter_data *args = malloc(sizeof(struct wavelets_filter_data));
+	struct wavelets_filter_data *args = calloc(1, sizeof(struct wavelets_filter_data));
 
 	args->Type = Type;
 	args->Nbr_Plan = Nbr_Plan;
 	args->fit = &gfit;
-	start_in_new_thread(extract_plans, args);
+	if (!start_in_new_thread(extract_plans, args))
+		free(args);
 }
 
 void on_button_extract_w_close_clicked(GtkButton *button, gpointer user_data) {
@@ -405,8 +402,10 @@ void on_spin_w5_value_changed(GtkSpinButton *button, gpointer user_data) {
 }
 
 void on_wavelet_preview_toggled(GtkToggleButton *button, gpointer user_data) {
-	if (wavelet_show_preview == TRUE) {
-		/* if user click very fast */
+	wavelet_show_preview = gtk_toggle_button_get_active(button);
+	cancel_pending_update();
+	if (!wavelet_show_preview) {
+	/* if user click very fast */
 		waiting_for_thread();
 		siril_preview_hide();
 	} else {
@@ -417,6 +416,5 @@ void on_wavelet_preview_toggled(GtkToggleButton *button, gpointer user_data) {
 		param->show_preview = TRUE;
 		notify_update((gpointer) param);
 	}
-	wavelet_show_preview = !wavelet_show_preview;
 }
 
