@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,6 @@
 #ifndef SRC_GUI_DOCUMENTATION_H_
 #define SRC_GUI_DOCUMENTATION_H_
 
-void siril_get_documentation();
+void siril_get_documentation(const gchar *page_path);
 
 #endif /* SRC_GUI_DOCUMENTATION_H_ */
