@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,37 +22,221 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
-#include "core/command.h"
+#include "core/icc_profile.h"
 #include "core/processing.h"
-#include "core/undo.h"
 #include "core/siril_world_cs.h"
-#include "algos/background_extraction.h"
+#include "algos/demosaicing.h"
 #include "algos/siril_wcs.h"
+#include "algos/photometry.h"
 #include "io/single_image.h"
 #include "io/sequence.h"
-#include "gui/open_dialog.h"
-#include "gui/PSF_list.h"
+#include "io/image_format_fits.h"
 #include "image_interactions.h"
+#include "gui/mouse_action_functions.h"
 #include "image_display.h"
 #include "gui/callbacks.h"
 #include "gui/utils.h"
 #include "progress_and_log.h"
-#include "message_dialog.h"
-#include "registration_preview.h"
+
+//#define DEBUG_SCROLL
+
+/* This filters the modifier keys that are checked (to avoid things like checks
+ * failing because NumLock is pressed). If we want to support additional modifiers
+ * in future (Alt perhaps) they must be added to eventmask. This variable should
+ * not be modified except using set_mouse_event_mask() at GUI startup.
+ */
+static GdkModifierType eventmask;
+
+void set_mouse_event_mask() {
+	eventmask = GDK_SHIFT_MASK | get_primary();
+}
 
 mouse_status_enum mouse_status;
-
-static gboolean double_middle_click_zooms_to_fit = FALSE;
-// Tracks whether double middle click will zoom to fit or zoom to 1:1, for the
-// toggle
-
-// caching widgets
-static GtkWidget *rotation_dlg = NULL;
-
-/* mouse callbacks */
+cut_method cutting;
 static double margin_size = 10;
+static release_action button_release = { 0, mouse_nullfunction };
 
-static gboolean is_over_the_left_side_of_sel(pointi zoomed, double zoom) {
+mouse_status_enum get_mouse_status() {
+	return mouse_status;
+}
+
+#define MAX_CALLBACKS_PER_EVENT 10
+/* selection zone event management */
+static selection_update_callback _registered_selection_callbacks[MAX_CALLBACKS_PER_EVENT];
+static int _nb_selection_callbacks = 0;
+
+mouse_action* create_mouse_action(guint button, GdkEventType type, GdkModifierType state, const mouse_function_metadata *metadata) {
+	mouse_action* action = (mouse_action*) malloc(sizeof(mouse_action));
+	action->button = button;
+	action->type = type;
+	action->state = state;
+	action->data = metadata;
+	return action;
+}
+
+scroll_action* create_scroll_action(GdkModifierType state, const scroll_function_metadata *metadata, SirilScrollDirection direction) {
+	scroll_action* action = (scroll_action*)malloc(sizeof(scroll_action));
+	action->direction = direction;
+	action->state = state;
+	action->data = metadata;
+	return action;
+}
+
+void initialize_mouse_actions() {
+	gui.mouse_actions = g_slist_prepend(gui.mouse_actions, create_mouse_action(GDK_BUTTON_PRIMARY, GDK_BUTTON_PRESS, 0, &main_action));
+	gui.mouse_actions = g_slist_prepend(gui.mouse_actions, create_mouse_action(GDK_BUTTON_PRIMARY, GDK_BUTTON_PRESS, get_primary(), &drag_action));
+	gui.mouse_actions = g_slist_prepend(gui.mouse_actions, create_mouse_action(GDK_BUTTON_PRIMARY, GDK_BUTTON_PRESS, get_primary() | GDK_SHIFT_MASK, &measure_action));
+	gui.mouse_actions = g_slist_prepend(gui.mouse_actions, create_mouse_action(GDK_BUTTON_PRIMARY, GDK_DOUBLE_BUTTON_PRESS, 0, &open_if_unloaded_action));
+	gui.mouse_actions = g_slist_prepend(gui.mouse_actions, create_mouse_action(GDK_BUTTON_MIDDLE, GDK_BUTTON_PRESS, 0, &drag_action));
+	gui.mouse_actions = g_slist_prepend(gui.mouse_actions, create_mouse_action(GDK_BUTTON_MIDDLE, GDK_BUTTON_PRESS, get_primary(), &photometry_box_action));
+	gui.mouse_actions = g_slist_prepend(gui.mouse_actions, create_mouse_action(GDK_BUTTON_MIDDLE, GDK_DOUBLE_BUTTON_PRESS, 0, &zoom_action));
+	gui.mouse_actions = g_slist_prepend(gui.mouse_actions, create_mouse_action(GDK_BUTTON_SECONDARY, GDK_BUTTON_PRESS, 0, &second_action));
+	gui.mouse_actions = g_slist_reverse(gui.mouse_actions);
+}
+
+void load_or_initialize_mouse_actions() {
+	if (com.pref.gui.mouse_cfg.mouse_actions_array) {
+		gui.mouse_actions = mouse_actions_config_to_list(com.pref.gui.mouse_cfg.mouse_actions_array);
+	} else {
+		initialize_mouse_actions();
+	}
+}
+
+void initialize_scroll_actions() {
+	gui.scroll_actions = g_slist_prepend(gui.scroll_actions, create_scroll_action(0, &scroll_zooms_action, MOUSE_VERTICAL_SCROLL));
+	gui.scroll_actions = g_slist_prepend(gui.scroll_actions, create_scroll_action(GDK_SHIFT_MASK, &scroll_traverses_seq_action, MOUSE_VERTICAL_SCROLL));
+	gui.scroll_actions = g_slist_reverse(gui.scroll_actions);
+}
+
+void load_or_initialize_scroll_actions() {
+	if (com.pref.gui.mouse_cfg.scroll_actions_array) {
+		gui.scroll_actions = scroll_actions_config_to_list(com.pref.gui.mouse_cfg.scroll_actions_array);
+	} else {
+		initialize_scroll_actions();
+	}
+}
+
+static inline gboolean mouse_action_match_to_event(GdkEventButton *event, mouse_action *action) {
+	return (action->button == event->button && action->type == event->type && action->state == (event->state & eventmask));
+}
+
+static gboolean scroll_action_match_to_event(GdkEventScroll *event, scroll_action *action) {
+	point delta;
+	gdk_event_get_scroll_deltas((GdkEvent*) event, &delta.x, &delta.y);
+	GdkModifierType filtered_event_state = event->state & eventmask;
+	SirilScrollDirection eventScrollDirection;
+	if (event->direction == GDK_SCROLL_DOWN || event->direction == GDK_SCROLL_UP)
+		eventScrollDirection = MOUSE_VERTICAL_SCROLL;
+	else if (event->direction == GDK_SCROLL_LEFT || event->direction == GDK_SCROLL_RIGHT)
+		eventScrollDirection = MOUSE_HORIZ_SCROLL;
+	else eventScrollDirection = MOUSE_SMOOTH_SCROLL;
+
+#ifdef DEBUG_SCROLL
+	siril_debug_print("Event direction: %u (raw direction: %u), delta x: %f, delta y: %f, modifiers: %u\n", eventScrollDirection, event->direction, delta.x, delta.y, event->state);
+	siril_debug_print("Action drection: %u, modifiers: %u\n", action->direction, action->state);
+#endif
+
+	if ((action->state == filtered_event_state) && (action->direction == eventScrollDirection)) {
+		return TRUE;
+	} else if (action->state == filtered_event_state && event->direction == GDK_SCROLL_SMOOTH) {
+		if (action->direction == MOUSE_HORIZ_SCROLL && delta.x != 0.0) {
+			return TRUE;
+		} else if (action->direction == MOUSE_VERTICAL_SCROLL && delta.y != 0.0) {
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
+void register_release_callback(mouse_function function, guint button) {
+	button_release.function = function;
+	button_release.button = button;
+}
+
+void clear_release_callback() {
+	button_release.function = mouse_nullfunction;
+	button_release.button = 0;
+}
+
+void register_selection_update_callback(selection_update_callback f) {
+	if (_nb_selection_callbacks < MAX_CALLBACKS_PER_EVENT) {
+		_registered_selection_callbacks[_nb_selection_callbacks] = f;
+		_nb_selection_callbacks++;
+	}
+}
+
+void unregister_selection_update_callback(const selection_update_callback f) {
+	int i;
+	for (i = 0; i < _nb_selection_callbacks; ++i) {
+		if (_registered_selection_callbacks[i] == f) {
+			_registered_selection_callbacks[i] =
+				_registered_selection_callbacks[_nb_selection_callbacks];
+			_registered_selection_callbacks[_nb_selection_callbacks] = NULL;
+			_nb_selection_callbacks--;
+			return;
+		}
+	}
+}
+
+// send the events
+gboolean new_selection_zone(gpointer user_data) {
+	int i;
+	siril_debug_print("selection: %d,%d,\t%dx%d\n", com.selection.x,
+			com.selection.y, com.selection.w, com.selection.h);
+	for (i = 0; i < _nb_selection_callbacks; ++i) {
+		if (_registered_selection_callbacks[i])
+			_registered_selection_callbacks[i]();
+	}
+	redraw(REDRAW_OVERLAY);
+	return FALSE;
+}
+
+void delete_selected_area() {
+	memset(&com.selection, 0, sizeof(rectangle));
+	if (!com.script)
+		gui_function(new_selection_zone, NULL);
+	if (gui.roi.active && com.pref.gui.roi_mode == ROI_AUTO)
+		on_clear_roi();
+}
+
+void reset_display_offset() {
+	siril_debug_print("resetting display offset\n");
+	gui.display_offset.x = 0;
+	gui.display_offset.y = 0;
+}
+
+void reset_menu_toggle_button() {
+	GtkApplicationWindow *app_win = GTK_APPLICATION_WINDOW(lookup_widget("control_window"));
+	GAction *action_tilt = g_action_map_lookup_action(G_ACTION_MAP(app_win), "show-tilt");
+	GAction *action_disto = g_action_map_lookup_action(G_ACTION_MAP(app_win), "show-disto");
+
+	GVariant *state = g_action_get_state(action_tilt);
+	if (g_variant_get_boolean(g_action_get_state(action_tilt))) {
+		g_action_change_state(action_tilt, g_variant_new_boolean(FALSE));
+	}
+	g_variant_unref(state);
+	state = g_action_get_state(action_disto);
+	if (g_variant_get_boolean(g_action_get_state(action_disto))) {
+		g_action_change_state(action_disto, g_variant_new_boolean(FALSE));
+	}
+	g_variant_unref(state);
+}
+
+void reset_zoom_default() {
+	gui.zoom_value = ZOOM_DEFAULT;
+	if (gui.zoom_value == ZOOM_FIT && !com.script) {
+		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(lookup_widget("zoom_to_fit_check_button")), TRUE);
+	}
+}
+
+void update_zoom_fit_button() {
+	GtkToggleToolButton *button = GTK_TOGGLE_TOOL_BUTTON(lookup_widget("zoom_to_fit_check_button"));
+	if (gtk_toggle_tool_button_get_active(button)) {
+		gtk_toggle_tool_button_set_active(button, FALSE);
+	}
+}
+
+gboolean is_over_the_left_side_of_sel(pointi zoomed, double zoom) {
 	if (com.selection.w == 0 && com.selection.h == 0) return FALSE;
 	int s = round_to_int(margin_size / zoom);
 	if (zoomed.x > com.selection.x - s && zoomed.x < com.selection.x + s) {
@@ -63,7 +247,7 @@ static gboolean is_over_the_left_side_of_sel(pointi zoomed, double zoom) {
 	return FALSE;
 }
 
-static gboolean is_over_the_right_side_of_sel(pointi zoomed, double zoom) {
+gboolean is_over_the_right_side_of_sel(pointi zoomed, double zoom) {
 	if (com.selection.w == 0 && com.selection.h == 0) return FALSE;
 	int s = round_to_int(margin_size / zoom);
 	if (zoomed.x > com.selection.x + com.selection.w - s
@@ -75,7 +259,7 @@ static gboolean is_over_the_right_side_of_sel(pointi zoomed, double zoom) {
 	return FALSE;
 }
 
-static gboolean is_over_the_bottom_of_sel(pointi zoomed, double zoom) {
+gboolean is_over_the_bottom_of_sel(pointi zoomed, double zoom) {
 	if (com.selection.w == 0 && com.selection.h == 0) return FALSE;
 	int s = round_to_int(margin_size / zoom);
 	if (zoomed.y > com.selection.y + com.selection.h - s
@@ -87,7 +271,7 @@ static gboolean is_over_the_bottom_of_sel(pointi zoomed, double zoom) {
 	return FALSE;
 }
 
-static gboolean is_over_the_top_of_sel(pointi zoomed, double zoom) {
+gboolean is_over_the_top_of_sel(pointi zoomed, double zoom) {
 	if (com.selection.w == 0 && com.selection.h == 0) return FALSE;
 	int s = round_to_int(margin_size / zoom);
 	if (zoomed.y > com.selection.y - s && zoomed.y < com.selection.y + s) {
@@ -98,7 +282,7 @@ static gboolean is_over_the_top_of_sel(pointi zoomed, double zoom) {
 	return FALSE;
 }
 
-static gboolean is_inside_of_sel(pointi zoomed, double zoom) {
+gboolean is_inside_of_sel(pointi zoomed, double zoom) {
 	if (com.selection.w == 0 && com.selection.h == 0) return FALSE;
 	int s = round_to_int(margin_size / zoom);
 	if (zoomed.x >= com.selection.x + s && zoomed.x <= com.selection.x + com.selection.w - s) {
@@ -132,158 +316,19 @@ static gboolean clamp2image(pointi* pt) {
 	return x_inside && y_inside;
 }
 
-#define MAX_CALLBACKS_PER_EVENT 10
-/* selection zone event management */
-static selection_update_callback _registered_selection_callbacks[MAX_CALLBACKS_PER_EVENT];
-static int _nb_selection_callbacks = 0;
-
-void register_selection_update_callback(selection_update_callback f) {
-	if (_nb_selection_callbacks < MAX_CALLBACKS_PER_EVENT) {
-		_registered_selection_callbacks[_nb_selection_callbacks] = f;
-		_nb_selection_callbacks++;
-	}
-}
-
-void unregister_selection_update_callback(const selection_update_callback f) {
-	int i;
-	for (i = 0; i < _nb_selection_callbacks; ++i) {
-		if (_registered_selection_callbacks[i] == f) {
-			_registered_selection_callbacks[i] =
-				_registered_selection_callbacks[_nb_selection_callbacks];
-			_registered_selection_callbacks[_nb_selection_callbacks] = NULL;
-			_nb_selection_callbacks--;
-			return;
-		}
-	}
-}
-
-// send the events
-void new_selection_zone() {
-	int i;
-	siril_debug_print("selection: %d,%d,\t%dx%d\n", com.selection.x,
-			com.selection.y, com.selection.w, com.selection.h);
-	for (i = 0; i < _nb_selection_callbacks; ++i) {
-		_registered_selection_callbacks[i]();
-	}
-	redraw(REDRAW_OVERLAY);
-}
-
-
-void delete_selected_area() {
-	memset(&com.selection, 0, sizeof(rectangle));
-	if (!com.script)
-		new_selection_zone();
-}
-
-void reset_display_offset() {
-	siril_debug_print("resetting display offset\n");
-	gui.display_offset.x = 0;
-	gui.display_offset.y = 0;
-}
-
-void reset_zoom_default() {
-	gui.zoom_value = ZOOM_DEFAULT;
-	if (gui.zoom_value == ZOOM_FIT && !com.script) {
-		gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(lookup_widget("zoom_to_fit_check_button")), TRUE);
-	}
-}
-
-void update_zoom_fit_button() {
-	GtkToggleToolButton *button = GTK_TOGGLE_TOOL_BUTTON(lookup_widget("zoom_to_fit_check_button"));
-	if (gtk_toggle_tool_button_get_active(button)) {
-		gtk_toggle_tool_button_set_active(button, FALSE);
-	}
-}
-
 /*
  * Button events
  */
 
-//static void do_popup_rgbmenu(GtkWidget *my_widget, GdkEventButton *event) {
-//	static GtkMenu *menu = NULL;
-//
-//	if (!menu) {
-//		menu = GTK_MENU(gtk_builder_get_object(gui.builder, "menurgb"));
-//		gtk_menu_attach_to_widget(GTK_MENU(menu), my_widget, NULL);
-//	}
-//
-//#if GTK_CHECK_VERSION(3, 22, 0)
-//	gtk_menu_popup_at_pointer(GTK_MENU(menu), NULL);
-//#else
-//	int button, event_time;
-//
-//	if (event) {
-//		button = event->button;
-//		event_time = event->time;
-//	} else {
-//		button = 0;
-//		event_time = gtk_get_current_event_time();
-//	}
-//
-//	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button,
-//			event_time);
-//#endif
-//}
-
-/* Gray popup menu */
-
-static void do_popup_graymenu(GtkWidget *my_widget, GdkEventButton *event) {
-	static GtkMenu *menu = NULL;
-
-	gboolean is_a_single_image_loaded = single_image_is_loaded() && (!sequence_is_loaded()
-			|| (sequence_is_loaded() && (com.seq.current == RESULT_IMAGE
-					|| com.seq.current == SCALED_IMAGE)));
-
-	if (!menu) {
-		menu = GTK_MENU(gtk_builder_get_object(gui.builder, "menugray"));
-		gtk_menu_attach_to_widget(GTK_MENU(menu), my_widget, NULL);
-	}
-
-	// selection submenu
-	double original_ratio = (double)gfit.rx / (double)gfit.ry;
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_free")), gui.ratio == 0.0);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_preserve")), gui.ratio == original_ratio);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_16_9")), gui.ratio == 16.0 / 9.0);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_4_3")), gui.ratio == 4.0 / 3.0);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_3_2")), gui.ratio == 3.0 / 2.0);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_1_1")), gui.ratio == 1.0 / 1.0);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_3_4")), gui.ratio == 3.0 / 4.0);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_2_3")), gui.ratio == 2.0 / 3.0);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_9_16")), gui.ratio == 9.0 / 16.0);
-	gtk_widget_set_sensitive(lookup_widget("menuitem_selection_preserve"), is_a_single_image_loaded || sequence_is_loaded());
-	gtk_widget_set_sensitive(lookup_widget("menuitem_selection_all"), is_a_single_image_loaded || sequence_is_loaded());
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_guides_0")), com.pref.gui.selection_guides == 0);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_guides_2")), com.pref.gui.selection_guides == 2);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_guides_3")), com.pref.gui.selection_guides == 3);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_guides_5")), com.pref.gui.selection_guides == 5);
-
-#if GTK_CHECK_VERSION(3, 22, 0)
-	gtk_menu_popup_at_pointer(GTK_MENU(menu), NULL);
-#else
-	int button, event_time;
-
-	if (event) {
-		button = event->button;
-		event_time = event->time;
-	} else {
-		button = 0;
-		event_time = gtk_get_current_event_time();
-	}
-
-	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, event_time);
-#endif
-}
-
-//gboolean rgb_area_popup_menu_handler(GtkWidget *widget) {
-//	do_popup_rgbmenu(widget, NULL);
-//	return TRUE;
-//}
-
 void init_mouse() {
 	mouse_status = MOUSE_ACTION_SELECT_REG_AREA;
 }
 
-static GdkModifierType get_primary() {
+void init_draw_poly() {
+	mouse_status = MOUSE_ACTION_DRAW_POLY;
+}
+
+GdkModifierType get_primary() {
 	return gdk_keymap_get_modifier_mask(
 			gdk_keymap_get_for_display(gdk_display_get_default()),
 			GDK_MODIFIER_INTENT_PRIMARY_ACCELERATOR);
@@ -320,195 +365,64 @@ void enforce_ratio_and_clamp() {
 	// clamp the selection inside the image (needed when enforcing a ratio or moving)
 	com.selection.x = set_int_in_interval(com.selection.x, 0, gfit.rx - com.selection.w);
 	com.selection.y = set_int_in_interval(com.selection.y, 0, gfit.ry - com.selection.h);
+
+	// If the image is CFA ensure the selection is aligned to a Bayer repeat
+	// This ensures CFA statistics (for Bayer patterns) will be valid
+	int cfa = get_cfa_pattern_index_from_string(gfit.keywords.bayer_pattern);
+	switch (cfa) {
+		case BAYER_FILTER_NONE:
+			break;
+		case BAYER_FILTER_RGGB: // Fallthrough intentional
+		case BAYER_FILTER_BGGR:
+		case BAYER_FILTER_GBRG:
+		case BAYER_FILTER_GRBG:
+			// Bayer (2x2 guarantees at least 1 pixel per subchannel for statistics)
+			if(com.selection.w < 2)
+				com.selection.w = 2;
+			if (com.selection.h < 2)
+				com.selection.h = 2;
+			break;
+		default: // X-trans (3x3 guarantees at least 1 pixel per subchannel for statistics)
+			if(com.selection.w < 3)
+				com.selection.w = 3;
+			if (com.selection.h < 3)
+				com.selection.h = 3;
+	}
+	if (cfa != BAYER_FILTER_NONE) {
+		com.selection.x &= ~1;
+		com.selection.y &= ~1;
+		com.selection.w &= ~1;
+		com.selection.h &= ~1;
+		if(com.selection.w < 2)
+			com.selection.w = 2;
+		if (com.selection.h < 2)
+			com.selection.h = 2;
+	}
 }
 
 gboolean on_drawingarea_button_press_event(GtkWidget *widget,
 		GdkEventButton *event, gpointer user_data) {
 
-	if (!rotation_dlg) {
-		rotation_dlg = lookup_widget("rotation_dialog");
-	}
-	/* when double clicking on drawing area (if no images loaded)
-	 * you can load an image This feature is in GIMP and I really
-	 * love it: lazy world :).
-	 */
-	if (!single_image_is_loaded() && !sequence_is_loaded()) {
-		if (event->button == GDK_BUTTON_PRIMARY
-				&& event->type == GDK_DOUBLE_BUTTON_PRESS) {
-			header_open_button_clicked();
-			launch_clipboard_survey();
-		}
-		return FALSE;
-	}
+	cache_widgets();
+	if (!gui.mouse_actions)
+		load_or_initialize_mouse_actions();
 
 	double zoom = get_zoom_val();
-
 	// evpos.x/evpos.y = cursor position in image coordinate
 	point evpos = { event->x, event->y };
 	cairo_matrix_transform_point(&gui.image_matrix, &evpos.x, &evpos.y);
-
 	// same as evpos but rounded to integer and clamped to image bounds
 	pointi zoomed = { (int)(evpos.x), (int)(evpos.y) };
 	gboolean inside = clamp2image(&zoomed);
 	//siril_debug_print("clicked at %g, %g, in image it's %d, %d (pointer is%s inside)\n",
 	//		event->x, event->y, zoomed.x, zoomed.y, inside ? "" : " not");
+	mouse_data data = {.widget = widget, .event = event, .zoom = zoom, .evpos = evpos, .zoomed = zoomed, .inside = inside, .mouse_status = &mouse_status, .cutting = &cutting };
 
-	if (inside) {
-		/* Double middle-click toggles between zoom to fit and zoom 1:1 centred on the click */
-		if (event->button == GDK_BUTTON_MIDDLE && event->type == GDK_DOUBLE_BUTTON_PRESS) {
-			update_zoom_fit_button();
-			switch (com.pref.gui.mmb_action) {
-				case MMB_ZOOM_FIT:
-					gui.zoom_value = ZOOM_FIT;
-					reset_display_offset();
-					break;
-				case MMB_ZOOM_100:
-					gui.zoom_value = ZOOM_NONE;
-					double x = evpos.x;
-					double y = evpos.y;
-					cairo_matrix_transform_point(&gui.display_matrix, &evpos.x, &evpos.y);
-					gui.display_offset.x = evpos.x - x;
-					gui.display_offset.y = evpos.y - y;
-					break;
-				case MMB_ZOOM_TOGGLE:
-					if (double_middle_click_zooms_to_fit) {
-						gui.zoom_value = ZOOM_FIT;
-						reset_display_offset();
-					} else {
-						gui.zoom_value = ZOOM_NONE;
-						double x = evpos.x;
-						double y = evpos.y;
-						cairo_matrix_transform_point(&gui.display_matrix, &evpos.x, &evpos.y);
-						gui.display_offset.x = evpos.x - x;
-						gui.display_offset.y = evpos.y - y;
-					}
-					break;
- 			}
-			update_zoom_label();
-			redraw(REDRAW_IMAGE);
-			double_middle_click_zooms_to_fit = !double_middle_click_zooms_to_fit;
-		}
-
-		/* Ctrl click or middle click to drag */
-		else if (((event->state & get_primary()) && (event->button == GDK_BUTTON_PRIMARY)) || (event->button == GDK_BUTTON_MIDDLE && !(event->state & get_primary()))) {
-			// viewport translation
-			gui.translating = TRUE;
-			gui.start.x = (int)(event->x);
-			gui.start.y = (int)(event->y);
-			return TRUE;
-		}
-
-		/* else, click on gray image */
-		else if (event->button == GDK_BUTTON_PRIMARY) {	// left click
-			if (mouse_status == MOUSE_ACTION_SELECT_REG_AREA) {
-				if (gui.drawing) {
-					gui.drawing = FALSE;
-				} else {
-					gui.drawing = TRUE;
-					if (is_inside_of_sel(zoomed, zoom)) {
-						// Move selection
-						gui.freezeX = gui.freezeY = TRUE;
-						gui.start = zoomed;
-						gui.origin.x = com.selection.x;
-						gui.origin.y = com.selection.y;
-					} else {
-						// Default values
-						gui.freezeX = gui.freezeY = FALSE;
-						// The order matters if the selection is so small that edge detection overlaps
-						// and need to be the same as in the on_drawingarea_motion_notify_event()
-						gboolean right = is_over_the_right_side_of_sel(zoomed, zoom);
-						gboolean left = is_over_the_left_side_of_sel(zoomed, zoom);
-						gboolean bottom = is_over_the_bottom_of_sel(zoomed, zoom);
-						gboolean top = is_over_the_top_of_sel(zoomed, zoom);
-						if (right || left || bottom || top) {
-							// Freeze one axis when grabbing an edge far enough from a corner
-							if (right) {
-								gui.start.x = com.selection.x;
-								if (!bottom && !top)
-									gui.freezeY = TRUE;
-							} else if (left) {
-								gui.start.x = com.selection.x + com.selection.w;
-								if (!bottom && !top)
-									gui.freezeY = TRUE;
-							}
-							if (bottom) {
-								gui.start.y = com.selection.y;
-								if (!left && !right)
-									gui.freezeX = TRUE;
-							} else if (top) {
-								gui.start.y = com.selection.y + com.selection.h;
-								if (!left && !right)
-									gui.freezeX = TRUE;
-							}
-						} else {
-							gui.start = zoomed;
-							com.selection.h = 0;
-							com.selection.w = 0;
-						}
-					}
-				}
-				redraw(REDRAW_OVERLAY);
-			} else if (mouse_status == MOUSE_ACTION_DRAW_SAMPLES) {
-				point pt;
-				int radius = get_background_sample_radius();
-
-				pt.x = (gdouble) zoomed.x;
-				pt.y = (gdouble) zoomed.y;
-
-				if (pt.x + radius < gfit.rx && pt.y + radius < gfit.ry
-						&& pt.x - radius > 0 && pt.y - radius > 0) {
-					com.grad_samples = add_background_sample(com.grad_samples, &gfit, pt);
-
-					redraw(REDRAW_OVERLAY);
-					redraw_previews();
-				}
-			} else if (mouse_status == MOUSE_ACTION_PHOTOMETRY) {
-				int s = com.pref.phot_set.outer * 1.2;
-				rectangle area = { zoomed.x - s, zoomed.y - s, s * 2, s * 2 };
-				if (zoomed.x - area.w > 0 && zoomed.x + area.w < gfit.rx
-						&& zoomed.y - area.h > 0 && zoomed.y + area.h < gfit.ry) {
-
-					struct phot_config *ps = phot_set_adjusted_for_image(&gfit);
-					gui.qphot = psf_get_minimisation(&gfit, select_vport(gui.cvport), &area, TRUE, ps, TRUE, com.pref.starfinder_conf.profile, NULL);
-					free(ps);
-					if (gui.qphot) {
-						gui.qphot->xpos = gui.qphot->x0 + area.x;
-						if (gfit.top_down)
-							gui.qphot->ypos = gui.qphot->y0 + area.y;
-						else
-							gui.qphot->ypos = area.y + area.h - gui.qphot->y0;
-						redraw(REDRAW_OVERLAY);
-						popup_psf_result(gui.qphot, &area, &gfit);
-					}
-				}
-			}
-		} else if (event->button == GDK_BUTTON_SECONDARY) {	// right click
-			if (mouse_status == MOUSE_ACTION_DRAW_SAMPLES) {
-				point pt;
-				int radius = (int) (25 / 2);
-
-				pt.x = (gdouble) zoomed.x;
-				pt.y = (gdouble) zoomed.y;
-
-				if (pt.x + radius <= gfit.rx && pt.y + radius <= gfit.ry
-						&& pt.x - radius >= 0 && pt.y - radius >= 0) {
-					com.grad_samples = remove_background_sample(com.grad_samples, &gfit, pt);
-
-					redraw(REDRAW_OVERLAY);
-					redraw_previews();
-				}
-			} else if (mouse_status == MOUSE_ACTION_PHOTOMETRY) {
-				if (sequence_is_loaded()) {
-					int s = com.pref.phot_set.outer * 1.2;
-					rectangle area = { zoomed.x - s, zoomed.y - s, s * 2, s * 2 };
-					if (zoomed.x - area.w > 0 && zoomed.x + area.w < gfit.rx
-							&& zoomed.y - area.h > 0 && zoomed.y + area.h < gfit.ry) {
-						memcpy(&com.selection, &area, sizeof(rectangle));
-						process_seq_psf(0);
-						delete_selected_area();
-					}
-				}
-			}
+	for (GSList *iter = gui.mouse_actions ; iter != NULL; iter = g_slist_next(iter)) {
+		mouse_action* action = (mouse_action*) iter->data;
+		if (mouse_action_match_to_event(event, action)) {
+			action->data->function(&data);
+			break;
 		}
 	}
 	return FALSE;
@@ -516,113 +430,28 @@ gboolean on_drawingarea_button_press_event(GtkWidget *widget,
 
 gboolean on_drawingarea_button_release_event(GtkWidget *widget,
 		GdkEventButton *event, gpointer user_data) {
-	// evpos.x/evpos.y = cursor position in image coordinate
-	point evpos = { event->x, event->y };
-	cairo_matrix_transform_point(&gui.image_matrix, &evpos.x, &evpos.y);
 
-	// same as evpos but rounded to integer and clamped to image bounds
-	pointi zoomed = { (int)(evpos.x), (int)(evpos.y) };
-	gboolean inside = clamp2image(&zoomed);
+	if (button_release.button == event->button) {
+		double zoom = get_zoom_val();
+		cache_widgets();
+		// evpos.x/evpos.y = cursor position in image coordinate
+		point evpos = { event->x, event->y };
+		cairo_matrix_transform_point(&gui.image_matrix, &evpos.x, &evpos.y);
+		// same as evpos but rounded to integer and clamped to image bounds
+		pointi zoomed = { (int)(evpos.x), (int)(evpos.y) };
+		gboolean inside = clamp2image(&zoomed);
+		mouse_data data = {.widget = widget, .event = event, .zoom = zoom, .evpos = evpos, .zoomed = zoomed, .inside = inside, .mouse_status = &mouse_status, .cutting = &cutting };
 
-	if (event->button == GDK_BUTTON_PRIMARY) {	// left click
-		if (gui.translating) {
-			gui.translating = FALSE;
-		} else if (gui.drawing && mouse_status == MOUSE_ACTION_SELECT_REG_AREA) {
-			gui.drawing = FALSE;
-			/* finalize selection rectangle coordinates */
-			if (!gui.freezeX) {
-				if (zoomed.x >= gui.start.x) {
-					com.selection.x = gui.start.x;
-					com.selection.w = zoomed.x - com.selection.x;
-				} else {
-					com.selection.x = zoomed.x;
-					com.selection.w = gui.start.x - zoomed.x;
-				}
-			}
-			if (!gui.freezeY) {
-				if (zoomed.y >= gui.start.y) {
-					com.selection.y = gui.start.y;
-					com.selection.h = zoomed.y - com.selection.y;
-				} else {
-					com.selection.y = zoomed.y;
-					com.selection.h = gui.start.y - zoomed.y;
-				}
-			}
-			// Clicking in displayed psf selects star in list if the DynamicPSF dialog is open
-			GtkWidget *dynpsf_dlg = lookup_widget("stars_list_window");
-			if (((com.selection.w == 0 || com.selection.w == 1) && (com.selection.h == 0 || com.selection.h == 1)) && gtk_widget_is_visible(dynpsf_dlg)) {
-				set_iter_of_clicked_psf((double) com.selection.x, (double) com.selection.y);
-			}
-
-			// never let selection be null if rotation_dlg is visible
-			// reinstate full image instead
-			if (!gui.freezeX && com.selection.w == 0 && gtk_widget_is_visible(rotation_dlg))
-				com.selection.w = gfit.rx;
-			if (!gui.freezeY && com.selection.h == 0 && gtk_widget_is_visible(rotation_dlg))
-				com.selection.h = gfit.ry;
-
-			if (gui.freezeX && gui.freezeY) { // Move selection
-				com.selection.x = (zoomed.x - gui.start.x) + gui.origin.x;
-				com.selection.y = (zoomed.y - gui.start.y) + gui.origin.y;
-			}
-
-			// Enforce a ratio and clamp selection to the image
-			enforce_ratio_and_clamp();
-
-			/* we have a new rectangular selection zone,
-			 * or an unselection (empty zone) */
-			new_selection_zone();
-
-			// Terminate any specific selection modification mode
-			gui.freezeX = gui.freezeY = FALSE;
-		} else if (mouse_status == MOUSE_ACTION_SELECT_PREVIEW1) {
-			set_preview_area(0, zoomed.x, zoomed.y);
-			mouse_status = MOUSE_ACTION_SELECT_REG_AREA;
-			// redraw to get the position of the new preview area
-			redraw(REDRAW_OVERLAY);
-		} else if (mouse_status == MOUSE_ACTION_SELECT_PREVIEW2) {
-			set_preview_area(1, zoomed.x, zoomed.y);
-			mouse_status = MOUSE_ACTION_SELECT_REG_AREA;
-			redraw(REDRAW_OVERLAY);
-		}
-	} else if (event->button == GDK_BUTTON_MIDDLE) {	// middle click, stop translating the image
-		if (gui.translating) {
-			gui.translating = FALSE;
-		}
-		/* Ctrl middle-click to set the photometry box */
-		else if (inside && event->state & get_primary()) {
-			double dX = 1.5 * com.pref.phot_set.outer;
-			double dY = dX;
-			double w = 3 * com.pref.phot_set.outer;
-			double h = w;
-
-			if ((dX <= zoomed.x) && (dY <= zoomed.y)
-					&& (zoomed.x - dX + w < gfit.rx)
-					&& (zoomed.y - dY + h < gfit.ry)) {
-
-				com.selection.x = zoomed.x - dX;
-				com.selection.y = zoomed.y - dY;
-				com.selection.w = w;
-				com.selection.h = h;
-
-				new_selection_zone();
-			}
-		}
-	} else if (event->button == GDK_BUTTON_SECONDARY) {	// right click
-		if (mouse_status != MOUSE_ACTION_DRAW_SAMPLES && mouse_status != MOUSE_ACTION_PHOTOMETRY) {
-			do_popup_graymenu(widget, NULL);
-		}
+		button_release.function(&data);
+		clear_release_callback();
+		gui_function(update_MenuItem, NULL);
 	}
-	update_MenuItem();
-
 	return FALSE;
 }
 
 gboolean on_drawingarea_motion_notify_event(GtkWidget *widget,
 		GdkEventMotion *event, gpointer user_data) {
-	if (!rotation_dlg) {
-		rotation_dlg = lookup_widget("rotation_dialog");
-	}
+	cache_widgets();
 	if ((!single_image_is_loaded() && !sequence_is_loaded())
 			|| gfit.type == DATA_UNSUPPORTED) {
 		return FALSE;
@@ -655,6 +484,11 @@ gboolean on_drawingarea_motion_notify_event(GtkWidget *widget,
 	gtk_label_set_text(GTK_LABEL(lookup_widget("label-rgb")), "");
 
 	if (inside) {
+		if (gui.measure_start.x != -1) {
+			gui.measure_end.x = zoomed.x;
+			gui.measure_end.y = zoomed.y;
+			redraw(REDRAW_OVERLAY);
+		}
 		if (gui.cvport == RGB_VPORT) {
 			static gchar buffer[256] = { 0 };
 			if (gfit.type == DATA_USHORT) {
@@ -687,9 +521,9 @@ gboolean on_drawingarea_motion_notify_event(GtkWidget *widget,
 			} else {
 				format_base_ushort = "x: %%.%dd y: %%.%dd";
 			}
-			if (gfit.hi >= 1000)
+			if (gfit.keywords.hi >= 1000)
 				val_width = 4;
-			if (gfit.hi >= 10000)
+			if (gfit.keywords.hi >= 10000)
 				val_width = 5;
 			g_sprintf(format, format_base_ushort,
 					coords_width, coords_width, val_width);
@@ -751,7 +585,13 @@ gboolean on_drawingarea_motion_notify_event(GtkWidget *widget,
 	if (blank_density_cvport && gtk_label_get_text(labels_density[gui.cvport])[0] != '\0')
 		gtk_label_set_text(labels_density[gui.cvport], " ");
 
-	if (gui.translating) {
+	if (gui.drawing_polygon) {
+		point *ev = malloc(sizeof(point));
+		ev->x = zoomed.x;
+		ev->y = zoomed.y;
+		gui.drawing_polypoints = g_slist_prepend(gui.drawing_polypoints, ev);
+		redraw(REDRAW_OVERLAY);
+	} else if (gui.translating) {
 		update_zoom_fit_button();
 
 		pointi ev = { (int)(event->x), (int)(event->y) };
@@ -761,6 +601,24 @@ gboolean on_drawingarea_motion_notify_event(GtkWidget *widget,
 		gui.display_offset.y += delta.y;
 		adjust_vport_size_to_image();
 		redraw(REDRAW_OVERLAY);
+	} else if (cutting) {	// button 1 down, dragging a line for the pixel profile cut
+		if (event->state & GDK_SHIFT_MASK)
+			cutting = CUT_VERT_OR_HORIZ;
+		else
+			cutting = CUT_UNCONSTRAINED;
+		pointi tmp;
+		tmp.x = zoomed.x;
+		tmp.y = zoomed.y;
+		if (cutting == CUT_VERT_OR_HORIZ) {
+			if (fabs(tmp.y - gui.cut.cut_start.y) > fabs(tmp.x - gui.cut.cut_start.x)) {
+				tmp.x = gui.cut.cut_start.x;
+			} else {
+				tmp.y = gui.cut.cut_start.y;
+			}
+		}
+		gui.cut.cut_end.x = tmp.x;
+		gui.cut.cut_end.y = tmp.y;
+		redraw(REDRAW_OVERLAY);
 	} else if (gui.drawing) {	// with button 1 down
 		if (!gui.freezeX) {
 			if (zoomed.x > gui.start.x) {
@@ -794,8 +652,9 @@ gboolean on_drawingarea_motion_notify_event(GtkWidget *widget,
 		redraw(REDRAW_OVERLAY);
 	}
 
-	/* don't change cursor if thread is running */
-	if (get_thread_run()) return FALSE;
+	/* don't change cursor if thread is running or if Python
+	 claims the thread */
+	if (get_thread_run() || com.python_claims_thread) return FALSE;
 
 	if (inside) {
 		if (mouse_status == MOUSE_ACTION_DRAW_SAMPLES) {
@@ -845,10 +704,22 @@ gboolean on_drawingarea_motion_notify_event(GtkWidget *widget,
 	return FALSE;
 }
 
+void on_drawingarea_enter_notify_event(GtkWidget *widget, GdkEvent *event,
+		gpointer user_data) {
+	if (single_image_is_loaded() || sequence_is_loaded()) {
+		if (get_thread_run() || com.python_claims_thread) {
+			set_cursor_waiting(TRUE);
+		} else {
+			/* trick to get default cursor */
+			set_cursor_waiting(FALSE);
+		}
+	}
+}
+
 void on_drawingarea_leave_notify_event(GtkWidget *widget, GdkEvent *event,
 		gpointer user_data) {
 	if (single_image_is_loaded() || sequence_is_loaded()) {
-		if (get_thread_run()) {
+		if (get_thread_run() || com.python_claims_thread) {
 			set_cursor_waiting(TRUE);
 		} else {
 			/* trick to get default cursor */
@@ -886,6 +757,11 @@ void update_zoom_label() {
 	gdk_threads_add_idle(set_label_zoom_text_idle, zoom_buffer);
 }
 
+gboolean update_zoom_label_idle(gpointer user_data) {
+	update_zoom_label();
+	return FALSE;
+}
+
 gboolean update_zoom(gdouble x, gdouble y, double scale) {
 	// event position in image coordinates before changing the zoom value
 	point evpos = { x, y };
@@ -915,32 +791,20 @@ gboolean update_zoom(gdouble x, gdouble y, double scale) {
 
 // scroll event is the wheel interception for zoom
 gboolean on_drawingarea_scroll_event(GtkWidget *widget, GdkEventScroll *event, gpointer user_data) {
-	gboolean handled = FALSE;
-
-	if (!single_image_is_loaded() && !sequence_is_loaded())
-		return FALSE;
-
+	if (!gui.scroll_actions)
+		load_or_initialize_scroll_actions();
 	point delta;
-
-	// The handler is written to act on either horizontal or vertical scroll events
-	// However the handler will only ever be called for one or the other depending
-	// on the configuration.
-	switch (event->direction) {
-		case GDK_SCROLL_SMOOTH:
-			gdk_event_get_scroll_deltas((GdkEvent*) event, &delta.x, &delta.y);
-			if (delta.y < 0) {
-				return update_zoom(event->x, event->y, ((ZOOM_IN - 1.0) * fabs(delta.y)) + 1.0);
-			} else if (delta.y > 0) {
-				return update_zoom(event->x, event->y, 1.0 / (((ZOOM_IN - 1.0) * delta.y) + 1.0));
-			}
-			break;
-		case GDK_SCROLL_DOWN:
-			return update_zoom(event->x, event->y, ZOOM_IN);
-			// Fallthrough intentional
-		case GDK_SCROLL_UP:
-			return update_zoom(event->x, event->y, ZOOM_OUT);
-		default:
+	gdk_event_get_scroll_deltas((GdkEvent *) event, &delta.x, &delta.y);
+	GSList *iter = gui.scroll_actions;
+	while (iter) {
+		scroll_action* action = (scroll_action*) iter->data;
+		if (scroll_action_match_to_event(event, action)) {
+			scroll_data data = { .event = event, .direction = action->direction };
+			action->data->function(&data);
 			break;
+		}
+		iter = g_slist_next(iter);
 	}
+
 	return FALSE;
 }
