@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,6 +42,8 @@
  *  ROTATION
  */
 static void rotate_gui(fits *fit) {
+	if (!check_ok_if_cfa())
+		return;
 	if (com.selection.w == 0 || com.selection.h == 0) return;
 	static GtkToggleButton *crop_rotation = NULL;
 	double angle = gtk_spin_button_get_value(
@@ -72,11 +74,11 @@ static void rotate_gui(fits *fit) {
 	memcpy(&com.selection, &area, sizeof(rectangle));
 	gtk_spin_button_set_value(
 			GTK_SPIN_BUTTON(lookup_widget("spinbutton_rotation")), 0.);
-	new_selection_zone();
+	gui_function(new_selection_zone, NULL);
 
 	update_zoom_label();
 	redraw(REMAP_ALL);
-	redraw_previews();
+	gui_function(redraw_previews, NULL);
 	set_cursor_waiting(FALSE);
 }
 void siril_rotate90() {
@@ -85,7 +87,7 @@ void siril_rotate90() {
 	verbose_rotate_fast(&gfit, 90); // fast rotation, no interpolation, no crop
 	update_zoom_label();
 	redraw(REMAP_ALL);
-	redraw_previews();
+	gui_function(redraw_previews, NULL);
 	set_cursor_waiting(FALSE);
 }
 
@@ -95,7 +97,7 @@ void siril_rotate270() {
 	verbose_rotate_fast(&gfit, -90); // fast rotation, no interpolation, no crop
 	update_zoom_label();
 	redraw(REMAP_ALL);
-	redraw_previews();
+	gui_function(redraw_previews, NULL);
 	set_cursor_waiting(FALSE);
 }
 
@@ -104,6 +106,12 @@ void on_button_rotation_close_clicked(GtkButton *button, gpointer user_data) {
 	siril_close_dialog("rotation_dialog");
 }
 
+gboolean rotation_hide_on_delete(GtkWidget *widget) {
+	delete_selected_area();
+	siril_close_dialog("rotation_dialog");
+	return TRUE;
+}
+
 void on_button_rotation_ok_clicked(GtkButton *button, gpointer user_data) {
 	rotate_gui(&gfit);
 }
@@ -119,7 +127,7 @@ void on_checkbutton_rotation_crop_toggled(GtkToggleButton *button, gpointer user
 	if (!gtk_toggle_button_get_active(button)) {
 		rectangle area = {0, 0, gfit.rx, gfit.ry};
 		memcpy(&com.selection, &area, sizeof(rectangle));
-		new_selection_zone();
+		gui_function(new_selection_zone, NULL);
 	}
 }
 
@@ -154,7 +162,7 @@ void mirrorx_gui(fits *fit) {
 	undo_save_state(fit, _("Mirror X"));
 	mirrorx(fit, TRUE);
 	redraw(REMAP_ALL);
-	redraw_previews();
+	gui_function(redraw_previews, NULL);
 	set_cursor_waiting(FALSE);
 }
 
@@ -163,7 +171,7 @@ void mirrory_gui(fits *fit) {
 	undo_save_state(fit, _("Mirror Y"));
 	mirrory(fit, TRUE);
 	redraw(REMAP_ALL);
-	redraw_previews();
+	gui_function(redraw_previews, NULL);
 	set_cursor_waiting(FALSE);
 }
 
@@ -172,6 +180,8 @@ void mirrory_gui(fits *fit) {
  */
 
 void on_button_binning_ok_clicked(GtkButton *button, gpointer user_data) {
+	if (!check_ok_if_cfa())
+		return;
 	if (confirm_delete_wcs_keywords(&gfit)) {
 		/* Switch to console tab */
 		control_window_switch_to_tab(OUTPUT_LOGS);
@@ -183,7 +193,7 @@ void on_button_binning_ok_clicked(GtkButton *button, gpointer user_data) {
 		undo_save_state(&gfit, _("Binning x%d (%s)"), factor, mean ? _("average") : _("sum"));
 		fits_binning(&gfit, factor, mean);
 
-		update_MenuItem(); // WCS not available anymore
+		gui_function(update_MenuItem, NULL); // WCS not available anymore
 		notify_gfit_modified();
 	}
 }
@@ -192,10 +202,17 @@ void on_button_binning_close_clicked(GtkButton *button, gpointer user_data) {
 	siril_close_dialog("binxy_dialog");
 }
 
+gboolean binxy_hide_on_delete(GtkWidget *widget) {
+	siril_close_dialog("binxy_dialog");
+	return TRUE;
+}
+
 /*************
  * RESAMPLE
  */
 void on_button_resample_ok_clicked(GtkButton *button, gpointer user_data) {
+	if (!check_ok_if_cfa())
+		return;
 	if (confirm_delete_wcs_keywords(&gfit)) {
 		/* Switch to console tab */
 		control_window_switch_to_tab(OUTPUT_LOGS);
@@ -212,7 +229,7 @@ void on_button_resample_ok_clicked(GtkButton *button, gpointer user_data) {
 		undo_save_state(&gfit, _("Resample (%g - %g)"), sample[0] / 100.0, sample[1] / 100.0);
 		verbose_resize_gaussian(&gfit, toX, toY, interpolation, clamp);
 
-		update_MenuItem(); // WCS not available anymore
+		gui_function(update_MenuItem, NULL); // WCS not available anymore
 		notify_gfit_modified();
 	}
 }
@@ -221,42 +238,125 @@ void on_button_resample_close_clicked(GtkButton *button, gpointer user_data) {
 	siril_close_dialog("resample_dialog");
 }
 
-void on_spinbutton_resample_X_value_changed(GtkSpinButton *spinbutton,
-		gpointer user_data) {
-	GtkToggleButton *ratio = GTK_TOGGLE_BUTTON(lookup_widget("button_sample_ratio"));
-	double xvalue = gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_X")));
-
-	if (gtk_toggle_button_get_active(ratio))
-		gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_Y")), xvalue);
+gboolean resample_hide_on_delete(GtkWidget *widget) {
+	siril_close_dialog("resample_dialog");
+	return TRUE;
 }
 
-void on_spinbutton_resample_Y_value_changed(GtkSpinButton *spinbutton,
-		gpointer user_data) {
-	GtkToggleButton *ratio = GTK_TOGGLE_BUTTON(
-			lookup_widget("button_sample_ratio"));
-	double yvalue = gtk_spin_button_get_value(
-			GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_Y")));
+void on_spinbutton_resample_X_value_changed(GtkSpinButton *spinbutton, gpointer user_data);
+void on_spinbutton_resample_Y_value_changed(GtkSpinButton *spinbutton, gpointer user_data);
+void on_spinbutton_resample_X_px_value_changed(GtkSpinButton *spinbutton, gpointer user_data);
+void on_spinbutton_resample_Y_px_value_changed(GtkSpinButton *spinbutton, gpointer user_data);
 
-	if (gtk_toggle_button_get_active(ratio))
-		gtk_spin_button_set_value(
-				GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_X")),
-				yvalue);
+static GtkWidget *spinbutton_resample_X = NULL, *spinbutton_resample_Y = NULL,
+	*spinbutton_resample_X_px = NULL, *spinbutton_resample_Y_px = NULL,
+	*button_sample_ratio = NULL, *combo_interpolation = NULL, *toggle_scale_clamp = NULL;
+
+static void initialize_resample_widgets_if_needed() {
+	if (!spinbutton_resample_X) {
+		spinbutton_resample_X = lookup_widget("spinbutton_resample_X");
+		spinbutton_resample_Y = lookup_widget("spinbutton_resample_Y");
+		spinbutton_resample_X_px = lookup_widget("spinbutton_resample_X_px");
+		spinbutton_resample_Y_px = lookup_widget("spinbutton_resample_Y_px");
+		button_sample_ratio = lookup_widget("button_sample_ratio");
+		combo_interpolation = lookup_widget("combo_interpolation");
+		toggle_scale_clamp = lookup_widget("toggle_scale_clamp");
+	}
+}
+
+static void pause_resample_signal_handlers() {
+	g_signal_handlers_block_by_func(spinbutton_resample_X, on_spinbutton_resample_X_value_changed, NULL);
+	g_signal_handlers_block_by_func(spinbutton_resample_Y, on_spinbutton_resample_Y_value_changed, NULL);
+	g_signal_handlers_block_by_func(spinbutton_resample_X_px, on_spinbutton_resample_X_px_value_changed, NULL);
+	g_signal_handlers_block_by_func(spinbutton_resample_Y_px, on_spinbutton_resample_Y_px_value_changed, NULL);
+}
+
+static void restart_resample_signal_handlers() {
+	g_signal_handlers_unblock_by_func(spinbutton_resample_X, on_spinbutton_resample_X_value_changed, NULL);
+	g_signal_handlers_unblock_by_func(spinbutton_resample_Y, on_spinbutton_resample_Y_value_changed, NULL);
+	g_signal_handlers_unblock_by_func(spinbutton_resample_X_px, on_spinbutton_resample_X_px_value_changed, NULL);
+	g_signal_handlers_unblock_by_func(spinbutton_resample_Y_px, on_spinbutton_resample_Y_px_value_changed, NULL);
+}
+
+void on_resample_dialog_show(GtkWidget *dialog, gpointer user_data) {
+	initialize_resample_widgets_if_needed();
+	pause_resample_signal_handlers();
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_X_px), gfit.rx);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_Y_px), gfit.ry);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_X), 100.0);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_Y), 100.0);
+	restart_resample_signal_handlers();
+}
+
+void on_spinbutton_resample_X_value_changed(GtkSpinButton *spinbutton, gpointer user_data) {
+	pause_resample_signal_handlers();
+	GtkToggleButton *ratio = GTK_TOGGLE_BUTTON(button_sample_ratio);
+	double xvalue = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton_resample_X));
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_X_px), round_to_int(gfit.rx * xvalue / 100.0));
+
+	if (gtk_toggle_button_get_active(ratio)) {
+		double yvalue = xvalue;
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_Y), yvalue);
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_Y_px), round_to_int(gfit.ry * yvalue / 100.0));
+	}
+	restart_resample_signal_handlers();
+}
+
+void on_spinbutton_resample_Y_value_changed(GtkSpinButton *spinbutton, gpointer user_data) {
+	pause_resample_signal_handlers();
+	GtkToggleButton *ratio = GTK_TOGGLE_BUTTON(button_sample_ratio);
+	double yvalue = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton_resample_Y));
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_Y_px), round_to_int(gfit.ry * yvalue / 100.0));
+
+	if (gtk_toggle_button_get_active(ratio)) {
+		double xvalue = yvalue;
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_X), xvalue);
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_X_px), round_to_int(gfit.rx * xvalue / 100.0));
+	}
+	restart_resample_signal_handlers();
+}
+
+void on_spinbutton_resample_X_px_value_changed(GtkSpinButton *spinbutton, gpointer user_data) {
+	pause_resample_signal_handlers();
+	GtkToggleButton *ratio_button = GTK_TOGGLE_BUTTON(button_sample_ratio);
+	double xpix = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton_resample_X_px));
+	double ratio = xpix / gfit.rx;
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_X), ratio * 100.0);
+
+	if (gtk_toggle_button_get_active(ratio_button)) {
+		double ypix = round_to_int(gfit.ry * ratio);
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_Y), ratio * 100.0);
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_Y_px), ypix);
+	}
+	restart_resample_signal_handlers();
+}
+
+void on_spinbutton_resample_Y_px_value_changed(GtkSpinButton *spinbutton, gpointer user_data) {
+	pause_resample_signal_handlers();
+	GtkToggleButton *ratio_button = GTK_TOGGLE_BUTTON(button_sample_ratio);
+	double ypix = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton_resample_Y_px));
+	double ratio = ypix / gfit.ry;
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_Y), ratio * 100.0);
+
+	if (gtk_toggle_button_get_active(ratio_button)) {
+		double xpix = round_to_int(gfit.rx * ratio);
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_X), ratio * 100.0);
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_X_px), xpix);
+	}
+	restart_resample_signal_handlers();
 }
 
 void on_button_sample_ratio_toggled(GtkToggleButton *button, gpointer user_data) {
-	double xvalue = gtk_spin_button_get_value(
-			GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_X")));
+	double xvalue = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbutton_resample_X));
 
 	if (gtk_toggle_button_get_active(button))
-		gtk_spin_button_set_value(
-				GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_Y")),
-				xvalue);
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_resample_Y), xvalue);
 }
 
 void on_combo_interpolation_changed(GtkComboBox *combo_box, gpointer user_data) {
 	gint idx = gtk_combo_box_get_active(combo_box);
 
-	gtk_widget_set_sensitive(lookup_widget("toggle_scale_clamp"), idx == OPENCV_CUBIC || idx == OPENCV_LANCZOS4);
+	gtk_widget_set_sensitive(toggle_scale_clamp, idx == OPENCV_CUBIC || idx == OPENCV_LANCZOS4);
 }
 
 /**************
@@ -267,9 +367,11 @@ void siril_crop() {
 			com.selection.x, com.selection.y, com.selection.w,
 			com.selection.h);
 	if (is_preview_active()) {
-		siril_message_dialog(GTK_MESSAGE_INFO, _("A live preview session is active"),
-				_("It is impossible to crop the image when a filter with preview session is active. "
-						"Please consider to close the filter dialog first."));
+		siril_message_dialog(GTK_MESSAGE_INFO, _("Image backup is active"),
+				_("It is impossible to crop the image when the image backup is active. "
+				"This occurs when a filter has a live preview active or when a filter with "
+				"on-demand ROI preview is open. Please close the filter dialog before "
+				"cropping."));
 		return;
 	}
 	clear_stars_list(TRUE);
@@ -283,9 +385,9 @@ void siril_crop() {
 	delete_selected_area();
 	reset_display_offset();
 	update_zoom_label();
-	adjust_cutoff_from_updated_gfit();
+	notify_gfit_modified();
 	redraw(REMAP_ALL);
-	redraw_previews();
+	gui_function(redraw_previews, NULL);
 }
 
 void on_crop_Apply_clicked(GtkButton *button, gpointer user_data) {
@@ -305,7 +407,7 @@ void on_crop_Apply_clicked(GtkButton *button, gpointer user_data) {
 		siril_log_message(_("Not a valid sequence for cropping.\n"));
 	}
 
-	struct crop_sequence_data *args = malloc(sizeof(struct crop_sequence_data));
+	struct crop_sequence_data *args = calloc(1, sizeof(struct crop_sequence_data));
 
 	GtkEntry *cropped_entry = GTK_ENTRY(lookup_widget("cropped_entry"));
 
@@ -321,3 +423,7 @@ void on_crop_close_clicked(GtkButton *button, gpointer user_data) {
 	siril_close_dialog("crop_dialog");
 }
 
+gboolean crop_hide_on_delete(GtkWidget *widget) {
+	siril_close_dialog("crop_dialog");
+	return TRUE;
+}
