@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include "core/proto.h"
 #include "core/OS_utils.h"
 #include "core/initfile.h"
+#include "core/icc_profile.h"
 #include "algos/sorting.h"
 #include "io/conversion.h"
 #include "io/films.h"
@@ -45,24 +46,6 @@
 
 #include "open_dialog.h"
 
-static void gtk_filter_add(GtkFileChooser *file_chooser, const gchar *title,
-		const gchar *pattern, gboolean set_default) {
-	gchar **patterns;
-	gint i;
-
-	GtkFileFilter *f = gtk_file_filter_new();
-	gtk_file_filter_set_name(f, title);
-	/* get the patterns */
-	patterns = g_strsplit(pattern, ";", -1);
-	for (i = 0; patterns[i] != NULL; i++)
-		gtk_file_filter_add_pattern(f, patterns[i]);
-	/* free the patterns */
-	g_strfreev(patterns);
-	gtk_file_chooser_add_filter(file_chooser, f);
-	if (set_default)
-		gtk_file_chooser_set_filter(file_chooser, f);
-}
-
 static void set_single_filter_dialog(GtkFileChooser *chooser, const gchar *name, const gchar *filter) {
 	gtk_filter_add(chooser, name, filter, TRUE);
 }
@@ -87,7 +70,7 @@ static void set_filters_dialog(GtkFileChooser *chooser, int whichdial) {
 		int i;
 
 		nb_raw = get_nb_raw_supported();
-		raw = calloc(sizeof(char), nb_raw * 12 + 1);// we assume the extension size of 3 char "*.xxx;*.XXX;" = 12
+		raw = calloc(sizeof(char), nb_raw * 12 + 1); // we assume the extension size of 3 char "*.xxx;*.XXX;" = 12
 		for (i = 0; i < nb_raw; i++) {
 			char *ext;
 			gchar *upcase;
@@ -120,9 +103,14 @@ static void set_filters_dialog(GtkFileChooser *chooser, int whichdial) {
 		s_pattern = g_string_append(s_pattern, "*.jpg;*.JPG;*.jpeg;*.JPEG;");
 #endif
 
+#ifdef HAVE_LIBJXL
+		s_supported_graph = g_string_append(s_supported_graph, ", *.jxl");
+		s_pattern = g_string_append(s_pattern, "*.jxl;*.JXL;");
+#endif
+
 #ifdef HAVE_LIBHEIF
-		s_supported_graph = g_string_append(s_supported_graph, ", *.heic, *.heif");
-		s_pattern = g_string_append(s_pattern, "*.heic;*.HEIC;*.heif;*.HEIF;");
+		s_supported_graph = g_string_append(s_supported_graph, ", *.heic, *.heif, *.avif");
+		s_pattern = g_string_append(s_pattern, "*.heic;*.HEIC;*.heif;*.HEIF;*.avif;*.AVIF;");
 #endif
 
 #ifdef HAVE_LIBPNG
@@ -132,36 +120,26 @@ static void set_filters_dialog(GtkFileChooser *chooser, int whichdial) {
 
 #ifdef HAVE_LIBTIFF
 		s_supported_graph = g_string_append(s_supported_graph, ", *.tif, *.tiff");
-		s_pattern = g_string_append(s_pattern, "*.tif;*.TIF;*.tiff;*.TIFF");
+		s_pattern = g_string_append(s_pattern, "*.tif;*.TIF;*.tiff;*.TIFF;");
+#endif
+
+#ifdef HAVE_LIBXISF
+		s_supported_graph = g_string_append(s_supported_graph, ", *.xisf");
+		s_pattern = g_string_append(s_pattern, "*.xisf;*.XISF");
 #endif
 		s_supported_graph = g_string_append(s_supported_graph, ")");
 
 		graphics_supported = g_string_free(s_supported_graph, FALSE);
 		graphics_filter = g_string_free(s_pattern, FALSE);
-		if (whichdial != OD_CONVERT && whichdial != OD_OPEN) {
-			gtk_filter_add(chooser, graphics_supported, graphics_filter,
-					gui.file_ext_filter == TYPEBMP || gui.file_ext_filter == TYPEJPG ||
-					gui.file_ext_filter == TYPEPNG || gui.file_ext_filter == TYPETIFF);
 
-			/* NETPBM FILES */
-			gtk_filter_add(chooser, _("Netpbm Files (*.ppm, *.pnm, *.pgm)"),
-					netpbm_filter, gui.file_ext_filter == TYPEPNM);
-			/* IRIS FILES */
-			gtk_filter_add(chooser, _("IRIS PIC Files (*.pic)"), pic_filter,
-					gui.file_ext_filter == TYPEPIC);
-			/* SER FILES */
-			gtk_filter_add(chooser, _("SER files (*.ser)"), ser_filter,
-					gui.file_ext_filter == TYPESER);
-		} else {
-			all_filter = g_string_append(all_filter, ";");
-			all_filter = g_string_append(all_filter, graphics_filter);
-			all_filter = g_string_append(all_filter, ";");
-			all_filter = g_string_append(all_filter, netpbm_filter);
-			all_filter = g_string_append(all_filter, ";");
-			all_filter = g_string_append(all_filter, pic_filter);
-			all_filter = g_string_append(all_filter, ";");
-			all_filter = g_string_append(all_filter, ser_filter);
-		}
+		all_filter = g_string_append(all_filter, ";");
+		all_filter = g_string_append(all_filter, graphics_filter);
+		all_filter = g_string_append(all_filter, ";");
+		all_filter = g_string_append(all_filter, netpbm_filter);
+		all_filter = g_string_append(all_filter, ";");
+		all_filter = g_string_append(all_filter, pic_filter);
+		all_filter = g_string_append(all_filter, ";");
+		all_filter = g_string_append(all_filter, ser_filter);
 
 #ifdef HAVE_FFMS2
 		/* FILM FILES */
@@ -184,15 +162,12 @@ static void set_filters_dialog(GtkFileChooser *chooser, int whichdial) {
 			g_free(ext);
 			g_free(upcase);
 		}
-		film_filter[strlen(film_filter) - 1] = '\0';
+		if (strlen(film_filter) > 0)
+			film_filter[strlen(film_filter) - 1] = '\0';
+
+		all_filter = g_string_append(all_filter, ";");
+		all_filter = g_string_append(all_filter, film_filter);
 
-		if (whichdial != OD_CONVERT && whichdial != OD_OPEN) {
-		gtk_filter_add(chooser, _("Film Files (*.avi, *.mpg, ...)"), film_filter,
-				gui.file_ext_filter == TYPEAVI);
-		} else {
-			all_filter = g_string_append(all_filter, ";");
-			all_filter = g_string_append(all_filter, film_filter);
-		}
 		free(film_filter);
 #endif
 		g_free(graphics_supported);
@@ -221,6 +196,25 @@ static void siril_add_debayer_toggle_button(GtkFileChooser *dialog) {
 	g_signal_connect(GTK_TOGGLE_BUTTON(toggle_debayer), "toggled", G_CALLBACK(on_debayer_toggled), (gpointer) main_debayer_button);
 }
 
+static gchar* get_calibration_file_directory(GtkWidget *entry) {
+    if (!GTK_IS_ENTRY(GTK_ENTRY(entry))) {
+        return NULL;
+    }
+
+    // Get the filename from the file chooser button (returns absolute path)
+    const gchar *filename = gtk_entry_get_text(GTK_ENTRY(entry));
+
+    if (!filename || filename[0] == '\0') {
+        // No file selected
+        return g_strdup(com.wd);
+    }
+
+    // Since gtk_file_chooser_get_filename() always returns absolute paths,
+    // we always extract the directory portion
+    gchar *dirname = g_path_get_dirname(filename);
+    return dirname;
+}
+
 static void opendial(int whichdial) {
 	SirilWidget *widgetdialog;
 	GtkFileChooser *dialog = NULL;
@@ -232,19 +226,37 @@ static void opendial(int whichdial) {
 	if (!com.wd)
 		return;
 
+	GtkWidget *entry = NULL;
 	switch (whichdial) {
 	case OD_NULL:
 		fprintf(stderr, "whichdial undefined, should not happen\n");
 		return;
 	case OD_FLAT:
+		if (whichdial == OD_FLAT)
+			entry = lookup_widget("flatname_entry");
 	case OD_DARK:
+		if (whichdial == OD_DARK)
+			entry = lookup_widget("darkname_entry");
 	case OD_OFFSET:
+		if (whichdial == OD_OFFSET)
+			entry = lookup_widget("offsetname_entry");
 	case OD_FLATLIB:
+		if (whichdial == OD_FLATLIB)
+			entry = lookup_widget("flatlib_entry");
 	case OD_DARKLIB:
+		if (whichdial == OD_DARKLIB)
+			entry = lookup_widget("darklib_entry");
 	case OD_OFFSETLIB:
+		if (whichdial == OD_OFFSETLIB)
+			entry = lookup_widget("biaslib_entry");
+	case OD_DISTOLIB:
+		if (whichdial == OD_DISTOLIB)
+			entry = lookup_widget("distolib_entry");
 		widgetdialog = siril_file_chooser_open(control_window, GTK_FILE_CHOOSER_ACTION_OPEN);
 		dialog = GTK_FILE_CHOOSER(widgetdialog);
-		gtk_file_chooser_set_current_folder(dialog, com.wd);
+		gchar *lastdir = get_calibration_file_directory(entry);
+		gtk_file_chooser_set_current_folder(dialog, lastdir);
+		g_free(lastdir);
 		gtk_file_chooser_set_local_only(dialog, FALSE);
 		if (whichdial == OD_FLATLIB) {
 			if (com.pref.prepro.flat_lib != NULL) {
@@ -264,9 +276,19 @@ static void opendial(int whichdial) {
 				gtk_file_chooser_set_current_folder(dialog, path);
 				g_free(path);
 			}
+		} else if (whichdial == OD_DISTOLIB) {
+			if (com.pref.prepro.disto_lib != NULL) {
+				gchar *path = g_path_get_dirname(com.pref.prepro.disto_lib);
+				gtk_file_chooser_set_current_folder(dialog, path);
+				g_free(path);
+			}
 		}
 		gtk_file_chooser_set_select_multiple(dialog, FALSE);
-		set_filters_dialog(dialog, whichdial);
+		if (whichdial == OD_DISTOLIB) {
+			set_single_filter_dialog(dialog, _("Master-distortion: file (*.wcs)"), "*.wcs;*.WCS");
+		} else {
+			set_filters_dialog(dialog, whichdial);
+		}
 		siril_file_chooser_add_preview(dialog, preview);
 		break;
 	case OD_CWD:
@@ -315,7 +337,7 @@ static void opendial(int whichdial) {
 		gboolean anything_loaded;
 		GtkFileChooser *chooser = GTK_FILE_CHOOSER(dialog);
 		GtkEntry *flat_entry, *dark_entry, *bias_entry, *bad_pixel_entry;
-		GtkEntry *flatlib_entry, *darklib_entry, *biaslib_entry;
+		GtkEntry *flatlib_entry, *darklib_entry, *biaslib_entry, *distolib_entry;
 		GtkToggleButton *flat_button, *dark_button, *bias_button;
 		GtkWidget *pbutton;
 
@@ -337,6 +359,7 @@ static void opendial(int whichdial) {
 		flatlib_entry = GTK_ENTRY(lookup_widget("flatlib_entry"));
 		darklib_entry = GTK_ENTRY(lookup_widget("darklib_entry"));
 		biaslib_entry = GTK_ENTRY(lookup_widget("biaslib_entry"));
+		distolib_entry = GTK_ENTRY(lookup_widget("distolib_entry"));
 		bad_pixel_entry = GTK_ENTRY(lookup_widget("pixelmap_entry"));
 
 		flat_button = GTK_TOGGLE_BUTTON(lookup_widget("useflat_button"));
@@ -376,13 +399,17 @@ static void opendial(int whichdial) {
 			gtk_entry_set_text(biaslib_entry, filename);
 			break;
 
+		case OD_DISTOLIB:
+			gtk_entry_set_text(distolib_entry, filename);
+			break;
+
 		case OD_CWD:
 			if (!siril_change_dir(filename, &err)) {
 				if (com.pref.wd)
 					g_free(com.pref.wd);
 				com.pref.wd = g_strdup(com.wd);
 				writeinitfile();
-				set_GUI_CWD();
+				gui_function(set_GUI_CWD, NULL);
 			} else {
 				siril_message_dialog(GTK_MESSAGE_ERROR, _("Error"), err);
 			}
@@ -391,6 +418,7 @@ static void opendial(int whichdial) {
 		case OD_OPEN:
 			set_cursor_waiting(TRUE);
 			retval = open_single_image(filename);
+			icc_auto_assign_or_convert(&gfit, ICC_ASSIGN_ON_LOAD);
 			set_cursor_waiting(FALSE);
 			if (retval == OPEN_IMAGE_CANCEL) goto wait;
 			break;
@@ -441,6 +469,10 @@ void on_flatlibfile_button_clicked(GtkButton *button, gpointer user_data) {
 	opendial(OD_FLATLIB);
 }
 
+void on_distolibfile_button_clicked(GtkButton *button, gpointer user_data) {
+	opendial(OD_DISTOLIB);
+}
+
 void header_open_button_clicked() {
 	opendial(OD_OPEN);
 }
@@ -470,10 +502,12 @@ void on_open_recent_action_item_activated(GtkRecentChooser *chooser,
 		g_warning("Could not convert uri \"%s\" to a local path: %s", uri,
 				error->message);
 		g_clear_error(&error);
+		g_free(uri);
 		return;
 	}
 
 	open_single_image(path);
+	icc_auto_assign_or_convert(&gfit, ICC_ASSIGN_ON_LOAD);
 
 	g_free(uri);
 	g_free(path);
