@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2025 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
  * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -168,7 +168,7 @@ gpointer gaia_check(gpointer user_data) {
     // Log results for all mirrors for transparency
     int working_mirrors = 0;
     for (int i = 0; i < num_mirrors; i++) {
-        if (response_times[i] != -1) {
+        if (response_times[i] > -1) {
             working_mirrors++;
             siril_debug_print("Mirror %s: %d ms\n", spcc_mirrors[i], response_times[i]);
             if (response_times[i] < best_responsetime) {
