@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,22 +22,27 @@
 #define SRC_GUI_PLOT_H_
 
 #include "core/siril.h"
+#include "kplot.h"
 
 void clear_all_photometry_and_plot();
 void reset_plot();
 void drawPlot();
 void notify_new_photometry();
 void init_plot_colors();
+void free_plot_data();
 
 /* for one curve */
 typedef struct plot_data_struct {
-	double *julian, *frame;
+	double *frame;
 	struct kpair *data, *err;
 	int nb;		// number of points in the plot (= number of images)
-
 	struct plot_data_struct *next;
 } pldata;
 
+typedef struct phot_data_struct {
+	int frame;
+	struct kpair data, err;
+} photdata_t;
 
 /* has to be the same as in the glade file */
 enum photometry_source {
@@ -120,4 +125,6 @@ typedef struct plot_draw_data {
 	int nbselected;
 } plot_draw_data_t;
 
+gboolean is_inside_grid(double x, double y, plot_draw_data_t *pddstruct);
+
 #endif /* SRC_GUI_PLOT_H_ */
