@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,12 +22,12 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/icc_profile.h"
 #include "algos/statistics.h"
 #include "algos/colors.h"
 #include "io/single_image.h"
 #include "io/image_format_fits.h"
 #include "io/sequence.h"
-#include "gui/image_display.h"
 #include "gui/utils.h"
 #include "gui/progress_and_log.h"
 #include "gui/dialogs.h"
@@ -38,19 +38,17 @@
 #include "gui/message_dialog.h"
 #include "gui/siril_preview.h"
 #include "gui/histogram.h"
-#include "core/undo.h"
 #include "core/arithm.h"
-#include "core/siril_app_dirs.h"
 #include "core/siril_log.h"
 #include "filters/ght.h"
 
-#include "gui/histogram.h"
 #include <gsl/gsl_histogram.h>
 
 // Invocation: 1 if called directly from starnet GUI,
 // 2 if called from Siril menu
 static int invocation = 0;
 
+static clip_mode_t clip_mode = RGBBLEND;
 static float leftD = 0.0f, rightD = 0.0f;
 static float leftB = 0.0f, rightB = 0.0f;
 static float leftLP = 0.0f, rightLP = 0.0f;
@@ -201,6 +199,8 @@ void display_remix_histo(gsl_histogram *histo, cairo_t *cr, int layer, int width
 			graph_height = bin_val;
 		current_bin++;
 	} while (i < nb_orig_bins && current_bin < nb_bins_allocated);
+	if (!graph_height)
+		return;
 	for (i = 0; i < nb_bins_allocated; i++) {
 		float bin_height = height - height * displayed_values[i] / graph_height;
 		cairo_line_to(cr, i, bin_height);
@@ -252,7 +252,7 @@ gboolean redraw_remix_histo_right(GtkWidget *widget, cairo_t *cr, gpointer data)
 static void update_remix_histo_left() {
 	if (!remix_histlayers_left[0]) return;
 	float norm = (float)gsl_histogram_bins(remix_histlayers_left[0]) - 1;
-	params_histo_left = (ght_params) { leftB, leftD, leftLP, leftSP, leftHP, leftBP, type_left, colour_left, TRUE, TRUE, TRUE };
+	params_histo_left = (ght_params) { leftB, leftD, leftLP, leftSP, leftHP, leftBP, type_left, colour_left, TRUE, TRUE, TRUE, clip_mode };
 
 	GHTsetup(&cp_histo_left, params_histo_left.B, params_histo_left.D, params_histo_left.LP, params_histo_left.SP, params_histo_left.HP, params_histo_left.stretchtype);
 	for (size_t i = 0; i < fit_left.naxes[2]; i++) {
@@ -271,7 +271,7 @@ static void update_remix_histo_left() {
 static void update_remix_histo_right() {
 	if (!remix_histlayers_right[0]) return;
 	float norm = (float)gsl_histogram_bins(remix_histlayers_right[0]) - 1;
-	params_histo_right = (ght_params) { rightB, rightD, rightLP, rightSP, rightHP, rightBP, type_right, colour_right, TRUE, TRUE, TRUE };
+	params_histo_right = (ght_params) { rightB, rightD, rightLP, rightSP, rightHP, rightBP, type_right, colour_right, TRUE, TRUE, TRUE, clip_mode };
 
 	int nlayers = fit_right.naxes[2];
 	GHTsetup(&cp_histo_right, params_histo_right.B, params_histo_right.D, params_histo_right.LP, params_histo_right.SP, params_histo_right.HP, params_histo_right.stretchtype);
@@ -388,14 +388,14 @@ void initialise_image() {
 	update_prepro_interface(TRUE);
 	adjust_sellabel();
 	display_filename();	// display filename in gray window
-	set_precision_switch(); // set precision on screen
+	gui_function(set_precision_switch, NULL); // set precision on screen
 	/* update menus */
-	update_MenuItem();
-	close_tab();
-	init_right_tab();
+	gui_function(update_MenuItem, NULL);
+	gui_function(close_tab, NULL);
+	gui_function(init_right_tab, NULL);
 }
 
-gboolean check_images_match(fits *fit1, fits *fit2) {
+static gboolean check_images_match(const fits *fit1, const fits *fit2) {
 	if (fit1->rx != fit2->rx) return FALSE;
 	if (fit1->ry != fit2->ry) return FALSE;
 	if (fit1->type != fit2->type) return FALSE;
@@ -465,8 +465,8 @@ int remixer() {
 	if(!permit_calculation)
 		return 1;
 
-	params_left = (ght_params) { leftB, leftD, leftLP, leftSP, leftHP, leftBP, type_left, colour_left, TRUE, TRUE, TRUE };
-	params_right = (ght_params) { rightB, rightD, rightLP, rightSP, rightHP, rightBP, type_right, colour_right, TRUE, TRUE, TRUE };
+	params_left = (ght_params) { leftB, leftD, leftLP, leftSP, leftHP, leftBP, type_left, colour_left, TRUE, TRUE, TRUE, clip_mode };
+	params_right = (ght_params) { rightB, rightD, rightLP, rightSP, rightHP, rightBP, type_right, colour_right, TRUE, TRUE, TRUE, clip_mode };
 
 	// Process left image
 	if (left_loaded && (left_changed || leftBP_changed)) {
@@ -507,13 +507,17 @@ int remixer() {
 	if (gfit.naxes[2] == 1) {
 		switch (gfit.type) {
 			case DATA_FLOAT:
-				for (size_t i = 0 ; i < npixels ; i++) {
-					gfit.fdata[i] = fit_left_calc.fdata[i] + fit_right_calc.fdata[i] - fit_left_calc.fdata[i] * fit_right_calc.fdata[i];
+				if (gfit.fdata) {
+					for (size_t i = 0 ; i < npixels ; i++) {
+						gfit.fdata[i] = fit_left_calc.fdata[i] + fit_right_calc.fdata[i] - fit_left_calc.fdata[i] * fit_right_calc.fdata[i];
+					}
 				}
 				break;
 			case DATA_USHORT:
-				for (size_t i = 0 ; i < npixels ; i++) {
-					gfit.data[i] = roundf_to_WORD(norm * (1.f - (1.f - (fit_left_calc.data[i]*invnorm))*(1.f - (fit_right_calc.data[i]*invnorm))));
+				if (gfit.data) {
+					for (size_t i = 0 ; i < npixels ; i++) {
+						gfit.data[i] = roundf_to_WORD(norm * (1.f - (1.f - (fit_left_calc.data[i]*invnorm))*(1.f - (fit_right_calc.data[i]*invnorm))));
+					}
 				}
 				break;
 			default:
@@ -522,92 +526,27 @@ int remixer() {
 	} else {
 		switch (gfit.type) {
 			case DATA_FLOAT:
-#ifdef _OPENMP
-#pragma omp parallel for num_threads(com.max_thread) schedule(static)
-#endif
-				for (size_t i = 0 ; i < npixels ; i++) {
-					float rinl, ginl, binl, rinr, ginr, binr, xl, yl, zl, xr, yr, zr;
-					float Ll, Al, Bl, Lr, Ar, Br, xo, yo, zo, rout, gout, bout;
-					if (left_loaded) {
-						rinl = fit_left_calc.fpdata[0][i];
-						ginl = fit_left_calc.fpdata[1][i];
-						binl = fit_left_calc.fpdata[2][i];
-					} else {
-						rinl = 0.f;
-						ginl = 0.f;
-						binl = 0.f;
-					}
-					if (right_loaded) {
-						rinr = fit_right_calc.fpdata[0][i];
-						ginr = fit_right_calc.fpdata[1][i];
-						binr = fit_right_calc.fpdata[2][i];
-					} else {
-						rinr = 0.f;
-						ginr = 0.f;
-						binr = 0.f;
-					}
-					linrgb_to_xyzf(rinl, ginl, binl, &xl, &yl, &zl, TRUE);
-					xyz_to_LABf(xl, yl, zl, &Ll, &Al, &Bl);
-					linrgb_to_xyzf(rinr, ginr, binr, &xr, &yr, &zr, TRUE);
-					xyz_to_LABf(xr, yr, zr, &Lr, &Ar, &Br);
-					float divisor = (Ll + Lr == 0.f) ? 1.f : Ll + Lr;
-					float ao = (Ll * Al + Lr * Ar) / divisor;
-					float bo = (Ll * Bl + Lr * Br) / divisor;
-					float Lo = Ll + Lr - Ll * Lr * 0.01f;
-					LAB_to_xyzf(Lo, ao, bo, &xo, &yo, &zo);
-					xyz_to_linrgbf(xo, yo, zo, &rout, &gout, &bout, TRUE);
-					gfit.fpdata[0][i] = rout;
-					gfit.fpdata[1][i] = gout;
-					gfit.fpdata[2][i] = bout;
+				if (left_loaded && !right_loaded) {
+					memcpy(gfit.fdata, fit_left_calc.fdata, npixels * 3 * sizeof(float));
+				} else if (right_loaded && !left_loaded) {
+					memcpy(gfit.fdata, fit_right_calc.fdata, npixels * 3 * sizeof(float));
+				} else {
+					screen(&fit_left_calc, &fit_right_calc, &gfit, !com.pref.force_16bit, com.max_thread);
 				}
 				break;
 			case DATA_USHORT:
-#ifdef _OPENMP
-#pragma omp parallel for num_threads(com.max_thread) schedule(static)
-#endif
-				for (size_t i = 0 ; i < npixels ; i++) {
-					float rinl, ginl, binl, rinr, ginr, binr, xl, yl, zl, xr, yr, zr;
-					float Ll, Al, Bl, Lr, Ar, Br, xo, yo, zo, rout, gout, bout;
-					if (left_loaded) {
-						rinl = fit_left_calc.pdata[0][i] * invnorm;
-						ginl = fit_left_calc.pdata[1][i] * invnorm;
-						binl = fit_left_calc.pdata[2][i] * invnorm;
-					} else {
-						rinl = 0.f;
-						ginl = 0.f;
-						binl = 0.f;
-					}
-					if (right_loaded) {
-						rinr = fit_right_calc.pdata[0][i] * invnorm;
-						ginr = fit_right_calc.pdata[1][i] * invnorm;
-						binr = fit_right_calc.pdata[2][i] * invnorm;
-					} else {
-						rinr = 0.f;
-						ginr = 0.f;
-						binr = 0.f;
-					}
-					linrgb_to_xyzf(rinl, ginl, binl, &xl, &yl, &zl, TRUE);
-					xyz_to_LABf(xl, yl, zl, &Ll, &Al, &Bl);
-					linrgb_to_xyzf(rinr, ginr, binr, &xr, &yr, &zr, TRUE);
-					xyz_to_LABf(xr, yr, zr, &Lr, &Ar, &Br);
-					float divisor = (Ll + Lr == 0.f) ? 1.f : Ll + Lr;
-					float ao = (Ll * Al + Lr * Ar) / divisor;
-					float bo = (Ll * Bl + Lr * Br) / divisor;
-					float Lo = Ll + Lr - Ll * Lr * 0.01f;
-					LAB_to_xyzf(Lo, ao, bo, &xo, &yo, &zo);
-					xyz_to_linrgbf(xo, yo, zo, &rout, &gout, &bout, TRUE);
-					gfit.pdata[0][i] = roundf_to_WORD(rout * norm);
-					gfit.pdata[1][i] = roundf_to_WORD(gout * norm);
-					gfit.pdata[2][i] = roundf_to_WORD(bout * norm);
+				if (left_loaded && !right_loaded) {
+					memcpy(gfit.data, fit_left_calc.data, npixels * 3 * sizeof(WORD));
+				} else if (right_loaded && !left_loaded) {
+					memcpy(gfit.data, fit_right_calc.data, npixels * 3 * sizeof(WORD));
+				} else {
+					screen(&fit_left_calc, &fit_right_calc, &gfit, !com.pref.force_16bit, com.max_thread);
 				}
 				break;
 			default:
 				break;
 		}
 	}
-	// If 16bit preference is set, check the images are 16bit
-	if (com.pref.force_16bit && gfit.type == DATA_FLOAT)
-		fit_replace_buffer(&gfit, float_buffer_to_ushort(gfit.fdata, ndata), DATA_USHORT);
 
 	notify_gfit_modified();
 
@@ -701,13 +640,13 @@ static void remixer_close() {
 	gtk_button_set_label(GTK_BUTTON(lookup_widget("remix_advanced")), _("Advanced"));
 	gtk_widget_set_tooltip_text(lookup_widget("remix_advanced"), _("Show advanced stretch options."));
 	advanced_interface = FALSE;
+
 	siril_close_dialog("dialog_star_remix");
 }
 
 void apply_remix_cancel() {
-	set_cursor_waiting(TRUE);
 	remixer_close();
-	if (left_loaded || right_loaded) {
+	if (left_loaded || right_loaded || (!single_image_is_loaded() && !sequence_is_loaded())) {
 		close_single_image();
 	}
 	set_cursor_waiting(FALSE);
@@ -716,6 +655,8 @@ void apply_remix_cancel() {
 /*** callbacks **/
 
 void on_dialog_star_remix_show(GtkWidget *widget, gpointer user_data) {
+	siril_set_file_filter(GTK_FILE_CHOOSER(lookup_widget("remix_filechooser_left")), "filefilter_fits", "FITS files");
+	siril_set_file_filter(GTK_FILE_CHOOSER(lookup_widget("remix_filechooser_right")), "filefilter_fits", "FITS files");
 	remixer_startup();
 	reset_controls_and_values();
 	remixer_show_preview = TRUE;
@@ -732,6 +673,7 @@ int toggle_remixer_window_visibility(int _invocation, fits* _fit_left, fits* _fi
 	if (gtk_widget_get_visible(lookup_widget("dialog_star_remix"))) {
 		set_cursor_waiting(TRUE);
 		reset_controls_and_values();
+
 		remixer_close();
 		set_cursor_waiting(FALSE);
 		siril_close_dialog("dialog_star_remix");
@@ -748,7 +690,6 @@ int toggle_remixer_window_visibility(int _invocation, fits* _fit_left, fits* _fi
 			copy_fits_metadata(_fit_left, &fit_left);
 			clearfits(_fit_left);
 			free(_fit_left);
-			_fit_left = NULL;
 			close_histograms(TRUE, TRUE);
 			remix_histo_startup_left();
 			copyfits(&fit_left, &fit_left_calc, (CP_ALLOC | CP_INIT | CP_FORMAT), 0);
@@ -762,16 +703,18 @@ int toggle_remixer_window_visibility(int _invocation, fits* _fit_left, fits* _fi
 			remix_histo_startup_right();
 			clearfits(_fit_right);
 			free(_fit_right);
-			_fit_right = NULL;
 			copyfits(&fit_right, &fit_right_calc, (CP_ALLOC | CP_INIT | CP_FORMAT), 0);
 			right_loaded = TRUE; // Mark RHS image as loaded
 			right_changed = TRUE; // Force update on initial draw
-			merge_fits_headers_to_result(&gfit, &fit_left, &fit_right, NULL);
+			merge_fits_headers_to_result(&gfit, FALSE, &fit_left, &fit_right, NULL);
 			// Avoid doubling STACKCNT and LIVETIME as we are merging starless and star parts of a single image
-			gfit.stackcnt = fit_left.stackcnt;
-			gfit.livetime = fit_left.livetime;
+			gfit.keywords.stackcnt = fit_left.keywords.stackcnt;
+			gfit.keywords.livetime = fit_left.keywords.livetime;
+			// Update the header
+			update_fits_header(&gfit);
 			initialise_image();
-
+			GtkWidget *clip = lookup_widget("remixer_clip_mode_settings");
+			gtk_widget_set_visible(clip, (fit_left.naxes[2] == 3));
 			gtk_widget_set_visible(lookup_widget("remix_filechooser_left"), FALSE);
 			gtk_widget_set_visible(lookup_widget("remix_filechooser_right"), FALSE);
 			update_image *param = malloc(sizeof(update_image));
@@ -786,22 +729,18 @@ int toggle_remixer_window_visibility(int _invocation, fits* _fit_left, fits* _fi
 			gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(lookup_widget("remix_filechooser_right")), com.wd);
 		}
 		// Set eyedropper icons to light or dark according to theme
-		gchar *image;
-		GtkWidget *v, *w;
+		GtkWidget *v = NULL, *w = NULL;
 		if (com.pref.gui.combo_theme == 0) {
-			image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "eyedropper_dark.svg", NULL);
-			v = gtk_image_new_from_file(image);
-			w = gtk_image_new_from_file(image);
+			v = gtk_image_new_from_resource("/org/siril/ui/pixmaps/eyedropper_dark.svg");
+			w = gtk_image_new_from_resource("/org/siril/ui/pixmaps/eyedropper_dark.svg");
 		} else {
-			image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "eyedropper.svg", NULL);
-			v = gtk_image_new_from_file(image);
-			w = gtk_image_new_from_file(image);
+			v = gtk_image_new_from_resource("/org/siril/ui/pixmaps/eyedropper.svg");
+			w = gtk_image_new_from_resource("/org/siril/ui/pixmaps/eyedropper.svg");
 		}
 		gtk_button_set_image(GTK_BUTTON(lookup_widget("eyedropper_SP_left")), v);
 		gtk_button_set_image(GTK_BUTTON(lookup_widget("eyedropper_SP_right")), w);
 		gtk_widget_show(v);
 		gtk_widget_show(w);
-		g_free(image);
 
 		// Hide the advanced widgets, these can be show using the Advanced button for full control
 		gtk_widget_set_visible(lookup_widget("ghtStretchTypecontrols3"), FALSE);
@@ -819,10 +758,11 @@ int toggle_remixer_window_visibility(int _invocation, fits* _fit_left, fits* _fi
 	return 0;
 }
 
-void on_remix_close_clicked(GtkButton *button, gpointer user_data) {
+gboolean on_remix_close_clicked(GtkButton *button, gpointer user_data) {
 	close_histograms(TRUE, TRUE);
 	remixer_close();
 	set_cursor_waiting(FALSE);
+	return FALSE;
 }
 
 void on_remix_reset_left_clicked(GtkButton *button, gpointer user_data) {
@@ -918,8 +858,7 @@ void on_spin_remix_B_right_value_changed(GtkSpinButton *button, gpointer user_da
 	if (fabsf(rightB) < 1.e-3f) {
 		rightB = 0.f;
 		gtk_spin_button_set_value(button, 0.f);
-	}
-	update_remix_histo_right();
+	}	update_remix_histo_right();
 	update_image *param = malloc(sizeof(update_image));
 	param->update_preview_fn = remixer_update_preview;
 	param->show_preview = TRUE;
@@ -1109,7 +1048,7 @@ void on_remix_filechooser_left_file_set(GtkFileChooser *filechooser, gpointer us
 		clearfits(&fit_left_calc);
 		left_loaded = FALSE;
 	}
-	filename_left = g_strdup(siril_file_chooser_get_filename(filechooser));
+	filename_left = siril_file_chooser_get_filename(filechooser);
 	if (readfits(filename_left, &fit_left, NULL, FALSE)) {
 		siril_message_dialog( GTK_MESSAGE_ERROR, _("Error: image could not be loaded"),
 			_("Image loading failed"));
@@ -1117,10 +1056,6 @@ void on_remix_filechooser_left_file_set(GtkFileChooser *filechooser, gpointer us
 		right_loaded = FALSE;
 		return;
 	}
-	if(fit_left.type == DATA_FLOAT && com.pref.force_16bit) {
-		const size_t ndata = fit_left.naxes[0] * fit_left.naxes[1] * fit_left.naxes[2];
-		fit_replace_buffer(&fit_left, float_buffer_to_ushort(fit_left.fdata, ndata), DATA_USHORT);
-	}
 	if (right_loaded) {
 		if(!check_images_match(&fit_left, &fit_right)) {
 			siril_message_dialog( GTK_MESSAGE_ERROR, _("Error: images do not match"),
@@ -1132,23 +1067,34 @@ void on_remix_filechooser_left_file_set(GtkFileChooser *filechooser, gpointer us
 		}
 		else {
 			left_loaded = TRUE;
-			merge_fits_headers_to_result(&gfit, &fit_left, &fit_right, NULL);
-			if (fit_left.filter[0] != '\0' && fit_right.filter[0] != '\0' && strlen(fit_left.filter) >= 8 && strlen(fit_right.filter) >= 8) {
-				gchar *temp_l = g_malloc(strlen(fit_left.filter) - 7);
-				g_strlcpy(temp_l, fit_left.filter, strlen(fit_left.filter) - 8);
-				gchar *temp_r = g_malloc(strlen(fit_right.filter) - 7);
-				g_strlcpy(temp_r, fit_right.filter, strlen(fit_right.filter) - 8);
-				if (strlen(fit_left.filter) == 8)
+			/* We are the second image to be loaded so we convert to the first
+			* image's color profile */
+			if (!profiles_identical(fit_left.icc_profile, fit_right.icc_profile)) {
+				if (fit_right.icc_profile) {
+					siril_colorspace_transform(&fit_left, fit_right.icc_profile);
+					siril_log_message(_("Color profiles did not match: left-hand image has been converted to right-hand image color profile.\n"));
+				} else {
+					fit_right.icc_profile = copyICCProfile(fit_left.icc_profile);
+					siril_log_message(_("Color profiles did not match: right-hand image has been assigned the left-hand image color profile.\n"));
+				}
+			}
+			merge_fits_headers_to_result(&gfit, FALSE, &fit_left, &fit_right, NULL);
+			if (fit_left.keywords.filter[0] != '\0' && fit_right.keywords.filter[0] != '\0' && strlen(fit_left.keywords.filter) >= 8 && strlen(fit_right.keywords.filter) >= 8) {
+				gchar *temp_l = g_malloc(strlen(fit_left.keywords.filter) - 7);
+				g_strlcpy(temp_l, fit_left.keywords.filter, strlen(fit_left.keywords.filter) - 8);
+				gchar *temp_r = g_malloc(strlen(fit_right.keywords.filter) - 7);
+				g_strlcpy(temp_r, fit_right.keywords.filter, strlen(fit_right.keywords.filter) - 8);
+				if (strlen(fit_left.keywords.filter) == 8)
 					temp_l[0] = '\0';
-				if (strlen(fit_right.filter) == 8)
+				if (strlen(fit_right.keywords.filter) == 8)
 					temp_r[0] = '\0';
 				if (!strcmp(temp_l, temp_r)) {
-					if (fit_left.livetime >= fit_right.livetime) {
-						gfit.livetime = fit_left.livetime;
-						gfit.stackcnt = fit_left.stackcnt;
+					if (fit_left.keywords.livetime >= fit_right.keywords.livetime) {
+						gfit.keywords.livetime = fit_left.keywords.livetime;
+						gfit.keywords.stackcnt = fit_left.keywords.stackcnt;
 					} else {
-						gfit.livetime = fit_right.livetime;
-						gfit.stackcnt = fit_right.stackcnt;
+						gfit.keywords.livetime = fit_right.keywords.livetime;
+						gfit.keywords.stackcnt = fit_right.keywords.stackcnt;
 					}
 				}
 				g_free(temp_l);
@@ -1157,16 +1103,20 @@ void on_remix_filechooser_left_file_set(GtkFileChooser *filechooser, gpointer us
 			  // value to use as the image is probably being combined from 2 different stacks
 		}
 	} else {
+		check_profile_correct(&fit_left);
 		close_single_image();
 		close_sequence(FALSE);
 		clearfits(&gfit);
 		copyfits(&fit_left, &gfit, (CP_ALLOC | CP_COPYA | CP_FORMAT), 0);
+		siril_log_message(_("Setting the output image ICC profile to the working color space.\n"));
 		initialise_image();
 		left_loaded = TRUE;
 		clearfits(&fit_right_calc);
 		copyfits(&fit_left, &fit_right_calc, (CP_ALLOC | CP_INIT | CP_FORMAT), 0);
 	}
 	remix_histo_startup_left();
+	GtkWidget *clip = lookup_widget("remixer_clip_mode_settings");
+	gtk_widget_set_visible(clip, (fit_left.naxes[2] == 3));
 	if (left_loaded || right_loaded)
 		permit_calculation = TRUE;
 	else
@@ -1187,7 +1137,7 @@ void on_remix_filechooser_right_file_set(GtkFileChooser *filechooser, gpointer u
 		clearfits(&fit_right_calc);
 		right_loaded = FALSE;
 	}
-	filename_right = g_strdup(siril_file_chooser_get_filename(filechooser));
+	filename_right = siril_file_chooser_get_filename(filechooser);
 	if (readfits(filename_right, &fit_right, NULL, FALSE)) {
 		siril_message_dialog( GTK_MESSAGE_ERROR, _("Error: image could not be loaded"),
 				_("Image loading failed"));
@@ -1195,10 +1145,6 @@ void on_remix_filechooser_right_file_set(GtkFileChooser *filechooser, gpointer u
 		right_loaded = FALSE;
 		return;
 	}
-	if(fit_right.type == DATA_FLOAT && com.pref.force_16bit) {
-		const size_t ndata = fit_right.naxes[0] * fit_right.naxes[1] * fit_right.naxes[2];
-		fit_replace_buffer(&fit_right, float_buffer_to_ushort(fit_right.fdata, ndata), DATA_USHORT);
-	}
 	if (left_loaded) {
 		if(!check_images_match(&fit_left, &fit_right)) {
 			siril_message_dialog( GTK_MESSAGE_ERROR, _("Error: images do not match"),
@@ -1210,23 +1156,38 @@ void on_remix_filechooser_right_file_set(GtkFileChooser *filechooser, gpointer u
 		}
 		else {
 			right_loaded = TRUE;
-			merge_fits_headers_to_result(&gfit, &fit_left, &fit_right, NULL);
-			if (fit_left.filter[0] != '\0' && fit_right.filter[0] != '\0' && strlen(fit_left.filter) >= 8 && strlen(fit_right.filter) >= 8) {
-				gchar *temp_l = g_malloc(strlen(fit_left.filter) - 7);
-				g_strlcpy(temp_l, fit_left.filter, strlen(fit_left.filter) - 8);
-				gchar *temp_r = g_malloc(strlen(fit_right.filter) - 7);
-				g_strlcpy(temp_r, fit_right.filter, strlen(fit_right.filter) - 8);
-				if (strlen(fit_left.filter) == 8)
+			/* We are the second image to be loaded so we convert to the first
+			* image's color profile */
+			if (!profiles_identical(fit_left.icc_profile, fit_right.icc_profile)) {
+				if (fit_left.icc_profile) {
+					siril_colorspace_transform(&fit_right, fit_left.icc_profile);
+					siril_log_message(_("Color profiles did not match: right-hand image has been converted to left-hand image color profile.\n"));
+				} else {
+					fit_left.icc_profile = copyICCProfile(fit_right.icc_profile);
+					siril_log_message(_("Color profiles did not match: left-hand image has been assigned the right-hand image color profile.\n"));
+				}
+			}
+			if (!profiles_identical(fit_left.icc_profile, fit_right.icc_profile)) {
+				siril_colorspace_transform(&fit_right, fit_left.icc_profile);
+				siril_log_message(_("Color profiles did not match: right-hand image has been converted to left-hand image color profile.\n"));
+			}
+			merge_fits_headers_to_result(&gfit, FALSE, &fit_left, &fit_right, NULL);
+			if (fit_left.keywords.filter[0] != '\0' && fit_right.keywords.filter[0] != '\0' && strlen(fit_left.keywords.filter) >= 8 && strlen(fit_right.keywords.filter) >= 8) {
+				gchar *temp_l = g_malloc(strlen(fit_left.keywords.filter) - 7);
+				g_strlcpy(temp_l, fit_left.keywords.filter, strlen(fit_left.keywords.filter) - 8);
+				gchar *temp_r = g_malloc(strlen(fit_right.keywords.filter) - 7);
+				g_strlcpy(temp_r, fit_right.keywords.filter, strlen(fit_right.keywords.filter) - 8);
+				if (strlen(fit_left.keywords.filter) == 8)
 					temp_l[0] = '\0';
-				if (strlen(fit_right.filter) == 8)
+				if (strlen(fit_right.keywords.filter) == 8)
 					temp_r[0] = '\0';
 				if (!strcmp(temp_l, temp_r)) {
-					if (fit_left.livetime >= fit_right.livetime) {
-						gfit.livetime = fit_left.livetime;
-						gfit.stackcnt = fit_left.stackcnt;
+					if (fit_left.keywords.livetime >= fit_right.keywords.livetime) {
+						gfit.keywords.livetime = fit_left.keywords.livetime;
+						gfit.keywords.stackcnt = fit_left.keywords.stackcnt;
 					} else {
-						gfit.livetime = fit_right.livetime;
-						gfit.stackcnt = fit_right.stackcnt;
+						gfit.keywords.livetime = fit_right.keywords.livetime;
+						gfit.keywords.stackcnt = fit_right.keywords.stackcnt;
 					}
 				}
 				g_free(temp_l);
@@ -1235,6 +1196,7 @@ void on_remix_filechooser_right_file_set(GtkFileChooser *filechooser, gpointer u
 			  // value to use as the image is probably being combined from 2 different stacks
 		}
 	} else {
+		check_profile_correct(&fit_right);
 		close_single_image();
 		close_sequence(FALSE);
 		clearfits(&gfit);
@@ -1245,6 +1207,8 @@ void on_remix_filechooser_right_file_set(GtkFileChooser *filechooser, gpointer u
 		copyfits(&fit_right, &fit_left_calc, (CP_ALLOC | CP_INIT | CP_FORMAT), 0);
 	}
 	remix_histo_startup_right();
+	GtkWidget *clip = lookup_widget("remixer_clip_mode_settings");
+	gtk_widget_set_visible(clip, (fit_right.naxes[2] == 3));
 	if (left_loaded || right_loaded)
 		permit_calculation = TRUE;
 	else
@@ -1408,3 +1372,15 @@ void on_remixer_help_okay_clicked(GtkButton *button, gpointer user_data) {
 void on_remix_histos_button_clicked(GtkButton *button, gpointer user_data) {
 	siril_open_dialog("remixer_histos");
 }
+
+void on_remixer_clipmode_changed(GtkComboBox *combo, gpointer user_data) {
+	left_changed = TRUE;
+	right_changed = TRUE;
+	clip_mode = (clip_mode_t) gtk_combo_box_get_active(combo);
+	update_remix_histo_left();
+	update_remix_histo_right();
+	update_image *param = malloc(sizeof(update_image));
+	param->update_preview_fn = remixer_update_preview;
+	param->show_preview = TRUE;
+	notify_update((gpointer) param);
+}
