@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,21 +23,20 @@
 #endif
 #include "core/siril.h"
 #include "core/proto.h"
-#include "core/siril_app_dirs.h"
 #include "core/siril_log.h"
 #include "gui/utils.h"
 #include "gui/callbacks.h"
-#include "gui/dialogs.h"
 #include "gui/message_dialog.h"
 #include "gui/image_display.h"
 #include "gui/progress_and_log.h"
 #include "gui/registration_preview.h"
 #include "gui/plot.h"
+#include "gui/registration.h"	// for update_reg_interface
+#include "gui/stacking.h"	// for update_stack_interface
 #include "io/sequence.h"
 #include "io/image_format_fits.h"
 #include "algos/PSF.h"
-#include "registration/registration.h"	// for update_reg_interface
-#include "stacking/stacking.h"	// for update_stack_interface
+#include "registration/registration.h"
 #include <dirent.h>
 
 #include "sequence_list.h"
@@ -196,16 +195,16 @@ static void add_image_to_sequence_list(sequence *seq, int index, int layer) {
 			switch (selected_source) {
 				case r_FWHM:
 					if (is_arcsec) {
-						bin = com.pref.binning_update ? (double) gfit.binning_x : 1.0;
-						convert_single_fwhm_to_arcsec_if_possible(seq->regparam[layer][index].fwhm, bin, (double) gfit.pixel_size_x, gfit.focal_length, &fwhm);
+						bin = com.pref.binning_update ? (double) gfit.keywords.binning_x : 1.0;
+						convert_single_fwhm_to_arcsec_if_possible(seq->regparam[layer][index].fwhm, bin, (double) gfit.keywords.pixel_size_x, gfit.keywords.focal_length, &fwhm);
 					} else {
 						fwhm = seq->regparam[layer][index].fwhm;
 					}
 					break;
 				case r_WFWHM:
 					if (is_arcsec) {
-						bin = com.pref.binning_update ? (double) gfit.binning_x : 1.0;
-						convert_single_fwhm_to_arcsec_if_possible(seq->regparam[layer][index].weighted_fwhm, bin, (double) gfit.pixel_size_x, gfit.focal_length, &fwhm);
+						bin = com.pref.binning_update ? (double) gfit.keywords.binning_x : 1.0;
+						convert_single_fwhm_to_arcsec_if_possible(seq->regparam[layer][index].weighted_fwhm, bin, (double) gfit.keywords.pixel_size_x, gfit.keywords.focal_length, &fwhm);
 					} else {
 						fwhm = seq->regparam[layer][index].weighted_fwhm;
 					}
@@ -429,7 +428,7 @@ void on_treeview1_cursor_changed(GtkTreeView *tree_view, gpointer user_data) {
 		if (idx != com.seq.current) {
 			fprintf(stdout, "loading image %d\n", idx);
 			if (seq_load_image(&com.seq, idx, TRUE)) // if loading fails, we fall back reloading the reference image
-				seq_load_image(&com.seq, com.seq.reference_image, TRUE); 
+				seq_load_image(&com.seq, com.seq.reference_image, TRUE);
 		}
 		g_value_unset(&value);
 	}
@@ -467,7 +466,7 @@ void fill_sequence_list(sequence *seq, int layer, gboolean as_idle) {
 	struct _seq_list *args;
 	if (seq == NULL || layer >= seq->nb_layers) return;
 
-	args = malloc(sizeof(struct _seq_list));
+	args = calloc(1, sizeof(struct _seq_list));
 	args->seq = seq;
 	args->layer = layer;
 	args->tview = GTK_TREE_VIEW(lookup_widget("treeview1"));
@@ -851,23 +850,19 @@ void sequence_list_select_row_from_index(int index, gboolean do_load_image) {
 
 	if (do_load_image) {
 		if (seq_load_image(&com.seq, index, TRUE)) // if loading fails, we fall back reloading the reference image
-			seq_load_image(&com.seq, com.seq.reference_image, TRUE); 
+			seq_load_image(&com.seq, com.seq.reference_image, TRUE);
 		update_reg_interface(FALSE);
 		redraw(REDRAW_OVERLAY);
 	}
 }
 
 void update_icons_sequence_list(gboolean is_dark) {
-	gchar *image;
 	GtkWidget *w;
 	if (is_dark) {
-		image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "frame_dark.svg", NULL);
-		w = gtk_image_new_from_file(image);
+		w = gtk_image_new_from_resource("/org/siril/ui/pixmaps/frame_dark.svg");
 	} else {
-		image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "frame.svg", NULL);
-		w = gtk_image_new_from_file(image);
+		w = gtk_image_new_from_resource("/org/siril/ui/pixmaps/frame.svg");
 	}
-	g_free(image);
 	gtk_button_set_image(GTK_BUTTON(GTK_TOGGLE_BUTTON(lookup_widget("drawframe_check"))), w);
 	gtk_widget_show(w);
 }
@@ -960,7 +955,7 @@ int update_sequences_list(const char *sequence_name_to_select) {
 #ifdef _WIN32
 	number_of_loaded_sequences = ListSequences(com.wd, seqname, seqcombo, &index_of_seq_to_load, &found);
 #else
-	struct dirent **list;
+	struct dirent **list = NULL;
 	int i, n;
 
 
@@ -1010,3 +1005,33 @@ int update_sequences_list(const char *sequence_name_to_select) {
 	return 0;
 }
 
+gboolean on_treeview1_query_tooltip(GtkWidget *widget, gint x, gint y,
+		gboolean keyboard_tip, GtkTooltip *tooltip, gpointer data) {
+	GtkTreeIter iter;
+	GtkTreeView *tree_view = GTK_TREE_VIEW(widget);
+	GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
+	GtkTreePath *path = NULL;
+
+	if (!sequence_is_loaded()) return FALSE;
+
+	if (!gtk_tree_view_get_tooltip_context(tree_view, &x, &y, keyboard_tip, &model, &path, &iter)) {
+		return FALSE;
+	}
+	char buffer[512];
+	fits fit = { 0 };
+	gint real_index = get_real_index_from_index_in_list(model, &iter);
+	if (seq_read_frame_metadata(&com.seq, real_index, &fit)) {
+		return FALSE;
+	}
+
+	if (fit.keywords.filename[0] == '\0') return FALSE;
+
+	g_snprintf(buffer, 511, "<b>Original filename:</b> %s", fit.keywords.filename);
+	clearfits(&fit);
+	gtk_tooltip_set_markup(tooltip, buffer);
+
+	gtk_tree_view_set_tooltip_row(tree_view, tooltip, path);
+	gtk_tree_path_free(path);
+
+	return TRUE;
+}
