@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,12 @@
 
 #include <gtk/gtk.h>
 
+typedef enum {
+	SIRIL_INTRO_GTK_POPOVER,
+	SIRIL_INTRO_WORKAROUND_POPOVER // this workaround is used when popover must be displayed on a child. popover is cropped, here we use a GtkDialog.
+} type_of_popover;
+
+
 typedef struct {
 	GtkWidget *popover;
 	GtkWidget *widget;
@@ -31,6 +37,7 @@ typedef struct {
 	const gchar *widget;
 	const gchar *tip;
 	const gint delay;
+	type_of_popover type;
 } SirilTipIntro;
 
 void start_intro_script();
