@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +21,10 @@
 #include "core/siril.h"
 #include "core/command.h"
 #include "core/command_line_processor.h"
+#include "core/processing.h"
 #include "algos/extraction.h"
 #include "io/sequence.h"
 #include "gui/dialogs.h"
-#include "gui/message_dialog.h"
 #include "gui/utils.h"
 #include "gui/progress_and_log.h"
 
@@ -38,63 +38,81 @@ void on_split_cfa_apply_clicked(GtkButton *button, gpointer user_data) {
 	gint method = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget("combo_split_cfa_method")));
 
 	if (gtk_toggle_button_get_active(seq) && sequence_is_loaded()) {
-		struct split_cfa_data *args = calloc(1, sizeof(struct split_cfa_data));
 
-		set_cursor_waiting(TRUE);
-		args->seq = &com.seq;
-		args->scaling = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget("combo_haoiii_scaling")));
-		args->seqEntry = strdup(gtk_entry_get_text(entrySplitCFA));
-		if ((com.seq.type == SEQ_SER || com.seq.type == SEQ_FITSEQ) && method == 0) {
-			siril_message_dialog( GTK_MESSAGE_ERROR, _("Error: sequence is SER or FITSEQ"),
-						_("Only FITS format is supported for sequence CFA splitting"));
-			free(args);
-			return;
-		}
-		switch (method) {
-			case 0:
-				if (args->seqEntry && args->seqEntry[0] == '\0')
-					args->seqEntry = strdup("CFA_");
-				apply_split_cfa_to_sequence(args);
-				break;
-			case 1:
-				if (args->seqEntry && args->seqEntry[0] == '\0')
-					args->seqEntry = strdup("Ha_");
-				apply_extractHa_to_sequence(args);
-				break;
-			case 2:
-				apply_extractHaOIII_to_sequence(args);
-				break;
-			case 3:
-				if (args->seqEntry && args->seqEntry[0] == '\0')
-					args->seqEntry = strdup("Green_");
-				apply_extractGreen_to_sequence(args);
-				break;
-			default:
-				fprintf(stderr, "unhandled case!\n");
-				free(args);
+		if (method == 0 || method == 2) {
+			struct multi_output_data *args = calloc(1, sizeof(struct multi_output_data));
+			set_cursor_waiting(TRUE);
+			args->seq = &com.seq;
+			args->user_data = calloc(1, sizeof(extraction_scaling));
+			*(extraction_scaling *)args->user_data = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget("combo_haoiii_scaling")));
+			args->seqEntry = strdup(gtk_entry_get_text(entrySplitCFA));
+
+			switch (method) {
+				case 0:
+					args->n = 4;
+					args->prefixes = calloc(5, sizeof(const char*));
+					if (!args->seqEntry || (args->seqEntry && args->seqEntry[0] == '\0')) {
+						free(args->seqEntry);
+						args->seqEntry = strdup("CFA");
+					}
+					size_t len = strlen(args->seqEntry);
+					// Strip any trailing '_' in order to insert the frame number
+					if (len > 0 && args->seqEntry[len - 1] == '_') {
+						args->seqEntry[len - 1] = '\0';  // Replace trailing '_' with null terminator
+					}
+					for (int i = 0 ; i < 4 ; i++) {
+						args->prefixes[i] = g_strdup_printf("%s%d_", args->seqEntry, i);
+					}
+					apply_split_cfa_to_sequence(args);
+					break;
+				case 2:
+					args->n = 2;
+					args->prefixes = calloc(3, sizeof(const char*));
+					args->prefixes[0] = g_strdup("Ha_");
+					args->prefixes[1] = g_strdup("OIII_");
+					apply_extractHaOIII_to_sequence(args);
+					break;
+			}
+		} else {
+			struct simple_extract_data *args = calloc(1, sizeof(struct simple_extract_data));
+			set_cursor_waiting(TRUE);
+			args->seq = &com.seq;
+			args->scaling = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget("combo_haoiii_scaling")));
+			args->seqEntry = strdup(gtk_entry_get_text(entrySplitCFA));
+
+			switch (method) {
+				case 1:
+					if (args->seqEntry && args->seqEntry[0] == '\0') {
+						free(args->seqEntry);
+						args->seqEntry = strdup("Ha_");
+					}
+					apply_extractHa_to_sequence(args);
+					break;
+				case 3:
+					if (args->seqEntry && args->seqEntry[0] == '\0') {
+						free(args->seqEntry);
+						args->seqEntry = strdup("Green_");
+					}
+					apply_extractGreen_to_sequence(args);
+					break;
+				default:
+					siril_debug_print("unhandled case!\n");
+					free(args->seqEntry);
+					free(args);
+			}
 		}
 	} else {
 		int scaling = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget("combo_haoiii_scaling")));
-		fprintf(stdout,"Scaling %d\n",scaling);
+		siril_debug_print("Scaling %d\n",scaling);
 		switch (method) {
 			case 0:
 				process_split_cfa(0);
 				break;
 			case 1:
-				process_extractHa(0);
+				extract_Ha(scaling);
 				break;
 			case 2:
-				switch (scaling) {
-					case 0:
-						processcommand("extract_HaOIII");
-						break;
-					case 1:
-						processcommand("extract_HaOIII -resample=ha");
-						break;
-					case 2:
-						processcommand("extract_HaOIII -resample=oiii");
-						break;
-				}
+				extract_HaOIII(scaling);
 				break;
 			case 3:
 				process_extractGreen(0);
