@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,10 +29,12 @@
 #include <string.h>
 
 #include "core/siril.h"
+#include "algos/siril_random.h"
 #include "core/siril_log.h"
 #include "gui/progress_and_log.h"
 #include "io/films.h"
 #include "io/image_format_fits.h"
+#include "core/icc_profile.h"
 
 static int pixfmt_gray, pixfmt_rgb, pixfmt_gray16, pixfmt_rgb48;
 
@@ -215,7 +217,7 @@ int film_open_file(const char *sourcefile, struct film_struct *film) {
 }
 
 static int randPixel(int nb_pixels) {
-	return rand() % nb_pixels;
+	return siril_random_uint() % nb_pixels;
 }
 
 static int *randomIndex(int n) {
@@ -326,7 +328,7 @@ int film_read_frame(struct film_struct *film, int frame_no, fits *fit) {
 		fit->pdata[BLAYER] = fit->data + nb_pixels * 2;
 	}
 	fit->bitpix = fit->orig_bitpix = BYTE_IMG;
-	g_snprintf(fit->row_order, FLEN_VALUE, "%s", "TOP-DOWN");
+	g_snprintf(fit->keywords.row_order, FLEN_VALUE, "%s", "TOP-DOWN");
 	//fit->mini = 0;
 	//fit->maxi = 255;
 	/* putting this above also requires the max[*] to be = 255. Besides, this overrides the
@@ -357,6 +359,12 @@ int film_read_frame(struct film_struct *film, int frame_no, fits *fit) {
 	}
 	fits_flip_top_to_bottom(fit);
 
+/* Film sequences are not color managed. Any sequence operations that rely on
+ * colorspace conversion (usually xRGB to xyz) will use a sensible default profile.
+ */
+	fit->icc_profile = NULL;
+	color_manage(fit, FALSE);
+
 	return FILM_SUCCESS;
 }
 
