@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +25,8 @@
  */
 
 #include "core/siril.h"
-
+#include "core/proto.h"
+#include "core/icc_profile.h"
 #include "io/image_format_fits.h"
 #include "gui/progress_and_log.h"
 #include "core/siril_log.h"
@@ -33,7 +34,7 @@
 #include "fits_sequence.h"
 
 static int fitseq_write_image_for_writer(struct seqwriter_data *writer, fits *image, int index);
-static int fitseq_prepare_for_multiple_read(fitseq *fitseq);
+static int fitseq_prepare_for_multiple_read(fitseq *fitseq, int iomode);
 static int fitseq_multiple_close(fitseq *fitseq);
 
 static int _find_hdus(fitsfile *fptr, int **hdus, int *nb_im) {
@@ -64,6 +65,15 @@ static int _find_hdus(fitsfile *fptr, int **hdus, int *nb_im) {
 
 		if (type != IMAGE_HDU) continue;
 
+		// Skip image HDUs named as ICC profiles or thumbnails
+		char extname[FLEN_VALUE], comment[FLEN_COMMENT];
+		int status2 = 0;
+		fits_read_key(fptr, TSTRING, "EXTNAME", &extname, comment, &status2);
+		if (g_str_has_prefix(extname, "ICCProfile")
+			|| g_str_has_prefix(extname, "Thumbnail")) {
+			continue; /* next HDU */
+		}
+
 		long naxes[3] = { 0L };
 		int naxis;
 		int bitpix;
@@ -82,6 +92,10 @@ static int _find_hdus(fitsfile *fptr, int **hdus, int *nb_im) {
 			} else {
 				//printf("naxes[2]=%ld, ref_naxes[2]=%ld\n", naxes[2], ref_naxes[2]);
 				if (naxes[2] != ref_naxes[2]) {
+					char extension[FLEN_VALUE], comment[FLEN_COMMENT];
+					fits_read_key(fptr, TSTRING, "EXTNAME", &extension, comment, &status);
+					if (!g_str_has_prefix(extension, "ICCProfile"))
+						continue;
 					siril_log_message(_("Several images were found in the FITS file but they have different number of layers, which is not allowed.\n"));
 					status = 1;
 					break;
@@ -119,7 +133,6 @@ static int _find_hdus(fitsfile *fptr, int **hdus, int *nb_im) {
 	return status;
 }
 
-
 // test if a file is a multi-extension FITS, a.k.a FITS cube or FITS sequence
 int fitseq_is_fitseq(const char *filename, int *frames) {
 	fitsfile *fptr;
@@ -150,30 +163,65 @@ void fitseq_init_struct(fitseq *fitseq) {
 	fitseq->writer = NULL;
 }
 
-int fitseq_open(const char *filename, fitseq *fitseq) {
+// opens a fitseq with iomode set to 0 (READONLY in most cases) or
+// 1 (READWRITE when the fitseq needs updating without creating a new file)
+int fitseq_open(const char *filename, fitseq *fitseq, int iomode) {
 	if (fitseq->fptr) {
 		fprintf(stderr, "FITS sequence: file already opened, or badly closed\n");
 		return -1;
 	}
 
+	// Check we haven't made a bad assumption about the extension
+	char *candidate_filename = NULL;
+
+	if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
+		// Original filename exists, use it
+		candidate_filename = strdup(filename);
+	} else {
+		// Try to find a file with different extensions
+		char *base = remove_all_ext_from_filename(filename);
+		const char *extensions[] = {
+			".fit", ".fits", ".fts", ".fit.fz", ".fits.fz", ".fts.fz",
+			".FIT", ".FITS", ".FTS", ".FIT.FZ", ".FITS.FZ", ".FTS.FZ"
+		}; // Not exhaustive but if people are using mixed-case extensions like .Fit then they should mend their ways
+
+		gboolean found = FALSE;
+		for (int i = 0; i < sizeof(extensions)/sizeof(extensions[0]); i++) {
+			char *test_filename = g_strdup_printf("%s%s", base, extensions[i]);
+			if (g_file_test(test_filename, G_FILE_TEST_EXISTS)) {
+				candidate_filename = strdup(test_filename);  // Use strdup for stdlib consistency
+				g_free(test_filename);  // Free the GLib-allocated string
+				found = TRUE;
+				break;
+			}
+			g_free(test_filename);
+		}
+		free(base);
+
+		if (!found) {
+			siril_log_color_message(_("Cannot find FITS file %s or any variant with supported extensions\n"), "red", filename);
+			return -1;
+		}
+	}
+
 	int status = 0;
-	siril_fits_open_diskfile_img(&(fitseq->fptr), filename, READONLY, &status);
+	siril_fits_open_diskfile_img(&(fitseq->fptr), candidate_filename, iomode, &status);
 	if (status) {
 		report_fits_error(status);
-		siril_log_color_message(_("Cannot open FITS file %s\n"), "red", filename);
+		siril_log_color_message(_("Cannot open FITS file %s\n"), "red", candidate_filename);
+		free(candidate_filename);
 		return -1;
 	}
-
 	if (_find_hdus(fitseq->fptr, &fitseq->hdu_index, &fitseq->frame_count) || fitseq->frame_count <= 1) {
-		siril_log_color_message(_("Cannot open FITS file %s: doesn't seem to be a FITS sequence\n"), "red", filename);
+		siril_log_color_message(_("Cannot open FITS file %s: doesn't seem to be a FITS sequence\n"), "red", candidate_filename);
+		free(candidate_filename);
 		return -1;
 	}
-
 	if (fits_movabs_hdu(fitseq->fptr, fitseq->hdu_index[0], NULL, &status)) {
 		report_fits_error(status);
+		free(candidate_filename);
 		return -1;
 	}
-
 	// we store the first image's dimensions in the struct
 	int naxis;
 	status = 0;
@@ -181,38 +229,35 @@ int fitseq_open(const char *filename, fitseq *fitseq) {
 	if (status || naxis <= 1 || fitseq->naxes[0] == 0 || fitseq->naxes[1] == 0) {
 		status = 0;
 		fits_close_file(fitseq->fptr, &status);
+		free(candidate_filename);
 		return -1;
 	}
 	if (naxis == 2)
 		fitseq->naxes[2] = 1;
-
 	manage_bitpix(fitseq->fptr, &(fitseq->bitpix), &(fitseq->orig_bitpix));
-
 	if (fitseq->bitpix == LONGLONG_IMG) {
 		siril_log_message(
 				_("FITS images with 64 bits signed integer per pixel.channel are not supported.\n"));
 		status = 0;
 		fits_close_file(fitseq->fptr, &status);
+		free(candidate_filename);
 		return -1;
 	}
-
-	fitseq->filename = strdup(filename);
+	fitseq->filename = candidate_filename; // Use the candidate filename instead of strdup(filename)
 	siril_debug_print("fitseq_open: sequence %s has %d frames, bitpix = %d, naxis = %d, naxes = { %ld, %ld, %ld }\n",
-			filename, fitseq->frame_count, fitseq->bitpix, naxis,
+			candidate_filename, fitseq->frame_count, fitseq->bitpix, naxis,
 			fitseq->naxes[0], fitseq->naxes[1], fitseq->naxes[2]);
-
 	if (fits_is_reentrant()) {
 		fitseq->is_mt_capable = TRUE;
 		fprintf(stdout, "cfitsio was compiled with multi-thread support,"
 				" parallel read of images will be possible\n");
-		fitseq_prepare_for_multiple_read(fitseq);
+		fitseq_prepare_for_multiple_read(fitseq, iomode);
 	} else {
 		fitseq->is_mt_capable = FALSE;
 		fprintf(stdout, "cfitsio was compiled without multi-thread support,"
 				" parallel read of images will be impossible\n");
 		siril_log_message(_("Your version of cfitsio does not support multi-threading\n"));
 	}
-
 	return 0;
 }
 
@@ -242,6 +287,8 @@ static int fitseq_read_frame_internal(fitseq *fitseq, int index, fits *dest, gbo
 	if (read_fits_with_convert(dest, fitseq->filename, force_float)) {
 		return -1;
 	}
+	dest->icc_profile = NULL;
+	color_manage(dest, FALSE);
 
 	return 0;
 }
@@ -283,6 +330,13 @@ int fitseq_read_partial_fits(fitseq *fitseq, int layer, int index, fits *dest, c
 	status = internal_read_partial_fits(fptr, fitseq->naxes[1], fitseq->bitpix,
 			dest->type == DATA_USHORT ? (void *)dest->data : (void *)dest->fdata,
 			layer, area);
+	if (!status) {
+		dest->orig_ry = fitseq->naxes[1];
+		dest->x_offset = area->x;
+		dest->y_offset = dest->orig_ry - area->y - area->h;
+		dest->icc_profile = NULL;
+		color_manage(dest, FALSE);
+	}
 	return status;
 }
 
@@ -307,7 +361,8 @@ int fitseq_read_partial(fitseq *fitseq, int layer, int index, void *buffer, cons
 
 	if (internal_read_partial_fits(fptr, fitseq->naxes[1], fitseq->bitpix, buffer, layer, area))
 		return 1;
-	flip_buffer(fitseq->bitpix, buffer, area);
+	if (!flip_buffer(fitseq->bitpix, buffer, area))
+		return 1;
 	return 0;
 }
 
@@ -391,13 +446,18 @@ static int fitseq_destroy(fitseq *fitseq, gboolean abort) {
 	fits_close_file(fitseq->fptr, &status);
 	if ((retval || !frame_count) && fitseq->filename) {
 		siril_log_message(_("Removing failed FITS sequence file: %s\n"), fitseq->filename);
+		report_fits_error(status);
 		if (g_unlink(fitseq->filename))
 			siril_debug_print("g_unlink() failed\n");
 	}
-	if (fitseq->filename)
+	if (fitseq->filename) {
 		free(fitseq->filename);
-	if (fitseq->hdu_index)
+		fitseq->filename = NULL;
+	}
+	if (fitseq->hdu_index) {
 		free(fitseq->hdu_index);
+		fitseq->hdu_index = NULL;
+	}
 	return retval;
 }
 
@@ -410,14 +470,14 @@ int fitseq_close_file(fitseq *fitseq) {
 }
 
 // to call after open to read with several threads in the file
-static int fitseq_prepare_for_multiple_read(fitseq *fitseq) {
+static int fitseq_prepare_for_multiple_read(fitseq *fitseq, int iomode) {
 	if (fitseq->thread_fptr) return 0;
 	if (!fitseq->is_mt_capable) return 0;
 	fitseq->num_threads = g_get_num_processors();
 	fitseq->thread_fptr = malloc(fitseq->num_threads * sizeof(fitsfile *));
 	for (guint i = 0; i < fitseq->num_threads; i++) {
 		int status = 0;
-		if (siril_fits_open_diskfile_img(&fitseq->thread_fptr[i], fitseq->filename, READONLY, &status)) {
+		if (siril_fits_open_diskfile_img(&fitseq->thread_fptr[i], fitseq->filename, iomode, &status)) {
 			report_fits_error(status);
 			return -1;
 		}
@@ -432,12 +492,14 @@ static int fitseq_multiple_close(fitseq *fitseq) {
 	for (guint i = 0; i < fitseq->num_threads; i++) {
 		int status = 0;
 		fits_close_file(fitseq->thread_fptr[i], &status);
-		if (status)
+		if (status) {
 			retval = 1;
+			report_fits_error(status);
+		}
 	}
 	free(fitseq->thread_fptr);
 	fitseq->thread_fptr = NULL;
-	siril_debug_print("closing FITS sequence fd for %d threads\n", fitseq->num_threads);
+	siril_debug_print("closed FITS sequence %s fd for %d threads\n", fitseq->filename, fitseq->num_threads);
 	return retval;
 }
 
