@@ -1,5 +1,5 @@
 // Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
-// Copyright (C) 2012-2025 team free-astro (see more in AUTHORS file)
+// Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
 // Reference site is https://siril.org
 // SPDX-License-Identifier: GPL-3.0-or-later
 
@@ -35,11 +35,15 @@
 #define M_PI 3.14159265358979323846  /* pi */
 #endif
 
-#define HEALPIX_DEBUG
+extern "C" {
+#include "core/siril_app_dirs.h"
+}
+
+//#define HEALPIX_DEBUG
 
 #define ZENODO_GAIA_XPSAMP_RECORD_ID "YOUR_RECORD_ID_HERE"
 
-extern const char* spcc_mirrors[];
+extern const char** spcc_mirrors;
 
 // Enum for Gaia version designator
 enum class GaiaVersion {
@@ -113,6 +117,75 @@ static std::vector<HealPixelRange> create_healpixel_ranges(const std::vector<int
     return ranges;
 }
 
+// Function to read the Healpix catalogue header
+static HealpixCatHeader read_healpix_cat_header(const std::string& filename, int* error_status) {
+    HealpixCatHeader header = {
+        "",                     // title: empty string
+        static_cast<GaiaVersion>(0),  // gaia_version: zero
+        0,                      // healpix_level
+        static_cast<CatalogueType>(0), // cat_type
+        0,                      // chunked
+        0,                      // chunk_level
+        0,                      // chunk_healpix
+        0,                      // chunk_first_healpixel
+        0,                      // chunk_last_healpixel
+        {}                      // spare: zero-initialized array
+    };
+
+    // Initialize error status
+    if (error_status) {
+        *error_status = 0;
+    }
+
+    // Open file in binary mode
+    std::ifstream file(filename, std::ios::binary);
+    if (!file.is_open()) {
+        if (error_status) {
+            *error_status = -1; // File open error
+        }
+        return header;
+    }
+
+    try {
+        // Read the fixed-size string (48 bytes)
+        char title_buffer[48] = {0};
+        file.read(title_buffer, 48);
+        if (file.fail()) {
+            if (error_status) {
+                *error_status = -2; // Read error
+            }
+            return header;
+        }
+        header.title = std::string(title_buffer, strnlen(title_buffer, 48));
+
+        // Read the remaining POD members
+        file.read(reinterpret_cast<char*>(&header.gaia_version), 1);
+        file.read(reinterpret_cast<char*>(&header.healpix_level), 1);
+        file.read(reinterpret_cast<char*>(&header.cat_type), 1);
+        file.read(reinterpret_cast<char*>(&header.chunked), 1);
+        file.read(reinterpret_cast<char*>(&header.chunk_level), 1);
+        file.read(reinterpret_cast<char*>(&header.chunk_healpix), sizeof(uint32_t));
+        file.read(reinterpret_cast<char*>(&header.chunk_first_healpixel), sizeof(uint32_t));
+        file.read(reinterpret_cast<char*>(&header.chunk_last_healpixel), sizeof(uint32_t));
+        file.read(reinterpret_cast<char*>(&header.spare), 63);
+
+        if (file.fail()) {
+            if (error_status) {
+                *error_status = -2; // Read error
+            }
+            return header;
+        }
+    }
+    catch (const std::exception&) {
+        if (error_status) {
+            *error_status = -3; // Exception during read
+        }
+        return header;
+    }
+
+    return header;
+}
+
 // This function queries a catalogue of packed EntryType objects (the template allows
 // use of SourceEntryAstro or SourceEntryXPsamp) for different uses. The catalogue MUST
 // be sorted by source_id otherwise the method will fail. Each entry that matches the
@@ -213,6 +286,24 @@ static bool header_compatible(HealpixCatHeader& a, HealpixCatHeader& b) {
     return same_version && same_chunk_level && same_index_level;
 }
 
+// Get Siril data directory and ensure the relevant subdir exists
+static std::filesystem::path get_or_create_cache_dir() {
+	std::filesystem::path target_path;
+
+    const char *uddir = siril_get_user_data_dir();
+    std::filesystem::path siril_subdir = std::filesystem::path(uddir) / "spcc-cache";
+    try {
+        if (!std::filesystem::exists(siril_subdir)) {
+		std::filesystem::create_directories(siril_subdir);
+        }
+        return siril_subdir;
+    } catch (const std::filesystem::filesystem_error& e) {
+        siril_debug_print(_("Can't create directory %s, falling back to system temp\n"), 
+			siril_subdir.string().c_str() );
+        return std::filesystem::temp_directory_path();
+    }
+}
+
 #ifdef HAVE_LIBCURL
 
 // Function to read header via HTTP range request using Siril's networking
@@ -234,52 +325,57 @@ static HealpixCatHeader read_healpix_cat_header_http_with_curl(CURL* curl, const
         *error_status = 0;
     }
 
-    gsize response_length;
-    int error;
-    char* buffer = fetch_url_range_with_curl((void*)curl, url.c_str(), 0, 128, &response_length, &error, FALSE);
+    // Setup Cache Path
+    auto tempdir = get_or_create_cache_dir();
+    std::string filename = url.substr(url.find_last_of("/\\") + 1);
+    std::string cache_path = (tempdir / (filename + ".header")).string();
 
-    if (error || !buffer || response_length < 128) {
-        if (error_status) {
-            *error_status = -1;
+    const size_t HEADER_SIZE = 128;
+
+    bool cache_exists = false;
+    std::error_code ec;
+
+    // Validate the cache file
+    if (std::filesystem::exists(cache_path, ec)) {
+        if (std::filesystem::file_size(cache_path, ec) == HEADER_SIZE) {
+            siril_debug_print(_("Header exists in cache\n"));
+            cache_exists = true;
+        } else {
+            siril_log_color_message(_("Cache file %s is corrupted or incomplete. Deleting...\n"),
+                                "salmon", cache_path.c_str());
+            std::filesystem::remove(cache_path, ec);
+        }
+    }
+
+    // Download if it doesn't exist
+    if (!cache_exists) {
+        siril_debug_print(_("Fetching header to cache: %s\n"), filename.c_str());
+
+        gsize response_length;
+        int error;
+        char* buffer = fetch_url_range_with_curl((void*)curl, url.c_str(), 0, HEADER_SIZE, &response_length, &error, FALSE);
+
+        if (error || !buffer || response_length < HEADER_SIZE) {
+            if (error_status) *error_status = -1;
+            free(buffer);
+            return {}; // Return empty header
+        }
+
+        // Save to disk
+        std::ofstream out(cache_path, std::ios::binary);
+        if (out.good()) {
+            out.write(buffer, HEADER_SIZE);
+            out.close();
         }
         free(buffer);
-        return header;
     }
 
-    try {
-        size_t offset = 0;
+    return read_healpix_cat_header(cache_path, error_status);
 
-        // Read title (48 bytes)
-        char title_buffer[48] = {0};
-        memcpy(title_buffer, buffer + offset, 48);
-        header.title = std::string(title_buffer, strnlen(title_buffer, 48));
-        offset += 48;
-
-        // Read remaining fields
-        memcpy(&header.gaia_version, buffer + offset, 1); offset += 1;
-        memcpy(&header.healpix_level, buffer + offset, 1); offset += 1;
-        memcpy(&header.cat_type, buffer + offset, 1); offset += 1;
-        memcpy(&header.chunked, buffer + offset, 1); offset += 1;
-        memcpy(&header.chunk_level, buffer + offset, 1); offset += 1;
-        memcpy(&header.chunk_healpix, buffer + offset, sizeof(uint32_t)); offset += sizeof(uint32_t);
-        memcpy(&header.chunk_first_healpixel, buffer + offset, sizeof(uint32_t)); offset += sizeof(uint32_t);
-        memcpy(&header.chunk_last_healpixel, buffer + offset, sizeof(uint32_t)); offset += sizeof(uint32_t);
-        memcpy(&header.spare, buffer + offset, 63);
-
-        free(buffer);
-    }
-    catch (const std::exception&) {
-        if (error_status) {
-            *error_status = -3;
-        }
-        free(buffer);
-        return header;
-    }
-
-    return header;
 }
 
 template<typename EntryType>
+
 static std::vector<EntryType> query_catalog_http_with_curl(CURL* curl,
                                                   const std::string& base_url,
                                                   const std::string& filename,
@@ -294,6 +390,7 @@ static std::vector<EntryType> query_catalog_http_with_curl(CURL* curl,
         uint32_t nside_chunks = 1 << header.chunk_level;
         uint32_t n_chunks = 12 * nside_chunks * nside_chunks;
         n_healpixels /= n_chunks;
+
         for (auto& range : healpixel_ranges) {
             range.start_id -= header.chunk_first_healpixel;
             range.end_id -= header.chunk_first_healpixel;
@@ -303,29 +400,74 @@ static std::vector<EntryType> query_catalog_http_with_curl(CURL* curl,
     size_t INDEX_SIZE = n_healpixels * sizeof(uint32_t);
     std::string full_url = base_url + "/" + filename;
 
-    // Function to read index entries via HTTP range request using provided CURL handle
-    auto read_index_entries = [curl, &full_url](uint32_t start_healpixel, uint32_t end_healpixel)
-        -> std::pair<bool, std::vector<uint32_t>> {
-        size_t start_pos = HEADER_SIZE + start_healpixel * sizeof(uint32_t);
-        size_t length = (end_healpixel - start_healpixel + 1) * sizeof(uint32_t);
+    auto tempdir = get_or_create_cache_dir();
+    std::string cache_path = (tempdir / (filename + ".index")).string();
+
+    // Load or download index
+    std::vector<uint32_t> full_index(n_healpixels);
+
+    bool cache_exists = false;
+    std::error_code ec;
+    if (std::filesystem::exists(cache_path, ec)) {
+        // Check if the size on disk matches our expected index size
+        if (std::filesystem::file_size(cache_path, ec) == INDEX_SIZE) {
+            cache_exists = true;
+        } else {
+            siril_log_color_message(_("Cache file %s is corrupted or incomplete. Deleting...\n"),
+                                "salmon", cache_path.c_str());
+            std::filesystem::remove(cache_path, ec);
+        }
+    }
+
+    // Ensure cache exists; if not, fetch and create it
+    if (!cache_exists) {
+        siril_debug_print(_("Fetching index to cache\n"));
 
         gsize response_length;
         int error;
-        char* buffer = fetch_url_range_with_curl((void*)curl, full_url.c_str(), start_pos, length,
-                                      &response_length, &error, FALSE);
 
-        if (error || !buffer) {
-            return {false, {}};
+        char* buffer = fetch_url_range_with_curl(
+            (void*)curl,
+            full_url.c_str(),
+            HEADER_SIZE,          // offset
+            INDEX_SIZE,           // length
+            &response_length,
+            &error,
+            FALSE
+        );
+
+        if (error || !buffer || response_length != INDEX_SIZE) {
+            siril_log_color_message(_("Failed to download index via HTTP\n"), "red");
+            return results;
         }
 
-        std::vector<uint32_t> indices(response_length / sizeof(uint32_t));
-        memcpy(indices.data(), buffer, response_length);
+        // Write buffer to cache file
+        std::ofstream out(cache_path, std::ios::binary);
+        if (!out.good()) {
+            siril_log_color_message(_("Failed to write index cache file\n"), "red");
+            free(buffer);
+            return results;
+        }
+
+        out.write(buffer, INDEX_SIZE);
+        out.close();
         free(buffer);
+    }
 
-        return {true, indices};
-    };
+    // At this point, the cache file is guaranteed to exist.
+    siril_debug_print(_("Loading index from cache\n"));
 
-    // Process each range
+    std::ifstream in(cache_path, std::ios::binary);
+    if (!in.good()) {
+        siril_log_color_message(_("Failed to read index cache file\n"), "red");
+        return results;
+    }
+
+    in.read(reinterpret_cast<char*>(full_index.data()), INDEX_SIZE);
+    in.close();
+
+    // Process each healpixel range
+    siril_debug_print(_("Fetching data\n"));
     for (const auto& range : healpixel_ranges) {
         uint32_t start_healpixel = range.start_id;
         uint32_t end_healpixel = range.end_id;
@@ -336,29 +478,35 @@ static std::vector<EntryType> query_catalog_http_with_curl(CURL* curl,
             return results;
         }
 
-        // Read the index entries
-        uint32_t index_start = (start_healpixel == 0) ? 0 : start_healpixel - 1;
-        auto [success, indices] = read_index_entries(index_start, end_healpixel);
+        // Index lookup from cached index
+        uint32_t index_start = (start_healpixel == 0) ? 0 : full_index[start_healpixel - 1];
+        uint32_t index_end   = full_index[end_healpixel];
 
-        if (!success) {
-            siril_log_color_message(_("Failed to read index entries via HTTP\n"), "red");
-            results.clear();
-            return results;
-        }
+        size_t num_records = index_end - index_start;
 
-        uint32_t start_offset = (start_healpixel == 0) ? 0 : indices[0];
-        uint32_t end_offset = indices[indices.size() - 1];
+	// It's possible that we're fetching a single healpixel in this range 
+	// and that the number of data records is zero
+	if (num_records == 0) continue;
+
+        size_t data_start_pos =
+            HEADER_SIZE + INDEX_SIZE + index_start * sizeof(EntryType);
 
-        // Calculate position in the data section
-        size_t data_start_pos = HEADER_SIZE + INDEX_SIZE + start_offset * sizeof(EntryType);
-        size_t num_records = end_offset - start_offset;
         size_t data_length = num_records * sizeof(EntryType);
 
-        // Read the data entries via HTTP range request using provided CURL handle
+        // Fetch data entries via HTTP
         gsize data_response_length;
         int data_error;
-        char* data_buffer = fetch_url_range_with_curl((void*)curl, full_url.c_str(), data_start_pos, data_length,
-                                           &data_response_length, &data_error, FALSE);
+
+        siril_debug_print(_("Fetching range %zu - %zu\n"), data_start_pos, data_length);
+        char* data_buffer = fetch_url_range_with_curl(
+            (void*)curl,
+            full_url.c_str(),
+            data_start_pos,
+            data_length,
+            &data_response_length,
+            &data_error,
+            FALSE
+        );
 
         if (data_error || !data_buffer) {
             siril_log_color_message(_("Failed to read data entries via HTTP\n"), "red");
@@ -366,18 +514,17 @@ static std::vector<EntryType> query_catalog_http_with_curl(CURL* curl,
             return results;
         }
 
-        // Convert buffer to EntryType vector
         std::vector<EntryType> buffer(num_records);
         memcpy(buffer.data(), data_buffer, data_response_length);
         free(data_buffer);
 
-        // Move entries to results vector
         results.insert(results.end(), buffer.begin(), buffer.end());
     }
 
     return results;
 }
 
+
 // Original template function now wraps the new one with a temporary CURL handle
 template<typename EntryType>
 static std::vector<EntryType> query_catalog_http(const std::string& base_url,
@@ -402,7 +549,6 @@ static std::vector<EntryType> query_catalog_http(const std::string& base_url,
  */
 static bool try_mirrors_and_update(CURL* curl, const char* path_suffix,
                                    HealpixCatHeader* header_out, int* error_status) {
-    extern const char* spcc_mirrors[];
 
     // First try the current mirror
     std::string current_url = std::string(com.spcc_remote_catalogue) + "/" + path_suffix;
@@ -456,7 +602,6 @@ static std::vector<EntryType> query_catalog_http_with_fallback(
                                     const std::string& filename,
                                     const std::vector<HealPixelRange>& healpixel_ranges,
                                     const HealpixCatHeader& header) {
-    extern const char* spcc_mirrors[];
     std::vector<EntryType> results;
 
     // Try current mirror first - make a copy since the function modifies ranges
@@ -504,75 +649,6 @@ static std::vector<EntryType> query_catalog_http_with_fallback(
 
 #endif
 
-// Function to read the Healpix catalogue header
-static HealpixCatHeader read_healpix_cat_header(const std::string& filename, int* error_status) {
-    HealpixCatHeader header = {
-        "",                     // title: empty string
-        static_cast<GaiaVersion>(0),  // gaia_version: zero
-        0,                      // healpix_level
-        static_cast<CatalogueType>(0), // cat_type
-        0,                      // chunked
-        0,                      // chunk_level
-        0,                      // chunk_healpix
-        0,                      // chunk_first_healpixel
-        0,                      // chunk_last_healpixel
-        {}                      // spare: zero-initialized array
-    };
-
-    // Initialize error status
-    if (error_status) {
-        *error_status = 0;
-    }
-
-    // Open file in binary mode
-    std::ifstream file(filename, std::ios::binary);
-    if (!file.is_open()) {
-        if (error_status) {
-            *error_status = -1; // File open error
-        }
-        return header;
-    }
-
-    try {
-        // Read the fixed-size string (48 bytes)
-        char title_buffer[48] = {0};
-        file.read(title_buffer, 48);
-        if (file.fail()) {
-            if (error_status) {
-                *error_status = -2; // Read error
-            }
-            return header;
-        }
-        header.title = std::string(title_buffer, strnlen(title_buffer, 48));
-
-        // Read the remaining POD members
-        file.read(reinterpret_cast<char*>(&header.gaia_version), 1);
-        file.read(reinterpret_cast<char*>(&header.healpix_level), 1);
-        file.read(reinterpret_cast<char*>(&header.cat_type), 1);
-        file.read(reinterpret_cast<char*>(&header.chunked), 1);
-        file.read(reinterpret_cast<char*>(&header.chunk_level), 1);
-        file.read(reinterpret_cast<char*>(&header.chunk_healpix), sizeof(uint32_t));
-        file.read(reinterpret_cast<char*>(&header.chunk_first_healpixel), sizeof(uint32_t));
-        file.read(reinterpret_cast<char*>(&header.chunk_last_healpixel), sizeof(uint32_t));
-        file.read(reinterpret_cast<char*>(&header.spare), 63);
-
-        if (file.fail()) {
-            if (error_status) {
-                *error_status = -2; // Read error
-            }
-            return header;
-        }
-    }
-    catch (const std::exception&) {
-        if (error_status) {
-            *error_status = -3; // Exception during read
-        }
-        return header;
-    }
-
-    return header;
-}
-
 
 #ifdef HEALPIX_DEBUG
 // Function to show entries for a specific healpixel
