@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,9 +31,11 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/siril_log.h"
-#include "gui/utils.h"
+#include "core/icc_profile.h"
+#include "core/processing.h"
 #include "gui/progress_and_log.h"
 #include "io/image_format_fits.h"
+#include "io/fits_keywords.h"
 
 #ifndef O_BINARY
 #define O_BINARY 0
@@ -62,6 +64,8 @@ static int bmp32tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry,
 		*rdata++ = (WORD) *rvb++;
 		rvb++;
 	}
+	set_all_keywords_default(fit);
+
 	fit->bitpix = fit->orig_bitpix = BYTE_IMG;
 	fit->naxis = 3;
 	fit->rx = rx;
@@ -69,7 +73,7 @@ static int bmp32tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry,
 	fit->naxes[0] = rx;
 	fit->naxes[1] = ry;
 	fit->naxes[2] = 3;
-	fit->binning_x = fit->binning_y = 1;
+	fit->keywords.binning_x = fit->keywords.binning_y = 1;
 	return 0;
 }
 
@@ -98,6 +102,8 @@ static int bmp24tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry,
 		}
 		rvb += padsize;
 	}
+	set_all_keywords_default(fit);
+
 	fit->bitpix = fit->orig_bitpix = BYTE_IMG;
 	fit->naxis = 3;
 	fit->rx = rx;
@@ -105,7 +111,7 @@ static int bmp24tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry,
 	fit->naxes[0] = rx;
 	fit->naxes[1] = ry;
 	fit->naxes[2] = 3;
-	fit->binning_x = fit->binning_y = 1;
+	fit->keywords.binning_x = fit->keywords.binning_y = 1;
 	return 0;
 }
 
@@ -132,6 +138,8 @@ static int bmp16tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry,
 		*gdata++ = ((pixel_data & 0x03e0) >> 5) * 255.0 / 31.0 + 0.5;
 		*bdata++ = ((pixel_data & 0x001f) >> 0) * 255.0 / 31.0 + 0.5;
 	}
+	set_all_keywords_default(fit);
+
 	fit->bitpix = fit->orig_bitpix = BYTE_IMG;
 	fit->naxis = 3;
 	fit->rx = rx;
@@ -139,7 +147,7 @@ static int bmp16tofits48(unsigned char *rvb, unsigned long rx, unsigned long ry,
 	fit->naxes[0] = rx;
 	fit->naxes[1] = ry;
 	fit->naxes[2] = 3;
-	fit->binning_x = fit->binning_y = 1;
+	fit->keywords.binning_x = fit->keywords.binning_y = 1;
 	return 0;
 }
 
@@ -165,6 +173,8 @@ static int bmp8tofits(unsigned char *rgb, unsigned long rx, unsigned long ry, fi
 		}
 		rgb += padsize;
 	}
+	set_all_keywords_default(fit);
+
 	fit->bitpix = fit->orig_bitpix = BYTE_IMG;
 	fit->rx = rx;
 	fit->ry = ry;
@@ -172,7 +182,7 @@ static int bmp8tofits(unsigned char *rgb, unsigned long rx, unsigned long ry, fi
 	fit->naxes[1] = ry;
 	fit->naxes[2] = 1;
 	fit->naxis = 2;
-	fit->binning_x = fit->binning_y = 1;
+	fit->keywords.binning_x = fit->keywords.binning_y = 1;
 	return 0;
 }
 
@@ -202,7 +212,7 @@ int readbmp(const char *name, fits *fit) {
 	FILE *file;
 	long int count;
 	unsigned char *buf;
-	unsigned long data_offset = 0;
+	long data_offset = 0;
 	unsigned long width = 0, height = 0;
 	unsigned short nbplane = 0;
 
@@ -255,6 +265,8 @@ int readbmp(const char *name, fits *fit) {
 	}
 	fclose(file);
 
+	set_all_keywords_default(fit);
+
 	switch (nbplane) {
 		case 1:
 			bmp8tofits(buf, width, height, fit);
@@ -274,6 +286,10 @@ int readbmp(const char *name, fits *fit) {
 	}
 	fit->type = DATA_USHORT;
 	free(buf);
+	// Initialize ICC profile. As the buffer is set to NULL, this sets the
+	// profile as sRGB (or Gray g22) which is what we want for 8-bit BMPs
+	fits_initialize_icc(fit, NULL, 0);
+
 	char *basename = g_path_get_basename(name);
 	siril_log_message(_("Reading BMP: file %s, %ld layer(s), %ux%u pixels\n"),
 			basename, fit->naxes[2], fit->rx, fit->ry);
@@ -305,6 +321,42 @@ int savebmp(const char *name, fits *fit) {
 
 	WORD *gbuf[3] = { fit->pdata[RLAYER], fit->pdata[GLAYER], fit->pdata[BLAYER] };
 	float *gbuff[3] = { fit->fpdata[RLAYER], fit->fpdata[GLAYER], fit->fpdata[BLAYER] };
+	// Apply the colorspace transform
+	void *buf = NULL;
+	void *dest = NULL;
+	gboolean src_is_float = (fit->type == DATA_FLOAT);
+
+	// Transform the data
+	if (fit->icc_profile && fit->color_managed) {
+		buf = src_is_float ? (void *) fit->fdata : (void *) fit->data;
+		size_t npixels = fit->rx * fit->ry;
+		size_t nchans = fit->naxes[2];
+		cmsUInt32Number trans_type;
+		if (src_is_float) {
+			dest = malloc(fit->rx * fit->ry * fit->naxes[2] * sizeof(float));
+			trans_type = nchans == 1 ? TYPE_GRAY_FLT : TYPE_RGB_FLT_PLANAR;
+		} else {
+			dest = malloc(fit->rx * fit->ry * fit->naxes[2] * sizeof(WORD));
+			trans_type = nchans == 1 ? TYPE_GRAY_16 : TYPE_RGB_16_PLANAR;
+		}
+		gboolean threaded = !get_thread_run();
+		cmsHTRANSFORM save_transform = sirilCreateTransformTHR((threaded ? com.icc.context_threaded : com.icc.context_single), fit->icc_profile, trans_type, (nchans == 1 ? com.icc.mono_out : com.icc.srgb_out), trans_type, com.pref.icc.export_intent, 0);
+		cmsUInt32Number data_format_size = gfit.type == DATA_FLOAT ? sizeof(float) : sizeof(WORD);
+		cmsUInt32Number bytesperline = gfit.rx * data_format_size;
+		cmsUInt32Number bytesperplane = npixels * data_format_size;
+		cmsDoTransformLineStride(save_transform, buf, dest, gfit.rx, gfit.ry, bytesperline, bytesperline, bytesperplane, bytesperplane);
+		cmsDeleteTransform(save_transform);
+		gbuf[0] = (WORD *) dest;
+		gbuf[1] = (WORD *) dest + (fit->rx * fit->ry);
+		gbuf[2] = (WORD *) dest + (fit->rx * fit->ry * 2);
+		gbuff[0] = (float *) dest;
+		gbuff[1] = (float *) dest + (fit->rx * fit->ry);
+		gbuff[2] = (float *) dest + (fit->rx * fit->ry * 2);
+		// No profile will be embedded as the version of BITMAPINFOHEADER
+		// we use does not support them: the assumption is that the data is
+		// sRGB (or possibly "Windows color space" which seems to be more or
+		// less the same thing).
+	}
 
 	unsigned int padsize = (4 - (width * 3) % 4) % 4;
 	size_t datasize = width * height * 3 + padsize * height;
@@ -399,6 +451,7 @@ int savebmp(const char *name, fits *fit) {
 	siril_log_message(_("Saving BMP: file %s, %ld layer(s), %ux%u pixels\n"), filename,
 			fit->naxes[2], fit->rx, fit->ry);
 	free(filename);
+	free(dest);
 	return 0;
 }
 
@@ -490,27 +543,30 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 		fclose(file);
 		return -1;
 	}
+	set_all_keywords_default(fit);
+
 	if (max_val == UCHAR_MAX) {
 		/* 8-bit file */
 		unsigned char *tmpbuf = NULL;
-		WORD *olddata;
 		if (fit->naxes[2] == 1)
 			stride = fit->rx;
 		else
 			stride = fit->rx * 3;
 		tmpbuf = malloc(stride * fit->ry);
-		olddata = fit->data;
-		fit->data = realloc(fit->data, stride * fit->ry * sizeof(WORD));
-		if (fit->data == NULL || tmpbuf == NULL) {
+		WORD *tmp = realloc(fit->data, stride * fit->ry * sizeof(WORD));
+		if (tmp == NULL || tmpbuf == NULL) {
 			PRINT_ALLOC_ERR;
 			fclose(file);
-			if (olddata && !fit->data)
-				free(olddata);
+			if (fit->data && !tmp)
+				free(fit->data);
 			if (tmpbuf)
 				free(tmpbuf);
+			if (tmp)
+				free(tmp);
 			fit->data = NULL;
 			return -1;
 		}
+		fit->data = tmp;
 		if (fread(tmpbuf, stride, fit->ry, file) < fit->ry) {
 			siril_log_color_message(_("Error reading 8-bit PPM image data.\n"), "red");
 			fclose(file);
@@ -525,7 +581,7 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 			rgb8bit_to_fits16bit(tmpbuf, fit);
 		free(tmpbuf);
 		fit->bitpix = BYTE_IMG;
-		fit->binning_x = fit->binning_y = 1;
+		fit->keywords.binning_x = fit->keywords.binning_y = 1;
 		fits_flip_top_to_bottom(fit);
 	} else if (max_val == USHRT_MAX || max_val == SHRT_MAX) {
 		/* 16-bit file */
@@ -558,20 +614,22 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 
 		} else {
 			/* RGB 16-bit image */
-			WORD *tmpbuf, *olddata = fit->data;
+			WORD *tmpbuf;
 			stride = fit->rx * 3 * sizeof(WORD);
 			tmpbuf = malloc(stride * fit->ry);
-			fit->data = realloc(fit->data, stride * fit->ry * sizeof(WORD));
-			if (fit->data == NULL || tmpbuf == NULL) {
+			WORD *tmp = realloc(fit->data, stride * fit->ry * sizeof(WORD));
+			if (tmp == NULL || tmpbuf == NULL) {
 				PRINT_ALLOC_ERR;
 				fclose(file);
-				if (olddata && !fit->data)
-					free(olddata);
+				if (fit->data && !tmp)
+					free(fit->data);
 				if (tmpbuf)
 					free(tmpbuf);
-				fit->data = NULL;
+				if (tmp)
+					free(tmp);
 				return -1;
 			}
+			fit->data = tmp;
 			if (fread(tmpbuf, stride, fit->ry, file) < fit->ry) {
 				siril_log_color_message(
 						_("Error reading 16-bit color PPM image data.\n"), "red");
@@ -585,7 +643,7 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 			free(tmpbuf);
 		}
 		fit->bitpix = USHORT_IMG;
-		fit->binning_x = fit->binning_y = 1;
+		fit->keywords.binning_x = fit->keywords.binning_y = 1;
 		fits_flip_top_to_bottom(fit);
 	} else {
 		siril_log_color_message(_("Not handled max value for PNM: %d.\n"), "red",
@@ -594,7 +652,11 @@ int import_pnm_to_fits(const char *filename, fits *fit) {
 		return -1;
 	}
 	fit->type = DATA_USHORT;
-	g_snprintf(fit->row_order, FLEN_VALUE, "%s", "TOP-DOWN");
+	g_snprintf(fit->keywords.row_order, FLEN_VALUE, "%s", "TOP-DOWN");
+
+	// Initialize ICC profile. As the buffer is set to NULL, this sets the
+	// profile as sRGB (or Gray g22) which is what we want
+	fits_initialize_icc(fit, NULL, 0);
 
 	fclose(file);
 	char *basename = g_path_get_basename(filename);
@@ -617,6 +679,38 @@ static int saveppm(const char *name, fits *fit) {
 	fprintf(fp, "P6\n%s\n%u %u\n%u\n", comment, fit->rx, fit->ry, USHRT_MAX);
 	WORD *gbuf[3] = { fit->pdata[RLAYER], fit->pdata[GLAYER], fit->pdata[BLAYER] };
 	float *gbuff[3] = { fit->fpdata[RLAYER], fit->fpdata[GLAYER], fit->fpdata[BLAYER] };
+
+	// Apply the colorspace transform
+	gboolean src_is_float = (fit->type == DATA_FLOAT);
+	void *buf = NULL;
+	void *dest = NULL;
+
+	// Colorspace transform the data
+	if (fit->color_managed && fit->icc_profile) {
+		buf = src_is_float ? (void *) fit->fdata : (void *) fit->data;
+		const size_t npixels = fit->rx * fit->ry;
+		if (src_is_float) {
+			dest = malloc(fit->rx * fit->ry * fit->naxes[2] * sizeof(float));
+		} else {
+			dest = malloc(fit->rx * fit->ry * fit->naxes[2] * sizeof(WORD));
+		}
+		gboolean threaded = !get_thread_run();
+		cmsColorSpaceSignature sig = cmsGetColorSpace(fit->icc_profile);
+		cmsUInt32Number trans_type = get_planar_formatter_type(sig, fit->type, FALSE);
+		cmsHTRANSFORM save_transform = sirilCreateTransformTHR((threaded ? com.icc.context_threaded : com.icc.context_single), fit->icc_profile, trans_type, com.icc.srgb_out, trans_type, com.pref.icc.export_intent, 0);
+		cmsUInt32Number datasize = gfit.type == DATA_FLOAT ? sizeof(float) : sizeof(WORD);
+		cmsUInt32Number bytesperline = gfit.rx * datasize;
+		cmsUInt32Number bytesperplane = npixels * datasize;
+		cmsDoTransformLineStride(save_transform, buf, dest, gfit.rx, gfit.ry, bytesperline, bytesperline, bytesperplane, bytesperplane);
+		cmsDeleteTransform(save_transform);
+		gbuf[0] = (WORD *) dest;
+		gbuf[1] = (WORD *) dest + (fit->rx * fit->ry);
+		gbuf[2] = (WORD *) dest + (fit->rx * fit->ry * 2);
+		gbuff[0] = (float *) dest;
+		gbuff[1] = (float *) dest + (fit->rx * fit->ry);
+		gbuff[2] = (float *) dest + (fit->rx * fit->ry * 2);
+	}
+
 	fits_flip_top_to_bottom(fit);
 	norm = (fit->orig_bitpix != BYTE_IMG) ? 1.0 : USHRT_MAX_DOUBLE / UCHAR_MAX_DOUBLE;
 	if (fit->type == DATA_USHORT) {
@@ -648,6 +742,7 @@ static int saveppm(const char *name, fits *fit) {
 	fits_flip_top_to_bottom(fit);
 	siril_log_message(_("Saving NetPBM: file %s, %ld layer(s), %ux%u pixels\n"),
 			name, fit->naxes[2], fit->rx, fit->ry);
+	free(dest);
 	return 0;
 }
 
@@ -657,6 +752,34 @@ static int savepgm(const char *name, fits *fit) {
 	double norm;
 	WORD *gbuf = fit->pdata[RLAYER];
 	float *gbuff = fit->fpdata[RLAYER];
+
+	// Apply the colorspace transform
+	void *buf = NULL;
+	void *dest = NULL;
+	gboolean src_is_float = (fit->type == DATA_FLOAT);
+
+	// Colorspace transform the data
+	if (fit->color_managed && fit->icc_profile) {
+		buf = src_is_float ? (void *) fit->fdata : (void *) fit->data;
+		const size_t npixels = fit->rx * fit->ry;
+		if (src_is_float) {
+			dest = malloc(fit->rx * fit->ry * fit->naxes[2] * sizeof(float));
+		} else {
+			dest = malloc(fit->rx * fit->ry * fit->naxes[2] * sizeof(WORD));
+		}
+		gboolean threaded = get_thread_run();
+		cmsColorSpaceSignature sig = cmsGetColorSpace(fit->icc_profile);
+		cmsUInt32Number trans_type = get_planar_formatter_type(sig, fit->type, FALSE);
+		cmsHTRANSFORM save_transform = sirilCreateTransformTHR((threaded ? com.icc.context_threaded : com.icc.context_single), fit->icc_profile, trans_type, com.icc.mono_out, trans_type, com.pref.icc.export_intent, 0);
+		cmsUInt32Number datasize = gfit.type == DATA_FLOAT ? sizeof(float) : sizeof(WORD);
+		cmsUInt32Number bytesperline = gfit.rx * datasize;
+		cmsUInt32Number bytesperplane = npixels * datasize;
+		cmsDoTransformLineStride(save_transform, buf, dest, gfit.rx, gfit.ry, bytesperline, bytesperline, bytesperplane, bytesperplane);
+		cmsDeleteTransform(save_transform);
+		gbuf = (WORD *) dest;
+		gbuff = (float *) dest;
+	}
+
 	const char *comment = "# CREATOR : SIRIL";
 
 	fp = g_fopen(name, "wb");
@@ -687,6 +810,7 @@ static int savepgm(const char *name, fits *fit) {
 	fits_flip_top_to_bottom(fit);
 	siril_log_message(_("Saving NetPBM: file %s, %ld layer(s), %ux%u pixels\n"),
 			name, fit->naxes[2], fit->rx, fit->ry);
+	free(dest);
 	return 0;
 }
 
@@ -833,6 +957,8 @@ int readpic(const char *name, fits *fit) {
 		return -1;
 	}
 
+	set_all_keywords_default(fit);
+
 	if (_pic_read_header(pic_file)) {
 		_pic_close_file(pic_file);
 		return -1;
@@ -840,10 +966,10 @@ int readpic(const char *name, fits *fit) {
 
 	fit->rx = (unsigned int) pic_file->width;
 	fit->ry = (unsigned int) pic_file->height;
-	fit->binning_x = (unsigned int) pic_file->bin[4];
-	fit->binning_y = (unsigned int) pic_file->bin[5];
-	fit->hi = pic_file->hi;
-	fit->lo = pic_file->lo;
+	fit->keywords.binning_x = (unsigned int) pic_file->bin[4];
+	fit->keywords.binning_y = (unsigned int) pic_file->bin[5];
+	fit->keywords.hi = pic_file->hi;
+	fit->keywords.lo = pic_file->lo;
 	fit->type = DATA_USHORT;
 
 	size_t nbdata = fit->rx * fit->ry;
@@ -891,7 +1017,7 @@ int readpic(const char *name, fits *fit) {
 			basename, fit->naxes[2], fit->rx, fit->ry);
 	siril_log_message("(%d,%d)-(%d,%d) - Binning %dx%d\n", pic_file->bin[0],
 			pic_file->bin[1], pic_file->bin[2], pic_file->bin[3],
-			fit->binning_x, fit->binning_y);
+			fit->keywords.binning_x, fit->keywords.binning_y);
 
 	if (pic_file->date[0] != 0x00) {
 		g_strchug(pic_file->date);	// removing left white spaces if exist
@@ -902,6 +1028,12 @@ int readpic(const char *name, fits *fit) {
 		siril_log_message(_("Time (of observation): %s\n"), pic_file->time);
 	}
 
+	// Initialize ICC profile. As the buffer is set to NULL, this sets the
+	// profile as sRGB (or Gray g22) which *I think* is what we want for
+	// PIC files. If anyone is still using them, please correct me if I'm
+	// wrong!
+	fits_initialize_icc(fit, NULL, 0);
+
 	_pic_close_file(pic_file);
 	g_free(basename);
 	return retval;
