@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
 #include <assert.h>
 
 #include "core/siril.h"
@@ -38,9 +37,9 @@
 #include "io/ser.h"
 #include "io/sequence.h"
 #include "core/proto.h"
-#include "gui/utils.h"
 #include "gui/progress_and_log.h"
 #include "registration/registration.h"
+#include "stacking/stacking.h"
 #ifdef HAVE_FFMS2
 #include "io/films.h"
 #endif
@@ -51,8 +50,16 @@
  * version 2 allowed regdata to be stored for CFA SER sequences, 0.9.11
  * version 3 introduced new weighted fwhm criteria, 0.99.0
  * version 4 introduced variable size sequences, extended registration data (incl. H), 1.1.0
+ * version 5:
+ * 	- removed upscale at stacking (U card) 1.3.4
+ *  - added D* cards containing distortion and astrometry information 1.3.4  - see enum disto_source
+ *  - added overlap statistics in the O* cards:
+ *  	=> ON i j areai.x areai.y areaj.x areaj.y areai.w areai.h Nij medij medji madij madji locij locji scaij scji
+ * 		=> with N the layer number and i,j the ith and jth images of the sequence
+ * version 6
+ * - added drrizle card for drizzled registration. Indicates there should be a drizzletmp folder and drizzle weights files
  */
-#define CURRENT_SEQFILE_VERSION 4	// to increment on format change
+#define CURRENT_SEQFILE_VERSION 6	// to increment on format change
 
 /* File format (lines starting with # are comments, lines that are (for all
  * something) need to be in all in sequence of this only type of line):
@@ -62,7 +69,7 @@
  * (for all images) I filenum incl [width,height] [stats+] <- stats added at some point, removed in 0.9.9
  * (for all layers (x)) Rx regparam+
  * TS | TA | TF (type for ser or film (avi) or fits)
- * U up-scale_ratio
+ * U up-scale_ratio -> discarded in v5
  * (for all images (y) and layers (x)) Mx-y stats+
  */
 
@@ -110,17 +117,22 @@ sequence * readseqfile(const char *name){
 				 * Such sequences don't exist anymore. */
 				assert(line[2] != '"');
 				if (line[2] == '\'')	/* new format, quoted string */
-					scanformat = "'%511[^']' %d %d %d %d %d %d %d %d";
-				else scanformat = "%511s %d %d %d %d %d %d %d %d";
+					scanformat = "'%511[^']' %d %d %d %d %d %d %d %d %d";
+				else
+					scanformat = "%511s %d %d %d %d %d %d %d %d %d";
 
-				if(sscanf(line+2, scanformat,
+				int nbtokens = sscanf(line+2, scanformat,
 							filename, &seq->beg, &seq->number,
 							&seq->selnum, &seq->fixed,
-							&seq->reference_image, &version, &seq->is_variable, &seq->fz) < 6 ||
-						allocated != 0){
+							&seq->reference_image, &version, &seq->is_variable, &seq->fz, &seq->is_drizzle);
+				if((nbtokens < 6 && version < 6) || (nbtokens < 10 && version >= 6) || allocated != 0) {
 					fprintf(stderr,"readseqfile: sequence file format error: %s\n",line);
 					goto error;
 				}
+				if (version < 0) {
+					fprintf(stderr, "readseqfile: sequence file format error: %s\n", line);
+					goto error;
+				}
 				if (seq->number == 0) {
 					fprintf(stderr, "readseqfile: sequence is empty?\n");
 					goto error;
@@ -162,6 +174,7 @@ sequence * readseqfile(const char *name){
 						seq->regparam = calloc(seq->nb_layers, sizeof(regdata*));
 						if (ser_is_cfa(seq->ser_file))
 							seq->regparam_bkp = calloc(3, sizeof(regdata*));
+						seq->distoparam = calloc(seq->nb_layers, sizeof(disto_params));
 					}
 				} else if (line[1] >= '0' && line[1] <= '9') {
 					/* in the future, wavelength and name of each layer will be added here */
@@ -220,7 +233,44 @@ sequence * readseqfile(const char *name){
 				}
 				++i;
 				break;
-
+			case 'D': // Distortion data - from version 5 onwards
+				current_layer = line[1] - '0';
+				if (current_layer < 0 || current_layer > seq->nb_layers) {
+					fprintf(stderr, "readseqfile: sequence file bad distortion layer: %s\n", line);
+					goto error;
+				}
+				int index;
+				char buf0[256], buf1[256], buf2[256];
+				nb_tokens = sscanf(line + 3, "%d %s %s %s\n",
+							&index,
+							buf0, buf1, buf2);
+				if (nb_tokens < 1 || nb_tokens > 4) {
+					fprintf(stderr, "readseqfile: sequence file bad distortion param: %s\n", line);
+					goto error;
+				}
+				if (!seq->distoparam)
+					seq->distoparam = calloc(seq->nb_layers, sizeof(disto_params));
+				seq->distoparam[current_layer].index = index;
+				if (index == DISTO_FILE || index == DISTO_MASTER) {
+					if (nb_tokens == 1) {
+						fprintf(stderr, "readseqfile: sequence file bad distortion param: %s\n", line);
+						goto error;
+					}
+					seq->distoparam[current_layer].filename = g_strdup(buf0);
+				}
+				if (index == DISTO_FILE_COMET) {
+					if (nb_tokens < 3) {
+						fprintf(stderr, "readseqfile: sequence file bad distortion param: %s\n", line);
+						goto error;
+					}
+					seq->distoparam[current_layer].velocity.x = (float)g_strtod(buf0, NULL);
+					seq->distoparam[current_layer].velocity.y = (float)g_strtod(buf1, NULL);
+					if (nb_tokens > 3) {
+						seq->distoparam[current_layer].filename = g_strdup(buf2);
+					}
+				}
+				++i;
+				break;
 			case 'R':
 				/* registration info */
 				if (line[1] == '*') {
@@ -281,6 +331,8 @@ sequence * readseqfile(const char *name){
 						seq->regparam_bkp[current_layer] = regparam;
 					else seq->regparam[current_layer] = regparam;
 				}
+				if (!seq->distoparam)
+					seq->distoparam = calloc(seq->nb_layers, sizeof(disto_params));
 				if (i >= seq->number) {
 					fprintf(stderr, "\nreadseqfile: out of array bounds in reg info!\n\n");
 					goto error;
@@ -335,6 +387,7 @@ sequence * readseqfile(const char *name){
 				}
 				else {
 					// version 4 without shifts and with homography matrix
+					// version 5
 					if (sscanf(line+3, "%g %g %g %lg %g %d H %lg %lg %lg %lg %lg %lg %lg %lg %lg",
 								&(regparam[i].fwhm),
 								&(regparam[i].weighted_fwhm),
@@ -394,13 +447,24 @@ sequence * readseqfile(const char *name){
 					seq->ext = get_com_ext(seq->fz) + 1;
 #endif
 					if (seq->fitseq_file) break;
-					seq->fitseq_file = malloc(sizeof(struct fits_sequence));
+					seq->fitseq_file = calloc(1, sizeof(struct fits_sequence));
 					fitseq_init_struct(seq->fitseq_file);
-					GString *fileString = g_string_new(filename);
-					g_string_append(fileString, get_com_ext(seq->fz));
-					seq->fitseq_file->filename = g_string_free(fileString, FALSE);
-					if (fitseq_open(seq->fitseq_file->filename, seq->fitseq_file)) {
-						g_free(seq->fitseq_file->filename);
+					int i = 0;
+					static const char *fitseq_ext[] = { ".fit", ".fits", ".fts", ".fit.fz", ".fits.fz", ".fts.fz", NULL };
+					// We only look for lowercase extensions as FITSEQ will only have been created by Siril, and should
+					// have used a lowercase extensions set in com.pref.ext
+					while (fitseq_ext[i]) {
+						GString *fileString = g_string_new(filename);
+						g_string_append(fileString, fitseq_ext[i++]);
+						gchar *test_filename = g_string_free(fileString, FALSE);
+						if (g_file_test(test_filename, G_FILE_TEST_EXISTS) && !fitseq_open(test_filename, seq->fitseq_file, READONLY)) {
+							seq->fitseq_file->filename = test_filename;
+							break;
+						} else {
+							g_free(test_filename);
+						}
+					}
+					if (!seq->fitseq_file->filename) {
 						free(seq->fitseq_file);
 						seq->fitseq_file = NULL;
 						goto error;
@@ -462,15 +526,8 @@ sequence * readseqfile(const char *name){
 #endif
 				break;
 
-			case 'U':
-				/* up-scale factor for stacking. Used in simplified stacking for
-				 * shift-only registrated sequences, up-scale will be done at
-				 * stack-time. */
-				if (line[1] == ' ' &&
-						sscanf(line+2, "%lg", &seq->upscale_at_stacking) != 1) {
-					fprintf(stderr,"readseqfile: sequence file format error: %s\n",line);
-					goto error;
-				}
+			case 'U': // for versions up to 4
+				siril_log_message(_("Seq file had info to upscale at stacking. This must now be passed as a stacking option\n"));
 				break;
 			case 'M':
 				/* stats may not exist for all images and layers so we use
@@ -540,6 +597,44 @@ sequence * readseqfile(const char *name){
 					goto error;
 				}
 				break;
+			case 'O':
+				current_layer = line[1] - '0';
+				if (!seq->ostats) {
+					seq->ostats = alloc_ostats(seq->nb_layers, seq->number);
+					if (!seq->ostats) {
+						PRINT_ALLOC_ERR;
+						goto error;
+					}
+				}
+				overlap_stats_t ostat = { 0 };
+				nb_tokens = sscanf(line + 3,
+					"%d %d %d %d %d %d %d %d %zu %g %g %g %g %g %g %g %g",
+					&(ostat.i),
+					&(ostat.j),
+					&(ostat.areai.x),
+					&(ostat.areai.y),
+					&(ostat.areaj.x),
+					&(ostat.areaj.y),
+					&(ostat.areai.w),
+					&(ostat.areai.h),
+					&(ostat.Nij),
+					&(ostat.medij),
+					&(ostat.medji),
+					&(ostat.madij),
+					&(ostat.madji),
+					&(ostat.locij),
+					&(ostat.locji),
+					&(ostat.scaij),
+					&(ostat.scaji));
+				if (nb_tokens != 17) {
+					fprintf(stderr, "readseqfile: sequence file format error: %s\n",line);
+					goto error;
+				}
+				ostat.areaj.w = ostat.areai.w;
+				ostat.areaj.h = ostat.areai.h;
+				int ijth = get_ijth_pair_index(seq->number, ostat.i, ostat.j);
+				seq->ostats[current_layer][ijth] = ostat;
+				break;
 		}
 	}
 	if (!allocated) {
@@ -598,10 +693,10 @@ int writeseqfile(sequence *seq){
 	free(filename);
 
 	fprintf(seqfile,"#Siril sequence file. Contains list of images, selection, registration data and statistics\n");
-	fprintf(seqfile,"#S 'sequence_name' start_index nb_images nb_selected fixed_len reference_image version variable_size fz_flag\n");
-	fprintf(seqfile,"S '%s' %d %d %d %d %d %d %d %d\n",
+	fprintf(seqfile,"#S 'sequence_name' start_index nb_images nb_selected fixed_len reference_image version variable_size fz_flag drizzle\n");
+	fprintf(seqfile,"S '%s' %d %d %d %d %d %d %d %d %d\n",
 			seq->seqname, seq->beg, seq->number, seq->selnum, seq->fixed,
-			seq->reference_image, CURRENT_SEQFILE_VERSION, seq->is_variable, seq->fz);
+			seq->reference_image, CURRENT_SEQFILE_VERSION, seq->is_variable, seq->fz, seq->is_drizzle);
 	if (seq->type != SEQ_REGULAR) {
 		char type;
 		switch (seq->type) {
@@ -616,14 +711,9 @@ int writeseqfile(sequence *seq){
 		fprintf(seqfile, "T%c\n", type);
 	}
 
-	if (seq->upscale_at_stacking != 1.0) {
-		// until we have a real drizzle
-		fprintf(seqfile, "U %g\n", seq->upscale_at_stacking);
-	}
-
 	fprintf(seqfile, "L %d\n", seq->nb_layers);
 
-	for(i=0; i < seq->number; ++i){
+	for(i = 0; i < seq->number; ++i){
 		if (seq->is_variable) {
 			fprintf(seqfile,"I %d %d %d,%d\n",
 					seq->imgparam[i].filenum,
@@ -639,7 +729,32 @@ int writeseqfile(sequence *seq){
 
 	for (layer = 0; layer < seq->nb_layers; layer++) {
 		if (seq->regparam && seq->regparam[layer]) {
-			for (i=0; i < seq->number; ++i) {
+			if (layer_has_distortion(seq, layer)) {
+				if (seq->distoparam[layer].index == DISTO_FILE)
+					fprintf(seqfile, "D%c %d %s\n",
+					seq->cfa_opened_monochrome ? '*' : '0' + layer,
+					DISTO_FILE,
+					seq->distoparam[layer].filename);
+				else if (seq->distoparam[layer].index == DISTO_FILES)
+					fprintf(seqfile, "D%c %d\n",
+					seq->cfa_opened_monochrome ? '*' : '0' + layer,
+					DISTO_FILES);
+				else if (seq->distoparam[layer].index == DISTO_MASTER) {
+					fprintf(seqfile, "D%c %d %s\n",
+					seq->cfa_opened_monochrome ? '*' : '0' + layer,
+					DISTO_MASTER,
+					seq->distoparam[layer].filename);
+				}
+				else if (seq->distoparam[layer].index == DISTO_FILE_COMET) {
+					fprintf(seqfile, "D%c %d %.3f %.3f %s\n",
+					seq->cfa_opened_monochrome ? '*' : '0' + layer,
+					DISTO_FILE_COMET,
+					seq->distoparam[layer].velocity.x,
+					seq->distoparam[layer].velocity.y,
+					seq->distoparam[layer].filename ? seq->distoparam[layer].filename : "");
+				}
+			}
+			for (i = 0; i < seq->number; ++i) {
 				fprintf(seqfile, "R%c %g %g %g %g %g %d H %g %g %g %g %g %g %g %g %g\n",
 						seq->cfa_opened_monochrome ? '*' : '0' + layer,
 						seq->regparam[layer][i].fwhm,
@@ -661,7 +776,7 @@ int writeseqfile(sequence *seq){
 			}
 		}
 		if (seq->stats && seq->stats[layer]) {
-			for (i=0; i < seq->number; ++i) {
+			for (i = 0; i < seq->number; ++i) {
 				if (!seq->stats[layer][i]) continue;
 
 				fprintf(seqfile, "M%c-%d %ld %ld %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg\n",
@@ -686,7 +801,7 @@ int writeseqfile(sequence *seq){
 	}
 	for (layer = 0; layer < 3; layer++) {
 		if (seq->regparam_bkp && seq->regparam_bkp[layer]) {
-			for (i=0; i < seq->number; ++i) {
+			for (i = 0; i < seq->number; ++i) {
 				fprintf(seqfile, "R%c %g %g %g %g %g %d H %g %g %g %g %g %g %g %g %g\n",
 						seq->cfa_opened_monochrome ? '0' + layer : '*',
 						seq->regparam_bkp[layer][i].fwhm,
@@ -708,7 +823,7 @@ int writeseqfile(sequence *seq){
 			}
 		}
 		if (seq->stats_bkp && seq->stats_bkp[layer]) {
-			for (i=0; i < seq->number; ++i) {
+			for (i = 0; i < seq->number; ++i) {
 				if (!seq->stats_bkp[layer][i]) continue;
 
 				fprintf(seqfile, "M%c-%d %ld %ld %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg %lg\n",
@@ -730,6 +845,34 @@ int writeseqfile(sequence *seq){
 			}
 		}
 	}
+	if (seq->ostats) {
+		for (layer = 0; layer < seq->nb_layers; layer++) {
+			int Npairs = seq->number * (seq->number - 1) / 2;
+			for (i = 0; i < Npairs; i++) {
+				if (seq->ostats[layer][i].i == -1)
+					continue;
+				fprintf(seqfile, "O%d %d %d %d %d %d %d %d %d %zu %g %g %g %g %g %g %g %g\n",
+					layer,
+					seq->ostats[layer][i].i,
+					seq->ostats[layer][i].j,
+					seq->ostats[layer][i].areai.x,
+					seq->ostats[layer][i].areai.y,
+					seq->ostats[layer][i].areaj.x,
+					seq->ostats[layer][i].areaj.y,
+					seq->ostats[layer][i].areai.w,
+					seq->ostats[layer][i].areai.h,
+					seq->ostats[layer][i].Nij,
+					seq->ostats[layer][i].medij,
+					seq->ostats[layer][i].medji,
+					seq->ostats[layer][i].madij,
+					seq->ostats[layer][i].madji,
+					seq->ostats[layer][i].locij,
+					seq->ostats[layer][i].locji,
+					seq->ostats[layer][i].scaij,
+					seq->ostats[layer][i].scaji);
+			}
+		}
+	}
 
 	fclose(seqfile);
 	seq->needs_saving = FALSE;
