@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,11 +40,9 @@
 #include "algos/star_finder.h"
 #include "registration/registration.h"
 #include "registration/matching/atpmatch.h"
-#include "registration/matching/match.h"
 #include "opencv/opencv.h"
 /* ******************* */
 #include "stacking/stacking.h"
-#include "stacking/sum.h"
 #include "algos/noise.h"
 #include "algos/statistics.h"
 #include "algos/demosaicing.h"
@@ -163,7 +161,9 @@ void stop_live_stacking_engine() {
 
 static int wait_for_file_to_be_written(const gchar *filename) {
 	int iter;
+#ifndef _WIN32
 	guint64 last_size = 0;
+#endif
 	GFile *fd = g_file_new_for_path(filename);
 	for (iter = 1; iter < WAIT_FILE_WRITTEN_ITERS; iter++) {
 		g_usleep(WAIT_FILE_WRITTEN_US);
@@ -211,7 +211,7 @@ static void file_changed(GFileMonitor *monitor, GFile *file, GFile *other,
 			if (!wait_for_file_to_be_written(filename)) {
 				fits dest = { 0 };
 				gchar *new = replace_ext(filename, com.pref.ext);
-				any_to_fits(TYPERAW, filename, &dest, FALSE, !com.pref.force_16bit, FALSE);
+				any_to_fits(TYPERAW, filename, &dest, FALSE, !com.pref.force_16bit);
 				savefits(new, &dest);
 				clearfits(&dest);
 			}
@@ -305,14 +305,14 @@ static void init_preprocessing_from_command(char *dark, char *flat, gboolean use
 				prepro->use_cosmetic_correction = FALSE;
 				siril_log_message(_("Calibration with color images is not yet supported\n"));
 			}
-			else if (strlen(prepro->dark->bayer_pattern) > 4) {
+			else if (strlen(prepro->dark->keywords.bayer_pattern) > 4) {
 				prepro->use_cosmetic_correction = FALSE;
 				prepro->fix_xtrans = TRUE;
 			} else {
 				prepro->use_cosmetic_correction = TRUE;
 				prepro->sigma[0] = -1.0;
 				prepro->sigma[1] = 3.5;
-				if (strlen(prepro->dark->bayer_pattern) >= 4)
+				if (strlen(prepro->dark->keywords.bayer_pattern) >= 4)
 					prepro->is_cfa = TRUE;
 			}
 			siril_log_message(_("Master dark %d x %d configured for live stacking (%s cosmetic correction)\n"), prepro->dark->rx, prepro->dark->ry, prepro->use_cosmetic_correction ? _("with") : _("without") );
@@ -334,7 +334,7 @@ static void init_preprocessing_from_command(char *dark, char *flat, gboolean use
 			} else {
 				prepro->use_flat = TRUE;
 				prepro->autolevel = TRUE;
-				if (strlen(prepro->flat->bayer_pattern) >= 4) {
+				if (strlen(prepro->flat->keywords.bayer_pattern) >= 4) {
 					prepro->is_cfa = TRUE;
 					prepro->equalize_cfa = TRUE;
 				}
@@ -439,7 +439,7 @@ int start_livestack_from_command(gchar *dark, gchar *flat, gboolean use_file_wat
 			return 1;
 		}
 
-		if (cvTransformImage(fit, H, FALSE, REGISTRATION_INTERPOLATION, TRUE)) {
+		if (cvTransformImage(fit, H, FALSE, REGISTRATION_INTERPOLATION, TRUE, NULL)) {
 			return 1;
 		}
 
@@ -470,27 +470,37 @@ static int live_stacking_star_align_prepare(struct generic_seq_args *args) {
 
 static int start_global_registration(sequence *seq) {
 	/* Register the sequence */
-	struct registration_args reg_args = { 0 };
-	reg_args.func = &register_star_alignment;
-	reg_args.seq = seq;
-	reg_args.reference_image = 0;
-	reg_args.layer = (seq->nb_layers == 3) ? 1 : 0;
-	reg_args.run_in_thread = FALSE;
-	reg_args.follow_star = FALSE;
-	reg_args.matchSelection = FALSE;
-	//memcpy(&reg_args.selection, &com.selection, sizeof(rectangle));
-	reg_args.x2upscale = FALSE;
-	reg_args.cumul = FALSE;
-	reg_args.min_pairs = 10;
-	reg_args.no_output = !reg_rotates;
-	reg_args.prefix = "r_";
-	reg_args.load_new_sequence = FALSE;
-	reg_args.interpolation = REGISTRATION_INTERPOLATION;
-	reg_args.type = reg_type;
-	reg_args.max_stars_candidates = 200;
+	struct registration_args regargs = { 0 };
+	regargs.func = &register_star_alignment;
+	regargs.seq = seq;
+	regargs.reference_image = 0;
+	regargs.layer = (seq->nb_layers == 3) ? 1 : 0;
+	regargs.run_in_thread = FALSE;
+	regargs.follow_star = FALSE;
+	regargs.matchSelection = FALSE;
+	//memcpy(&regargs.selection, &com.selection, sizeof(rectangle));
+	regargs.output_scale = 1.f;
+	regargs.min_pairs = 10;
+	regargs.no_output = !reg_rotates;
+	regargs.prefix = "r_";
+	regargs.load_new_sequence = FALSE;
+	regargs.interpolation = REGISTRATION_INTERPOLATION;
+	regargs.type = reg_type;
+	regargs.max_stars_candidates = 200;
+	cvGetEye(&regargs.framingd.Htransf);
+	cvGetEye(&regargs.framingd.Hshift);
+	regargs.framingd.roi_out = (framing_roi){ 0, 0, seq->rx, seq->ry};
+
+	// preparing detection params
+	regargs.sfargs = calloc(1, sizeof(struct starfinder_data));
+	regargs.sfargs->im.from_seq = regargs.seq;
+	regargs.sfargs->layer = regargs.layer;
+	regargs.sfargs->keep_stars = TRUE;
+	regargs.sfargs->save_to_file = FALSE;
+	regargs.sfargs->max_stars_fitted = regargs.max_stars_candidates;
 
 	struct generic_seq_args *args = create_default_seqargs(seq);
-	if (reg_args.filters.filter_included) {
+	if (regargs.filters.filter_included) {
 		args->filtering_criterion = seq_filter_included;
 		args->nb_filtered_images = seq->selnum;
 	}
@@ -502,12 +512,12 @@ static int start_global_registration(sequence *seq) {
 	args->has_output = reg_rotates;
 	args->output_type = get_data_type(seq->bitpix);
 	args->upscale_ratio = 1.0;
-	args->new_seq_prefix = reg_args.prefix;
+	args->new_seq_prefix = strdup(regargs.prefix);
 	args->load_new_sequence = FALSE;
 	args->already_in_a_thread = TRUE;
 	if (!sadata) {
 		sadata = calloc(1, sizeof(struct star_align_data));
-		sadata->regargs = &reg_args;
+		sadata->regargs = &regargs;
 	}
 	args->user = sadata;
 	/* registration will work in 16 bits if input data is 16 bits */
@@ -518,10 +528,10 @@ static int start_global_registration(sequence *seq) {
 	// hack to not free the regparam, because it's referenced by
 	// sadata->current_regdata because of the first call to the prepare,
 	// and used for reference frame params in the registration
-	regparam_bkp = seq->regparam[reg_args.layer];
-	seq->regparam[reg_args.layer] = NULL;
+	regparam_bkp = seq->regparam[regargs.layer];
+	seq->regparam[regargs.layer] = NULL;
 	free_sequence(seq, FALSE);
-
+	free(regargs.sfargs);
 	return retval || !sadata->success[1];
 }
 
@@ -568,7 +578,7 @@ static gpointer live_stacker(gpointer arg) {
 				clearfits(&fit);
 				break;
 			}
-			gboolean is_CFA = fit.bayer_pattern[0] != '\0';
+			gboolean is_CFA = fit.keywords.bayer_pattern[0] != '\0';
 			use_demosaicing = is_CFA ? BOOL_TRUE : BOOL_FALSE;
 			if (prepro)
 				prepro->debayer = is_CFA;
@@ -583,14 +593,16 @@ static gpointer live_stacker(gpointer arg) {
 			/* we want to load the image while avoiding GTK+ calls
 			 * from this thread, setting com.script is a hack to
 			 * avoid this, but this doesn't display the loaded
-			 * image, so we still need the extra idle in
-			 * complete_image_loading
+			 * image, so we still need the extra idle called by
+			 * execute_idle_and_wait_for_it() (this immediately
+			 * returns if headless)
 			 */
 			gboolean script_bkp = com.script;
 			com.script = TRUE;
 			open_single_image(filename);
 			com.script = script_bkp;
-			complete_image_loading();
+			if (!com.headless)
+				execute_idle_and_wait_for_it(end_image_loading, NULL);
 		}
 
 		siril_debug_print("Adding file to input sequence\n");
@@ -643,7 +655,8 @@ static gpointer live_stacker(gpointer arg) {
 			if (buildseqfile(&seq, 1) || seq.number == 1) {
 				index++;
 				livestacking_display(_("Waiting for second image"), FALSE);
-				livestacking_update_number_of_images(1, gfit.exposure, -1.0, NULL);
+				livestacking_update_number_of_images(1, gfit.keywords.exposure, -1.0, NULL);
+				free(seq.seqname);
 				continue;
 			}
 			first_loop = FALSE;
@@ -746,7 +759,7 @@ static gpointer live_stacker(gpointer arg) {
 		stackparam.use_32bit_output = get_data_type(r_seq.bitpix) == DATA_FLOAT ||
 			(use_32bits && (prepro || use_demosaicing == BOOL_TRUE));
 		stackparam.reglayer = (r_seq.nb_layers == 3) ? 1 : 0;
-		stackparam.apply_nbstack_weights = TRUE;
+		stackparam.weighting_type = NBSTACK_WEIGHT;
 
 		reserve_thread(); // hack: generic function fails otherwise
 		main_stack(&stackparam);
@@ -795,7 +808,7 @@ static gpointer live_stacker(gpointer arg) {
 				gdk_threads_add_idle(livestacking_first_result_idle, NULL);
 				first_stacking_result = FALSE;
 			} else {
-				queue_redraw(REMAP_ALL);
+				queue_redraw(REMAP_ALL); // TODO: is this safe enough if the livestacking is running from a python command?
 			}
 		}
 		g_free(result_filename);
@@ -812,7 +825,7 @@ static gpointer live_stacker(gpointer arg) {
 		const char *total_time = format_time_diff(tv_start, tv_end);
 		siril_log_color_message(_("Time to process the last image for live stacking: %s\n"),
 				"green", total_time);
-		livestacking_update_number_of_images(number_of_images_stacked, gfit.livetime, noise, total_time);
+		livestacking_update_number_of_images(number_of_images_stacked, gfit.keywords.livetime, noise, total_time);
 	} while (1);
 
 	siril_debug_print("===== exiting live stacking thread =====\n");
