@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,16 +20,15 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/icc_profile.h"
 #include "core/processing.h"
 #include "core/OS_utils.h"
 #include "core/initfile.h"
 #include "core/siril_log.h"
-#include "algos/siril_wcs.h"
 #include "algos/statistics.h"
 #include "gui/utils.h"
 #include "gui/dialogs.h"
 #include "gui/dialog_preview.h"
-#include "gui/image_display.h"
 #include "gui/message_dialog.h"
 #include "gui/open_dialog.h"
 #include "gui/progress_and_log.h"
@@ -41,6 +40,8 @@
 #include "tinyexpr.h"
 #include "pixel_math_runner.h"
 
+#define T_CURRENT "gfit"
+
 static const gchar *variables[] = {
 		"I1",
 		"I2",
@@ -318,7 +319,7 @@ static gboolean check_files_dimensions(guint *width, guint* height, guint *chann
 
 static gboolean end_pixel_math_operation(gpointer p) {
 	struct pixel_math_data *args = (struct pixel_math_data *)p;
-	stop_processing_thread();// can it be done here in case there is no thread?
+	stop_processing_thread();
 
 	if (!args->ret) {
 		/* write to gfit in the graphical thread */
@@ -328,10 +329,10 @@ static gboolean end_pixel_math_operation(gpointer p) {
 		invalidate_gfit_histogram();
 
 		memcpy(&gfit, args->fit, sizeof(fits));
-
+		icc_auto_assign(&gfit, ICC_ASSIGN_ON_COMPOSITION);
 		com.seq.current = UNRELATED_IMAGE;
 		create_uniq_from_gfit(strdup(_("Pixel Math result")), FALSE);
-		open_single_image_from_gfit();
+		gui_function(open_single_image_from_gfit, NULL);
 	}
 	else clearfits(args->fit);
 
@@ -456,6 +457,10 @@ static gboolean is_pm_rescale_checked() {
 	return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("rescale_pm_button")));
 }
 
+static gboolean is_cumulate_checked() {
+	return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("cumulate_pm_button")));
+}
+
 static float get_min_rescale_value() {
 	return (float) gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spin_pm_low")));
 }
@@ -464,7 +469,7 @@ static float get_max_rescale_value() {
 	return (float) gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spin_pm_high")));
 }
 
-static void update_metadata(fits *fit) {
+static void update_metadata(fits *fit, gboolean do_sum) {
 	fits **f = malloc((MAX_IMAGES + 1) * sizeof(fits *));
 	int j = 0;
 	for (int i = 0; i < MAX_IMAGES ; i++)
@@ -472,24 +477,22 @@ static void update_metadata(fits *fit) {
 			f[j++] = &var_fit[i];
 	f[j] = NULL;
 
-	if (!f[0])
+	if (!f[0] && single_image_is_loaded() )
 		// if no fit used (only constants),
-		// we copy the metadata from first image of the list
-		copy_fits_metadata(var_fit, fit);
+		// we copy the metadata from gfit
+		copy_fits_metadata(&gfit, fit);
 	else
-		merge_fits_headers_to_result2(fit, f);
-	load_WCS_from_memory(fit);
+		merge_fits_headers_to_result2(fit, f, do_sum);
+	update_fits_header(fit);
 	free(f);
 }
 
-static gchar *parse_image_functions(gpointer p, int idx, int c) {
-	struct pixel_math_data *args = (struct pixel_math_data *)p;
-
+static gchar* parse_image_functions(gpointer p, int idx, int c) {
+	struct pixel_math_data *args = (struct pixel_math_data*) p;
 	gchar *expression;
 	gchar **image = args->varname;
 	int nb_images = args->nb_rows;
-	switch(idx) {
-	default:
+	switch (idx) {
 	case 1:
 		expression = args->expression1;
 		break;
@@ -499,113 +502,142 @@ static gchar *parse_image_functions(gpointer p, int idx, int c) {
 	case 3:
 		expression = args->expression3;
 		break;
+	default:
+		return NULL;
 	}
-	if (!expression) return expression;
-	for (int i = 0; i < MAX_IMAGE_FUNCTIONS; i++) {
-		/* First we need to find each occurrences of functions
-		 * Then if args->varname[i] is inside function parameters
-		 * Then apply statistics on args->varname[i]
-		 * Then transform it to numerical expression
-		 */
-		const gchar *function = image_functions[i].name;
-		int len = strlen(function);
-		int total_len = strlen(expression);
+	if (!expression)
+		return expression;
 
-		for (int pos = 0; pos < total_len - len; pos++) {
-			// Match word at current position
-			int found = 1;
-			for (int j = 0; j < len; j++) {
-				// If word is not matched
-				if (expression[pos + j] != function[j]) {
-					found = 0;
-					break;
+	gchar *result = g_strdup(expression);
+	GRegex *regex = g_regex_new("(\\w+)\\((\\w+)\\)", 0, 0, NULL);
+
+	gboolean replaced = TRUE;
+	while (replaced) {
+		replaced = FALSE;
+		GMatchInfo *match_info;
+		g_regex_match(regex, result, 0, &match_info);
+
+		if (g_match_info_matches(match_info)) {
+			gchar *function = g_match_info_fetch(match_info, 1);
+			gchar *param = g_match_info_fetch(match_info, 2);
+			gchar *full_match = g_match_info_fetch(match_info, 0);
+			double median = 0.0, mean = 0.0, min = 0.0, max = 0.0, noise = 0.0, adev = 0.0, bwmv = 0.0, mad = 0.0, sdev = 0.0;
+			double w = 0.0, h = 0.0;
+			imstats *stats = NULL;
+
+			if (g_strcmp0(param, T_CURRENT) == 0) {
+				stats = statistics(NULL, -1, &gfit, c, NULL, STATS_MAIN, MULTI_THREADED);
+				if (!stats) {
+					g_free(full_match);
+					g_free(function);
+					g_free(param);
+					g_match_info_free(match_info);
+					g_regex_unref(regex);
+					return result;
 				}
-			}
-
-			// If word have been found then print found message
-			if (found == 1) {
-				if (expression[pos + len] == '(') {
-					for (int j = 0; j < nb_images; j++) {
-						int len2 = strlen(image[j]);
-						if (expression[pos + len + len2 + 1] == ')') {
-							// possible match
-							gchar *str = malloc(len2 + 1);
-							g_strlcpy(str, expression + pos + len + 1, len2 + 1);
-							if (!g_strcmp0(str, image[j])) {
-								gchar *replace = NULL;
-								double median = 0.0, mean = 0.0, min = 0.0,
-										max = 0.0, noise = 0.0, adev = 0.0,
-										bwmv = 0.0, mad = 0.0, sdev = 0.0;
-								double w = 0.0, h = 0.0;
-								imstats *stats = statistics(NULL, -1, &var_fit[j], c, NULL, STATS_MAIN, MULTI_THREADED);
-								if (!stats) return expression;
-
-								median = stats->median;
-								mean = stats->mean;
-								min = stats->min;
-								max = stats->max;
-								noise = stats->bgnoise;
-								adev = stats->avgDev;
-								bwmv = stats->sqrtbwmv * stats->sqrtbwmv;
-								mad = stats->mad;
-								sdev = stats->sigma;
-
-								w = (double) var_fit[j].rx;
-								h = (double) var_fit[j].ry;
-
-								free_stats(stats);
-
-								if (!g_strcmp0(function, "mean")) {
-									replace = g_strdup_printf("%g", mean);
-								} else if ((!g_strcmp0(function, "med") || !(g_strcmp0(function, "median")))) {
-									replace = g_strdup_printf("%g", median);
-								} else if (!g_strcmp0(function, "min")) {
-									replace = g_strdup_printf("%g", min);
-								} else if (!g_strcmp0(function, "max")) {
-									replace = g_strdup_printf("%g", max);
-								} else if (!g_strcmp0(function, "noise")) {
-									replace = g_strdup_printf("%g", noise);
-								} else if (!g_strcmp0(function, "adev")) {
-									replace = g_strdup_printf("%g", adev);
-								} else if (!g_strcmp0(function, "bwmv")) {
-									replace = g_strdup_printf("%g", bwmv);
-								} else if ((!g_strcmp0(function, "mad") || !(g_strcmp0(function, "mdev")))) {
-									replace = g_strdup_printf("%g", mad);
-								} else if (!g_strcmp0(function, "sdev")) {
-									replace = g_strdup_printf("%g", sdev);
-								} else if ((!g_strcmp0(function, "width") || !(g_strcmp0(function, "w")))) {
-									replace = g_strdup_printf("%g", w);
-								} else if ((!g_strcmp0(function, "height") || !(g_strcmp0(function, "h")))) {
-									replace = g_strdup_printf("%g", h);
-								}
-								if (replace) {
-									GString *string = g_string_new(expression);
-									g_string_erase(string, pos, len + len2 + 2);
-									g_string_insert(string, pos, replace);
-									g_free(expression);
-									expression = g_string_free(string, FALSE);
-									total_len = strlen(expression);
-									pos = pos + strlen(replace);
-									g_free(replace);
-									siril_debug_print("Expression%d: %s\n", c, expression);
-								}
-							}
-							free(str);
+				median = stats->median;
+				mean = stats->mean;
+				min = stats->min;
+				max = stats->max;
+				noise = stats->bgnoise;
+				adev = stats->avgDev;
+				bwmv = stats->sqrtbwmv * stats->sqrtbwmv;
+				mad = stats->mad;
+				sdev = stats->sigma;
+				w = (double) gfit.rx;
+				h = (double) gfit.ry;
+				free_stats(stats);
+			} else {
+				for (int j = 0; j < nb_images; j++) {
+					if (g_strcmp0(param, image[j]) == 0) {
+						stats = statistics(NULL, -1, &var_fit[j], c, NULL, STATS_MAIN, MULTI_THREADED);
+						if (!stats) {
+							g_free(full_match);
+							g_free(function);
+							g_free(param);
+							g_match_info_free(match_info);
+							g_regex_unref(regex);
+							return result;
 						}
+						median = stats->median;
+						mean = stats->mean;
+						min = stats->min;
+						max = stats->max;
+						noise = stats->bgnoise;
+						adev = stats->avgDev;
+						bwmv = stats->sqrtbwmv * stats->sqrtbwmv;
+						mad = stats->mad;
+						sdev = stats->sigma;
+						w = (double) var_fit[j].rx;
+						h = (double) var_fit[j].ry;
+						free_stats(stats);
+						break;
 					}
 				}
 			}
+
+			gchar *replace = NULL;
+			if (!g_strcmp0(function, "mean")) {
+				replace = g_strdup_printf("%g", mean);
+			} else if (!g_strcmp0(function, "med") || !g_strcmp0(function, "median")) {
+				replace = g_strdup_printf("%g", median);
+			} else if (!g_strcmp0(function, "min")) {
+				replace = g_strdup_printf("%g", min);
+			} else if (!g_strcmp0(function, "max")) {
+				replace = g_strdup_printf("%g", max);
+			} else if (!g_strcmp0(function, "noise")) {
+				replace = g_strdup_printf("%g", noise);
+			} else if (!g_strcmp0(function, "adev")) {
+				replace = g_strdup_printf("%g", adev);
+			} else if (!g_strcmp0(function, "bwmv")) {
+				replace = g_strdup_printf("%g", bwmv);
+			} else if (!g_strcmp0(function, "mad") || !g_strcmp0(function, "mdev")) {
+				replace = g_strdup_printf("%g", mad);
+			} else if (!g_strcmp0(function, "sdev")) {
+				replace = g_strdup_printf("%g", sdev);
+			} else if (!g_strcmp0(function, "width") || !g_strcmp0(function, "w")) {
+				replace = g_strdup_printf("%g", w);
+			} else if (!g_strcmp0(function, "height") || !g_strcmp0(function, "h")) {
+				replace = g_strdup_printf("%g", h);
+			}
+
+			if (replace) {
+				gchar *temp = result;
+				// Replace the specific matched string with the calculated value
+				gchar **split = g_strsplit(result, full_match, 2);
+				if (split[0] && split[1]) {
+					result = g_strconcat(split[0], replace, split[1], NULL);
+				} else {
+					result = g_strdup(result);
+				}
+				g_strfreev(split);
+				g_free(temp);
+				g_free(replace);
+				replaced = TRUE;
+				siril_debug_print("Expression%d: %s\n", c, result);
+			}
+
+			g_free(full_match);
+			g_free(function);
+			g_free(param);
 		}
+
+		g_match_info_free(match_info);
 	}
+
+	g_regex_unref(regex);
+
 	for (int j = 0; j < nb_images; j++) {
-		gchar *test =  g_strrstr(expression, image[j]);
+		const gchar *test = g_strrstr(result, image[j]);
 		if (test) {
 			var_fit_mask[j] = TRUE;
-			siril_debug_print("found image name %s in the expression %s\n", image[j], expression);
+			siril_debug_print("found image name %s in the expression %s\n", image[j], result);
 		}
 	}
-	return expression;
+
+	return result;
 }
+
 gpointer apply_pixel_math_operation(gpointer p) {
 	struct pixel_math_data *args = (struct pixel_math_data *)p;
 
@@ -616,8 +648,10 @@ gpointer apply_pixel_math_operation(gpointer p) {
 	args->ret = 0;
 	float maximum = -FLT_MAX;
 	float minimum = +FLT_MAX;
+	long width, height, nchan;
 
 	if (args->single_rgb && args->fit->naxes[2] > 1) {
+		// No need to null check these two as they will be NULL if args->single_rgb is TRUE
 		args->expression2 = g_strdup(args->expression1);
 		args->expression3 = g_strdup(args->expression1);
 
@@ -630,14 +664,15 @@ gpointer apply_pixel_math_operation(gpointer p) {
 		args->expression3 = parse_image_functions(args, 3, RLAYER);
 	}
 
-
 #ifdef _OPENMP
 #pragma omp parallel num_threads(com.max_thread) firstprivate(n1,n2,n3)
 #endif
 	{
 		// we build the expressions in parallel because tr_eval() is not thread-safe
-		te_variable *vars = malloc(nb_rows * sizeof(te_variable));
-		double *x = malloc(nb_rows * sizeof(double));
+		int k = 0;
+		if (args->has_gfit) k = 1;
+		te_variable *vars = malloc((nb_rows + k) * sizeof(te_variable));
+		double *x = malloc((nb_rows + k) * sizeof(double));
 		if (!vars || !x) {
 			failed = TRUE;
 			goto failure;
@@ -648,8 +683,14 @@ gpointer apply_pixel_math_operation(gpointer p) {
 			vars[i].context = NULL;
 			vars[i].type = 0;
 		}
+		if (args->has_gfit) {
+			vars[nb_rows].name = g_strdup(T_CURRENT);
+			vars[nb_rows].address = &x[nb_rows];
+			vars[nb_rows].context = NULL;
+			vars[nb_rows].type = 0;
+		}
 		int err = 0;
-		n1 = te_compile(args->expression1, vars, nb_rows, &err);
+		n1 = te_compile(args->expression1, vars, nb_rows + k, &err);
 		if (!n1) {
 #ifdef _OPENMP
 			if (omp_get_thread_num() == 0)
@@ -660,7 +701,7 @@ gpointer apply_pixel_math_operation(gpointer p) {
 		}
 
 		if (args->expression2) {
-			n2 = te_compile(args->expression2, vars, nb_rows, &err);
+			n2 = te_compile(args->expression2, vars, nb_rows + k, &err);
 			if (!n2) {
 #ifdef _OPENMP
 				if (omp_get_thread_num() == 0)
@@ -670,7 +711,7 @@ gpointer apply_pixel_math_operation(gpointer p) {
 				goto failure;
 			}
 
-			n3 = te_compile(args->expression3, vars, nb_rows, &err);
+			n3 = te_compile(args->expression3, vars, nb_rows + k, &err);
 			if (!n3) {
 #ifdef _OPENMP
 				if (omp_get_thread_num() == 0)
@@ -680,37 +721,97 @@ gpointer apply_pixel_math_operation(gpointer p) {
 				goto failure;
 			}
 		}
+		if (args->has_gfit && nb_rows == 0) {
+			width = gfit.naxes[0];
+			height = gfit.naxes[1];
+			nchan = gfit.naxes[2];
+		} else {
+			width = var_fit[0].naxes[0];
+			height = var_fit[0].naxes[1];
+			nchan = var_fit[0].naxes[2];
+		}
+		if (com.pref.force_16bit) {
 #ifdef _OPENMP
 #pragma omp for schedule(static) reduction(max:maximum) reduction(min:minimum)
 #endif
-		for (size_t px = 0; px < var_fit[0].naxes[0] * var_fit[0].naxes[1] * var_fit[0].naxes[2]; px++) {
-			/* The variables can be changed here, and eval can be called as many
-			 * times as you like. This is fairly efficient because the parsing has
-			 * already been done. */
-			for (int i = 0; i < nb_rows; i++) {
-				x[i] = var_fit[i].fdata[px];
-			}
-
-			if (!args->single_rgb) { // in that case var_fit[0].naxes[2] == 1, but we built RGB
-				fit->fpdata[RLAYER][px] = (float) te_eval(n1);
-				fit->fpdata[GLAYER][px] = (float) te_eval(n2);
-				fit->fpdata[BLAYER][px] = (float) te_eval(n3);
-
-				/* may not be used but (only if rescale) at least it is computed */
-				maximum = max(maximum, max(fit->fpdata[RLAYER][px], max(fit->fpdata[GLAYER][px], fit->fpdata[BLAYER][px])));
-				minimum = min(minimum, min(fit->fpdata[RLAYER][px], min(fit->fpdata[GLAYER][px], fit->fpdata[BLAYER][px])));
-			} else {
-				if (px < (var_fit[0].naxes[0] * var_fit[0].naxes[1])) {
-					fit->fdata[px] = (float) te_eval(n1);
-				} else if (px < 2 * (var_fit[0].naxes[0] * var_fit[0].naxes[1])) {
-					fit->fdata[px] = (float) te_eval(n2);
-				} else {
-					fit->fdata[px] = (float) te_eval(n3);
+			for (size_t px = 0; px < width * height * nchan; px++) {
+				/* The variables can be changed here, and eval can be called as many
+				 * times as you like. This is fairly efficient because the parsing has
+				 * already been done. */
+				for (int i = 0; i < nb_rows; i++) {
+					x[i] = (double) var_fit[i].fdata[px];
+				}
+				if (args->has_gfit) {
+					if (gfit.type == DATA_USHORT) {
+						x[nb_rows] = (double) gfit.data[px] / USHRT_MAX_DOUBLE;
+					} else {
+						x[nb_rows] = (double) gfit.fdata[px];
+					}
 				}
 
-				/* may not be used but (only if rescale) at least it is computed */
-				maximum = max(maximum, fit->fdata[px]);
-				minimum = min(minimum, fit->fdata[px]);
+				if (!args->single_rgb) { // in that case var_fit[0].naxes[2] == 1, but we built RGB
+					fit->pdata[RLAYER][px] =  roundf_to_WORD((float) te_eval(n1) * USHRT_MAX_SINGLE);
+					fit->pdata[GLAYER][px] =  roundf_to_WORD((float) te_eval(n2) * USHRT_MAX_SINGLE);
+					fit->pdata[BLAYER][px] =  roundf_to_WORD((float) te_eval(n3) * USHRT_MAX_SINGLE);
+
+					/* may not be used but (only if rescale) at least it is computed */
+					maximum = max(maximum, max(fit->pdata[RLAYER][px], max(fit->pdata[GLAYER][px], fit->pdata[BLAYER][px])));
+					minimum = min(minimum, min(fit->pdata[RLAYER][px], min(fit->pdata[GLAYER][px], fit->pdata[BLAYER][px])));
+				} else {
+					if (px < (var_fit[0].naxes[0] * var_fit[0].naxes[1])) {
+						fit->data[px] =  roundf_to_WORD((float) te_eval(n1) * USHRT_MAX_SINGLE);
+					} else if (px < 2 * (var_fit[0].naxes[0] * var_fit[0].naxes[1])) {
+						fit->data[px] =  roundf_to_WORD((float) te_eval(n2) * USHRT_MAX_SINGLE);
+					} else {
+						fit->data[px] =  roundf_to_WORD((float) te_eval(n3) * USHRT_MAX_SINGLE);
+					}
+
+					/* may not be used but (only if rescale) at least it is computed */
+					maximum = max(maximum, fit->data[px]);
+					minimum = min(minimum, fit->data[px]);
+				}
+			}
+
+		} else {
+#ifdef _OPENMP
+#pragma omp for schedule(static) reduction(max:maximum) reduction(min:minimum)
+#endif
+			for (size_t px = 0; px < width * height * nchan; px++) {
+				/* The variables can be changed here, and eval can be called as many
+				 * times as you like. This is fairly efficient because the parsing has
+				 * already been done. */
+				for (int i = 0; i < nb_rows; i++) {
+					x[i] = var_fit[i].fdata[px];
+				}
+				if (args->has_gfit) {
+					if (gfit.type == DATA_USHORT) {
+						x[nb_rows] = gfit.data[px] / USHRT_MAX_DOUBLE;
+					} else {
+						x[nb_rows] = gfit.fdata[px];
+					}
+				}
+
+				if (!args->single_rgb) { // in that case var_fit[0].naxes[2] == 1, but we built RGB
+					fit->fpdata[RLAYER][px] = (float) te_eval(n1);
+					fit->fpdata[GLAYER][px] = (float) te_eval(n2);
+					fit->fpdata[BLAYER][px] = (float) te_eval(n3);
+
+					/* may not be used but (only if rescale) at least it is computed */
+					maximum = max(maximum, max(fit->fpdata[RLAYER][px], max(fit->fpdata[GLAYER][px], fit->fpdata[BLAYER][px])));
+					minimum = min(minimum, min(fit->fpdata[RLAYER][px], min(fit->fpdata[GLAYER][px], fit->fpdata[BLAYER][px])));
+				} else {
+					if (px < (width * height)) {
+						fit->fdata[px] = (float) te_eval(n1);
+					} else if (px < 2 * (width * height)) {
+						fit->fdata[px] = (float) te_eval(n2);
+					} else {
+						fit->fdata[px] = (float) te_eval(n3);
+					}
+
+					/* may not be used but (only if rescale) at least it is computed */
+					maximum = max(maximum, fit->fdata[px]);
+					minimum = min(minimum, fit->fdata[px]);
+				}
 			}
 		}
 
@@ -725,17 +826,28 @@ failure: // failure before the eval loop
 	} // end of parallel block
 
 	if (args->rescale) {
+		if (com.pref.force_16bit) {
+			args->min *= USHRT_MAX_SINGLE;
+			args->max *= USHRT_MAX_SINGLE;
 #ifdef _OPENMP
 #pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
-		for (int i = 0; i < fit->rx * fit->ry * fit->naxes[2]; i++) {
-			fit->fdata[i] = (((args->max - args->min) * (fit->fdata[i] - minimum)) / (maximum - minimum)) + args->min;
+			for (int i = 0; i < fit->rx * fit->ry * fit->naxes[2]; i++) {
+				fit->data[i] = roundf_to_WORD((float)(args->max - args->min) * (float)(fit->data[i] - minimum) / (float)(maximum - minimum) + args->min);
+			}
+		} else {
+#ifdef _OPENMP
+#pragma omp parallel for num_threads(com.max_thread) schedule(static)
+#endif
+			for (int i = 0; i < fit->rx * fit->ry * fit->naxes[2]; i++) {
+				fit->fdata[i] = (((args->max - args->min) * (fit->fdata[i] - minimum)) / (maximum - minimum)) + args->min;
+			}
 		}
 	}
 
 	if (failed)
 		args->ret = 1;
-	else update_metadata(args->fit);
+	else update_metadata(args->fit, args->do_sum);
 
 	/* free memory */
 	g_free(args->expression1);
@@ -754,7 +866,6 @@ failure: // failure before the eval loop
 		if (!failed) {
 			clearfits(&gfit);
 			memcpy(&gfit, args->fit, sizeof(fits));
-
 			com.seq.current = UNRELATED_IMAGE;
 			create_uniq_from_gfit(strdup(_("Pixel Math result")), FALSE);
 		}
@@ -762,10 +873,9 @@ failure: // failure before the eval loop
 		free(args->fit);
 		free(args);
 	}
-	else if (com.script)
+	else {
 		execute_idle_and_wait_for_it(end_pixel_math_operation, args);
-	else
-		siril_add_idle(end_pixel_math_operation, args);
+	}
 	return GINT_TO_POINTER((gint)failed);
 }
 
@@ -874,11 +984,13 @@ static int parse_parameters(gchar **expression1, gchar **expression2, gchar **ex
 	return 0;
 }
 
+
 int load_pm_var(const gchar *var, int index, int *w, int *h, int *c) {
 	if (index > MAX_IMAGES - 1) {
 		siril_log_message(_("A maximum of %d images can be used in a single expression.\n"), MAX_IMAGES);
 		return 1;
 	}
+
 	if (readfits(var, &var_fit[index], NULL, TRUE)) {
 		*w = *h = *c = -1;
 		return 1;
@@ -896,6 +1008,49 @@ void free_pm_var(int nb) {
 	}
 }
 
+static int replace_t_with_gfit(struct pixel_math_data *args) {
+	int retval = 0;
+	/* Check if single_rgb is FALSE and if $T is present in any expression */
+	if (args->single_rgb == FALSE &&
+		((args->expression1 && g_strstr_len(args->expression1, -1, "$T") != NULL) ||
+		 (args->expression2 && g_strstr_len(args->expression2, -1, "$T") != NULL) ||
+		 (args->expression3 && g_strstr_len(args->expression3, -1, "$T") != NULL))) {
+		retval = 1;
+	}
+
+	/* If we reach here, either single_RGB is FALSE or no $T is present */
+	if ((args->expression1 && g_strstr_len(args->expression1, -1, "$T") != NULL) ||
+		(args->expression2 && g_strstr_len(args->expression2, -1, "$T") != NULL) ||
+		(args->expression3 && g_strstr_len(args->expression3, -1, "$T") != NULL)) {
+
+		const gchar *pattern = "\\$T";
+		const gchar *replacement = T_CURRENT;
+
+		args->has_gfit = TRUE;
+
+		if (args->expression1 != NULL) {
+			gchar *new_expression1 = g_regex_replace(g_regex_new(pattern, 0, 0, NULL), args->expression1, -1, 0, replacement, 0, NULL);
+			g_free(args->expression1);
+			args->expression1 = new_expression1;
+		}
+
+		if (args->expression2 != NULL) {
+			gchar *new_expression2 = g_regex_replace(g_regex_new(pattern, 0, 0, NULL), args->expression2, -1, 0, replacement, 0, NULL);
+			g_free(args->expression2);
+			args->expression2 = new_expression2;
+		}
+
+		if (args->expression3 != NULL) {
+			gchar *new_expression3 = g_regex_replace(g_regex_new(pattern, 0, 0, NULL), args->expression3, -1, 0, replacement, 0, NULL);
+			g_free(args->expression3);
+			args->expression3 = new_expression3;
+		}
+	}
+
+	/* Success - no errors encountered */
+	return retval;
+}
+
 static int pixel_math_evaluate(gchar *expression1, gchar *expression2, gchar *expression3) {
 	int nb_rows = 0;
 
@@ -903,67 +1058,136 @@ static int pixel_math_evaluate(gchar *expression1, gchar *expression2, gchar *ex
 	int height = -1;
 	int channel = -1;
 
+	int retval = 0;
+
+	gboolean icc_warning_given = FALSE;
 	gboolean single_rgb = is_pm_use_rgb_button_checked();
 	gboolean rescale = is_pm_rescale_checked();
+	gboolean do_sum = is_cumulate_checked();
 	float min = get_min_rescale_value();
 	float max = get_max_rescale_value();
 
 	while (nb_rows < get_pixel_math_number_of_rows() && nb_rows < MAX_IMAGES) {
 		const gchar *path = get_pixel_math_var_paths(nb_rows);
-		if (readfits(path, &var_fit[nb_rows], NULL, TRUE)) return -1;
-		if (channel == - 1) {
+		if (readfits(path, &var_fit[nb_rows], NULL, TRUE)) {
+			retval = 1;
+			goto free_expressions;
+		}
+
+		// Check ICC profiles are defined and the same
+		if (nb_rows > 0) {
+			if (!profiles_identical(var_fit[nb_rows].icc_profile, var_fit[0].icc_profile)) {
+				if (!icc_warning_given) {
+					siril_log_color_message(_("ICC profiles are inconsistent. The output color profile will be based on the first layer to be loaded.\n"), "salmon");
+					icc_warning_given = TRUE;
+				}
+				if (var_fit[0].icc_profile)
+					siril_log_color_message(_("ICC profile of layer %d does not match the first image. Converting it to match.\n"), "salmon", nb_rows + 1);
+				else
+					siril_log_color_message(_("The first layer loaded had no color profile. All input layers will be treated as raw data.\n"), "salmon");
+				// This also takes care of the stuation where a file doesn't have an
+				// ICC profile - in that case it is assigned a profile to match.
+				// Ultimately it is much better to use images that all have the same
+				// color profile - when they don't we're reduced to guesswork about the
+				// user's intention.
+				siril_colorspace_transform(&var_fit[nb_rows], var_fit[0].icc_profile);
+			}
+		}
+		// Check channels are compatible
+		if (channel == -1) {
 			width = var_fit[nb_rows].rx;
 			height = var_fit[nb_rows].ry;
 			channel = var_fit[nb_rows].naxes[2];
 		}
 
 		if (channel == 3 && !single_rgb) {
-			siril_message_dialog(GTK_MESSAGE_ERROR, _("Incompatible parameters"),
+			queue_message_dialog(GTK_MESSAGE_ERROR, _("Incompatible parameters"),
 					_("3 channel images are incompatible with the \"Use single RGB/K expression\" unchecked."));
-			return 1;
+			retval = 1;
+			goto free_expressions;
 		}
 		nb_rows++;
 	}
 
-	if (!nb_rows) {
-		siril_message_dialog(GTK_MESSAGE_ERROR, _("No images loaded"), _("You must load images first."));
-		return 1;
-	}
-
 	channel = single_rgb ? channel : 3;
 
-	fits *fit = NULL;
-	if (new_fit_image(&fit, width, height, channel, DATA_FLOAT))
-		return 1;
-
-	struct pixel_math_data *args = malloc(sizeof(struct pixel_math_data));
+	struct pixel_math_data *args = calloc(1, sizeof(struct pixel_math_data));
 
 	if (parse_parameters(&expression1, &expression2, &expression3)) {
-		siril_message_dialog(GTK_MESSAGE_ERROR, _("Parameter error"),
-				_("Parameter symbols could not be parsed."));
-		return 1;
+		queue_message_dialog(GTK_MESSAGE_ERROR, _("Parameter error"), _("Parameter symbols could not be parsed."));
+		retval = 1;
+		goto free_expressions;
+	}
+
+	args->expression1 = g_strdup(expression1);
+	if (!single_rgb) {
+		args->expression2 = g_strdup(expression2);
+		args->expression3 = g_strdup(expression3);
 	}
 
-	args->expression1 = expression1;
-	args->expression2 = single_rgb ? NULL : expression2;
-	args->expression3 = single_rgb ? NULL : expression3;
 	args->single_rgb = single_rgb;
 	args->rescale = rescale;
+	args->do_sum = do_sum;
 	args->min = min;
 	args->max = max;
-	args->fit = fit;
 	args->nb_rows = nb_rows;
 	args->ret = 0;
 	args->from_ui = TRUE;
+	args->has_gfit = FALSE;
 
 	args->varname = malloc(nb_rows * sizeof(gchar *));
 	for (int i = 0; i < nb_rows; i++) {
 		args->varname[i] = g_strdup(get_pixel_math_var_name(i));
 	}
 
-	start_in_new_thread(apply_pixel_math_operation, args);
+	if (replace_t_with_gfit(args) && gfit.naxes[2] > 1) {
+		queue_message_dialog(GTK_MESSAGE_ERROR, _("Incorrect formula"), _("RGB $T cannot be used in this context."));
+		retval = 1;
+		goto free_expressions;
+	}
 
-	return 0;
+	fits *fit = NULL;
+	if (args->has_gfit) { // in the case where no images are loaded, at least gfit must be laded
+		width = gfit.rx;
+		height = gfit.ry;
+		channel = args->single_rgb ? gfit.naxes[2] : 3;
+		if (nb_rows > 0) {
+			if (width != var_fit[0].naxes[0] ||
+				height != var_fit[0].naxes[1]) {
+					queue_message_dialog(GTK_MESSAGE_ERROR, _("Images have different size"),
+						_("The image currently displayed must be the same size as the other images loaded into PixelMath."));
+				retval = 1;
+				goto free_expressions;
+			}
+		}
+	}
+
+	if (!nb_rows && !args->has_gfit) {
+		queue_message_dialog(GTK_MESSAGE_ERROR, _("No images loaded"), _("You must load images first."));
+		retval = 1;
+		goto free_expressions;
+	}
+
+	if (new_fit_image(&fit, width, height, channel, com.pref.force_16bit ? DATA_USHORT : DATA_FLOAT)) {
+		retval = 1;
+		goto free_expressions;
+	}
+
+	args->fit = fit;
+
+	if (!start_in_new_thread(apply_pixel_math_operation, args)) {
+		g_free(args->expression1);
+		g_free(args->expression2);
+		g_free(args->expression3);
+		free(args);
+	}
+
+free_expressions:
+	g_free(expression1);
+	g_free(expression2);
+	g_free(expression3);
+
+	return retval;
 }
 
 static gboolean allowed_char(char c) {
@@ -1026,23 +1250,6 @@ static void add_image_to_variable_list(const gchar *path, const gchar *var, gcha
 
 }
 
-static void gtk_filter_add(GtkFileChooser *file_chooser, const gchar *title,
-		const gchar *pattern) {
-	gchar **patterns;
-	gint i;
-
-	GtkFileFilter *f = gtk_file_filter_new();
-	gtk_file_filter_set_name(f, title);
-	/* get the patterns */
-	patterns = g_strsplit(pattern, ";", -1);
-	for (i = 0; patterns[i] != NULL; i++)
-		gtk_file_filter_add_pattern(f, patterns[i]);
-	/* free the patterns */
-	g_strfreev(patterns);
-	gtk_file_chooser_add_filter(file_chooser, f);
-	gtk_file_chooser_set_filter(file_chooser, f);
-}
-
 static int search_for_free_index() {
 	int i;
 	for (i = 0; i < MAX_IMAGES; i++) {
@@ -1073,7 +1280,7 @@ static void select_image(int nb) {
 	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), com.wd);
 	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
 	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), FALSE);
-	gtk_filter_add(GTK_FILE_CHOOSER(dialog), _("FITS Files (*.fit, *.fits, *.fts, *.fit.fz, *.fits.fz, *.fts.fz)"), FITS_EXTENSIONS);
+	gtk_filter_add(GTK_FILE_CHOOSER(dialog), _("FITS Files (*.fit, *.fits, *.fts, *.fit.fz, *.fits.fz, *.fts.fz)"), FITS_EXTENSIONS, gui.file_ext_filter == TYPEFITS);
 	siril_file_chooser_add_preview(GTK_FILE_CHOOSER(dialog), preview);
 
 	res = gtk_dialog_run(GTK_DIALOG(dialog));
@@ -1108,11 +1315,17 @@ static void select_image(int nb) {
 						g_free(str);
 
 					} else {
-						if (f.filter[0] != '\0') {
-							memcpy(filter, f.filter, FLEN_VALUE);
+						if (f.keywords.filter[0] != '\0') {
+							memcpy(filter, f.keywords.filter, FLEN_VALUE);
 						}
 
 						int idx = search_for_free_index();
+						if (idx >= MAX_IMAGES) {
+							siril_log_color_message(_("Error: maximum variable index exceeded - too many variables!\n"), "red");
+							g_free(filename);
+							clearfits(&f);
+							break;
+						}
 						add_image_to_variable_list(filename, variables[idx], filter, f.naxes[2], f.naxes[0], f.naxes[1]);
 
 						pos++;
@@ -1131,7 +1344,6 @@ static void select_image(int nb) {
 
 	siril_preview_free(preview);
 	gtk_widget_destroy(dialog);
-
 }
 
 gboolean on_pixel_math_entry_r_focus_in_event(GtkWidget *widget,
@@ -1219,7 +1431,8 @@ static void apply_pixel_math() {
 	gchar *expression3 = get_pixel_math_expression3();
 	remove_spaces_from_str(expression3);
 
-	pixel_math_evaluate(expression1, expression2, expression3);
+	if (pixel_math_evaluate(expression1, expression2, expression3))
+		siril_log_color_message(_("Error evaluating pixelmath expression.\n"), "red");
 }
 
 void on_pixel_math_entry_activate(GtkEntry *entry, gpointer user_data) {
@@ -1238,7 +1451,7 @@ void on_pixel_math_treeview_row_activated(GtkTreeView *tree_view,
 	GtkEntry *entry = get_entry_with_focus();
 
 	GtkEntryBuffer *buffer = gtk_entry_get_buffer(entry);
-	gint *i = gtk_tree_path_get_indices(path);
+	const gint *i = gtk_tree_path_get_indices(path);
 	const gchar *str = get_pixel_math_var_name(i[0]);
 
 	if (str) {
@@ -1362,7 +1575,7 @@ void on_pixel_math_treeview_functions_row_activated(GtkTreeView *tree_view,
 		GtkTreePath *path, GtkTreeViewColumn *column) {
 	GtkEntry *entry = get_entry_with_focus();
 	GtkEntryBuffer *buffer = gtk_entry_get_buffer(entry);
-	gint *i = gtk_tree_path_get_indices(path);
+	const gint *i = gtk_tree_path_get_indices(path);
 	const gchar *str = get_function_name(i[0]);
 
 	if (str) {
@@ -1379,7 +1592,7 @@ void on_pixel_math_treeview_operators_row_activated(GtkTreeView *tree_view,
 		GtkTreePath *path, GtkTreeViewColumn *column) {
 	GtkEntry *entry = get_entry_with_focus();
 	GtkEntryBuffer *buffer = gtk_entry_get_buffer(entry);
-	gint *i = gtk_tree_path_get_indices(path);
+	const gint *i = gtk_tree_path_get_indices(path);
 	const gchar *str = get_operator_name(i[0]);
 
 	if (str) {
@@ -1524,7 +1737,7 @@ void on_pixel_math_treeview_presets_row_activated(GtkTreeView *tree_view,
 	GtkEntry *entry = get_entry_with_focus();
 
 	GtkEntryBuffer *buffer = gtk_entry_get_buffer(entry);
-	gint *i = gtk_tree_path_get_indices(path);
+	const gint *i = gtk_tree_path_get_indices(path);
 	const gchar *str = get_preset_expr(i[0]);
 
 	if (str) {
