@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,15 +27,19 @@
 #include "core/proto.h"
 #include "core/OS_utils.h"
 #include "core/siril_log.h"
+#include "core/siril_alloc.h"
 #include "io/sequence.h"
 #include "io/ser.h"
 #include "io/image_format_fits.h"
 #include "gui/progress_and_log.h"
 #include "algos/sorting.h"
 #include "algos/statistics.h"
+#include "algos/siril_wcs.h"
 #include "stacking/stacking.h"
 #include "stacking/siril_fit_linear.h"
+#include "stacking/blending.h"
 #include "registration/registration.h"
+#include "opencv/opencv.h"
 
 static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean);
 
@@ -62,26 +66,47 @@ int stack_open_all_files(struct stacking_args *args, int *bitpix, int *naxis, lo
 	int nb_frames = args->nb_images_to_stack;
 	guint stackcnt = 0;
 	double livetime = 0.0;
+	gboolean drizzle = TRUE;
 	*bitpix = 0;
 	*naxis = 0;
 	*naxes = 0;
+	// for max framing
+	double xmin = DBL_MAX;
+	double xmax = -DBL_MAX;
+	double ymin = DBL_MAX;
+	double ymax = -DBL_MAX;
+	if (args->maximize_framing)	{
+		g_assert(args->seq->regparam);
+		g_assert(args->seq->regparam[args->reglayer]);
+	}
 	set_progress_bar_data(_("Opening images for stacking"), PROGRESS_NONE);
 
 	if (args->seq->type == SEQ_REGULAR || args->seq->type == SEQ_FITSEQ) {
-		if (args->apply_nbstack_weights) {
+		if (args->weighting_type == NBSTACK_WEIGHT) {
 			int nb_layers = args->seq->nb_layers;
 			args->weights = malloc(nb_layers * nb_frames * sizeof(double));
 		}
 		if (args->seq->type == SEQ_FITSEQ) {
 			g_assert(args->seq->fitseq_file);
-			/* we assume that all images have the same dimensions and bitpix */
+			/* we assume that all images have the same dimensions and bitpix 
+			unless we are using max framing, which will recompute dest image size
+			and skip forcing same image size */
 			memcpy(naxes, args->seq->fitseq_file->naxes, sizeof args->seq->fitseq_file->naxes);
 			*naxis = naxes[2] == 3 ? 3 : 2;
 			*bitpix = args->seq->fitseq_file->bitpix;
 		}
-
+		double scale = (args->upscale_at_stacking) ? 2. : 1.;
 		for (int i = 0; i < nb_frames; ++i) {
 			int image_index = args->image_indices[i]; // image index in sequence
+			if (args->seq->is_drizzle) {
+				const gchar *drizztmp = get_sequence_cache_filename(args->seq, image_index, "drizztmp", "fit", NULL);
+				if (!g_file_test(drizztmp, G_FILE_TEST_EXISTS)) {
+					gchar *basename = g_path_get_basename(drizztmp);
+					siril_log_color_message(_("Drizzle file %s not found in ./drizztmp folder, aborting\n"), "red", basename);
+					g_free(basename);
+					drizzle = FALSE;
+				}
+			}
 			if (!get_thread_run())
 				return ST_GENERIC_ERROR;
 			if (i % 20 == 0)
@@ -95,7 +120,7 @@ int stack_open_all_files(struct stacking_args *args, int *bitpix, int *naxis, lo
 				}
 
 				fptr = args->seq->fptr[image_index];
-				if (check_fits_params(fptr, bitpix, naxis, naxes)) {
+				if (check_fits_params(fptr, bitpix, naxis, naxes, args->maximize_framing)) {
 					siril_log_message(_("Opening image %d failed\n"), image_index);
 					return ST_SEQUENCE_ERROR;
 				}
@@ -122,7 +147,7 @@ int stack_open_all_files(struct stacking_args *args, int *bitpix, int *naxis, lo
 			if (image_index == args->ref_image)
 				import_metadata_from_fitsfile(fptr, fit);
 
-			if (args->apply_nbstack_weights) {
+			if (args->weighting_type == NBSTACK_WEIGHT) {
 				int nb_layers = args->seq->nb_layers;
 				double weight = (double)stack_count;
 				siril_debug_print("weight for image %d: %d\n", i, stack_count);
@@ -132,16 +157,59 @@ int stack_open_all_files(struct stacking_args *args, int *bitpix, int *naxis, lo
 					args->weights[nb_frames * 2 + i] = weight;
 				}
 			}
+			/*we compute the dest size if maximize_framing*/
+			if (args->maximize_framing) {
+				regdata *regdat = args->seq->regparam[args->reglayer];
+				int rx = (args->seq->is_variable) ? args->seq->imgparam[image_index].rx : args->seq->rx;
+				int ry = (args->seq->is_variable) ? args->seq->imgparam[image_index].ry : args->seq->ry;
+				xmin = (xmin > regdat[image_index].H.h02 * scale) ? regdat[image_index].H.h02 * scale : xmin;
+				ymin = (ymin > regdat[image_index].H.h12 * scale) ? regdat[image_index].H.h12 * scale : ymin;
+				xmax = (xmax < regdat[image_index].H.h02 * scale + rx) ? regdat[image_index].H.h02 * scale + rx : xmax;
+				ymax = (ymax < regdat[image_index].H.h12 * scale + ry) ? regdat[image_index].H.h12 * scale + ry : ymax;
+			}
 		}
 		if (stackcnt <= 0)
 			stackcnt = nb_frames;
-		fit->stackcnt = stackcnt;
-		fit->livetime = livetime;
+		fit->keywords.stackcnt = stackcnt;
+		fit->keywords.livetime = livetime;
 		// keeping exposure of the reference frame
 
 		if (naxes[2] == 0)
 			naxes[2] = 1;
 		g_assert(naxes[2] <= 3);
+
+		gboolean update_wcs = TRUE;
+		if (args->maximize_framing) {
+			// using same formulas as in applyreg::compute_roi
+			naxes[0] = (int)xmax - (int)xmin + 1;
+			naxes[1] = (int)ymax - (int)ymin + 1;
+			args->offset[0] =  (int)xmin;
+			args->offset[1] = -(int)ymin;
+			siril_debug_print("new size: %ld %ld\n", naxes[0], naxes[1]);
+			siril_debug_print("new origin: %d %d\n", args->offset[0], args->offset[1]);
+		} else if (layer_has_registration(args->seq, args->reglayer)) {
+			double dx, dy;
+			translation_from_H(args->seq->regparam[args->reglayer][args->ref_image].H, &dx, &dy);
+			args->offset[0] = (int)dx;
+			args->offset[1] = (int)dy;
+		} else {
+			update_wcs = FALSE;
+		}
+		if (update_wcs && has_wcs(fit)) {
+			Homography Hs = { 0 };
+			cvGetEye(&Hs);
+			double dx, dy;
+			translation_from_H(args->seq->regparam[args->reglayer][args->ref_image].H, &dx, &dy);
+			// siril_debug_print("ref shift: %d %d\n", (int)dx, (int)dy);
+			// siril_debug_print("crpix: %.1f %.1f\n", fit->keywords.wcslib->crpix[0], fit->keywords.wcslib->crpix[1]);
+			Hs.h02 = dx - args->offset[0];
+			Hs.h12 = args->offset[1] - dy;
+			// int orig_rx = (args->seq->is_variable) ? args->seq->imgparam[args->seq->reference_image].rx : args->seq->rx;
+			int orig_ry = (args->seq->is_variable) ? args->seq->imgparam[args->seq->reference_image].ry : args->seq->ry;
+			// siril_debug_print("size: %d %d\n", orig_rx, orig_ry);
+			cvApplyFlips(&Hs, orig_ry, naxes[1]);
+			reframe_wcs(fit->keywords.wcslib, &Hs);
+		}
 	}
 	else if (args->seq->type == SEQ_SER) {
 		g_assert(args->seq->ser_file);
@@ -153,11 +221,6 @@ int stack_open_all_files(struct stacking_args *args, int *bitpix, int *naxis, lo
 			type_ser = SER_MONO;
 		naxes[2] = type_ser == SER_MONO ? 1 : 3;
 		*naxis = type_ser == SER_MONO ? 2 : 3;
-		/* case of Super Pixel not handled yet */
-		if (com.pref.debayer.open_debayer && com.pref.debayer.bayer_inter == BAYER_SUPER_PIXEL) {
-			siril_log_message(_("Super-pixel is not handled yet for on the fly SER stacking\n"));
-			return ST_GENERIC_ERROR;
-		}
 
 		import_metadata_from_serfile(args->seq->ser_file, fit);
 		for (int i = 0; i < nb_frames; ++i) {
@@ -166,16 +229,25 @@ int stack_open_all_files(struct stacking_args *args, int *bitpix, int *naxis, lo
 			if (dt)
 				*list_date = g_list_prepend(*list_date,	new_date_item(dt, 0.0));
 		}
-		fit->stackcnt = nb_frames;
-		fit->livetime = fit->exposure * nb_frames;
+		fit->keywords.stackcnt = nb_frames;
+		fit->keywords.livetime = fit->keywords.exposure * nb_frames;
 		// keeping the fallacious exposure based on fps from the header
 	} else {
 		siril_log_message(_("Rejection stacking is only supported for FITS images/sequences and SER sequences.\nUse \"Sum Stacking\" instead.\n"));
 		return ST_SEQUENCE_ERROR;
 	}
 
+	if (args->seq->is_drizzle) { 
+		if (drizzle) {
+			siril_log_color_message(_("Drizzle stacking will be used.\n"), "blue");
+			args->drizzle = TRUE;
+		} else {
+			siril_log_color_message(_("Drizzle stacking cannot be performed because drizzle weights are missing.\n"), "red");
+			return ST_GENERIC_ERROR;
+		}
+	}
 	set_progress_bar_data(NULL, PROGRESS_DONE);
-	siril_debug_print("stack count: %u, livetime: %f\n", fit->stackcnt, fit->livetime);
+	siril_debug_print("stack count: %u, livetime: %f\n", fit->keywords.stackcnt, fit->keywords.livetime);
 	return ST_OK;
 }
 
@@ -282,6 +354,30 @@ int stack_compute_parallel_blocks(struct _image_block **blocksptr, long max_numb
 	return ST_OK;
 }
 
+// This function reaaranges data that was written continuously to the buffer by 
+// seq_opened_read_region to the actual stride of block_data. The rest of each line
+// is padded with zeros.
+// For instance, if img has a stride rx_img = 3 while block data has rx = 5 over
+// 2 lines (ry = 2), this will transform abcdef0000 to abc00def00
+static void rearrange_block_data(void *buffer, data_type itype, int rx, int ry, int rx_img) {
+	if (rx == rx_img) // nothing to rearrange
+		return;
+	int ielem_size = itype == DATA_FLOAT ? sizeof(float) : sizeof(WORD);
+	size_t full_stride = rx * ielem_size;
+	size_t img_stride = rx_img * ielem_size;
+	size_t padding_stride = full_stride - img_stride;
+	size_t src = (ry - 1) * img_stride;
+	size_t dst = (ry - 1) * full_stride;
+	for (int j = ry - 1 ; j >= 0; j--) { // we start from the end of the array to rearrange in place
+		// using memmove instead of memcpy to avoid error in case of overlap
+		memmove(buffer + dst, buffer + src, img_stride); 
+		memset(buffer + dst + img_stride, 0, padding_stride);
+		src -= img_stride;
+		dst -= full_stride;
+	}
+	return;
+}
+
 static int stack_read_block_data(struct stacking_args *args,
 		struct _image_block *my_block, struct _data_block *data,
 		long *naxes, data_type itype, int thread_id) {
@@ -289,12 +385,20 @@ static int stack_read_block_data(struct stacking_args *args,
 	int ielem_size = itype == DATA_FLOAT ? sizeof(float) : sizeof(WORD);
 	/* store the layer info to retrieve normalization coeffs*/
 	data->layer = (int)my_block->channel;
+	gboolean masking = (args->feather_dist > 0);
 	/* Read the block from all images, store them in pix[image] */
 	for (int frame = 0; frame < args->nb_images_to_stack; ++frame) {
 		gboolean clear = FALSE, readdata = TRUE;
 		long offset = 0;
+		int image_index = args->image_indices[frame]; // image index in sequence
 		/* area in C coordinates, starting with 0, not cfitsio coordinates. */
-		rectangle area = {0, my_block->start_row, naxes[0], my_block->height};
+		int rx = naxes[0];
+		int ry = naxes[1];
+		if (args->maximize_framing) {
+			rx = (args->seq->is_variable) ? args->seq->imgparam[image_index].rx : args->seq->rx;
+			ry = (args->seq->is_variable) ? args->seq->imgparam[image_index].ry : args->seq->ry;
+		}
+		rectangle area = {0, my_block->start_row, rx, my_block->height};
 
 		if (!get_thread_run())
 			return ST_CANCEL;
@@ -306,31 +410,40 @@ static int stack_read_block_data(struct stacking_args *args,
 			 * shift is managed in the main loop after the read. */
 			regdata *layerparam = args->seq->regparam[args->reglayer];
 			if (layerparam) {
-				double scale = args->seq->upscale_at_stacking;
+				double scale = (args->upscale_at_stacking) ? 2. : 1.;
 				double dx, dy;
 				translation_from_H(layerparam[args->image_indices[frame]].H, &dx, &dy);
+				dy -=args->offset[1];
 				int shifty = round_to_int(dy * scale);
 #ifdef STACK_DEBUG
 				fprintf(stdout, "shifty for image %d: %d\n", args->image_indices[frame], shifty);
 #endif
-				if (area.y + area.h + shifty < 0 || area.y + shifty >= naxes[1]) {
+				if (area.y + area.h + shifty <= 0 || area.y + shifty >= ry) {
 					// entirely outside image below or above: all black pixels
 					clear = TRUE; readdata = FALSE;
 				} else if (area.y + shifty < 0) {
 					/* we read only the bottom part of the area here, which
-					 * requires an offset in pix */
+					* requires an offset in pix */
 					clear = TRUE;
 					area.h += area.y + shifty;	// cropping the height
+					area.h = min(area.h, ry);
 					offset = -naxes[0] * (area.y + shifty);	// positive
 					area.y = 0;
-				} else if (area.y + area.h + shifty >= naxes[1]) {
+				} else if (area.y + area.h + shifty >= ry) {
 					/* we read only the upper part of the area here */
 					clear = TRUE;
 					area.y += shifty;
-					area.h += naxes[1] - (area.y + area.h);
+					area.h += ry - (area.y + area.h);
 				} else {
 					area.y += shifty;
 				}
+				if (area.h <= 0) { // as a last safety net
+					clear = TRUE; readdata = FALSE;
+				}
+				if (area.h == INT_MIN) { // mainly to avoid static checker warning
+					siril_debug_print("Error: image #%d has a wrong area height\n", args->image_indices[frame] + 1);
+					area.h += 1;
+				}
 			}
 #ifdef STACK_DEBUG
 			else fprintf(stderr, "NO REGPARAM\n");
@@ -340,6 +453,10 @@ static int stack_read_block_data(struct stacking_args *args,
 				/* we are reading outside an image, fill with
 				 * zeros and attempt to read lines that fit */
 				memset(data->pix[frame], 0, my_block->height * naxes[0] * ielem_size);
+				if (masking)
+					memset(data->mask[frame], 0, my_block->height * naxes[0] * sizeof(float));
+				if (args->drizzle)
+					memset(data->drizz[frame], 0, my_block->height * naxes[0] * sizeof(float));
 			}
 		}
 
@@ -347,18 +464,80 @@ static int stack_read_block_data(struct stacking_args *args,
 			// reading pixels from current frame
 			void *buffer;
 			if (itype == DATA_FLOAT)
-				buffer = ((float*)data->pix[frame])+offset;
-			else 	buffer = ((WORD *)data->pix[frame])+offset;
+				buffer = ((float*)data->pix[frame]) + offset;
+			else
+				buffer = ((WORD *)data->pix[frame]) + offset;
 			int retval = seq_opened_read_region(args->seq, my_block->channel,
 					args->image_indices[frame], buffer, &area, thread_id);
 			if (retval) {
-#ifdef _OPENMP
-				int tid = omp_get_thread_num();
-				if (tid == 0)
-#endif
-					siril_log_color_message(_("Error reading one of the image areas\n"), "red");
+					siril_log_color_message(_("Error reading one of the image areas (%d: %d %d %d %d)\n"), "red", args->image_indices[frame] + 1,
+					area.x, area.y, area.w, area.h);
 				return ST_SEQUENCE_ERROR;
 			}
+			if (args->maximize_framing) {
+				rearrange_block_data(buffer, itype, naxes[0], area.h, rx);
+			}
+		}
+		
+		if (masking && (args->reglayer < 0 || readdata)) {
+			// we need to compute the correct mask area
+			// We load the corresponding downscaled portion of the mask file (distances to black are already included)
+			// Upcsale it to the mask buffer
+			// Re-arrange it if required (as for the image block) for maximize_framing
+			// Normalize it to 1. (all values > feather_dist -> 1., values < feather_dist -> val/feather_dist)
+			// And finally apply the ramping function which has been precomputed on  RAMP_PACE + 1 points
+			const gchar *maskfile = get_sequence_cache_filename(args->seq, args->image_indices[frame], "cache", "msk", NULL);
+			float *mask_scaled;
+			int scaled_rx = 0, scaled_ry = 0;
+			double fx = 0., fy = 0.;
+			int rx = (args->seq->is_variable) ? args->seq->imgparam[image_index].rx : args->seq->rx;
+			int ry = (args->seq->is_variable) ? args->seq->imgparam[image_index].ry : args->seq->ry;
+			compute_downscaled_mask_size(rx, ry, &scaled_rx, &scaled_ry, &fx, &fy);
+			rectangle maskscaled_area = { 0, (int)(fy * area.y), scaled_rx, (int)(fy * area.h)};
+			if (area.h == 0 || area.w == 0 || maskscaled_area.w == 0 || maskscaled_area.h == 0)
+				continue;
+			mask_scaled = siril_malloc((size_t)(maskscaled_area.h * maskscaled_area.w * sizeof(float)));
+			if (read_mask_fits_area(maskfile, &maskscaled_area, scaled_ry, mask_scaled)) {
+				free(mask_scaled);
+				siril_log_color_message(_("Error reading one of the masks areas (%d: %d %d %d %d)\n"), "red", args->image_indices[frame] + 1,
+				maskscaled_area.x, maskscaled_area.y, maskscaled_area.w, maskscaled_area.h);
+				return ST_SEQUENCE_ERROR;
+			}
+			float *mbuffer = data->mask[frame] + offset;
+			cvUpscaleBlendMask(maskscaled_area.w, maskscaled_area.h, rx, area.h, mask_scaled, mbuffer);
+			siril_free(mask_scaled);
+			if (args->maximize_framing) {
+				rearrange_block_data(mbuffer, DATA_FLOAT, naxes[0], area.h, rx);
+			}
+			float distf = (float)args->feather_dist;
+			float invdistf = 1.f / distf;
+			size_t block_nb_pix = my_block->height * naxes[0];
+			// we normalize and apply the ramping function for all values above 0.
+			for (size_t i = 0; i < block_nb_pix; i++) {
+				if (data->mask[frame][i]) {
+					data->mask[frame][i] = (data->mask[frame][i] > distf) ? 1.f : get_ramped_value(data->mask[frame][i] * invdistf);
+				}
+			}
+		}
+		if (args->drizzle && (args->reglayer < 0 || readdata)) {
+			const gchar *drizzfile = get_sequence_cache_filename(args->seq, args->image_indices[frame], "drizztmp", "fit", NULL);
+			float *dbuffer = data->drizz[frame] + offset;
+			int rx = (args->seq->is_variable) ? args->seq->imgparam[image_index].rx : args->seq->rx;
+			int ry = (args->seq->is_variable) ? args->seq->imgparam[image_index].ry : args->seq->ry;
+			int layer = args->seq->nb_layers == 3 ? (int)my_block->channel : 0;
+			if (read_drizz_fits_area(drizzfile, layer, &area, ry, dbuffer)) {
+				siril_log_color_message(_("Error reading one of the drizzle weights areas (%d: %d %d %d %d)\n"), "red", args->image_indices[frame] + 1,
+				area.x, area.y, area.w, area.h);
+				return ST_SEQUENCE_ERROR;
+			}
+			if (!flip_buffer(FLOAT_IMG, dbuffer, &area)) {
+				siril_log_color_message(_("Error reading one of the drizzle weights areas (%d: %d %d %d %d)\n"), "red", args->image_indices[frame] + 1,
+					area.x, area.y, area.w, area.h);
+				return ST_SEQUENCE_ERROR;
+			}
+			if (args->maximize_framing) {
+				rearrange_block_data(dbuffer, DATA_FLOAT, naxes[0], area.h, rx);
+			}
 		}
 	}
 	return ST_OK;
@@ -502,7 +681,7 @@ int check_G_values(float Gs, float Gc) {
 	return (Gs > Gc);
 }
 
-void confirm_outliers(struct outliers *out, int N, double median, int *rejected, int rej[2]) {
+void confirm_outliers(struct ESD_outliers *out, int N, double median, int *rejected, int rej[2]) {
 	int i = N - 1;
 
 	while (i > 1 && !out[i].out) {
@@ -533,19 +712,31 @@ static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struc
 
 	memcpy(o_stack, stack, N * sizeof(WORD)); /* making a copy of unsorted stack to apply weights (before the median sorts in place)*/
 
-	/* remove null pixels */
-	for (int frame = 0; frame < N; frame++) {
-		if (stack[frame] > 0) {
-			if (frame != kept) {
-				stack[kept] = stack[frame];
+	/* remove null pixels (or null drizzle weights or both)*/
+	if (args->drizzle) {
+		float *d_stack = (float*) data->dstack;
+		for (int frame = 0; frame < N; frame++) {
+			if (stack[frame] > 0 && d_stack[frame] != 0.f) {
+				if (frame != kept) {
+					stack[kept] = stack[frame];
+				} 
+				kept++;
+			}
+		}
+	} else {
+		for (int frame = 0; frame < N; frame++) {
+			if (stack[frame] != 0.f) {
+				if (frame != kept) {
+					stack[kept] = stack[frame];
+				} 
+				kept++;
 			}
-			kept++;
 		}
 	}
 	/* Preventing problems
 	   0: should not happen but just in case.
 	   1 or 2: no need to reject */
-	if (kept <= 2) {
+	if (kept <= 1) {
 		return kept;
 	}
 	removed = N - kept;
@@ -726,7 +917,7 @@ static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struc
 				return kept;
 			}
 			max_outliers -= removed;
-			struct outliers *out = malloc(max_outliers * sizeof(struct outliers));
+			struct ESD_outliers *out = malloc(max_outliers * sizeof(struct ESD_outliers));
 
 			memcpy(w_stack, stack, N * sizeof(WORD));
 			memset(rejected, 0, N * sizeof(int));
@@ -764,15 +955,18 @@ static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struc
 static double mean_and_reject(struct stacking_args *args, struct _data_block *data,
 		int stack_size, data_type itype, int rej[2]) {
 	double mean;
-
+	gboolean masking = (args->feather_dist > 0);
+	gboolean weighting = args->weighting_type > NO_WEIGHT;
 	int layer = data->layer;
 	if (itype == DATA_USHORT) {
 		int kept_pixels = apply_rejection_ushort(data, stack_size, args, rej);
 		if (kept_pixels == 0)
 			mean = quickmedian(data->stack, stack_size);
 		else {
-			if (args->apply_noise_weights || args->apply_nbstack_weights || args->apply_wfwhm_weights || args->apply_nbstars_weights) {
-				double *pweights = args->weights + layer * stack_size;
+			if (weighting || masking || args->drizzle) {
+				double *pweights =  NULL;
+				if (weighting)
+					pweights = args->weights + layer * stack_size;
 				/* min and max computed here instead of rejection step to avoid dealing
 				 * with too many particular cases. For rejection, the original stack
 				 * (o_stack) is used to keep the weight order, stack being sorted, we can
@@ -790,14 +984,28 @@ static double mean_and_reject(struct stacking_args *args, struct _data_block *da
 				for (int frame = 0; frame < stack_size; ++frame) {
 					WORD val = ((WORD*)data->o_stack)[frame];
 					if (val >= pmin && val <= pmax && val > 0) {
-						sum += (float)val * pweights[frame];
-						norm += pweights[frame];
+						double n = 1.;
+						if (args->drizzle)
+							n *= data->dstack[frame];
+						if (masking)
+							n *= data->mstack[frame];
+						if (weighting)
+							n *= pweights[frame];
+						sum += (double)val * n;
+						norm += n;
 					}
 				}
-				if (norm <= 1.e-2) { // if norm is 0, sum is 0 too
-					// if the kept pixel has a weight of 0, bad luck, still take it,
-					// that will not look good, but it's better than a black pixel
-					mean = ((WORD*)data->stack)[0];
+				if (norm == 0. || sum == 0.) { // if norm is 0, sum is 0 too
+					// We replace by the sum without weighting
+					// Will not look good, but it's better than a black pixel
+					sum = 0.;
+					for (int frame = 0; frame < stack_size; ++frame) {
+						WORD val = ((WORD*)data->o_stack)[frame];
+						if (val >= pmin && val <= pmax && val > 0) {
+							sum += (double)val;
+						}
+					}
+					mean = sum / (double)kept_pixels;
 				}
 				else mean = sum / norm;
 			} else {
@@ -813,9 +1021,9 @@ static double mean_and_reject(struct stacking_args *args, struct _data_block *da
 		if (kept_pixels == 0)
 			mean = quickmedian_float(data->stack, stack_size);
 		else {
-			if (args->apply_noise_weights || args->apply_nbstack_weights || args->apply_wfwhm_weights || args->apply_nbstars_weights) {
+			if (weighting || masking || args->drizzle) {
 				double *pweights = args->weights + layer * stack_size;
-				float pmin = 10000.0, pmax = -10000.0; /* min and max computed here instead of rejection step to avoid dealing with too many particular cases */
+				float pmin = FLT_MAX, pmax = -FLT_MAX; /* min and max computed here instead of rejection step to avoid dealing with too many particular cases */
 				for (int frame = 0; frame < kept_pixels; ++frame) {
 					if (pmin > ((float*)data->stack)[frame]) pmin = ((float*)data->stack)[frame];
 					if (pmax < ((float*)data->stack)[frame]) pmax = ((float*)data->stack)[frame];
@@ -826,13 +1034,29 @@ static double mean_and_reject(struct stacking_args *args, struct _data_block *da
 				for (int frame = 0; frame < stack_size; ++frame) {
 					float val = ((float*)data->o_stack)[frame];
 					if (val >= pmin && val <= pmax && val != 0.f) {
-						sum += val * pweights[frame];
-						norm += pweights[frame];
+						double n = 1.;
+						if (args->drizzle)
+							n *= data->dstack[frame];
+						if (masking)
+							n *= data->mstack[frame];
+						if (weighting)
+							n *= pweights[frame];
+						sum += (double)val * n;
+						norm += n;
 					}
 				}
-				if (norm == 0.0)
+				if (norm == 0. || sum == 0.) { // if norm is 0, sum is 0 too
+					// We replace by the sum without weighting
+					// Will not look good, but it's better than a black pixel
+					sum = 0.;
+					for (int frame = 0; frame < stack_size; ++frame) {
+						float val = ((float*)data->o_stack)[frame];
+						if (val >= pmin && val <= pmax && val > 0) {
+							sum += (double)val;
+						}
+					}
 					mean = sum / (double)kept_pixels;
-				else mean = sum / norm;
+				} else mean = sum / norm;
 			} else {
 				double sum = 0.0;
 				for (int frame = 0; frame < kept_pixels; ++frame) {
@@ -915,6 +1139,8 @@ static int compute_wfwhm_weights(struct stacking_args *args) {
 			}
 		}
 		norm /= (double) nb_frames;
+		if (!norm)
+			return ST_GENERIC_ERROR;
 
 		for (int i = 0; i < args->nb_images_to_stack; i++) {
 			pweights[layer][i] /= norm;
@@ -974,10 +1200,12 @@ static int compute_nbstars_weights(struct stacking_args *args) {
 
 /* How many rows fit in memory, based on image size, number and available memory.
  * It returns at most the total number of rows of the image (naxes[1] * naxes[2]) */
-static long stack_get_max_number_of_rows(long naxes[3], data_type type, int nb_images_to_stack, int nb_rejmaps) {
+static long stack_get_max_number_of_rows(long naxes[3], data_type type, int nb_images_to_stack, int nb_rejmaps, gboolean masking, gboolean drizzle) {
 	int max_memory = get_max_memory_in_MB();
 	long total_nb_rows = naxes[1] * naxes[2];
 	int elem_size = type == DATA_FLOAT ? sizeof(float) : sizeof(WORD);
+	int mask_elem_size = (masking) ? sizeof(float) : 0;
+	int drizz_elem_size = (drizzle) ? sizeof(float) : 0;
 
 	guint64 size_of_result = naxes[0] * naxes[1] * naxes[2] * elem_size;
 	guint64 size_of_rejmaps = naxes[0] * naxes[1] * naxes[2] * sizeof(WORD);
@@ -987,8 +1215,11 @@ static long stack_get_max_number_of_rows(long naxes[3], data_type type, int nb_i
 		max_memory = 0;
 
 	siril_log_message(_("Using %d MB memory maximum for stacking\n"), max_memory);
+	// for each datablock, we store the pixel values
+	// if masking, we also need to store all the masks in float + 1 mask in 32b to conpute the smoothing (this mask is freed for every frame)
+	// if drizzle, we need to store the drizzle weights in float
 	guint64 number_of_rows = (guint64)max_memory * BYTES_IN_A_MB /
-		((guint64)naxes[0] * nb_images_to_stack * elem_size);
+		(nb_images_to_stack * naxes[0] * (elem_size + mask_elem_size + drizz_elem_size) + (masking) * naxes[0] * sizeof(float));
 	// this is how many rows we can load in parallel from all images of the
 	// sequence and be under the limit defined in config in megabytes.
 	if (total_nb_rows < number_of_rows)
@@ -1007,6 +1238,10 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 	guint64 irej[3][2] = {{0,0}, {0,0}, {0,0}};
 	regdata *layerparam = NULL;
 
+	gboolean masking = (args->feather_dist > 0);
+	if (masking)
+		init_ramp(); // we cache the values of the masks ramping function
+
 	int nb_frames = args->nb_images_to_stack; // number of frames actually used
 	naxes[0] = naxes[1] = 0; naxes[2] = 1;
 
@@ -1038,7 +1273,7 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 		retval = ST_SEQUENCE_ERROR;
 		goto free_and_close;
 	}
-	if (naxes[0] != args->seq->rx || naxes[1] != args->seq->ry) {
+	if (!args->maximize_framing && (naxes[0] != args->seq->rx || naxes[1] != args->seq->ry)) {
 		siril_log_color_message(_("Rejection stack error: sequence has wrong image size (%dx%d for sequence, %ldx%ld for images)\n"), "red", args->seq->rx, args->seq->ry, naxes[0], naxes[1]);
 		retval = ST_SEQUENCE_ERROR;
 		goto free_and_close;
@@ -1075,6 +1310,11 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 		}
 	}
 
+	/* prepare the downscaled 8b masks if masking is allowed */
+	if (masking && (retval = compute_masks(args))) {
+		goto free_and_close;
+	}
+
 	/* manage threads */
 	int nb_threads;
 #ifdef _OPENMP
@@ -1110,7 +1350,7 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 			nb_rejmaps = 1;
 		else nb_rejmaps = 2;
 	}
-	long max_number_of_rows = stack_get_max_number_of_rows(naxes, itype, args->nb_images_to_stack, nb_rejmaps);
+	long max_number_of_rows = stack_get_max_number_of_rows(naxes, itype, args->nb_images_to_stack, nb_rejmaps, masking, args->drizzle);
 	/* Compute parallel processing data: the data blocks, later distributed to threads */
 	if ((retval = stack_compute_parallel_blocks(&blocks, max_number_of_rows, naxes, nb_threads,
 					&largest_block_height, &nb_blocks))) {
@@ -1128,11 +1368,20 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 	size_t npixels_in_block = largest_block_height * naxes[0];
 	g_assert(npixels_in_block > 0);
 	int ielem_size = itype == DATA_FLOAT ? sizeof(float) : sizeof(WORD);
+	int ielem_mask_size = (masking) ? sizeof(float) : 0;
+	int ielem_drizz_size = (args->drizzle) ? sizeof(float) : 0;
 
 	fprintf(stdout, "allocating data for %d threads (each %lu MB)\n", pool_size,
 			(unsigned long)(nb_frames * npixels_in_block * ielem_size) / BYTES_IN_A_MB);
 	data_pool = calloc(pool_size, sizeof(struct _data_block));
 	size_t bufferSize = ielem_size * nb_frames * (npixels_in_block + 1ul) + 4ul; // buffer for tmp and stack, added 4 byte for alignment
+	// the +1ul pixel is for storing the current pixel stack
+	if (masking)
+		bufferSize += ielem_mask_size * nb_frames * (npixels_in_block + 1ul) + 4ul; // buffer for masks and mask stack, added 4 byte for alignment
+		// the +1ul pixel is for storing the current mask stack
+	if (args->drizzle)
+		bufferSize += ielem_drizz_size * nb_frames * (npixels_in_block + 1ul) + 4ul; // buffer for drizz weights and weights stack, added 4 byte for alignment
+		// the +1ul pixel is for storing the current weight stack
 	if (is_mean) {
 		bufferSize += nb_frames * sizeof(int); // for rejected
 		bufferSize += ielem_size * nb_frames; // for o_stack
@@ -1146,9 +1395,13 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 		}
 	}
 	for (i = 0; i < pool_size; i++) {
-		data_pool[i].pix = malloc(nb_frames * sizeof(void *));
-		data_pool[i].tmp = malloc(bufferSize);
-		if (!data_pool[i].pix || !data_pool[i].tmp) {
+		data_pool[i].pix = siril_malloc(nb_frames * sizeof(void *));
+		if (masking)
+			data_pool[i].mask = siril_malloc(nb_frames * sizeof(float *));
+		if (args->drizzle)
+			data_pool[i].drizz = siril_malloc(nb_frames * sizeof(float *));
+		data_pool[i].tmp = siril_malloc(bufferSize);
+		if (!data_pool[i].pix || !data_pool[i].tmp || (masking && !data_pool[i].mask) || (args->drizzle && !data_pool[i].drizz)) {
 			PRINT_ALLOC_ERR;
 			gchar *available = g_format_size_full(get_available_memory(), G_FORMAT_SIZE_IEC_UNITS);
 			fprintf(stderr, "Cannot allocate %zu (free memory: %s)\n", bufferSize / BYTES_IN_A_MB, available);
@@ -1158,14 +1411,33 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 			retval = ST_ALLOC_ERROR;
 			goto free_and_close;
 		}
-		data_pool[i].stack = (void*) ((char*) data_pool[i].tmp
+		data_pool[i].stack = (void *)((char *)data_pool[i].tmp
 				+ nb_frames * npixels_in_block * ielem_size);
-		if (is_mean) {
-			size_t offset = (size_t)ielem_size * nb_frames * (npixels_in_block + 1);
-			int temp = offset % sizeof(int);
+		size_t stack_offset = (size_t)ielem_size * nb_frames * (npixels_in_block + 1);
+		int temp = stack_offset % sizeof(int);
+		if (temp > 0) { // align buffer
+			stack_offset += sizeof(int) - temp;
+		}
+		size_t mask_offset = 0;
+		if (masking) {
+			data_pool[i].mstack = (float *)((char *)data_pool[i].tmp + stack_offset + ielem_mask_size * nb_frames * npixels_in_block); // mask stack is stored after the pixels
+			mask_offset = (size_t)ielem_mask_size * nb_frames * (npixels_in_block + 1);
+			temp = mask_offset % sizeof(int);
 			if (temp > 0) { // align buffer
-				offset += sizeof(int) - temp;
+				mask_offset += sizeof(int) - temp;
 			}
+		}
+		size_t drizz_offset = 0;
+		if (args->drizzle) {
+			data_pool[i].dstack = (float *)((char *)data_pool[i].tmp + stack_offset + mask_offset + ielem_drizz_size * nb_frames * npixels_in_block); // drizz weight stack is stored after the masks
+			drizz_offset = (size_t)ielem_drizz_size * nb_frames * (npixels_in_block + 1);
+			temp = drizz_offset % sizeof(int);
+			if (temp > 0) { // align buffer
+				drizz_offset += sizeof(int) - temp;
+			}
+		}
+		if (is_mean) {
+			size_t offset = stack_offset + mask_offset + drizz_offset;
 			data_pool[i].rejected = (int*)((char*)data_pool[i].tmp + offset);
 			data_pool[i].o_stack = (void*)((char*)data_pool[i].rejected + sizeof(int) * nb_frames);
 
@@ -1174,7 +1446,7 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 			} else if (args->type_of_rejection == GESDT) {
 				data_pool[i].w_stack = (void*)((char*)data_pool[i].o_stack + ielem_size * nb_frames);
 				int max_outliers = (int) floor(nb_frames * args->sig[0]);
-				args->critical_value = malloc(max_outliers * sizeof(float));
+				args->critical_value = malloc(max_outliers * sizeof(float)); // why do we malloc here? space has already been booked in tmp
 				for (int j = 0, size = nb_frames; j < max_outliers; j++, size--) {
 					float t_dist = gsl_cdf_tdist_Pinv(1 - args->sig[1] / (2 * size), size - 2);
 					float numerator = (size - 1) * t_dist;
@@ -1200,7 +1472,12 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 		for (int j = 0; j < nb_frames; ++j) {
 			if (itype == DATA_FLOAT)
 				data_pool[i].pix[j] = ((float*)data_pool[i].tmp) + j * npixels_in_block;
-			else 	data_pool[i].pix[j] = ((WORD *)data_pool[i].tmp) + j * npixels_in_block;
+			else 
+				data_pool[i].pix[j] = ((WORD *)data_pool[i].tmp) + j * npixels_in_block;
+			if (masking)
+				data_pool[i].mask[j] = (float *)((char*)data_pool[i].tmp + stack_offset + j * npixels_in_block * ielem_mask_size);
+			if (args->drizzle)
+				data_pool[i].drizz[j] = (float *)((char*)data_pool[i].tmp + stack_offset + mask_offset + j * npixels_in_block * ielem_drizz_size);
 		}
 	}
 
@@ -1208,33 +1485,30 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 		args->sd_calculator = nb_frames < 65536 ? siril_stats_ushort_sd_32 : siril_stats_ushort_sd_64;
 		args->mad_calculator = siril_stats_ushort_mad;
 	}
-
-	if (args->apply_noise_weights) {
-		siril_log_message(_("Computing weights based on noise...\n"));
-		retval = compute_noise_weights(args);
-		if (retval) {
-			retval = ST_GENERIC_ERROR;
-			goto free_and_close;
-		}
+	switch (args->weighting_type) {
+		default:
+		case NO_WEIGHT:
+			retval = ST_OK;
+			break;
+		case NOISE_WEIGHT:
+			siril_log_message(_("Computing weights based on noise...\n"));
+			retval = compute_noise_weights(args);
+			break;
+		case WFWHM_WEIGHT:
+			siril_log_message(_("Computing weights based on wFWHM...\n"));
+			retval = compute_wfwhm_weights(args);
+			break;
+		case NBSTARS_WEIGHT:
+			siril_log_message(_("Computing weights based on number of stars...\n"));
+			retval = compute_nbstars_weights(args);
+			break;
+		case NBSTACK_WEIGHT:
+			siril_log_message(_("Computing weights based on number of stacked images...\n"));
+			break;
 	}
-	if (args->apply_wfwhm_weights) {
-		siril_log_message(_("Computing weights based on wFWHM...\n"));
-		retval = compute_wfwhm_weights(args);
-		if (retval) {
-			retval = ST_GENERIC_ERROR;
-			goto free_and_close;
-		}
-	}
-	if (args->apply_nbstars_weights) {
-		siril_log_message(_("Computing weights based on number of stars...\n"));
-		retval = compute_nbstars_weights(args);
-		if (retval) {
-			retval = ST_GENERIC_ERROR;
-			goto free_and_close;
-		}
-	}
-	if (args->apply_nbstack_weights) {
-		siril_log_message(_("Computing weights based on number of stacked images...\n"));
+	if (retval) {
+		retval = ST_GENERIC_ERROR;
+		goto free_and_close;
 	}
 
 	siril_log_message(_("Starting stacking...\n"));
@@ -1249,7 +1523,7 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 	for (i = 0; i < nb_blocks; i++)
 	{
 		/**** Step 1: get allocated memory for the current thread ****/
-		struct _image_block *my_block = blocks+i;
+		struct _image_block *my_block = blocks + i;
 		struct _data_block *data;
 		int data_idx = 0;
 		guint64 brej[2] = {0, 0}; // rejection counts for the block
@@ -1295,10 +1569,7 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 			if (retval) break;
 
 			// update progress bar
-#ifdef _OPENMP
-#pragma omp atomic
-#endif
-			cur_nb++;
+			g_atomic_int_inc(&cur_nb);
 
 			if (!get_thread_run()) {
 				retval = ST_CANCEL;
@@ -1314,11 +1585,13 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 					int pix_idx = line_idx + x;
 					if (layerparam) {
 						int shiftx = 0;
-						double scale = args->seq->upscale_at_stacking;
+						double scale = (args->upscale_at_stacking) ? 2. : 1.;
 						double dx, dy;
 						translation_from_H(layerparam[args->image_indices[frame]].H, &dx, &dy);
+						dx -= args->offset[0];
 						shiftx = round_to_int(dx * scale);
 
+
 						if (shiftx && (x - shiftx >= naxes[0] || x - shiftx < 0)) {
 							/* outside bounds, images are black. We could
 							 * also set the background value instead, if available */
@@ -1373,13 +1646,19 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 							// multiplicative + scale (offset[frame] = 0)
 							if (itype == DATA_FLOAT) {
 								tmp = fpixel * args->coeff.pscale[layer][frame];
-								((float*)data->stack)[frame] = (float)(tmp * args->coeff.pmul[layer][frame]);
+								((float *)data->stack)[frame] = (float)(tmp * args->coeff.pmul[layer][frame]);
 							} else {
 								tmp = (double)pixel * args->coeff.pscale[layer][frame];
 								((WORD *)data->stack)[frame] = round_to_WORD(tmp * args->coeff.pmul[layer][frame]);
 							}
 							break;
 					}
+					if (masking) {
+						data->mstack[frame] = data->mask[frame][pix_idx];
+					}
+					if (args->drizzle) {
+						data->dstack[frame] = data->drizz[frame][pix_idx];
+					}
 				}
 
 				double result; // resulting pixel value, either mean or median
@@ -1465,6 +1744,9 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 		norm_to_0_1_range(&fit);
 	compute_date_time_keywords(list_date, &fit);
 	memcpy(&args->result, &fit, sizeof(fits));
+	if (has_wcs(&args->result)) {
+		update_wcsdata_from_wcs(&args->result);
+	}
 
 free_and_close:
 	fprintf(stdout, "free and close (%d)\n", retval);
@@ -1474,8 +1756,10 @@ free_and_close:
 
 	if (data_pool) {
 		for (i=0; i<pool_size; i++) {
-			if (data_pool[i].pix) free(data_pool[i].pix);
-			if (data_pool[i].tmp) free(data_pool[i].tmp);
+			if (data_pool[i].pix) siril_free(data_pool[i].pix);
+			if (data_pool[i].mask) siril_free(data_pool[i].mask);
+			if (data_pool[i].drizz) siril_free(data_pool[i].drizz);
+			if (data_pool[i].tmp) siril_free(data_pool[i].tmp);
 		}
 		free(data_pool);
 	}
