@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,8 @@
 #include "io/sequence.h"
 #include "io/ser.h"
 #include "registration/registration.h"
+#include "algos/siril_wcs.h"
+#include "opencv/opencv.h"
 
 #include "stacking/stacking.h"
 
@@ -65,6 +67,22 @@ static int stack_addminmax(struct stacking_args *args, gboolean ismax) {
 		siril_log_message(_("No frame selected for stacking (select at least 2). Aborting.\n"));
 		return ST_GENERIC_ERROR;
 	}
+	int output_size[2], offset[2];
+	gboolean update_wcs = FALSE;
+	if (args->maximize_framing) {
+		compute_max_framing(args, output_size, offset);
+		update_wcs = TRUE;
+	} else {
+		output_size[0] = args->seq->rx;
+		output_size[1] = args->seq->ry;
+		double dx = 0., dy = 0.;
+		if (reglayer >= 0) {
+			translation_from_H(args->seq->regparam[reglayer][args->ref_image].H, &dx, &dy);
+			update_wcs = TRUE;
+		}
+		offset[0] = (int)dx;
+		offset[1] = (int)dy;
+	}
 
 	final_pixel[0] = NULL;
 	ffinal_pixel[0] = NULL;
@@ -102,13 +120,14 @@ static int stack_addminmax(struct stacking_args *args, gboolean ismax) {
 		/* first loaded image: init data structures for stacking */
 		if (!nbdata) {
 			is_float = fit.type == DATA_FLOAT;
-			nbdata = fit.naxes[0] * fit.naxes[1];
+			nbdata = output_size[0] * output_size[1];
+			size_t nbpixels = nbdata * fit.naxes[2];
 			if (is_float) {
 				if (ismax)
-					ffinal_pixel[0] = calloc(nbdata * fit.naxes[2], sizeof(float));
+					ffinal_pixel[0] = calloc(nbpixels, sizeof(float));
 				else {
-					ffinal_pixel[0] = malloc(nbdata * fit.naxes[2] * sizeof(float));
-					for (long k = 0; k < nbdata * fit.naxes[2]; k++)
+					ffinal_pixel[0] = malloc(nbpixels * sizeof(float));
+					for (long k = 0; k < nbpixels; k++)
 						ffinal_pixel[0][k] = 1.0;
 				}
 				if (!ffinal_pixel[0]) {
@@ -122,10 +141,10 @@ static int stack_addminmax(struct stacking_args *args, gboolean ismax) {
 				}
 			} else {
 				if (ismax)
-					final_pixel[0] = calloc(nbdata * fit.naxes[2], sizeof(WORD));
+					final_pixel[0] = calloc(nbpixels, sizeof(WORD));
 				else {
-					final_pixel[0] = malloc(nbdata * fit.naxes[2] * sizeof(WORD));
-					for (long k = 0; k < nbdata * fit.naxes[2]; k++)
+					final_pixel[0] = malloc(nbpixels * sizeof(WORD));
+					for (long k = 0; k < nbpixels; k++)
 						final_pixel[0][k] = USHRT_MAX;
 				}
 				if (!final_pixel[0]) {
@@ -138,7 +157,7 @@ static int stack_addminmax(struct stacking_args *args, gboolean ismax) {
 					final_pixel[2] = final_pixel[1] + nbdata;
 				}
 			}
-		} else if (fit.ry * fit.rx != nbdata) {
+		} else if (fit.ry * fit.rx != nbdata && !args->maximize_framing) {
 			siril_log_message(_("Stacking: image in sequence doesn't has the same dimensions\n"));
 			retval = ST_SEQUENCE_ERROR;
 			goto free_and_reset_progress_bar;
@@ -148,10 +167,17 @@ static int stack_addminmax(struct stacking_args *args, gboolean ismax) {
 		int shiftx, shifty;
 		if (reglayer != -1 && args->seq->regparam[reglayer]) {
 			double dx, dy;
-			double scale = args->seq->upscale_at_stacking;
+			double scale = (args->upscale_at_stacking) ? 2. : 1.;
 			translation_from_H(args->seq->regparam[reglayer][j].H, &dx, &dy);
-			shiftx = round_to_int(dx * scale);
-			shifty = round_to_int(dy * scale);
+			dx *= scale;
+			dy *= scale;
+			dx -= offset[0];
+			dy -= offset[1];
+			if (args->maximize_framing)
+				dy -= (double)fit.ry;
+			shiftx = round_to_int(dx);
+			shifty = round_to_int(dy);
+			siril_debug_print("img %d dx %d dy %d\n", j, shiftx, shifty);
 		} else {
 			shiftx = 0;
 			shifty = 0;
@@ -161,24 +187,32 @@ static int stack_addminmax(struct stacking_args *args, gboolean ismax) {
 #endif
 
 		/* Summing the exposure */
-		livetime += fit.exposure;
+		livetime += fit.keywords.exposure;
 
-		if (fit.date_obs) {
-			GDateTime *date = g_date_time_ref(fit.date_obs);
-			list_date = g_list_prepend(list_date, new_date_item(date, fit.exposure));
+		if (fit.keywords.date_obs) {
+			GDateTime *date = g_date_time_ref(fit.keywords.date_obs);
+			list_date = g_list_prepend(list_date, new_date_item(date, fit.keywords.exposure));
 		}
 
 		/* stack current image */
+		if (shiftx == INT_MIN) { // mainly to avoid static checker warning
+			siril_debug_print("Error: image #%d has a wrong shiftx value\n", j + 1);
+			shiftx += 1;
+		}
+		if (shifty == INT_MIN) { // mainly to avoid static checker warning
+			siril_debug_print("Error: image #%d has a wrong shifty value\n", j + 1);
+			shifty += 1;
+		}
 		size_t i = 0;	// index in final_pixel[0]
-		for (int y = 0; y < fit.ry; ++y) {
-			for (int x = 0; x < fit.rx; ++x) {
+		for (int y = 0; y < output_size[1]; ++y) {
+			int ny = y - shifty;
+			for (int x = 0; x < output_size[0]; ++x) {
 				int nx = x - shiftx;
-				int ny = y - shifty;
 				//printf("x=%d y=%d sx=%d sy=%d i=%d ii=%d\n",x,y,shiftx,shifty,i,ii);
 				if (nx >= 0 && nx < fit.rx && ny >= 0 && ny < fit.ry) {
 					size_t ii = ny * fit.rx + nx;		// index in final_pixel[0] too
 					//printf("shiftx=%d shifty=%d i=%d ii=%d\n",shiftx,shifty,i,ii);
-					if (ii > 0 && ii < fit.rx * fit.ry) {
+					if (ii < nbdata) {
 						for (int layer = 0; layer < args->seq->nb_layers; ++layer) {
 							if (is_float) {
 								float current_pixel = fit.fpdata[layer][ii];
@@ -211,10 +245,10 @@ static int stack_addminmax(struct stacking_args *args, gboolean ismax) {
 
 	fits *result = &args->result;
 	if (is_float) {
-		if (new_fit_image_with_data(&result, args->seq->rx, args->seq->ry, args->seq->nb_layers, DATA_FLOAT, ffinal_pixel[0]))
+		if (new_fit_image_with_data(&result, output_size[0], output_size[1], args->seq->nb_layers, DATA_FLOAT, ffinal_pixel[0]))
 			return ST_GENERIC_ERROR;
 	} else {
-		if (new_fit_image_with_data(&result, args->seq->rx, args->seq->ry, args->seq->nb_layers, DATA_USHORT, final_pixel[0]))
+		if (new_fit_image_with_data(&result, output_size[0], output_size[1], args->seq->nb_layers, DATA_USHORT, final_pixel[0]))
 			return ST_GENERIC_ERROR;
 	}
 
@@ -225,20 +259,36 @@ static int stack_addminmax(struct stacking_args *args, gboolean ismax) {
 			import_metadata_from_fitsfile(args->seq->fptr[ref], result);
 			result->orig_bitpix = result->bitpix = args->seq->bitpix;
 			seq_close_image(args->seq, ref);
-		result->livetime = livetime;
+		result->keywords.livetime = livetime;
 		}
 	} else if (args->seq->type == SEQ_FITSEQ) {
 		if (!fitseq_set_current_frame(args->seq->fitseq_file, ref)) {
 			import_metadata_from_fitsfile(args->seq->fitseq_file->fptr, result);
 			result->orig_bitpix = result->bitpix = args->seq->fitseq_file->bitpix;
 		}
-		result->livetime = livetime;
+		result->keywords.livetime = livetime;
 	} else if (args->seq->type == SEQ_SER) {
 		import_metadata_from_serfile(args->seq->ser_file, result);
 		result->orig_bitpix = result->bitpix = (args->seq->ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) ? BYTE_IMG : USHORT_IMG;
-		result->livetime = result->exposure * args->nb_images_to_stack; // livetime is null for ser as fit has no exposure data
+		result->keywords.livetime = result->keywords.exposure * args->nb_images_to_stack; // livetime is null for ser as fit has no exposure data
+	}
+	result->keywords.stackcnt = args->nb_images_to_stack;
+	if (update_wcs && has_wcs(result)) {
+		Homography Hs = { 0 };
+		cvGetEye(&Hs);
+		double dx, dy;
+		translation_from_H(args->seq->regparam[args->reglayer][args->ref_image].H, &dx, &dy);
+		siril_debug_print("ref shift: %d %d\n", (int)dx, (int)dy);
+		siril_debug_print("crpix: %.1f %.1f\n", result->keywords.wcslib->crpix[0], result->keywords.wcslib->crpix[1]);
+		Hs.h02  = dx - offset[0];
+		Hs.h12 -= dy - offset[1];
+		int orig_rx = (args->seq->is_variable) ? args->seq->imgparam[args->seq->reference_image].rx : args->seq->rx;
+		int orig_ry = (args->seq->is_variable) ? args->seq->imgparam[args->seq->reference_image].ry : args->seq->ry;
+		siril_debug_print("size: %d %d\n", orig_rx, orig_ry);
+		cvApplyFlips(&Hs, orig_ry, 0);
+		reframe_wcs(result->keywords.wcslib, &Hs);
+		update_wcsdata_from_wcs(result);
 	}
-	result->stackcnt = args->nb_images_to_stack;
 
 	compute_date_time_keywords(list_date, result);
 	g_list_free_full(list_date, (GDestroyNotify) free_list_date);
