@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,21 +31,21 @@
 #include "io/image_format_fits.h"
 #include "gui/progress_and_log.h"
 #include "gui/image_interactions.h"
-#include "gui/utils.h" // for delete_selected_area()
 #include "opencv/opencv.h"
 
 #include "stacking.h"
 
 #define TMP_UPSCALED_PREFIX "tmp_upscaled_"
 
-void remove_tmp_drizzle_files(struct stacking_args *args) {
+void remove_tmp_upscaled_files(struct stacking_args *args) {
 	int i;
-	if (args->seq->upscale_at_stacking < 1.05)
+	if (!args->upscale_at_stacking)
 		return;
 
 	gchar *basename = g_path_get_basename(args->seq->seqname);
 	if (!g_str_has_prefix(basename, TMP_UPSCALED_PREFIX)) {
 		remove_prefixed_sequence_files(args->seq, TMP_UPSCALED_PREFIX);
+		g_free(basename);
 		return;
 	}
 	// else means that we are removing files after processing and that
@@ -67,6 +67,8 @@ void remove_tmp_drizzle_files(struct stacking_args *args) {
 		for (i = 0; i < args->seq->number; i++) {
 			char filename[500];
 			// FIXME: no preallocation of file name
+			// don't use the fit_sequence_get_image_filename_checkext function here as these
+			// temp files are created with the preferred extension anyway.
 			fit_sequence_get_image_filename(args->seq, args->image_indices[i], filename, TRUE);
 			siril_debug_print("Removing %s\n", filename);
 			if (g_unlink(filename))
@@ -110,8 +112,8 @@ struct upscale_args {
 static int upscale_image_hook(struct generic_seq_args *args, int o, int i, fits *fit, rectangle *_, int threads) {
 	double factor = ((struct upscale_args *)args->user)->factor;
 	/* updating pixel size if exist */
-	fit->pixel_size_x /= factor;
-	fit->pixel_size_y /= factor;
+	fit->keywords.pixel_size_x /= factor;
+	fit->keywords.pixel_size_y /= factor;
 
 	return cvResizeGaussian(fit,
 			round_to_int(fit->rx * factor),
@@ -119,13 +121,13 @@ static int upscale_image_hook(struct generic_seq_args *args, int o, int i, fits
 }
 
 int upscale_sequence(struct stacking_args *stackargs) {
-	if (stackargs->seq->upscale_at_stacking <= 1.05)
+	if (!stackargs->upscale_at_stacking)
 		return 0;
 
 	struct generic_seq_args *args = create_default_seqargs(stackargs->seq);
-	struct upscale_args *upargs = malloc(sizeof(struct upscale_args));
+	struct upscale_args *upargs = calloc(1, sizeof(struct upscale_args));
 
-	upargs->factor = stackargs->seq->upscale_at_stacking;
+	upargs->factor = 2.;
 
 	args->filtering_criterion = stackargs->filtering_criterion;
 	args->filtering_parameter = stackargs->filtering_parameter;
@@ -137,7 +139,7 @@ int upscale_sequence(struct stacking_args *stackargs) {
 	args->has_output = TRUE;
 	args->output_type = get_data_type(args->seq->bitpix);
 	args->upscale_ratio = upargs->factor;
-	args->new_seq_prefix = TMP_UPSCALED_PREFIX;
+	args->new_seq_prefix = strdup(TMP_UPSCALED_PREFIX);
 	args->user = upargs;
 	args->already_in_a_thread = TRUE;
 
@@ -145,13 +147,14 @@ int upscale_sequence(struct stacking_args *stackargs) {
 	int nb_threads = seq_compute_mem_limits(args, FALSE);
 	if (nb_threads == 0) {
 		siril_log_color_message(_("Stacking will be done without up-scaling (disabling 'drizzle')\n"), "red");
-		stackargs->seq->upscale_at_stacking = 1.0;
-		free(args);
+		stackargs->upscale_at_stacking = FALSE;
+		free(upargs);
+		free_generic_seq_args(args, TRUE);
 		return 0;
 	}
 	args->max_parallel_images = nb_threads;
 
-	remove_tmp_drizzle_files(stackargs);
+	remove_tmp_upscaled_files(stackargs);
 
 	generic_sequence_worker(args);
 
@@ -205,13 +208,12 @@ int upscale_sequence(struct stacking_args *stackargs) {
 		newseq->reference_image = find_refimage_in_indices(stackargs->image_indices,
 				stackargs->nb_images_to_stack, stackargs->ref_image);
 		stackargs->ref_image = newseq->reference_image;
-		newseq->upscale_at_stacking = oldseq->upscale_at_stacking;
 		newseq->regparam[stackargs->reglayer] = malloc(stackargs->nb_images_to_stack * sizeof(regdata));
 		int i;
 		for (i = 0; i < stackargs->nb_images_to_stack; i++) {
 			regdata *data = &oldseq->regparam[stackargs->reglayer][stackargs->image_indices[i]];
 			memcpy(&newseq->regparam[stackargs->reglayer][i], data, sizeof(regdata));
-			// TODO: why don't we modify the shifts here already?
+			// TODO: why don't we modify the shifts here already? indeed!
 		}
 		stackargs->retval = stack_fill_list_of_unfiltered_images(stackargs);
 
