@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2026 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@ static float compute_mean(const float *f, int size) {
 	return sum / size;
 }
 
-/*static void print_outliers(struct outliers *rej, int N) {
+/*static void print_outliers(struct ESD_outliers *rej, int N) {
 	fprintf(stderr, "outliers are: ");
 	for (int i = 0; i < N; i++) {
 		if (rej[i].out) fprintf(stderr, "%f ", rej[i].x);
@@ -69,7 +69,7 @@ static float calculate_critical_value(int size, float alpha) {
 }
 
 static float ESD_test(float *stack, int size, float alpha, int max_outliers) {
-	struct outliers *out = malloc(max_outliers * sizeof(struct outliers));
+	struct ESD_outliers *out = malloc(max_outliers * sizeof(struct ESD_outliers));
 
 	quicksort_f(stack, size);
 	double median = gsl_stats_float_median_from_sorted_data(stack, 1, size);
